/*
 *  armboot - Startup Code for Xarina/ARM1176 from OMP2420/ARM1136
 *
 *  Copyright (c) 2004	Texas Instruments <r-woodruff2@ti.com>
 *  Copyright (c) 2007, 2011	Sony Corporation.
 *
 *  Copyright (c) 2001	Marius Grger <mag@sysgo.de>
 *  Copyright (c) 2002	Alex ZEke <azu@sysgo.de>
 *  Copyright (c) 2002	Gary Jennejohn <gj@denx.de>
 *  Copyright (c) 2003	Richard Woodruff <r-woodruff2@ti.com>
 *  Copyright (c) 2003	Kshitij <kshitij@ti.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#ifdef CONFIG_XARINA
#include <asm/arch/xarina.h>
#endif  /* CONFIG_XARINA */

#ifdef CONFIG_XARINA
#if !defined(CONFIG_XARINA_SPL)
#if !defined(CONFIG_XARINA_NOR_BOOT)
.globl _start
_start:
.globl _ddr_early_vector_start
_ddr_early_vector_start:
	ldr	pc, _reset_handler
	ldr	pc, _undefined_instruction_handler
	ldr	pc, _software_interrupt_handler
	ldr	pc, _prefetch_abort_handler
	ldr	pc, _data_abort_handler
	ldr	pc, _not_used_handler
	ldr	pc, _irq_handler
	ldr	pc, _fiq_handler

_reset_handler:                 .word reset
_undefined_instruction_handler: .word undefined_instruction_handler
_software_interrupt_handler:    .word software_interrupt_handler
_prefetch_abort_handler:        .word prefetch_abort_handler
_data_abort_handler:            .word data_abort_handler
_not_used_handler:              .word not_used_handler
_irq_handler:                   .word irq_handler
_fiq_handler:                   .word fiq_handler
.global _ddr_early_vector_end
_ddr_early_vector_end:

 /* vector table running on ddr  */
.globl _ddr_vector_start
_ddr_vector_start:
	ldr	pc, _reset
	ldr	pc, _undefined_instruction
	ldr	pc, _software_interrupt
	ldr	pc, _prefetch_abort
	ldr	pc, _data_abort
	ldr	pc, _not_used
	ldr	pc, _irq
	ldr	pc, _fiq

_reset:                 .word reset
_undefined_instruction: .word undefined_instruction
_software_interrupt:    .word software_interrupt
_prefetch_abort:        .word prefetch_abort
_data_abort:            .word data_abort
_not_used:              .word not_used
_irq:                   .word irq
_fiq:                   .word fiq
.global _ddr_vector_end
_ddr_vector_end:

#else	/* !defined(CONFIG_XARINA_NOR_BOOT) */
 /* vector table running on nor flash */
.globl _start
_start:
	b reset
	b undefined_instruction_handler
	b software_interrupt_handler
	b prefetch_abort_handler
	b data_abort_handler
	b not_used_handler
	b irq_handler
	b fiq_handler

_pad0:			.word 0x12345678
_pad1:			.word 0x12345678
_pad2:			.word 0x12345678
_pad3:			.word 0x12345678
_pad4:			.word 0x12345678
_pad5:			.word 0x12345678
_pad6:			.word 0x12345678
_pad7:			.word 0x12345678
.global _end_vect
_end_vect:

 /* vector table early running on ddr sdram */
.globl _ddr_early_vector_start
_ddr_early_vector_start:
	ldr	pc, _reset_handler
	ldr	pc, _undefined_instruction_handler
	ldr	pc, _software_interrupt_handler
	ldr	pc, _prefetch_abort_handler
	ldr	pc, _data_abort_handler
	ldr	pc, _not_used_handler
	ldr	pc, _irq_handler
	ldr	pc, _fiq_handler

_reset_handler:                 .word reset
_undefined_instruction_handler: .word undefined_instruction_handler
_software_interrupt_handler:    .word software_interrupt_handler
_prefetch_abort_handler:        .word prefetch_abort_handler
_data_abort_handler:            .word data_abort_handler
_not_used_handler:              .word not_used_handler
_irq_handler:                   .word irq_handler
_fiq_handler:                   .word fiq_handler
.global _ddr_early_vector_end
_ddr_early_vector_end:

 /* vector table running on ddr sdram */
.globl _ddr_vector_start
_ddr_vector_start:
	ldr	pc, _reset
	ldr	pc, _undefined_instruction
	ldr	pc, _software_interrupt
	ldr	pc, _prefetch_abort
	ldr	pc, _data_abort
	ldr	pc, _not_used
	ldr	pc, _irq
	ldr	pc, _fiq

_reset:                 .word reset
_undefined_instruction: .word undefined_instruction
_software_interrupt:    .word software_interrupt
_prefetch_abort:        .word prefetch_abort
_data_abort:            .word data_abort
_not_used:              .word not_used
_irq:                   .word irq
_fiq:                   .word fiq
.global _ddr_vector_end
_ddr_vector_end:
#endif	/* !defined(CONFIG_XARINA_NOR_BOOT) */

#else  /* CONFIG_XARINA_SPL */
/* vector table running on audio memory */
.globl _start
_start:
	ldr	pc, _reset_handler
	ldr	pc, _undefined_instruction_handler
	ldr	pc, _software_interrupt_handler
	ldr	pc, _prefetch_abort_handler
	ldr	pc, _data_abort_handler
	ldr	pc, _not_used_handler
	ldr	pc, _irq_handler
	ldr	pc, _fiq_handler

_reset_handler:                 .word reset
_undefined_instruction_handler: .word undefined_instruction_handler
_software_interrupt_handler:    .word software_interrupt_handler
_prefetch_abort_handler:        .word prefetch_abort_handler
_data_abort_handler:            .word data_abort_handler
_not_used_handler:              .word not_used_handler
_irq_handler:                   .word irq_handler
_fiq_handler:                   .word fiq_handler

.global _end_vect
_end_vect:

#if !defined(CONFIG_NAND_SPL)
 /* vector table running on ddr sdram */
.globl _ddr_vector_start
_ddr_vector_start:
	ldr	pc, _reset
	ldr	pc, _undefined_instruction
	ldr	pc, _software_interrupt
	ldr	pc, _prefetch_abort
	ldr	pc, _data_abort
	ldr	pc, _not_used
	ldr	pc, _irq
	ldr	pc, _fiq

_reset:                 .word reset
_undefined_instruction: .word undefined_instruction
_software_interrupt:    .word software_interrupt
_prefetch_abort:        .word prefetch_abort
_data_abort:            .word data_abort
_not_used:              .word not_used
_irq:                   .word irq
_fiq:                   .word fiq
.global _ddr_vector_end
_ddr_vector_end:
#endif /* CONFIG_NAND_SPL */

#endif  /* CONFIG_XARINA_SPL */

#else  /* CONFIG_XARINA */

.globl _start
_start: b	reset
	ldr	pc, _undefined_instruction
	ldr	pc, _software_interrupt
	ldr	pc, _prefetch_abort
	ldr	pc, _data_abort
	ldr	pc, _not_used
	ldr	pc, _irq
	ldr	pc, _fiq

_undefined_instruction: .word undefined_instruction
_software_interrupt:	.word software_interrupt
_prefetch_abort:	.word prefetch_abort
_data_abort:		.word data_abort
_not_used:		.word not_used
_irq:			.word irq
_fiq:			.word fiq
_pad:			.word 0x12345678 /* now 16*4=64 */
.global _end_vect
_end_vect:

#endif  /* CONFIG_XARINA */

	.balignl 16,0xdeadbeef
/*
 *************************************************************************
 *
 * Startup Code (reset vector)
 *
 * do important init only if we don't start from memory!
 * setup Memory and board specific bits prior to relocation.
 * relocate armboot to ram
 * setup stack
 *
 *************************************************************************
 */

_TEXT_BASE:
	.word	TEXT_BASE

.globl _armboot_start
_armboot_start:
	.word _start

/*
 * These are defined in the board-specific linker script.
 */
.globl _bss_start
_bss_start:
	.word __bss_start

.globl _bss_end
_bss_end:
	.word _end

#ifdef CONFIG_USE_IRQ
/* IRQ stack memory (calculated at run-time) */
.globl IRQ_STACK_START
IRQ_STACK_START:
	.word	0x0badc0de

/* IRQ stack memory (calculated at run-time) */
.globl FIQ_STACK_START
FIQ_STACK_START:
	.word 0x0badc0de
#endif

/*
 * the actual reset code
 */

reset:
	/*
	 * set the cpu to SVC32 mode
	 */
	mrs	r0,cpsr
	bic	r0,r0,#0x1f
	orr	r0,r0,#0xd3
	msr	cpsr,r0

#if defined(CONFIG_XARINA_NOR_BOOT)
	/* jump nor mirror to nor */
	adr	r0, _start		/* r0 <- current position of code   */
	ldr	r1, _TEXT_BASE		/* test if we run from flash or RAM */
	sub	r4, r1, r0		/* r4 <- _start _TEXT_BASE equal flag */
	cmp	r0, r1			/* don't reloc during debug	    */
	beq	jump_to_substance_end

	/* This code starts from 0x0000_0000 */
	ldr r0, =CONFIG_FLASH_BASE
	mov r1, pc
	/* b   reset */
	add r1, r1, #4
	add pc, r1, r0
	nop
	nop
	nop
	nop
	nop

jump_to_substance_end:
#endif /* defined(CONFIG_XARINA) */

#if defined(CONFIG_XARINA)
	bl xa_pll1_sel_start
#endif

#if defined(CONFIG_XARINA_NOR_BOOT)
	/* initial unreset */
	bl	xarina_init_unreset
#endif


	/* the mask ROM code should have PLL and others stable */
#ifndef CONFIG_SKIP_LOWLEVEL_INIT
	bl  cpu_init_crit
#endif


#if defined(CONFIG_XARINA_NOR_BOOT) && !defined(CONFIG_SKIP_RELOCATE_UBOOT)
relocate:				/* relocate U-Boot to RAM	    */
	adr	r0, _start		/* r0 <- current position of code   */
	ldr	r1, _TEXT_BASE		/* test if we run from flash or RAM */
	cmp	r0, r1			/* don't reloc during debug	    */
	beq	stack_setup

	ldr	r2, _armboot_start
	ldr	r3, _bss_start
	sub	r2, r3, r2		/* r2 <- size of armboot	    */
	add	r2, r0, r2		/* r2 <- source end address	    */

copy_loop:
	ldmia	r0!, {r3-r10}		/* copy from source address [r0]    */
	stmia	r1!, {r3-r10}		/* copy to   target address [r1]    */
	cmp	r0, r2			/* until source end addreee [r2]    */
	ble	copy_loop
#endif	/* CONFIG_SKIP_RELOCATE_UBOOT */


#if !defined(CONFIG_XARINA_SPL)
enable_mmu:
	/* enable domain access */
	ldr	r5, =0x0000ffff
	mcr	p15, 0, r5, c3, c0, 0	/* load domain access register */

	/* Set the TTB register */
	ldr	r0, _mmu_table_base
	ldr	r1, =_TEXT_BASE
	ldr	r2, =0xfff00000
	bic	r0, r0, r2
	orr	r1, r0, r1
	mcr	p15, 0, r1, c2, c0, 0

	/* Enable the MMU */
	mrc	p15, 0, r0, c1, c0, 0
	orr	r0, r0, #1		/* Set CR_M to enable MMU */	

	/* Prepare to enable the MMU */
	adr	r1, skip_hw_init
	and	r1, r1, #0x3fc
	ldr	r2, _TEXT_BASE
	ldr	r3, =0xfff00000
	and	r2, r2, r3
	orr	r2, r2, r1
	b	mmu_enable

	.align 5
	/* Run in a single cache-line */
mmu_enable:

	mcr	p15, 0, r0, c1, c0, 0
	nop
	nop
skip_hw_init:
#endif

	/* Set up the stack						    */
stack_setup:
#if !defined(CONFIG_XARINA_SPL)
	ldr	r0, _TEXT_BASE		/* upper 128 KiB: relocated uboot   */
#else /* defined(CONFIG_XARINA) && !defined(CONFIG_XARINA_SPL) */
	ldr	r0, =XARINA_SPL_WORK_AREA_TOP		/* upper 128 KiB: relocated uboot   */
#endif /* defined(CONFIG_XARINA) && !defined(CONFIG_XARINA_SPL) */
	sub	r0, r0, #CONFIG_SYS_MALLOC_LEN /* malloc area			    */
	sub	r0, r0, #CONFIG_SYS_GBL_DATA_SIZE /* bdinfo			    */
#ifdef CONFIG_USE_IRQ
	sub	r0, r0, #(CONFIG_STACKSIZE_IRQ+CONFIG_STACKSIZE_FIQ)
#endif
	sub	sp, r0, #12		/* leave 3 words for abort-stack    */

clear_bss:
	ldr	r0, _bss_start		/* find start of bss segment	    */
	ldr	r1, _bss_end		/* stop here			    */
	mov	r2, #0x00000000		/* clear			    */

clbss_l:str	r2, [r0]		/* clear loop...		    */
	add	r0, r0, #4
	cmp	r0, r1
	bne	clbss_l
#if !defined(CONFIG_NAND_SPL)
        
#ifdef CONFIG_XARINA_DDR_TEST
        ldr     pc, _start_ddr_test
        
_start_ddr_test: .word start_ddr_test
        
#else	
	ldr	pc, _start_armboot

_start_armboot: .word start_armboot
        
#endif /* CONFIG_XARINA_DDR_TEST */
#else
	ldr	pc, _nand_boot

_nand_boot: .word nand_boot


/*	b	nand_boot */
/*	.word nand_boot */
#endif


#if !defined(CONFIG_XARINA_SPL)
_mmu_table_base:
	.word mmu_table
#endif

/*
 *************************************************************************
 *
 * CPU_init_critical registers
 *
 * setup important registers
 * setup memory timing
 *
 *************************************************************************
 */
cpu_init_crit:
	/*
	 * flush v4 I/D caches
	 */
	mov	r0, #0
	mcr	p15, 0, r0, c7, c7, 0	/* flush v3/v4 cache */
	mcr	p15, 0, r0, c8, c7, 0	/* flush v4 TLB */

	/*
	 * disable MMU stuff and caches
	 */
	mrc	p15, 0, r0, c1, c0, 0
	bic	r0, r0, #0x00002300	@ clear bits 13, 9:8 (--V- --RS)
	bic	r0, r0, #0x00000087	@ clear bits 7, 2:0 (B--- -CAM)
	orr	r0, r0, #0x00000002	@ set bit 2 (A) Align
	orr	r0, r0, #0x00001000	@ set bit 12 (I) I-Cache
	orr	r0, r0, #0x00000004	@ set bit 2 (D) D-Cache
	mcr	p15, 0, r0, c1, c0, 0

	/*
	 * enable VFP
	 */
	ldr	r0, =0x00f00000
	mcr	p15, 0, r0, c1, c0, 2	/* Co-Processor-Access CP11/CP10=all access */
	ldr	r0, =0x40000000
	fmxr	fpexc, r0		/* FMXR                EN=1 */

	/*
	 * Jump to board specific initialization... The Mask ROM will have already initialized
	 * basic memory.  Go here to bump up clock rate and handle wake up conditions.
	 */

	ldr	sp, =XARINA_E2PROM_I2C_SP_TOP	/* reset stack area to ARM9 SRAM */
	stmdb	sp!,{lr}	/* persevere link reg across call */

	bl	lowlevel_init	/* go setup pll,mux,memory */

	ldmia	sp!,{pc}	/* retrieve the link reg and return */

#if !defined(CONFIG_NAND_SPL)
/*
 *************************************************************************
 *
 * Interrupt handling
 *
 *************************************************************************
 */
@
@ IRQ stack frame.
@
#define S_FRAME_SIZE	72

#define S_OLD_R0	68
#define S_PSR		64
#define S_PC		60
#define S_LR		56
#define S_SP		52

#define S_IP		48
#define S_FP		44
#define S_R10		40
#define S_R9		36
#define S_R8		32
#define S_R7		28
#define S_R6		24
#define S_R5		20
#define S_R4		16
#define S_R3		12
#define S_R2		8
#define S_R1		4
#define S_R0		0

#define MODE_SVC 0x13
#define I_BIT	 0x80

/*
 * use bad_save_user_regs for abort/prefetch/undef/swi ...
 * use irq_save_user_regs / irq_restore_user_regs for IRQ/FIQ handling
 */

	.macro	bad_save_user_regs
	sub	sp, sp, #S_FRAME_SIZE		@ carve out a frame on current user stack
	stmia	sp, {r0 - r12}			@ Save user registers (now in svc mode) r0-r12

#if !defined(CONFIG_XARINA_SPL)
	ldr	r2, _armboot_start
#else /* !defined(CONFIG_XARINA_XPL) */
	ldr	r2, =XARINA_SPL_WORK_AREA_TOP
#endif /* !defined(CONFIG_XARINA_XPL) */
	sub	r2, r2, #(CONFIG_SYS_MALLOC_LEN)
	sub	r2, r2, #(CONFIG_SYS_GBL_DATA_SIZE+8)	@ set base 2 words into abort stack
	ldmia	r2, {r2 - r3}			@ get values for "aborted" pc and cpsr (into parm regs)
	add	r0, sp, #S_FRAME_SIZE		@ grab pointer to old stack

	add	r5, sp, #S_SP
	mov	r1, lr
	stmia	r5, {r0 - r3}			@ save sp_SVC, lr_SVC, pc, cpsr
	mov	r0, sp				@ save current stack into r0 (param register)
	.endm

	.macro	irq_save_user_regs
	sub	sp, sp, #S_FRAME_SIZE
	stmia	sp, {r0 - r12}			@ Calling r0-r12
	add	r8, sp, #S_PC			@ !!!! R8 NEEDS to be saved !!!! a reserved stack spot would be good.
	stmdb	r8, {sp, lr}^			@ Calling SP, LR
	str	lr, [r8, #0]			@ Save calling PC
	mrs	r6, spsr
	str	r6, [r8, #4]			@ Save CPSR
	str	r0, [r8, #8]			@ Save OLD_R0
	mov	r0, sp
	.endm

	.macro	irq_restore_user_regs
	ldmia	sp, {r0 - lr}^			@ Calling r0 - lr
	mov	r0, r0
	ldr	lr, [sp, #S_PC]			@ Get PC
	add	sp, sp, #S_FRAME_SIZE
	subs	pc, lr, #4			@ return & move spsr_svc into cpsr
	.endm

	.macro get_bad_stack
#if !defined(CONFIG_XARINA_SPL)
	ldr	r13, _armboot_start		@ setup our mode stack (enter in banked mode)
#else /* !defined(CONFIG_XARINA_XPL) */
	ldr	r13, =XARINA_SPL_WORK_AREA_TOP	@ setup our mode stack (enter in banked mode)
#endif /* !defined(CONFIG_XARINA_XPL) */
	sub	r13, r13, #(CONFIG_SYS_MALLOC_LEN)	@ move past malloc pool
	sub	r13, r13, #(CONFIG_SYS_GBL_DATA_SIZE+8) @ move to reserved a couple spots for abort stack

	str	lr, [r13]			@ save caller lr in position 0 of saved stack
	mrs	lr, spsr			@ get the spsr
	str	lr, [r13, #4]			@ save spsr in position 1 of saved stack

	mov	r13, #MODE_SVC			@ prepare SVC-Mode
	@ msr	spsr_c, r13
	msr	spsr, r13			@ switch modes, make sure moves will execute
	mov	lr, pc				@ capture return pc
	movs	pc, lr				@ jump to next instruction & switch modes.
	.endm

	.macro get_bad_stack_swi
	sub	r13, r13, #4			@ space on current stack for scratch reg.
	str	r0, [r13]			@ save R0's value.
#if !defined(CONFIG_XARINA_SPL)
	ldr	r0, _armboot_start		@ get data regions start
#else /* !defined(CONFIG_XARINA_SPL) */
	ldr	r0, =XARINA_SPL_WORK_AREA_TOP	@ get data regions start
#endif /* !defined(CONFIG_XARINA_SPL) */
	sub	r0, r0, #(CONFIG_SYS_MALLOC_LEN)	@ move past malloc pool
	sub	r0, r0, #(CONFIG_SYS_GBL_DATA_SIZE+8)	@ move past gbl and a couple spots for abort stack
	str	lr, [r0]			@ save caller lr in position 0 of saved stack
	mrs	lr, spsr			@ get the spsr
	str	lr, [r0, #4]			@ save spsr in position 1 of saved stack
	ldr	r0, [r13]			@ restore r0
	add	r13, r13, #4			@ pop stack entry
	.endm

	.macro get_irq_stack			@ setup IRQ stack
	ldr	sp, IRQ_STACK_START
	.endm

	.macro get_fiq_stack			@ setup FIQ stack
	ldr	sp, FIQ_STACK_START
	.endm
        
/*
 * exception handlers
 */
	.align	5
undefined_instruction:
	get_bad_stack
	bad_save_user_regs
	bl	do_undefined_instruction

	.align	5
software_interrupt:
	get_bad_stack_swi
	bad_save_user_regs
	bl	do_software_interrupt

	.align	5
prefetch_abort:
	get_bad_stack
	bad_save_user_regs
	bl	do_prefetch_abort

	.align	5
data_abort:
	get_bad_stack
	bad_save_user_regs
	bl	do_data_abort

	.align	5
not_used:
	get_bad_stack
	bad_save_user_regs
	bl	do_not_used

#ifdef CONFIG_USE_IRQ

	.align	5
irq:
	get_irq_stack
	irq_save_user_regs
	bl	do_irq
	irq_restore_user_regs

	.align	5
fiq:
	get_fiq_stack
	/* someone ought to write a more effiction fiq_save_user_regs */
	irq_save_user_regs
	bl	do_fiq
	irq_restore_user_regs

#else

	.align	5
irq:
	get_bad_stack
	bad_save_user_regs
	bl	do_irq

	.align	5
fiq:
	get_bad_stack
	bad_save_user_regs
	bl	do_fiq

#endif

#endif /* !defined(CONFIG_NAND_SPL) */

/*
 * exception handlers (boot -> remap)
 */
	.align	5
undefined_instruction_handler:
	b	undefined_instruction_handler

	.align	5
software_interrupt_handler:
	b	software_interrupt_handler

	.align	5
prefetch_abort_handler:
	b	prefetch_abort_handler

	.align	5
data_abort_handler:
	b	data_abort_handler

	.align	5
not_used_handler:
	b	not_used_handler

	.align	5
irq_handler:
	b	irq_handler

	.align	5
fiq_handler:
	b	fiq_handler


	.align 5
.global arm1176_cache_flush
arm1176_cache_flush:
		mcr	p15, 0, r1, c7, c5, 0	@ invalidate I cache
		mov	pc, lr			@ back to caller

