/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <common.h>

#ifdef CONFIG_I2C00_I2C
#ifdef CONFIG_HARD_I2C

#include <command.h>
#include <i2c.h>		/* Functional interface */
#include <asm/errno.h>
#include "i2c00.h"

/**************************/
/* prototype              */
/**************************/
static void i2c00_init(unsigned int saddr, unsigned int rate);
#ifdef CONFIG_XARINA_VSS
static void i2c00_bus_reset();
#endif /* CONFIG_XARINA_VSS */

/**************************/
/* global variable        */
/**************************/
static unsigned int g_bus  = CONFIG_I2C00_DEFAULT_NUM;
static unsigned int g_rate[] = {
	CONFIG_SYS_I2C_SPEED,
	CONFIG_SYS_I2C_SPEED,
};
static int g_slaveaddr = I2C00_SLAVEADDR_DEFAULT;

static unsigned long g_base_addr[] = {
	I2C00_BASE_00,
	I2C00_BASE_01,
};


/**************************/
/* I2C00 Utility          */
/**************************/
static unsigned int speed2rate(unsigned int speed)
{
	unsigned int rate = I2C00_100KBPS;

	switch(speed) {
	case I2C00_SPEED_400KBPS:
		rate = I2C00_400KBPS;
		break;
	case I2C00_SPEED_100KBPS:
		/* Do Nothing */
	default:
		rate = I2C00_100KBPS;
		break;
	}

	return rate;
}


static unsigned int rate2speed(unsigned int rate)
{
	unsigned int speed = I2C00_SPEED_100KBPS;

	/* speed value convert */
	switch(rate) {
	case I2C00_400KBPS:
		speed = I2C00_SPEED_400KBPS;
		break;
	case I2C00_100KBPS:
		/* Do Nothing */
	default:
		speed = I2C00_SPEED_100KBPS;
		break;
	}

	return speed;
}


/**************************/
/* I2C00 registger access */
/**************************/
static void i2c00_start(void) 
{
	ROUT(ICCM  , (RIN(ICCM)  | ATM));
	ROUT(ICATMC, (I2C00_SEMI | APC));

	return;
}


static void i2c00_repstart(void) 
{
	ROUT(ICCM  , (RIN(ICCM)  | ATM));
	ROUT(ICATMC, (I2C00_SEMI | APC));

	return;
}


static void i2c00_stop(void) 
{
	ROUT(ICATMC, I2C00_SP);

	return;
}


static void i2c00_soft_reset(void)
{
	ROUT(ICATMC, IRST);

	udelay(100);
	i2c00_init(g_slaveaddr, *(g_rate + g_bus));

	return;
}


static int i2c00_rate_change(unsigned int rate)
{
	if (RIN(ICCS) & BB) {
		return -EBUSY;
		printf("i2c00_rate_change error\n");
	}

	printf("i2c00_rate_change %d %d %d\n",
	       GET_ICCCL(rate),
	       GET_ICCCH(rate),
	       GET_ICDS(rate));

	ROUT(ICCCL , GET_ICCCL(rate));
	ROUT(ICCCH , GET_ICCCH(rate));
	ROUT(ICDS  , GET_ICDS(rate));

	return 0;
}


static void i2c00_init(unsigned int saddr, unsigned int rate)
{
	ROUT(ICCS  , 0x00);
	ROUT(ICINTC, (INTI2CSC | INTATMC | INTI2CC));
	ROUT(ICATMC, APC);
	ROUT(ICAL  , 0x00);
	ROUT(ICCCL , GET_ICCCL(rate));
	ROUT(ICCCH , GET_ICCCH(rate));
	ROUT(ICDS  , GET_ICDS(rate));

	ROUT(ICAR  , saddr << 1);
	ROUT(ICCM  , ((I2C00_CFG_ICDH << 8) | ACK));

	ROUT(ICINT , (I2CSEN | ATMEN | I2CEN));
	ROUT(ICCS  , ES0);

	return;
}

#ifdef CONFIG_XARINA_VSS
static void i2c00_bus_reset()
{
    int i;

    i2c00_soft_reset();

	ROUT(ICBE, TSDAOE | TSCLOE);
	ROUT(ICBE, TSDAOE | TSCLOE | TSDAO | TSCLO);
	udelay(5);
	ROUT(ICBE, TSDAOE | TSCLOE | TSCLO);
	udelay(5);
	ROUT(ICBE, TSDAOE | TSCLOE);
	udelay(5);
	ROUT(ICBE, TSDAOE | TSCLOE | TSDAO);
	udelay(5);

    for (i = 0; i < 64; i++) {
        ROUT(ICBE, TSDAOE | TSCLOE | TSDAO | TSCLO);
        udelay(5);
        ROUT(ICBE, TSDAOE | TSCLOE | TSDAO);
        udelay(5);
    }

	ROUT(ICBE, TSDAOE | TSCLOE);
	udelay(5);
	ROUT(ICBE, TSDAOE | TSCLOE | TSCLO);
	udelay(5);
	ROUT(ICBE, TSDAOE | TSCLOE | TSDAO | TSCLO);
	udelay(5);
	ROUT(ICBE, 0x0);
	udelay(5);

	ROUT(ICATMC, IRST);

	udelay(100);
}
#endif /* CONFIG_XARINA_VSS */

static int interrupt_check(void)
{
	unsigned long fact;
	int ret;

	fact = RIN(ICINT) & (INTI2CS | INTATM | INTI2C);
	if (!fact) {
		/* interrupt nothing */
		return I2C00_E_NO_INT;
	}

	/* interrupt clear */
	ROUT(ICINTC, fact);

	ret = I2C00_E_OK;
	switch(fact) {
	case (INTATM):
		/* Auto transmit fin */
		if (RIN(ICATMS) == 0x05) {
			ret = I2C00_E_NOACK;
		}
		break;
	default:
		/* Stop condition or Other */
		if (RIN(ICATMC) & ERRST) {
			// Error
			ROUT(ICATMC, RIN(ICATMC) & ~(ERRST | AT1 | AT0));
			if ( ((RIN(ICCM) & SCL) != 0) &&
			     ((RIN(ICCS) & BB ) == 0) ) { 
				// Ignore
				ROUT(ICCM, RIN(ICCM) & ~ATM);
				ret = I2C00_E_ERR;
			} else {
				// Freeze
				ret = I2C00_E_FREEZE;
			}
		}
		break;
	}

	return ret;
}


static int wait_ack(void)
{
	unsigned long start, now;
	int ret;

	start = get_timer (0);
	while(1) {
		ret = interrupt_check();
		if (ret != I2C00_E_NO_INT) {
			break;
		}

		now = get_timer(start);
		if ((now / get_tbclk()) >= I2C00_WAIT_ACK_TIMEOUT) {
			ret = I2C00_E_TIMEOUT;
			break;
		}

	}

	return ret;
}


static int i2c00_read(unsigned char saddr, unsigned char *p_buf, int len, int first)
{
	unsigned char slave_addr;	/* slave addr (8bit) */
	int rest = len;			/* rest read len */

	int rdcount = 0;		/* total read data len */
	int trbytes;			/* a read len */
	int idx;			/* buf data start index */
	int ret;			/* wait_ack() return value */
	int i;

	while(rest > 0) {
		/* slave addr set */
		slave_addr = (saddr << 1) | I2C00_RW_READ;
		ROUT(ICRAM, slave_addr);
		idx = 1;

		/* read size compute */
		trbytes = (ICRAMSIZE < idx + rest) ?
			   ICRAMSIZE : idx + rest;

		ROUT(ICTCR, (trbytes - 1));

		/* semi-auto transfer start */
		if (first) {
			if (RIN(ICCS) & BB) {
				return -EBUSY;
			}
			i2c00_start();
		} else {
			i2c00_repstart();
		}

		/* transfer end wait */
		ret = wait_ack();
		if(ret > 0) {
			/* read data get */
			for (i = idx; i < trbytes; i++) {
				*p_buf++ = RIN(ICRAM + i*4);
			}
			/* size compute */
			rest    -= trbytes - idx;
			rdcount += trbytes - idx;
		} else if (ret != I2C00_E_FREEZE) {
			/* stop & end */
			i2c00_stop();
			wait_ack();
			return ret;
		} else {
			i2c00_soft_reset();
			return ret;
		}

		first = I2C00_HOLD;
	}
	
	return rdcount;
}


static int i2c00_write(unsigned char saddr, int subaddr, int alen, unsigned char *p_buf, int len, int first)
{
	unsigned char slave_addr;	/* slave addr (8bit) */
	int rest;			/* len add subaddr size. and copy to rest */

	int wrcount = 0;		/* total write data len */
	int trbytes;			/* a write len */
	int idx;			/* buf data start index */
	int ret;			/* wait_ack() return value */
	int i;
	int subaddr_size = 0;
	
	len += alen;
	rest = len;

	while(rest > 0) {
		/* slave addr set */
		slave_addr = (saddr << 1) & I2C00_RW_WRITE;
		ROUT(ICRAM, slave_addr);
		idx = 1;

		/* read size compute */
		trbytes = (ICRAMSIZE < idx + rest) ?
			   ICRAMSIZE : idx + rest;

		/* subaddress set */
		subaddr_size = 0;
		if (rest == len) {
			if (alen == 2) {
				ROUT(ICRAM + idx*4, (subaddr >> 8) & 0x0F);
				ROUT(ICRAM + idx*8, subaddr & 0xFF);
			} else {
				ROUT(ICRAM + idx*4, subaddr);
			}
			subaddr_size = alen;
		}

		/* write data set */
		for (i = idx+subaddr_size; i < trbytes; i++) {
			ROUT(ICRAM + i*4, *p_buf++);
		}

		ROUT(ICTCR, (trbytes - 1));

		/* semi-auto transfer start */
		if (first) {
			if (RIN(ICCS) & BB) {
				return -EBUSY;
			}
			i2c00_start();
		} else {
			i2c00_repstart();
		}

		/* transfer end wait */
		ret = wait_ack();
		if (ret > 0) {
			/* size compute */
			rest    -= trbytes - idx;
			wrcount += trbytes - idx;
		} else if (ret != I2C00_E_FREEZE) {
			/* stop & end */
			i2c00_stop();
			wait_ack();
			return ret;
		} else {
			i2c00_soft_reset();
			return ret;
		}

		first = I2C00_HOLD;
	}

	return wrcount;
}


/**************************/
/* I2C driver I/F         */
/**************************/
static void err_msg(int err)
{
#if 1
	switch(err) {
	case I2C00_E_TIMEOUT:
		printf("I2C00 ERROR: timeout\n");
		break;
	case I2C00_E_NOACK:
		printf("I2C00 ERROR: noack from slave\n");
		break;
	case I2C00_E_ERR:
		printf("I2C00 ERROR: error detect, but ignore\n");
		break;
	case I2C00_E_FREEZE:
		printf("I2C00 ERROR: freeze.\n");
		break;
	case -EBUSY:
		printf("I2C00 ERROR: bus busy.\n");
		break;
	default:
		break;
	}
#endif
	return;
}


void i2c_init(int speed, int slaveadd)
{
    puts ("i2c_init ");
    int i;
	/* speed value convert */
    for (i = 0; i < I2C00_BUS_MAX; i++) {
        *(g_rate + i) = speed2rate(speed);
    }

	/* slaveaddr stored */
	g_slaveaddr = slaveadd;

#ifdef CONFIG_XARINA_VSS
    i2c00_bus_reset();
#endif /* CONFIG_XARINA_VSS */

	/* initialize */
	i2c00_init(g_slaveaddr, *(g_rate + g_bus));

	return;
}


int i2c_read(uchar chip, uint addr, int alen, uchar *buffer, int len)
{
	int count;

	udelay(10000);

	count = i2c00_write(chip, addr, alen, NULL, 0, I2C00_FIRST);
	if (count < 0) {
		err_msg(count);
		return -1;
	}

	count = i2c00_read(chip, buffer, len, I2C00_HOLD);
	if (count < 0) {
		err_msg(count);
		return -1;
	}

	i2c00_stop();
	wait_ack();

	return 0;
}


int i2c_write(uchar chip, uint addr, int alen, uchar *buffer, int len)
{
	int count;

	count = i2c00_write(chip, addr, alen, buffer, len, I2C00_FIRST);
	if (count < 0) {
		err_msg(count);
		return -1;
	}

	i2c00_stop();
	wait_ack();

	return 0;
}


int i2c_probe(uchar chip)
{
	int tmp;
	int count;

	udelay(10000);

	count = i2c00_write(chip, 0, 1, NULL, 0, I2C00_FIRST);
	if (count < 0) {
		return -1;
	}

	count = i2c00_read(chip, &tmp, 1, I2C00_HOLD);
	if (count < 0) {
		return -1;
	}

	i2c00_stop();
	wait_ack();

	return 0;
}


uchar i2c_reg_read(uchar chip, uchar reg)
{
	uchar buf = 0;

	i2c_read(chip, reg, 1, &buf, 1);

	return buf;
}


void i2c_reg_write(uchar chip, uchar reg, uchar val)
{
	i2c_write(chip, reg, 1, &val, 1);

	return;
}


int i2c_set_bus_num(unsigned int bus)
{
	if (bus >= I2C00_BUS_MAX) {
		return -1;
	}

	g_bus = bus;
	i2c00_init(g_slaveaddr, *(g_rate + g_bus));

	return 0;
}


unsigned int i2c_get_bus_num(void)
{
	return g_bus;
}


int i2c_set_bus_speed(unsigned int speed)
{
	int ret;

	if (!(speed == I2C00_SPEED_100KBPS || speed == I2C00_SPEED_400KBPS)) {
		return -1;
	}

	*(g_rate + g_bus) = speed2rate(speed);
	ret = i2c00_rate_change(*(g_rate + g_bus));

	return ret ? -1 : 0;
}


unsigned int i2c_get_bus_speed(void)
{
	return rate2speed(*(g_rate + g_bus));
}

#endif /* CONFIG_HARD_I2C */

#endif /* CONFIG_I2C00_I2C */
