/*
 * (C) Copyright 2011 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/arch/postedwrite_sync.h>
#include <asm/arch/xarina_dmac.h>

#define DUMMY_DMA_DEF_CONTROL		(0x10000001)	/* prot[0]=ON,size=1 */
#define DUMMY_DMA_DEF_CONF		(0x00000000)	/* none */
#define DUMMY_DMA_LOOP_TIMEOUT		(0x1000)	/* more than 10ms */
#define A11_DMAC_AHB2			(0x03000000)	/* src and dest AHB2 */
#define DMAC_ACTIVE			(0x00020000)
#define NON_CACHE_MEM_ADDR		(0x67F00000)

#if defined(CONFIG_POSTEDWRITE_WORKAROUND)

 __attribute__ ((aligned (32)))
volatile unsigned char dummy_tx_buf[32];
 __attribute__ ((aligned (32)))
volatile unsigned char dummy_rx_buf[32];

void postedwrite_sync(enum postedwrite_ic ic)
{
	unsigned long bus = 0;
	volatile unsigned long dummy;
	int i;
	switch (ic) {
	case POSTEDWRITE_DRAM:
		/* DRAM dummy read */
		dummy = *(unsigned long *)NON_CACHE_MEM_ADDR;
		break;
	case POSTEDWRITE_A11DMAC_AHB2:
		bus = A11_DMAC_AHB2;
		/* fall through */
	case POSTEDWRITE_A11DMAC_AHB1:
		/* A11 DMAC dummy memory to memory transfer */

		DMAC_ROUT(DMAC_SRCADDR_1, (unsigned long)dummy_tx_buf);
		DMAC_ROUT(DMAC_DSTADDR_1, (unsigned long)dummy_rx_buf);
		DMAC_ROUT(DMAC_LLI_1, DMAC_LLI_VAL);
		DMAC_ROUT(DMAC_DEF_LLI_1, 0);
		DMAC_ROUT(DMAC_DEF_CONTROL_1, DUMMY_DMA_DEF_CONTROL | bus);
		DMAC_ROUT(DMAC_DEF_CONFIG_1, DUMMY_DMA_DEF_CONF | DMAC_CONFIG_E);

		for (i=0 ; i<DUMMY_DMA_LOOP_TIMEOUT ; i++) {
			if (!(DMAC_RIN(DMAC_DEF_CONFIG_1) & DMAC_ACTIVE))
				break;
		}
		if (i >= DUMMY_DMA_LOOP_TIMEOUT)
			DMAC_ROUT(DMAC_DEF_CONFIG_1, 0);

		break;
	case POSTEDWRITE_PCIEDMAC:
		/* not use U-Boot */
		break;
	default:
		break;
	}
}

#endif /* defined(CONFIG_POSTEDWRITE_WORKAROUND) */
