/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */


#include <common.h>
#include <mmc.h>
#include <malloc.h>
#ifdef CONFIG_POSTEDWRITE_WORKAROUND
#include <asm/arch/postedwrite_sync.h>
#endif
#include "xarina_sd.h"

/* #define XARINA_SD_DEBUG */

/********************************/
/* DEBUG PRINT                  */
/********************************/
#if defined(XARINA_SD_DEBUG)
#define DEBUG_PRINTK(frmt,args...)  printf("[MMC DEBUG]" frmt ,##args)
#else
#define DEBUG_PRINTK(frmt,args...)
#endif

#define ADMA_DESC_ATTR_TRANS	(0x20)
#define ADMA_DESC_ATTR_VALID	(0x01)
#define ADMA_DESC_ATTR_END	(0x02)
#define ADMA_TRANS_MAX		(0x10000)

#if defined(CONFIG_MMC)

struct s_adma2_desc {
	unsigned short attr : 6;
	unsigned short rsv  : 10;
	unsigned short len;
	unsigned long addr;

};

struct mmc xarina_mmc;

static struct s_adma2_desc *desc;


static void xarina_sd_reset(void)
{
	unsigned long data;

	/* all reset */
	data = SDIF_RIN(SD_SRST_CTL);
	data |= SRST_ALL;
	SDIF_ROUT(SD_SRST_CTL, data);

}

static void xarina_sd_amba_ctrl_init(void)
{
	unsigned long data;

	data = SDIF_RIN(SD_AMBA_CTL_0);
	data &= ~BURST_MASK;
	data |= BURST_INCR16;
	SDIF_ROUT(SD_AMBA_CTL_0, data);

	data = SDIF_RIN(SD_AMBA_CTL_1);
	data &= ~LOCK_TRAN;
	SDIF_ROUT(SD_AMBA_CTL_1, data);

}

static void xarina_sd_int_enable(void)
{
	unsigned long data;

	/* normal intrupt enbale */
	data = SDIF_RIN(SD_INT_STS_EN);
	data |= SD_INT_STS_EN_ALL;
	SDIF_ROUT(SD_INT_STS_EN, data);

	/* error intrupt enbale */
	data = SDIF_RIN(SD_EINT_STS_EN);
	data |= SD_EINT_STS_EN_ALL;
	SDIF_ROUT(SD_EINT_STS_EN, data);

}

static void xarina_sd_clock_delay_init(void)
{
	unsigned long data;

	data = SDIF_RIN(SD_CLK_DLY_CTL);
	data &= ~SD_CLK_DLY_CTL_MASK;
	data |= SD_CLK_DLY_CTL_INIT;
	SDIF_ROUT(SD_CLK_DLY_CTL, data);


}

static void xarina_sd_maxc_init(void)
{
	unsigned long data = SD_MAXC_CTL_DATA; /* force 0 */

	SDIF_ROUT(SD_MAXC_CTL, data);

}

static void xarina_sd_set_clock(struct mmc *mmc, unsigned int clock)
{
	unsigned int set_clock = clock;
	unsigned int sd_clock = mmc->f_max;
	unsigned int div = 0;
	int i;
	unsigned long data;
	unsigned char clock_table[] = {	0x00,
					0x01,
					0x02,
					0x04,
					0x08,
					0x10,
					0x20,
					0x40,
					0x80};

	/* clock stop */
	data = SDIF_RIN(SD_CLK_CTL);
	data &= ~(SDCLK_EN_ENABLE | INT_CLK_EN);
	SDIF_ROUT(SD_CLK_CTL, data);


	if (clock > mmc->f_max) {
		set_clock = mmc->f_max;
	} else if (clock < mmc->f_min) {
		set_clock = mmc->f_min;
	}

	for (i = 0; i < 9; i++) {
		if ((sd_clock >> i) <= set_clock) {
			div = i;
			break;
		}
	}

	/* clock select */
	data = SDIF_RIN(SD_CLK_CTL);
	data &= ~SDCLK_FRQ_MASK;
	data |= (clock_table[div] << 8);
	SDIF_ROUT(SD_CLK_CTL, data);

	/* SD clock enable */
	data = SDIF_RIN(SD_CLK_CTL);
	data |= SDCLK_EN_ENABLE;
	SDIF_ROUT(SD_CLK_CTL, data);

	/* internal clock enable */
	data = SDIF_RIN(SD_CLK_CTL);
	data |= INT_CLK_EN;
	SDIF_ROUT(SD_CLK_CTL, data);

	data = SDIF_RIN(SD_TMO_CTL);
	data &= ~0xf0000;
	data |= 0xe0000;
	SDIF_ROUT(SD_TMO_CTL, data);

	/* wait until clock is stable */
	while (!(SDIF_RIN(SD_CLK_CTL) & INT_CLK_STA));

	DEBUG_PRINTK("clock set = 0x%02x\n", clock_table[i]);


}

static void xarina_sd_set_bus_width(struct mmc *mmc, unsigned int width)
{
	unsigned long data;

	data = SDIF_RIN(SD_HOST_CTL);
	data &= ~DAT_WIDTH_MASK;

	switch (width) {
	case 1:
		data |= DAT_WIDTH_1BIT;
		DEBUG_PRINTK("buswith = 1bit\n");
		break;
	case 4:
		data |= DAT_WIDTH_4BIT;
		DEBUG_PRINTK("buswith = 4bit\n");
		break;
	case 8:
		data |= DAT_WIDTH_8BIT;
		DEBUG_PRINTK("buswith = 8bit\n");
		break;
	default:
		data |= DAT_WIDTH_1BIT;
		DEBUG_PRINTK("buswith = 1bit error\n");
		break;
	}
	SDIF_ROUT(SD_HOST_CTL, data);


}



static void xarina_sd_set_ios(struct mmc *mmc)
{

	unsigned long data;

	/* Set the clock speed */
	xarina_sd_set_clock(mmc, mmc->clock);

	/* Set the bus width */
	xarina_sd_set_bus_width(mmc, mmc->bus_width);

	data = SDIF_RIN(SD_PWR_CTL);
	data |= BUS_VOL_33V | BUS_PWR_ON;
	SDIF_ROUT(SD_PWR_CTL, data);

}


static int xarina_sd_init(struct mmc *mmc)
{
	xarina_sd_reset();
	xarina_sd_amba_ctrl_init();
	xarina_sd_maxc_init();
	xarina_sd_clock_delay_init();
	xarina_sd_int_enable();

	/* wait card state stable */
	while (!(SDIF_RIN(SD_STATE) & SD_STATE_CD_STA));

	if (!(SDIF_RIN(SD_STATE) & SD_STATE_CD_INS)) {
		/* SD card is not inserted */
		printf("SD Card is absent.\n");
		return NO_CARD_ERR;
	}

	DEBUG_PRINTK("SD Card is inserted\n");


	return 0;


}

static void xarina_sd_prepare_data(struct mmc_data *data)
{
	unsigned long tmp;
	unsigned long line_num;
	int i;
	int total = 0;
	
	line_num = data->blocks * data->blocksize / ADMA_TRANS_MAX + 1;

	/* descripter init */
	memset(desc, 0 , sizeof(struct s_adma2_desc) * line_num);

	for (i = 0; i < line_num; i++) {
		desc[i].addr = (unsigned long)data->dest + i * ADMA_TRANS_MAX;
		if (i < (line_num - 1)) {
			desc[i].len = 0;	/* 0 means 65536 byte	*/
			desc[i].attr = ADMA_DESC_ATTR_TRANS | ADMA_DESC_ATTR_VALID;
		} else {
			desc[i].len = data->blocks * data->blocksize - i * ADMA_TRANS_MAX;
			desc[i].attr = ADMA_DESC_ATTR_TRANS | ADMA_DESC_ATTR_VALID | ADMA_DESC_ATTR_END;
		}
		total += ADMA_TRANS_MAX;
	}
	flush_dcache_range((unsigned long)&desc[0], (unsigned long)&desc[line_num]);

#if defined (CONFIG_POSTEDWRITE_WORKAROUND)
	postedwrite_sync(POSTEDWRITE_DRAM);
#endif

	/* address set */
	SDIF_ROUT(SD_ADMS_SYS_ADR, (unsigned long)desc);

	DEBUG_PRINTK("data->dest = 0x%08x\n", data->dest);


	/* select ADMA2 */
	tmp = SDIF_RIN(SD_HOST_CTL);
	tmp &= ~DMA_SEL_MASK;
	tmp |= DMA_SEL_ADMA2;
	SDIF_ROUT(SD_HOST_CTL, tmp);

	/* set block size */
	tmp = SDIF_RIN(SD_BLK_SIZE);
	tmp &= ~BLK_SIZE_MASK;
	tmp |= ((7 << 12) |data->blocksize);
	SDIF_ROUT(SD_BLK_SIZE, tmp);
	DEBUG_PRINTK("block size = %d\n", data->blocksize);

	/* set block count */
	tmp = SDIF_RIN(SD_BLK_CNT);
	tmp &= ~BLK_CNT_MASK;
	tmp |= (data->blocks << 16);
	SDIF_ROUT(SD_BLK_CNT, tmp);
	DEBUG_PRINTK("block count = %d\n", data->blocks);


}


static int xarina_sd_send_cmd(struct mmc *mmc, struct mmc_cmd *cmd, struct mmc_data *data)
{
	unsigned long cmd_flags;
	unsigned int retry = 0x100000;
	unsigned long int_status;
	int i;

	DEBUG_PRINTK("cmd send start\n");
	/* wait until command can be sent */
	while ((SDIF_RIN(SD_STATE) & SD_STATE_CMD_INH) ||
		(SDIF_RIN(SD_STATE) & SD_STATE_DAT_INH));

	/* wait until DAT line is inactive */
	while (SDIF_RIN(SD_STATE) & SD_STATE_USE_DAT_LINE);

	if (data) {
		unsigned long line_num;
		line_num = data->blocks * data->blocksize / ADMA_TRANS_MAX + 1;

		desc = (struct s_adma2_desc *)malloc(sizeof(struct s_adma2_desc) * line_num);

		if (desc == NULL) {
			printf("can not prepare data\n");
			return -1;
		}
		xarina_sd_prepare_data(data);
	}
	/* set argument */
	SDIF_ROUT(SD_ARG, cmd->cmdarg);
	DEBUG_PRINTK("cmd->arg = 0x%08x\n", cmd->cmdarg);

	cmd_flags = SDIF_RIN(SD_CMD); 

	if (data) {
		cmd_flags &= ~SD_TRN_MASK;
		cmd_flags |= DMA_EN;

		if (data->blocks > 1) {
			/* multiple block transfer */
			cmd_flags |= TRN_MOD_MULTI;
			cmd_flags |= A_CMD12_EN;
			DEBUG_PRINTK("multiple block mode\n");
		}
		if (data->flags & MMC_DATA_READ) {
			cmd_flags |= TRN_DIR_READ;
			DEBUG_PRINTK("read mode\n");
		}
		invalidate_dcache_range((unsigned long)data->dest, 
			(unsigned long)data->dest + data->blocks * data->blocksize);

	}

	cmd_flags &= ~SD_CMD_MASK;

	if (!(cmd->resp_type & MMC_RSP_PRESENT)) {
		cmd_flags |= RES_TYPE_NORES;
	} else if (cmd->resp_type & MMC_RSP_136) {
		cmd_flags |= RES_TYPE_136;
	} else if (cmd->resp_type & MMC_RSP_BUSY) {
		cmd_flags |= RES_TYPE_48_BUSY;
	} else {
		cmd_flags |= RES_TYPE_48;
	}

	if (cmd->resp_type & MMC_RSP_CRC) {
		cmd_flags |= CRC_CHK_EN;
	}
	if (cmd->resp_type & MMC_RSP_OPCODE) {
		cmd_flags |= IDX_CHK_EN;
	}
	if (data) {
		cmd_flags |= DAT_PSEL;
	}
	DEBUG_PRINTK("cmd->cmdidx = %d\n", cmd->cmdidx);
	DEBUG_PRINTK("cmd_flags = 0x%08lx\n", cmd_flags);
	DEBUG_PRINTK("0x3800D004 = 0x%08lx\n", *(unsigned long *)0x3800D004);
	DEBUG_PRINTK("0x3800D00C = 0x%08lx\n", cmd->cmdidx << 24 | cmd_flags);

	/* send command */
	SDIF_ROUT(SD_CMD, cmd->cmdidx << 24 | cmd_flags);


	for (i = 0; i < retry; i++) {
		int_status = SDIF_RIN(SD_INT_STS);
		/* Command Complete */
		if (int_status & CMD_COMP) {
			if (!data) {
				/* int clear */
				SDIF_ROUT(SD_INT_STS, CMD_COMP);
			}
			break;
		}
	}

	if (i == retry) {
		printf("%s: waiting for status update\n", __func__);
		DEBUG_PRINTK("INT STATUS: 0x%08lx\n", SDIF_RIN(SD_INT_STS));
		return TIMEOUT;
	}

	if (int_status & CTMO_ERR) {
		/* Timeout Error */
		DEBUG_PRINTK("timeout: %08x cmd %d\n", int_status, cmd->cmdidx);
		return TIMEOUT;
	} else if (int_status & ERR_INT) {
		/* Error Interrupt */
		DEBUG_PRINTK("error: %08x cmd %d\n", int_status, cmd->cmdidx);
		return -1;
	}

	if (cmd->resp_type & MMC_RSP_PRESENT) {
		if (cmd->resp_type & MMC_RSP_136) {
			/* CRC is stripped so we need to do some shifting. */
			unsigned long tmp_res[4];
			for (i = 0; i < 4; i++) {
				tmp_res[i] = SDIF_RIN(SD_RES_0 + i * 4);
				DEBUG_PRINTK("cmd->resp[%d]: %08x\n",
						i, tmp_res[i]);
			}
			cmd->response[0] = (tmp_res[3] << 8) | (tmp_res[2] >> 24);
			cmd->response[1] = (tmp_res[2] << 8) | (tmp_res[1] >> 24);
			cmd->response[2] = (tmp_res[1] << 8) | (tmp_res[0] >> 24);
			cmd->response[3] = (tmp_res[0] << 8);

		} else if (cmd->resp_type & MMC_RSP_BUSY) {
			for (i = 0; i < retry; i++) {
				/* PRNTDATA[23:20] : DAT[3:0] Line Signal */
				if (SDIF_RIN(SD_STATE) & SD_STATE_DAT0)
					break;
			}

			if (i == retry) {
				printf("%s: card is still busy\n", __func__);
				return TIMEOUT;
			}

			cmd->response[0] =  SDIF_RIN(SD_RES_0);
			DEBUG_PRINTK("cmd->resp[0]: %08x\n", cmd->response[0]);
		} else {
			cmd->response[0] =  SDIF_RIN(SD_RES_0);
			DEBUG_PRINTK("cmd->resp[0]: %08x\n", cmd->response[0]);
		}
	}

	if (data) {
		while (1) {
			int_status = SDIF_RIN(SD_INT_STS);

			if (int_status & ERR_INT) {
				/* Error Interrupt */
				SDIF_ROUT(SD_INT_STS, int_status);
				printf("%s: error during transfer: 0x%08lx\n",
						__func__, int_status);
				free(desc);
				return -1;
			} else if (int_status & CARDRMOV) {
				/* SD card remove */
				printf("SD card is removed.\n");
				SDIF_ROUT(SD_INT_STS, int_status);
				free(desc);
				return -1;
			} else if (int_status & DMA_INT) {
				SDIF_ROUT(SD_INT_STS, DMA_INT);
				SDIF_ROUT(SD_SYS_ADR, SDIF_RIN(SD_SYS_ADR));
				DEBUG_PRINTK("DMA end\n");
			} else if (int_status & TRAN_COMP) {
				/* Transfer Complete */
				DEBUG_PRINTK("read is done\n");
				break;
			}

		}
		SDIF_ROUT(SD_INT_STS, int_status);
#if defined(CONFIG_POSTEDWRITE_WORKAROUND)
	postedwrite_sync(POSTEDWRITE_A11DMAC_AHB1);
#endif
		free(desc);
	}

	DEBUG_PRINTK("cmd send end\n");

	return 0;

}

extern __u8* do_fat_read_block;
extern __u8* get_dentfromdir_block;
extern __u8* get_vfatname_block;

int xarina_sd_initialize(bd_t *bis)
{

	unsigned long data;

	memset(&xarina_mmc, 0, sizeof(struct mmc));

	sprintf(xarina_mmc.name, "XARINA SD");
	xarina_mmc.priv = NULL;
	xarina_mmc.set_ios = xarina_sd_set_ios;
	xarina_mmc.send_cmd = xarina_sd_send_cmd;
	xarina_mmc.init = xarina_sd_init;

#if defined(CONFIG_XARINA_EVBRD_PRE)
	xarina_mmc.f_min = 1000000;
	xarina_mmc.f_max = 13500000;
#else
	xarina_mmc.f_min = 200000;
	xarina_mmc.f_max = 40000000;
#endif

	/* base clock and time out clock setting */
	data = SDIF_RIN(SD_CAP_CTL);
	data &= ~(0x003f | 0x3f00);
	data |= (40 << 8 | 40);
	SDIF_ROUT(SD_CAP_CTL, data);


	/* voltage setting */
	xarina_mmc.voltages = 0;
	if (SDIF_RIN(SD_CAP) & SD_CAP_V18SUP) {
		xarina_mmc.voltages |= MMC_VDD_165_195;
	}
	if (SDIF_RIN(SD_CAP) & SD_CAP_V30SUP) {
		xarina_mmc.voltages |= MMC_VDD_30_31;
	}
	if (SDIF_RIN(SD_CAP) & SD_CAP_V33SUP) {
		xarina_mmc.voltages |= MMC_VDD_33_34;	
	}

	/* capabilities setting */
	xarina_mmc.host_caps = MMC_MODE_4BIT;

	if (SDIF_RIN(SD_CAP) & SD_CAP_HSPSUP) {
		xarina_mmc.host_caps |= MMC_MODE_HS;
	}
	if (SDIF_RIN(SD_CAP) & SD_CAP_MMC4_SUP) {
		xarina_mmc.host_caps |= MMC_MODE_8BIT;
	}


	mmc_register(&xarina_mmc);

	/* memory allocation for FAT */
	do_fat_read_block = (__u8*)malloc(65536);
	if (!do_fat_read_block) {
		printf("error: memory allocation\n");
		return -1;
	}
	get_dentfromdir_block = (__u8*)malloc(65536);
	if (!get_dentfromdir_block) {
		printf("error: memory allocation\n");
		return -1;
	}
	get_vfatname_block = (__u8*)malloc(65536);
	if (!get_vfatname_block) {
		printf("error: memory allocation\n");
		return -1;
	}

	return 0;
}




#endif /* CONFIG_MMC */


