/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */


#ifndef __XARINA_SD_H
#define __XARINA_SD_H
#include <asm/io.h>
#define XARNIA_SDIF_BASE	(0x3800D000)

#define SD_SYS_ADR		(0x0000)
#define SD_BLK_SIZE		(0x0004)
#define SD_BLK_CNT		(0x0004)
#define SD_ARG			(0x0008)
#define SD_TRN			(0x000C)
#define SD_CMD			(0x000C)
#define SD_RES_0		(0x0010)
#define SD_RES_1		(0x0014)
#define SD_RES_2		(0x0018)
#define SD_RES_3		(0x001C)
#define SD_DAT_BUF		(0x0020)
#define SD_STATE		(0x0024)
#define SD_HOST_CTL		(0x0028)
#define SD_PWR_CTL		(0x0028)
#define SD_BGAP_CTL		(0x0028)
#define SD_WAKE_CTL		(0x0028)
#define SD_CLK_CTL		(0x002C)
#define SD_TMO_CTL		(0x002C)
#define SD_SRST_CTL		(0x002C)
#define SD_INT_STS		(0x0030)
#define SD_EINT_STS		(0x0030)
#define SD_INT_STS_EN		(0x0034)
#define SD_EINT_STS_EN		(0x0034)
#define SD_INT_SIG_EN		(0x0038)
#define SD_EINT_SIG_EN		(0x0038)
#define SD_A12_ERR_STS		(0x003C)
#define SD_CAP			(0x0040)
#define SD_MAXC			(0x0048)
#define SD_FEVENT_A12		(0x0050)
#define SD_FEVENT_ERR		(0x0050)
#define SD_ADMA_ERR_STS		(0x0054)
#define SD_ADMS_SYS_ADR		(0x0058)
#define SD_CEATA_CTL		(0x0080)
#define SD_VER			(0x00FC)
#define SD_AMBA_CTL_0		(0x0100)
#define SD_AMBA_CTL_1		(0x0104)
#define SD_CAP_CTL		(0x0200)
#define SD_MAXC_CTL		(0x0208)
#define SD_CEATA_CAP_CTL	(0x0210)
#define SD_CLK_DLY_CTL		(0x0214)
#define SD_TST_CTL		(0x0218)


/* SD_BLK_SIZE */
#define BLK_SIZE_MASK		(0x00000FFF)
#define BLK_SIZE_512BYTE	(0x00000200)
#define DMA_BOUND_MASK		(0x00007000)
#define DMA_BOUND_512KBYTE	(0x00007000)

/* SD_BLK_CNT */
#define BLK_CNT_MASK		(0xFFFF0000)

/* SD_TRN */
#define SD_TRN_MASK		(0x0000003F)
#define DMA_EN			(0x00000001)
#define BLK_CNT_EN		(0x00000002)
#define TRN_DIR_READ		(0x00000010)
#define TRN_MOD_MULTI		(0x00000020)
#define A_CMD12_EN		(0x00000004)

/* SD_CMD */
#define SD_CMD_MASK		(0xFFFF0000)
#define RES_TYPE_NORES		(0x00000000)
#define RES_TYPE_136		(0x00010000)
#define RES_TYPE_48		(0x00020000)
#define RES_TYPE_48_BUSY	(0x00030000)
#define CRC_CHK_EN		(0x00080000)
#define IDX_CHK_EN		(0x00100000)
#define DAT_PSEL		(0x00200000)

/* SD_STATE */
#define SD_STATE_CMD_INH	(0x00000001)
#define SD_STATE_DAT_INH	(0x00000002)
#define SD_STATE_USE_DAT_LINE	(0x00000004)
#define SD_STATE_DAT0		(0x00100000)

/* SD_HOST_CTL */
#define DAT_WIDTH_MASK		(0x00000022)
#define DAT_WIDTH_1BIT		(0x00000000)
#define DAT_WIDTH_4BIT		(0x00000002)
#define DAT_WIDTH_8BIT		(0x00000020)
#define DMA_SEL_MASK		(0x00000018)
#define DMA_SEL_SDMA		(0x00000000)
#define DMA_SEL_ADMA1		(0x00000008)
#define DMA_SEL_ADMA2		(0x00000010)

/* SD_PWR_CTL */
#define BUS_VOL_MASK		(0x00000E00)
#define BUS_VOL_33V		(0x00000E00)
#define BUS_PWR_ON		(0x00000100)

/* SD_CLK_CTL */
#define SDCLK_FRQ_MASK		(0x0000FF00)
#define SDCLK_FRQ_DIV_1		(0x00000000)
#define SDCLK_FRQ_DIV_2		(0x00000100)
#define SDCLK_EN_ENABLE		(0x00000004)
#define INT_CLK_STA		(0x00000002)
#define INT_CLK_EN		(0x00000001)

/* SD_SRST_CTL */
#define SRST_ALL		(0x01000000)

/* SD_INT_STS */
#define CMD_COMP		(0x00000001)
#define TRAN_COMP		(0x00000002)
#define DMA_INT			(0x00000008)
#define BUF_WR_RDY		(0x00000010)
#define BUF_RD_RDY		(0x00000020)
#define CARDRMOV		(0x00000080)
#define ERR_INT			(0x00008000)

/* SD_EINT_STS */
#define CTMO_ERR		(0x00010000)

/* SD_INT_STS_EN */
#define SD_INT_STS_EN_ALL	(0x000001FF)

/* SD_EINT_STS_EN */
#define SD_EINT_STS_EN_ALL	(0x3FFF0000)

/* SD_AMBA_CTL_0 */
#define BURST_MASK		(0x00000007)
#define BURST_INCR4		(0x00000000)
#define BURST_INCR8		(0x00000001)
#define BURST_INCR16		(0x00000002)
#define BURST_INCR		(0x00000003)
#define BURST_SINGLE		(0x00000004)

/* SD_AMBA_CTL_1 */
#define LOCK_TRAN		(0x00000001)

/* SD_MAXC_CTL */
#define SD_MAXC_CTL_DATA	(0x00000000)

/* SD_STATE */
#define SD_STATE_CD_INS		(0x00010000)
#define SD_STATE_CD_STA		(0x00020000)

/* SD_CAP */
#define SD_CAP_V18SUP		(0x04000000)
#define SD_CAP_V30SUP		(0x02000000)
#define SD_CAP_V33SUP		(0x01000000)
#define SD_CAP_HSPSUP		(0x00200000)
#define SD_CAP_MMC4_SUP		(0x00040000)

/* SD_CLK_DLY_CTL */
#define SD_CLK_DLY_CTL_MASK	(0x00000007)
#define SD_CLK_DLY_CTL_INIT	(0x00000003) /* recommended for Xarina */

/********************************/
/* I/O                          */
/********************************/
#define SDIF_RIN(n)			readl(XARNIA_SDIF_BASE+(n))
#define SDIF_ROUT(n, v)			writel((v), XARNIA_SDIF_BASE+(n))





#endif






