#ifndef _ASM_ARCH_XARINA_H_
#define _ASM_ARCH_XARINA_H_

#include <config.h>

#if defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL)
#include "xarina_vss.h"
#endif /* defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL) */


/************************************/
/* vector copy address              */
/************************************/
#define XARINA_BOOTRAM_BASE		0x00003F80	/* BootRAM Address */

/************************************/
/* device type                      */
/************************************/
#define XARINA_DEVICE_ES1		(0x00)
#define XARINA_DEVICE_ES2		(0x01)
#define XARINA_DEVICE_ES3		(0x02)
#define XARINA_DEVICE_UNKNOWN		(0xFF)


/************************************/
/* system type                      */
/************************************/
#define XARINA_SYSTEM_PACKAGE		(0xFF)
#define XARINA_SYSTEM_MODULE_H		(0x01)
#define XARINA_SYSTEM_MODULE_S		(0x02)
#define XARINA_SYSTEM_MODULE_H2		(0x03)
#define XARINA_SYSTEM_MODULE_S2		(0x04)

/************************************/
/* board type                       */
/************************************/
#define XARINA_BOARD_EVB_1_0		(0x00)
#define XARINA_BOARD_EVB_1_1		(0x01)
#define XARINA_BOARD_RD_IMX122		(0x02)
#define XARINA_BOARD_RD_ICX692		(0x03)
#define XARINA_BOARD_RD_IMX122AF	(0x04)
#define XARINA_BOARD_RD_IMX139		(0x05)
#define XARINA_BOARD_RD_IMX140		(0x06)
#define XARINA_BOARD_RD_IMX139AF	(0x07)
#define XARINA_BOARD_RD_IMX140AF	(0x08)
#define XARINA_BOARD_RD_IMX140LA	(0x09)
#define XARINA_BOARD_RD_ICX693  	(0x0A)

#define XARINA_BOARD_UNKNOWN		(0xFF)

/************************************/
/* boot type                        */
/************************************/
#define XARINA_BOOT_NONE		(-1)
#define XARINA_BOOT_NAND_BOOT		(0)
#define XARINA_BOOT_UART_BOOT		(1)
#define XARINA_BOOT_SD_BOOT		(2)
#define XARINA_BOOT_TYPE_ADDR		(0x5402A000)

/************************************/
/* DDR init result                  */
/************************************/
#define XARINA_SYSRAM_INIT_RESULT_ADDR	(0x5402A010)
#define XARINA_VRAM_INIT_RESULT_ADDR	(0x5402A020)
#define XARINA_DDRTOP_INIT_RESULT_ADDR	(0x5402A030)
#define XARINA_DDR_INIT_SUCCESS		(0x00000000)
#define XARINA_DDR_INIT_FAILURE		(0x000000FF)

/************************************/
/* bootparam load result            */
/************************************/
#define XARINA_BOOT_PARAM_LOAD_ERROR	(1)
#define XARINA_BOOT_PARAM_LOAD_SUCCESS	(2)

/************************************/
/* GbE Mode	                    */
/************************************/
#define XARINA_GBE_RGMII	(0)
#define XARINA_GBE_MII		(1)

/************************************/
/* NAND Controller type             */
/************************************/
#define XARINA_NAND_CONTROLLER1		(0)
#define XARINA_NAND_CONTROLLER2		(1)


/************************************/
/* DDR init(EEPROM ACCESS via I2C)  */
/************************************/
#define XARINA_E2PROM_MAGIC_BYTES	4
#define XARINA_E2PROM_MAGIC_NUM		0x00a11102
#define XARINA_E2PROM_HEADER_BYTES	8
#define XARINA_E2PROM_DATA_BUF_ADDR	0x54020000
#define XARINA_E2PROM_DATA_BUF_SIZE	0xC0
#define XARINA_E2PROM_I2C_SP_TOP	0x54022000

/************************************/
/* only spl                         */
/************************************/
#ifdef CONFIG_XARINA_SPL
#define XARINA_SPL_WORK_AREA_TOP	CONFIG_SPL_WORK_AREA_TOP	/* spl work area top */
#define XARINA_SPL_VECTOR_TABLE_SIZE	0x40		/* Xarina U-Boot vector table size */
#endif /* CONFIG_XARINA_SPL */

#endif  /* _ASM_ARCH_XARINA_H_ */
