/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef _XARINA_NAND1_H
#define _XARINA_NAND1_H
#include <config.h>


#define XARINA_NAND1_PAGESIZE			(2048)
#define XARINA_NAND1_OOBSIZE		64

#define XARINA_NAND1_ECC_SIZE			(512)
#define XARINA_NAND1_ECC_STEPS		(XARINA_NAND1_PAGESIZE / XARINA_NAND1_ECC_SIZE)
#define XARINA_NAND1_ECC_BYTES		(8) /* data 4 + oob 4 */
#define XARINA_NAND1_ECC_TOTAL		(XARINA_NAND1_ECC_BYTES * XARINA_NAND1_ECC_STEPS)

#if defined(CONFIG_XARINA_FPGA) | defined(CONFIG_XARINA_EVBRD_PRE)
#define XARINA_NAND1_MODE_CLK_RATE			(XARINA_NAND1_MODE_CLK_RATE_2)
#else /* defined(CONFIG_XARINA_FPGA) | defined(CCONFIG_XARINA_EVBRD_PRE) */ 
#define XARINA_NAND1_MODE_CLK_RATE			(XARINA_NAND1_MODE_CLK_RATE_4)
#endif /* defined(CONFIG_XARINA_FPGA) | defined(CONFIG_XARINA_EVBRD_PRE) */

#define XARINA_NAND1_MODE_CLK_RATE_1		(0x1) /* 1/1  ..... - 20MHz */
#define XARINA_NAND1_MODE_CLK_RATE_2		(0x2) /* 1/2  20MHz - 40MHz */
#define XARINA_NAND1_MODE_CLK_RATE_3		(0x3) /* 1/3  40MHz - 60MHz */
#define XARINA_NAND1_MODE_CLK_RATE_4		(0x0) /* 1/4  60MHz - 80MHz */

/* Mode Register */
#define XARINA_NAND1_MODE_WAIT_ACT			(0x0000)
#define XARINA_NAND1_MODE_REGNUM			(0x0010)
#define XARINA_NAND1_MODE_ENDIAN			(0x0000)

#define XARINA_NAND1_BIT_WAIT_ACT			0x0080
#define XARINA_NAND1_BIT_READ_ID			0x0050
#define XARINA_NAND1_BIT_READ_STATUS		0x0010
#define XARINA_NAND1_BIT_ENDIAN			0x0004



#define XARINA_NAND1_WP_ON		1
#define XARINA_NAND1_WP_OFF		0

#define XARINA_NAND1_NUM_OF_CHIP 	(1)
/* chip delay */
#define XARINA_NAND1_CHIP_DELAY		(5) /* dummy parameter */

#define XARINA_NAND1_SHIFT_8			8
#define XARINA_NAND1_SHIFT_16		16
#define XARINA_NAND1_MASK_8BIT		0x000000ff
#define XARINA_NAND1_MASK_16BIT		0x0000ffff
#define XARINA_NAND1_ADDR_INVALID	(-1)
#define XARINA_NAND1_CMD_INVALID		(-1)

#define XARINA_NAND1_DEVICE_READY	1
#define XARINA_NAND1_DEVICE_BUSY		0

#define XARINA_NAND1_STATUS_FLAG		0xff00


/* XNAND 16bit Registers */
/* Devided into 2 parts, MSB8 & LSB8 */
#define XARINA_NAND1_MODE_REG			(0x00<<1)	/* Mode Register (LSB8) */
#define XARINA_NAND1_FLASH_DATA_SIZE_REG		(0x02<<1)	/* Flash Data Size Register */
#define XARINA_NAND1_TASK_REG			(0x04<<1)	/* Task Register */
#define XARINA_NAND1_STATUS_CONTROL_REG		(0x06<<1)	/* Status(MSB8)/Control(LSB8) Register */
#define XARINA_NAND1_STATUS_ID_READ12_REG		(0x08<<1)	/* Status/ID Read Register1(LSB8)/2(MSB8) */
#define XARINA_NAND1_STATUS_ID_READ34_REG		(0x0A<<1)	/* Status/ID Read Register3(LSB8)/4(MSB8) */
#define XARINA_NAND1_STATUS_ID_READ56_REG		(0x0C<<1)	/* Status/ID Read Register5(LSB8)/6(MSB8) */
#define XARINA_NAND1_COMMAND1_REG			(0x10<<1)	/* Command1 Register (LSB8) */
#define XARINA_NAND1_COL_ADDR0_REG			(0x12<<1)	/* Column Address0 Register (LSB8) */
#define XARINA_NAND1_COL_ADDR1_REG			(0x12<<1)	/* Column Address1 Register (MSB8) */
#define XARINA_NAND1_PAGE_ADDR0_REG			(0x14<<1)	/* Page Address0 Register (LSB8) */
#define XARINA_NAND1_PAGE_ADDR1_REG			(0x14<<1)	/* Page Address1 Register (MSB8) */
#define XARINA_NAND1_PAGE_ADDR2_REG			(0x16<<1)	/* Page Address2 Register (LSB8) */
#define XARINA_NAND1_COMMAND2_REG			(0x18<<1)	/* Command2 Register (LSB8) */
#define XARINA_NAND1_COMMAND3_REG			(0x1A<<1)	/* Command3 Register (LSB8) */
#define XARINA_NAND1_DATA_REG			(0x1C<<1)	/* Data0 Register (LSB8)/ Data1 (MSB8) */
#define XARINA_NAND1_MIRROR_DATA_REG		(0x1E<<1)	/* Data2 Register (LSB8)/ Data3 (MSB8) mirror_reg for data12 */

#define XARINA_NAND1_ECC_CLEAR_REG			(0x20<<1)	/* ECC Clear Register (LSB8) */
#define XARINA_NAND1_ECC_CHECK_REG			(0x22<<1)	/* ECC Check Register (LSB8) */
#define XARINA_NAND1_ECC_RES_LOW_MID_REG		(0x24<<1)	/* ECC Result Low(LSB8)/Middle(MSB8) Register */
#define XARINA_NAND1_ECC_RES_HIGH_STATUS_REG	(0x26<<1)	/* ECC Result High(LSB8)/Status(MSB8) Register */
/* STATUS_REG = STATUS(read), Data4(write) */
#define XARINA_NAND1_ECC_ERR_BIT_REG		(0x28<<1)	/* ECC Error bit Register */
#define XARINA_NAND1_ECC_ERR_ADDR_REG		(0x2C<<1)	/* ECC Error Address Register */
#define XARINA_NAND1_NAND_MODE_REG			(0x30<<1)	/* NAND Mode Register (LSB8) */

/* for nand dma if registers */
#define XARINA_NAND1_NF_AUTOCTL			(0x0100)	/* DMA Setting */
#define XARINA_NAND1_NF_STOP				(0x0104)	/* Control DMA Transfer autostop */
#define XARINA_NAND1_NF_RIS				(0x0110)	/* DMA RAW Interrupt Status */
#define XARINA_NAND1_NF_IS				(0x0114)	/* DMA Interrupt Status */
#define XARINA_NAND1_NF_IEN				(0x0118)	/* DMA Interrupt Enable for Sector autotransfer */
#define XARINA_NAND1_NF_ICR				(0x011C)	/* DMA Interrupt Clear */
#define XARINA_NAND1_NF_FIFO				(0x1000)	/* DMA FIFO for Burst Transfer (32bit reg) */

#define XARINA_NAND1_NF_BIT_DM_TYPE		0x0004 /* transfer type m2m/p2m */
#define XARINA_NAND1_NF_BIT_AUTO_EN		0x0002 /* transfer start */
#define XARINA_NAND1_NF_BIT_WXR			0x0001 /* Read/Write 1:write */



/* Mode Register */
#define XARINA_NAND1_BIT_WAIT_ACT			0x0080
#define XARINA_NAND1_BIT_REG_NUM			0x0070
#define XARINA_NAND1_BIT_READ_ID			0x0050
#define XARINA_NAND1_BIT_READ_STATUS		0x0010
#define XARINA_NAND1_BIT_ENDIAN			0x0004

/* Task Register */
#define XARINA_NAND1_BIT_TASK_ALL					0xffff
#define XARINA_NAND1_BIT_TASK_RECV2REG		0x8000
#define XARINA_NAND1_BIT_TASK_WAIT_READY2		0x4000
#define XARINA_NAND1_BIT_TASK_COMMAND3		0x2000
#define XARINA_NAND1_BIT_TASK_SEND_ECC		0x1000
#define XARINA_NAND1_BIT_TASK_SEND_DATA		0x0800
#define XARINA_NAND1_BIT_TASK_CHK_ECC		0x0400
#define XARINA_NAND1_BIT_TASK_RECV_DATA		0x0200
#define XARINA_NAND1_BIT_TASK_WAIT_READY1		0x0100
#define XARINA_NAND1_BIT_TASK_COMMAND2		0x0080
#define XARINA_NAND1_BIT_TASK_PAGE_ADR2		0x0040
#define XARINA_NAND1_BIT_TASK_PAGE_ADR1		0x0020
#define XARINA_NAND1_BIT_TASK_PAGE_ADR0		0x0010
#define XARINA_NAND1_BIT_TASK_COL_ADR1		0x0008
#define XARINA_NAND1_BIT_TASK_COL_ADR0		0x0004
#define XARINA_NAND1_BIT_TASK_COMMAND1		0x0002
#define XARINA_NAND1_BIT_TASK_CLR			0x0001

/* Control/Status Register */
#define XARINA_NAND1_BIT_FIFO_FULL			0x0800
#define XARINA_NAND1_BIT_FIFO_EMPTY			0x0400
#define XARINA_NAND1_BIT_FBSYN			0x0100
#define XARINA_NAND1_BIT_FCE1N			0x0008
#define XARINA_NAND1_BIT_FCE0N			0x0004
#define XARINA_NAND1_BIT_FCDISABLE			0x000C
#define XARINA_NAND1_BIT_FWPN			0x0002
#define XARINA_NAND1_BIT_FOUT			0x0001

/* ECC Check Register */
#define XARINA_NAND1_CERR				(0x0004) /* correctable */
#define XARINA_NAND1_UCERR				(0x0002) /* uncorrectable */
#define XARINA_NAND1_CBERR				(0x0001) /* ecc code error */

#define XARINA_NAND1_BIT_CERR			0x0004 /* correctable */
#define XARINA_NAND1_BIT_UCERR			0x0002 /* uncorrectable */
#define XARINA_NAND1_BIT_CBERR			0x0001 /* ecc code error */


/* NAND Mode Register */
#define XARINA_NAND1_F8X16				(2)
#define XARINA_NAND1_BSIZE				(1)
#define XARINA_NAND1_ACYC				(0)



#endif	/* _XARINA_NAND1_H */

