/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <common.h>
#include <command.h>
#include <asm/arch/xarina.h>
#include "corefw_boot_param.h"
#include "corefw_comm_param.h"
#include "cam_set_param.h"
#include "cam_comm_param.h"
#include "boot_param.h"
#include "xarina_bool.h"
#include "hw_reg_ctrl.h"
#include "pinmux.h"
#include "menu_update.h"

#define PARAM_TYPE_COREFW_COMM		(0)
#define PARAM_TYPE_CAMFW_SET		(1)
#define PARAM_TYPE_CAMFW_DEFECT		(2)
#define PARAM_TYPE_CAMFW_SHADING	(3)

extern char cmd_buff[];
extern char console_buffer[];	/* readline buffer */
extern int data_down_load(unsigned int type, unsigned long addr);

static unsigned long camfw_param_addr = CAMFW_PARAM_DEFAULT_ADDR;
static unsigned long corefw_param_addr = COREFW_PARAM_DEFAULT_ADDR;
static int corefw_param_load_flag = FALSE;

unsigned long boot_param_cam_addr_get(void)
{
	return camfw_param_addr;
}

unsigned long boot_param_core_addr_get(void)
{
	return corefw_param_addr;
}


static int is_param_data_file(struct s_param_data_image_header *p_header, const char *identifier)
{

	size_t n = strlen(identifier);
	if (memcmp((const void *)p_header->identifier, (const void *)identifier, n) != 0) {
		return FALSE;
	}
	return TRUE;
}


int boot_param_data_expansion(struct s_param_data_image_header *p_header,
		const char *identifier)
{
	if (is_param_data_file(p_header, identifier)) {
		if(p_header->addr < CONFIG_XARINA_COREFW_BOOT_INFO_ADDR) {
			return E_CMD_EXPANSION_ADDR_ERROR;
		}
		if(p_header->size > CONFIG_XARINA_PARAM_DATA_MAX_SIZE) {
			return E_CMD_EXPANSION_SIZE_ERROR;
		}
		memcpy((void *)p_header->addr, 
			(const void *)p_header 
				+ sizeof(struct s_param_data_image_header),
			(size_t)p_header->size);
		flush_dcache_range((unsigned long)p_header->addr, 
				(unsigned long)p_header->addr + p_header->size);
	} else {
		return E_CMD_EXPANSION_ID_ERROR;
	}
	
	if (strcmp(identifier, CAM_SET_PARAM_FILE_IDENTIFIER) == 0) {
		camfw_param_addr = p_header->addr;
	} else if (strcmp(identifier, COREFW_PARAM_FILE_IDENTIFIER) == 0) {
		corefw_param_addr = p_header->addr;
		corefw_param_load_flag = TRUE;
	} else {
		/* do nothing */
	}
	
	return E_CMD_EXPANSION_SUCCESS;
}


int boot_param_load_file_env(char *env, u32 addr, u32 size)
{
	int ret;
	char file_name[256];

	ret = getenv_f(env, file_name, 256);

	if (ret == -1) {
		return FALSE;
	}

	sprintf(cmd_buff, "ubifsload %x %s %x", addr, file_name, size);	
	ret = run_command(cmd_buff, 0);
		
	return !ret;
}
int boot_param_load_file_name(char *name, u32 addr, u32 size)
{
	int ret;

	if (name == NULL) {
		return FALSE;
	}

	sprintf(cmd_buff, "ubifsload %x %s %x", addr, name, size);	
	ret = run_command(cmd_buff, 0);
	
	return !ret;
}

int boot_param_check_load_corefw_param(void)
{
	return corefw_param_load_flag;
}

static int load_corefw_param(void)
{
	int ret;
	struct s_param_data_image_header *p_header 
		= (struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;
	
	/* load saved parameter */
	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_COREFW_COMM_PARAM,
				CONFIG_SYS_LOAD_ADDR, 0);
	printf("load CoreFW comm param ... ");
	if (ret != TRUE) {
		printf("not found!\n");
		return FALSE;
	}
			
	if (boot_param_data_expansion(p_header, COREFW_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done! at 0x%08lx size 0x%08lx\n", p_header->addr, p_header->size);
	} else {
		printf("invalid file!\n");
		return FALSE;
	}
	

	return TRUE;

}
static int load_corefw_param_default(void)
{
	int ret;
	struct s_param_data_image_header *p_header 
		= (struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;
	
	/* load saved parameter */
	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_COREFW_COMM_PARAM_D,
				CONFIG_SYS_LOAD_ADDR, 0);
	printf("load default CoreFW comm param ... ");
	if (ret != TRUE) {
		printf("not found!\n");
		return FALSE;
	}
			
	if (boot_param_data_expansion(p_header, COREFW_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done! at 0x%08lx size 0x%08lx\n", p_header->addr, p_header->size);
	} else {
		printf("invalid file!\n");
		return FALSE;
	}
	

	return TRUE;

}

static int load_camfw_param_default(void)
{
	int ret;
	char file_name[256];
	struct s_param_data_image_header *p_header = 
		(struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;

	/* load saved parameter */
	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM_D,
				 CONFIG_SYS_LOAD_ADDR, 0);

	printf("load Cam FW set param ... ");

	if (ret != TRUE) {
		printf("not found!\n");
		return FALSE;
	}
			
	if (boot_param_data_expansion(p_header, CAM_SET_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done!\n");
	} else {
		printf("invalid file!\n");
		return FALSE;
	}

	ret = getenv_f(CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM_D, file_name, 256);

	if (ret != -1) {
		setenv(CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM, file_name);
	}
	

	return TRUE;

}


static int load_cam_set_param(void)
{
	int ret;

	struct s_param_data_image_header *p_header 
		= (struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;

	/* load saved parameter */
	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM,
				CONFIG_SYS_LOAD_ADDR, 0);

	printf("load CAM FW setting param ... ");
	if (ret != TRUE) {
		printf("not found!\n");
		return FALSE;
	}
			
	if (boot_param_data_expansion(p_header, CAM_SET_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done! at 0x%08lx size 0x%08lx\n", p_header->addr, p_header->size);
		cam_param_evr_type_ddr_set(xarina_hw_evr_type_get());
	} else {
		printf("invalid file!\n");
		return FALSE;
	}
	
	return TRUE;

}
static int load_cam_defect_data(void)
{
	int ret;
	struct s_param_data_image_header *p_header 
		= (struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;

	/* load saved parameter */
	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_CAMFW_DEFECT_DATA,
				CONFIG_SYS_LOAD_ADDR,
				0);

	printf("load cam defect compensation data ... ");

	if (ret != TRUE) {
		printf("not found!\n");
		return FALSE;
	}
	if (boot_param_data_expansion(p_header, CAMFW_DEFECT_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done! at 0x%08lx size 0x%08lx\n", p_header->addr, p_header->size);
	} else {
		printf("invalid file!\n");
		return FALSE;
	}


	return TRUE;

}

static int load_cam_shading_data(void)
{
	int ret;
	struct s_param_data_image_header *p_header 
		= (struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;

	/* load saved parameter */
	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_CAMFW_SHADING_DATA,
				CONFIG_SYS_LOAD_ADDR,
				0);

	printf("load cam shading correct data ... ");

	if (ret != TRUE) {
		printf("not found!\n");
		return FALSE;
	}
	if (boot_param_data_expansion(p_header, CAMFW_SHADING_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done! at 0x%08lx size 0x%08lx\n", p_header->addr, p_header->size);
	} else {
		printf("invalid file!\n");
		return FALSE;
	}


	return TRUE;

}

int boot_param_load(void)
{
	int ret_corefw;
	int ret_camfw;
	int ret_defect;
	int ret_shading;
	int ret;

	/* Load coreFW comm param */
	ret_corefw = load_corefw_param();
	/* Load cam set_param */
	ret_camfw = load_cam_set_param();
	/* Load cam defect data */
	ret_defect = load_cam_defect_data();
	/* Load cam shading data */
	ret_shading = load_cam_shading_data();

	ret = (ret_corefw && ret_camfw && ret_defect && ret_shading) ? 
		XARINA_BOOT_PARAM_LOAD_SUCCESS : 
		XARINA_BOOT_PARAM_LOAD_ERROR;

	return ret;
}

int boot_param_corefw_param_restore(void)
{
	return load_corefw_param_default();
}

int boot_param_camfw_param_restore(void)
{
	return load_camfw_param_default();
}


int do_load_param(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int param_type = 0;
	int load_type = 0;
	int ret;
	struct s_param_data_image_header *p_header 
		= (struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;
	
	if (argc != 3) {
		cmd_usage(cmdtp);
		return 1;
	}

	if (strcmp(argv[1], "copa") == 0) {
		param_type = PARAM_TYPE_COREFW_COMM;
	} else if (strcmp(argv[1], "capa") == 0) {
		param_type = PARAM_TYPE_CAMFW_SET;	
	} else if (strcmp(argv[1], "deco") == 0) {
		param_type = PARAM_TYPE_CAMFW_DEFECT;	
	} else {
		printf("arg[1] must be copa | capa | deco\n");
		return 1;  
	}
	
	if (strcmp(argv[2], "t") == 0) {
		load_type = DOWNLOAD_TYPE_TFTP;
	} else if (strcmp(argv[2], "x") == 0) {
		load_type = DOWNLOAD_TYPE_LOADX;	
	} else if (strcmp(argv[2], "u") == 0) {
		load_type = DOWNLOAD_TYPE_UBIFS;
	} else {
		printf("arg[2] must be t | x | u\n");
		return 1;  
	}

	if ((load_type == DOWNLOAD_TYPE_LOADX) |
	    (load_type == DOWNLOAD_TYPE_TFTP)) {	
		/* not create filesize variable when tftp download size 0 file */
		run_command("setenv filesize", 0);	/* delete filesize */
	
		ret = data_down_load(load_type, CONFIG_SYS_LOAD_ADDR);
		printf("load file ... ");
	
		if (ret <= 0 || getenv("filesize") == NULL) {
			printf("error\n");
			return 1;
		}
	} else {
		ret = readline("FILENAME: ");
		if (ret <= 0) {
			return 1;
		}
		ret = boot_param_load_file_name(console_buffer, CONFIG_SYS_LOAD_ADDR, 0);
		if (!ret) {
			printf("error\n");
			return 1;
		}
	}
	
	switch (param_type) {
	case PARAM_TYPE_COREFW_COMM:
	if (boot_param_data_expansion(p_header, COREFW_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
			printf("done!\n");
		} else {
			printf("invalid file!\n");
			return 1;
		}	
		break;
	case PARAM_TYPE_CAMFW_SET:
	if (boot_param_data_expansion(p_header, CAM_SET_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
			printf("done!\n");
		} else {
			printf("invalid file!\n");
			return 1;
		}	
		break;
	case PARAM_TYPE_CAMFW_DEFECT:
	if (boot_param_data_expansion(p_header, CAMFW_DEFECT_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
			printf("done!\n");
		} else {
			printf("invalid file!\n");
			return 1;
		}	
		break;
	default:
		/* do nothing */
		break;
	}


	return 0;
}


U_BOOT_CMD(
	loadprm, 3, 0,	do_load_param,
	"load paramter",
	"<parameter (copa | capa | deco)> <download type (t | x | u)>\n"
	"    - load 'parameter' by 'download type'\n"
	"    - 'download type' t is TFTP\n"
	"    - 'download type' x is XMODEM\n"
	"    - 'download type' u is UBIFS\n"
);


