/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <common.h>
#include <flash.h>
#include "cam_comm_param.h"
#include "cam_set_param.h"
#include "boot_param.h"
#include "command_exec.h"
#include "xarina_bool.h"


/************************************/
/* parameter member operation       */
/************************************/
/* CAM_MODE */
static unsigned char cam_param_cam_mode_get(unsigned long base)
{
	return ((struct s_cam_comm_param *)base)->cam_mode;
}

static void cam_param_cam_mode_set(unsigned long base, unsigned char data)
{
	((struct s_cam_comm_param *)base)->cam_mode = data;
	return;
}
/* SENSSYSTEM */
static unsigned char cam_param_senssystem_get(unsigned long base)
{
	return ((struct s_cam_comm_param *)base)->senssystem;
}

static void cam_param_senssystem_set(unsigned long base, unsigned char data)
{
	((struct s_cam_comm_param *)base)->senssystem = data;
	return;
}
/* VIDEOSIZE */
static unsigned char cam_param_videosize_get(unsigned long base)
{
	return ((struct s_cam_comm_param *)base)->videosize;
}

static void cam_param_videosize_set(unsigned long base, unsigned char data)
{
	((struct s_cam_comm_param *)base)->videosize = data;
	return;
}

/* SENSSMODE */
static unsigned char cam_param_sensmode_get(unsigned long base)
{
	
	return ((struct s_cam_comm_param *)base)->sensmode;
}

static void cam_param_sensmode_set(unsigned long base, unsigned char data)
{
	((struct s_cam_comm_param *)base)->sensmode = data;
	return;
}

/* WDMODE */
static unsigned char cam_param_wdmode_get(unsigned long base)
{
	return ((struct s_cam_comm_param *)base)->wdmode;
}

static void cam_param_wdmode_set(unsigned long base, unsigned char data)
{
	((struct s_cam_comm_param *)base)->wdmode = data;
	return;
}
/* SENSBIT */
static unsigned char cam_param_sensbit_get(unsigned long base)
{
	return ((struct s_cam_comm_param *)base)->sensbit;
}

static void cam_param_sensbit_set(unsigned long base, unsigned char data)
{
	((struct s_cam_comm_param *)base)->sensbit = data;
	return;
}
/* VIDEOFRAME */
static unsigned char cam_param_videoframe_get(unsigned long base)
{
	return ((struct s_cam_comm_param *)base)->videoframe;
}

static void cam_param_videoframe_set(unsigned long base, unsigned char data)
{
	((struct s_cam_comm_param *)base)->videoframe = data;
	return;
}
/* VIDEOIF */
static unsigned char cam_param_videoif_get(unsigned long base)
{
	return ((struct s_cam_comm_param *)base)->videoif;
}

static void cam_param_videoif_set(unsigned long base, unsigned char data)
{
	((struct s_cam_comm_param *)base)->videoif = data;
	return;
}
/* VIDEOSYNC */
static unsigned char cam_param_videosync_get(unsigned long base)
{
	return ((struct s_cam_comm_param *)base)->videosync;
}

static void cam_param_videosync_set(unsigned long base, unsigned char data)
{
	((struct s_cam_comm_param *)base)->videosync = data;
	return;
}

/* VIDEORANGE */
static unsigned char cam_param_videorange_get(unsigned long base)
{
	return ((struct s_cam_comm_param *)base)->videorange;
}

static void cam_param_videorange_set(unsigned long base, unsigned char data)
{
	((struct s_cam_comm_param *)base)->videorange = data;
	return;
}
/* EVR TYPE*/
static unsigned char cam_param_evr_type_get(unsigned long base)
{
	return ((struct s_cam_comm_param *)base)->evr_type;
}

static void cam_param_evr_type_set(unsigned long base, unsigned char data)
{
	((struct s_cam_comm_param *)base)->evr_type = data;
	return;
}

/* MIRIS */
static unsigned char cam_param_sysmiris_get(unsigned long base)
{
	unsigned char miris;
	unsigned char ret;

	miris = ((struct s_cam_comm_param *)base)->sysmiris;
	if ((miris & CAM_PARAM_MIRIS_SYSMIRIS) != 0) {
		ret = CAM_PARAM_MIRIS_SYSMIRIS_YES;
	} else {
		ret = CAM_PARAM_MIRIS_SYSMIRIS_NONE;
	}

	return ret;
}


static void cam_param_sysmiris_set(unsigned long base, unsigned char data)
{
	unsigned char miris;

	miris = ((struct s_cam_comm_param *)base)->sysmiris;
	if (data != 0) {
		miris |= CAM_PARAM_MIRIS_SYSMIRIS;
	} else {
		miris &= ~CAM_PARAM_MIRIS_SYSMIRIS;
	}
	((struct s_cam_comm_param *)base)->sysmiris = miris;

	return;
}

/* VRAMBANDWIDTH */
static unsigned long cam_param_vramband_get(unsigned long base)
{
	return ((struct s_cam_comm_param *)base)->vramband;
}

static void cam_param_vramband_set(unsigned long base, unsigned long data)
{
	((struct s_cam_comm_param *)base)->vramband = data;
	return;
}
/* PLL2_CLK */
static unsigned long cam_param_pll2_clk_get(unsigned long base)
{
	return ((struct s_cam_comm_param *)base)->pll2_clk;
}

static void cam_param_pll2_clk_set(unsigned long base, unsigned char data)
{
	((struct s_cam_comm_param *)base)->vramband = data;
	return;
}

/* UART BAUDRATE */
static unsigned char cam_param_uart_baudrate_get(unsigned long base)
{
	return ((struct s_cam_comm_param *)base)->uart_baudrate;
}

static void cam_param_uart_baudrate_set(unsigned long base, unsigned char data)
{
	((struct s_cam_comm_param *)base)->uart_baudrate = data;
	return;
}
/* SSPTIMER */
static unsigned short cam_param_ssptimer_get(unsigned long base)
{
	return ((struct s_cam_comm_param *)base)->ssptimer;
}

static void cam_param_ssptimer_set(unsigned long base, unsigned short data)
{
	((struct s_cam_comm_param *)base)->ssptimer = data;
	return;
}
/* ADDRESS TILING SEL */
static unsigned char cam_param_tiling_sel_get(unsigned long base)
{
	return ((struct s_cam_comm_param *)base)->address_tiling_sel;
}

static void cam_param_tiling_sel_set(unsigned long base, unsigned char data)
{
	((struct s_cam_comm_param *)base)->address_tiling_sel = data;
	return;
}
/************************************/
/* parameter member operation       */
/*      for only comm memory on ddr */
/************************************/
/* CAM MODE */
unsigned char cam_param_cam_mode_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_cam_mode_get(addr);
}
void cam_param_cam_mode_ddr_set(unsigned char data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_cam_mode_set(addr, data);
}
/* SENSSYSTEM */
unsigned char cam_param_senssystem_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_senssystem_get(addr);
}

void cam_param_senssystem_ddr_set(unsigned char data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_senssystem_set(addr, data);
	return;
}

/* VIDEOSIZE */
unsigned char cam_param_videosize_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_videosize_get(addr);
}

void cam_param_videosize_ddr_set(unsigned char data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_videosize_set(addr, data);
	return;
}
/* SENSMODE */
unsigned char cam_param_sensmode_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_sensmode_get(addr);
}

void cam_param_sensmode_ddr_set(unsigned char data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_sensmode_set(addr, data);
	return;
}
/* WDMODE */
unsigned char cam_param_wdmode_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_wdmode_get(addr);
}

void cam_param_wdmode_ddr_set(unsigned char data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_wdmode_set(addr, data);
	return;
}
/* SENSBIT */
unsigned char cam_param_sensbit_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_sensbit_get(addr);
}

void cam_param_sensbit_ddr_set(unsigned char data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_sensbit_set(addr, data);
	return;
}
/* VIDEOFRAME */
unsigned char cam_param_videoframe_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_videoframe_get(addr);
}

void cam_param_videoframe_ddr_set(unsigned char data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_videoframe_set(addr, data);
	return;
}
/* VIDEOIF */
unsigned char cam_param_videoif_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_videoif_get(addr);
}

void cam_param_videoif_ddr_set(unsigned char data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_videoif_set(addr, data);
	return;
}
/* VIDEOSYNC */
unsigned char cam_param_videosync_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_videosync_get(addr);
}

void cam_param_videosync_ddr_set(unsigned char data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_videosync_set(addr, data);
	return;
}
/* VIDEORANGE */
unsigned char cam_param_videorange_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_videorange_get(addr);
}

void cam_param_videorange_ddr_set(unsigned char data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_videorange_set(addr, data);
	return;
}
/* EVR TYPE */
unsigned char cam_param_evr_type_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_evr_type_get(addr);
}

void cam_param_evr_type_ddr_set(unsigned char data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_evr_type_set(addr, data);
	return;
}
/* MIRIS */
unsigned char cam_param_sysmiris_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_sysmiris_get(addr);
}

void cam_param_sysmiris_ddr_set(unsigned char data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_sysmiris_set(addr, data);
	return;
}

/* VRAMBANDWIDTH */
unsigned long cam_param_vramband_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_vramband_get(addr);
}

void cam_param_vramband_ddr_set(unsigned long data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_vramband_set(addr, data);
	return;
}
/* PLL2 CLK */
unsigned char cam_param_pll2_clk_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_pll2_clk_get(addr);
}

void cam_param_uart_pll2_clk_ddr_set(unsigned char data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_pll2_clk_set(addr, data);
	return;
}
/* UART BAUDRATE */
unsigned char cam_param_uart_baudrate_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_uart_baudrate_get(addr);
}

void cam_param_uart_baudrate_ddr_set(unsigned char data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_uart_baudrate_set(addr, data);
	return;
}
/* SSPTIMER */
unsigned short cam_param_ssptimer_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_ssptimer_get(addr);
}

void cam_param_ssptimer_ddr_set(unsigned short data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_ssptimer_set(addr, data);
	return;
}
/* ADDRESS TILING SEL */
unsigned char cam_param_tiling_sel_ddr_get(void)
{
	unsigned long addr = boot_param_cam_addr_get();
	return cam_param_tiling_sel_get(addr);
}

void cam_param_tiling_sel_ddr_set(unsigned char data)
{
	unsigned long addr = boot_param_cam_addr_get();
	cam_param_tiling_sel_set(addr, data);
	return;
}



