/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <exports.h>
#include "gpio_vss.h"

static unsigned long get_ch(const char* arg)
{
    const char c = arg[0];

    if ( (c == 'A') || c == 'a' ) {
        return GPIO_A_BASE;
    }
    if ( (c == 'B') || c == 'b' ) {
        return GPIO_B_BASE;
    }
    if ( (c == 'C') || c == 'c' ) {
        return GPIO_C_BASE;
    }
    if ( (c == 'E') || c == 'e' ) {
        return GPIO_E_BASE;
    }
    if ( (c == 'F') || c == 'f' ) {
        return GPIO_F_BASE;
    }
    if ( (c == 'G') || c == 'g' ) {
        return GPIO_G_BASE;
    }
    if ( (c == 'H') || c == 'h' ) {
        return GPIO_H_BASE;
    }
    if ( (c == 'I') || c == 'i' ) {
        return GPIO_I_BASE;
    }

    return 0;
}

static int do_gpio(cmd_tbl_t * cmdtp, int flag, int argc, char * const argv[])
{
    unsigned long ch  = 0;
    unsigned long bit = 0;
    unsigned long val = 0;
    int ret = 0;

    if (argc < 4) {
        return cmd_usage(cmdtp);
    }

    ch  = get_ch(argv[2]);
    bit = simple_strtoul(argv[3], NULL, 10);

    if (bit > 7) {
        printf("invalid bit: %d\n", bit);
        return cmd_usage(cmdtp);
    }

    if (strncmp(argv[1], "read", 4) == 0) {
        ret = gpio_read(ch, bit);
        #if 0
        printf("gpio read 0x%x bit%d %d\n", ch, bit, ret);
        #else
        printf("%d\n", ret);
        #endif
        return 0;
    }

    if (strncmp(argv[1], "write", 5) == 0) {
        if (argc < 5) {
            return cmd_usage(cmdtp);
        }
        val = simple_strtoul(argv[4], NULL, 10);
        if (val > 1) {
            return cmd_usage(cmdtp);
        }

        ret = gpio_write(ch, bit, val);
        #if 0
        printf("gpio write 0x%x bit%d %d\n", ch, bit, val);
        #endif
        return ret;
    }

    return cmd_usage(cmdtp);;
}

U_BOOT_CMD(
    gpio, 5, 0, do_gpio,
    "read/write gpio",
    "read|write ch(A-C,E-I) bit(0-7) val(0-1, write only)\n"
);
