/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <command.h>
#include <common.h>
#include <linux/ctype.h>
#include "cmd_menu.h"
#include "menu_top.h"
#include "xarina_bool.h"


static int menu_no_str2val(int c)
{
	if (c == MENU_SELECT_CHAR_RET_MENU) {
		return MENU_SELECT_VAL_RET_MENU;
	}
	if (c == MENU_SELECT_CHAR_EXIT_MENU) {
		return MENU_SELECT_VAL_EXIT_MENU;
	}

	if (c >= '1' && c <= '9') {
		return c - '0';
	}

	if (c >= 'a' && c <= 'z') {
		return c - 'a' + 10;
	}

	return -1;
}


static int menu_no_val2str(int val)
{
	if (val == MENU_SELECT_VAL_RET_MENU) {
		return MENU_SELECT_CHAR_RET_MENU;
	}
	if (val == MENU_SELECT_VAL_EXIT_MENU) {
		return MENU_SELECT_CHAR_EXIT_MENU;
	}

	if (val >= 1 && val <= 9) {
		return '0' + val;
	}

	if (val >= 10 && val <= 35) {
		return 'a' + (val - 10);
	}

	return '\0';
}

/*
static int menu_option(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	if (p_item->option >= MENU_OPTION_NONE && p_item->option < MENU_OPTION_MAX) {
		switch(p_item->option) {
		default:
			break;
		}
	}

	return 0;
}
*/

static struct menu_item *menu_item_get(struct menu_info *p_info, int select_num)
{
	if (select_num == MENU_SELECT_VAL_RET_MENU) {
		return p_info->p_ret_item;
	}
	if (select_num == MENU_SELECT_VAL_EXIT_MENU) {
		return p_info->p_exit_item;
	}

	return p_info->p_item[select_num - 1];
}


static int menu_select_num_check(struct menu_info *p_info, struct menu_proc *p_proc)
{
	/* return select */
	if (p_proc->select_num == MENU_SELECT_VAL_RET_MENU) {
		if (p_info->p_ret_item == NULL) {
			return FALSE;
		} else {
			return TRUE;
		}
	}

	/* exit select */
	if (p_proc->select_num == MENU_SELECT_VAL_EXIT_MENU) {
		if (p_info->p_exit_item == NULL) {
			return FALSE;
		} else {
			return TRUE;
		}
	}

	/* normal select range check */
	if (!(p_proc->select_num > 0 && p_proc->select_num <= p_info->item_num)) {
		return FALSE;
	}

	/* disable check */
	if (p_proc->b_disable[p_proc->select_num - 1]) {
		return FALSE;
	}

	return TRUE;
}


static void menu_str_out(struct menu_info *p_info, struct menu_proc *p_proc)
{
	int i;

	/* menu name output */
	printf("<<%s>>\n", p_info->p_str);

	/* string output */
	if (p_info->p_print_func != NULL) {
		p_info->p_print_func(p_info, p_proc);
	}

	/* menu item output */
	for(i = 0; i < p_info->item_num; i++) {
		if (p_proc->b_disable[i]) {
			printf(" [-] .......... --------\n");
		} else {
			printf(" [%c] .......... %s", menu_no_val2str(i+1), p_info->p_item[i]->p_str);

			if (p_proc->b_choice[i]) {
				printf("(*)");
			}

			printf("\n");
		}
	}

	/* return item output */
	if (p_info->p_ret_item != NULL) {
		printf(" [%c] .......... %s\n",	menu_no_val2str(MENU_SELECT_VAL_RET_MENU),
						p_info->p_ret_item->p_str);
	}

	/* exit item output */
	if (p_info->p_exit_item != NULL) {
		printf(" [%c] .......... %s\n",	menu_no_val2str(MENU_SELECT_VAL_EXIT_MENU),
						p_info->p_exit_item->p_str);
	}

	/* additional strings output */
	if (p_info->p_add_str != NULL) {
		printf("%s\n", p_info->p_add_str);
	}

	return;
}


static void menu_item_all_enable(int *p_disable)
{
	int i;

	for(i = 0; i < MENU_ITEM_NUM_MAX; i++) {
		*(p_disable + i) = FALSE;
	}

	return;
}

static void menu_item_all_unselect(int *p_choice)
{
	int i;

	for(i = 0; i < MENU_ITEM_NUM_MAX; i++) {
		*(p_choice + i) = FALSE;
	}

	return;
}

int do_menu (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	struct menu_info *p_info;	/* current menu info ptr */
	struct menu_item *p_item;	/* user select menu item */
	struct menu_proc proc;		/* do_menu process variable */
	int ret;			/* return value */
	int c;				/* intput char */
	int b_exit = FALSE;

	/***********************************/
	/* init                            */
	/***********************************/
	p_info = &info_top;
	while(!b_exit) {
		putc('\n');
		
		if (p_info == NULL) {
			printf("system error.\n");
			b_exit = TRUE;
			continue;
		}

		/***********************************/
		/* preprocess                      */
		/***********************************/
		menu_item_all_enable(proc.b_disable);
		menu_item_all_unselect(proc.b_choice);

		if (p_info->p_pre_func != NULL) {
			ret = p_info->p_pre_func(p_info, &proc);
		}

		/***********************************/
		/* menu strings output             */
		/***********************************/
		menu_str_out(p_info, &proc);

		/***********************************/
		/* input number                    */
		/***********************************/
		printf("Enter menu item No.: ");
		c = getc();
		printf("%c\n\n", c);

		proc.select_num = menu_no_str2val(c);


		/***********************************/
		/* input number check              */
		/***********************************/
		if (!menu_select_num_check(p_info, &proc)) {
			printf("input error.\n\n");
			continue;
		}

		/***********************************/
		/* menu item get                   */
		/***********************************/
		p_item = menu_item_get(p_info, proc.select_num);
		if (p_item == NULL) {
			printf("system error.\n");
			b_exit = TRUE;
			continue;
		}

		printf("%s\n", p_item->p_str);

		/***********************************/
		/* menu item exec                  */
		/***********************************/
		if (p_item->p_func != NULL) {
			ret = p_item->p_func(p_info, p_item, &proc);
			if (ret < 0) {
				/* Do Nothing */
			}
		}

		/***********************************/
		/* menu option process             */
		/***********************************/
/*		menu_option(p_info, p_item, &proc); */

		/***********************************/
		/* menu transition                 */
		/***********************************/
		if (p_item->next != NULL) {
			p_info = p_item->next;
		}

		/***********************************/
		/* exit check                      */
		/***********************************/
		if (proc.select_num == MENU_SELECT_VAL_EXIT_MENU) {
			b_exit = TRUE;
		}
	}

	printf("exit menu command.\n\n");
	return 0;
}


U_BOOT_CMD(
	menu, 1, 0,	do_menu,
	"menu    - go menu mode",
	"\n    - go menu mode\n "
);

