/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef COREFW_BOOT_PARAM_H
#define COREFW_BOOT_PARAM_H
/*-----------------------------------------------------------------------------
 * Macro definition
 *---------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * Shared Memory Address Size
 *---------------------------------------------------------------------------*/
#define SC_SYS_SHM_TOP_ADR                      (0x60000000)    /* Shared Memory Area top address                            */
#define SC_SYS_SHM_SIZE                         (0x00008000)    /* Shared Memory Size (32KB)                                 */
#define SC_SYS_SHM_CAM_FW_PARAM_ADR             (0x60001000)    /* CamFW Setting Parameter Area top address                  */
#define SC_SYS_SHM_CAM_FW_PARAM_SIZE            (0x00010000)    /* CamFW Setting Parameter Area size (64KB)                  */
#define SC_SYS_SHM_CAM_PREDEF_ADR               (0x60011000)    /* CamFW Defect Correcting Parameter Area top address        */
#define SC_SYS_SHM_CAM_PREDEF_SIZE              (0x00001400)    /* CamFW Defect Correcting Parameter Area size (5KB)         */
#define SC_SYS_SHM_CAM_SHD_DATA_ADR             (0x60012400)    /* CamFW Shading Informations Area top address               */
#define SC_SYS_SHM_CAM_SHD_DATA_SIZE            (0x00001000)    /* CamFW Shading Informations Area size (4KB)                */
#define SC_SYS_SHM_CAM_INFO_NOTIFY_ADR          (0x60013400)    /* CamFW Information Notification Parameter Area top address */
#define SC_SYS_SHM_CAM_INFO_NOTIFY_SIZE         (0x00000400)    /* CamFW Information Notification Parameter Area size (1KB)  */

/*-----------------------------------------------------------------------------
 * Outside Module Program Address
 *---------------------------------------------------------------------------*/
#define SC_SYS_OM_CAM_SENSER_ADR                (0x60218000)    /* CamFW Senser Program Area top address        */
#define SC_SYS_OM_AUDIO_CPU2_VEC_ADR            (0x60198000)    /* CPU2(Audio) forwarding address(for SRC side) */

/*-----------------------------------------------------------------------------
 * CPU1 Boot Parameter
 *---------------------------------------------------------------------------*/
/* Boot Response Code */
#define SC_SYS_BOOT_RES_NO_INIT                 (0x00000000)    /* no response            */
#define SC_SYS_BOOT_RES_PF_INIT_DONE            (0x00000001)    /* SPF/OS init complite   */
#define SC_SYS_BOOT_RES_RM_INIT_DONE            (0x00000002)    /* RM init complite       */
#define SC_SYS_BOOT_RES_VOUT_BOOT_DONE          (0x00000004)    /* VideoOut Boot complite */
#define SC_SYS_BOOT_RES_CAM_INIT_DONE           (0x00000008)    /* CamFW init complite    */

#define SC_SYS_BOOT_RES_MODE_NORMAL_INIT_DONE   (0x00000010)    /* init complite(Normal Mode)  */
#define SC_SYS_BOOT_RES_MODE_DIRECT_INIT_DONE   (0x00000020)    /* init complite(Direct Mode)  */
#define SC_SYS_BOOT_RES_MODE_TRIGGER_INIT_DONE  (0x00000040)    /* init complite(Trigger Mode) */

#define SC_SYS_BOOT_RES_W_MASK                  (0x00020000)    /* error of Startup Mask Config(Warning)    */
#define SC_SYS_BOOT_RES_W_OSD                   (0x00040000)    /* error of Startup OSD Config(Warning)     */
#define SC_SYS_BOOT_RES_W_VIDEO_OTHER           (0x00080000)    /* error of Video[except Mask/OSD](Warning) */

#define SC_SYS_BOOT_RES_ERR                     (0xFFF00000)    /* error response */
#define SC_SYS_BOOT_RES_E_PARAM                 (SC_SYS_BOOT_RES_ERR | 0x00000100)    /* error of Boot Parameter     */
#define SC_SYS_BOOT_RES_E_SPF_INIT              (SC_SYS_BOOT_RES_ERR | 0x00000200)    /* error of SPF init           */
#define SC_SYS_BOOT_RES_E_DRV_INIT              (SC_SYS_BOOT_RES_ERR | 0x00000400)    /* error of Normal Driver init */
#define SC_SYS_BOOT_RES_E_RM_INIT               (SC_SYS_BOOT_RES_ERR | 0x00000800)    /* error of RM init            */
#define SC_SYS_BOOT_RES_E_VIDEO_INIT            (SC_SYS_BOOT_RES_ERR | 0x00001000)    /* error of VideoFW init       */
#define SC_SYS_BOOT_RES_E_AUDIO_INIT            (SC_SYS_BOOT_RES_ERR | 0x00002000)    /* error of AudioFW init       */
#define SC_SYS_BOOT_RES_E_CAM_INIT              (SC_SYS_BOOT_RES_ERR | 0x00004000)    /* error of CamFW init         */
#define SC_SYS_BOOT_RES_E_VIDEO_OPEN            (SC_SYS_BOOT_RES_ERR | 0x00008000)    /* error of VideoFW open       */
#define SC_SYS_BOOT_RES_E_AUDIO_OPEN            (SC_SYS_BOOT_RES_ERR | 0x00010000)    /* error of AudioFW open       */

/* Notify Type of Boot Response(BootMode, use Mailbox or not) */
#define SC_SYS_BOOT_MODE_NORMAL_NOTIFY_ON       (0x00000001)    /* Normal Mode & use Mailbox      */
#define SC_SYS_BOOT_MODE_NORMAL_NOTIFY_OFF      (0x00000002)    /* Normal Mode & not use Mailbox  */
#define SC_SYS_BOOT_MODE_DIRECT_NOTIFY_ON       (0x00000003)    /* Direct Mode & use Mailbox      */
#define SC_SYS_BOOT_MODE_DIRECT_NOTIFY_OFF      (0x00000004)    /* Direct Mode & not use Mailbox  */
#define SC_SYS_BOOT_MODE_TRIGGER_NOTIFY_ON      (0x00000005)    /* Trigger Mode & use Mailbox     */
#define SC_SYS_BOOT_MODE_TRIGGER_NOTIFY_OFF     (0x00000006)    /* Trigger Mode & not use Mailbox */

/* Rotation */
#define SC_SYS_ROTATION_DISABLE                 (0x00000000)    /* rotation disable */
#define SC_SYS_ROTATION_ENABLE                  (0x00000001)    /* rotation enable  */

/* CPU1 Error Type(for Debug) */
#define SC_SYS_CPU1_UDF_ERR                     (0x80000001)
#define SC_SYS_CPU1_SWI_ERR                     (0x80000002)
#define SC_SYS_CPU1_PABT_ERR                    (0x80000003)
#define SC_SYS_CPU1_DABT_ERR                    (0x80000004)
#define SC_SYS_CPU1_FNC_ERR                     (0x80000010)

/*-----------------------------------------------------------------------------
 * CoreFW Common Parameter(Memory Info)
 *---------------------------------------------------------------------------*/
/* Memory Size */
#define SC_SYS_MEM_SIZE_0B                      (0)             /* memory size = 0B     */
#define SC_SYS_MEM_SIZE_4KB                     (1)             /* memory size = 4KB    */
#define SC_SYS_MEM_SIZE_8KB                     (2)             /* memory size = 8KB    */
#define SC_SYS_MEM_SIZE_16KB                    (3)             /* memory size = 16KB   */
#define SC_SYS_MEM_SIZE_32KB                    (4)             /* memory size = 32KB   */
#define SC_SYS_MEM_SIZE_64KB                    (5)             /* memory size = 64KB   */
#define SC_SYS_MEM_SIZE_128KB                   (6)             /* memory size = 128KB  */
#define SC_SYS_MEM_SIZE_256KB                   (7)             /* memory size = 256KB  */
#define SC_SYS_MEM_SIZE_512KB                   (8)             /* memory size = 512KB  */
#define SC_SYS_MEM_SIZE_1MB                     (9)             /* memory size = 1MB    */
#define SC_SYS_MEM_SIZE_2MB                     (10)            /* memory size = 2MB    */
#define SC_SYS_MEM_SIZE_4MB                     (11)            /* memory size = 4MB    */
#define SC_SYS_MEM_SIZE_8MB                     (12)            /* memory size = 8MB    */
#define SC_SYS_MEM_SIZE_16MB                    (13)            /* memory size = 16MB   */
#define SC_SYS_MEM_SIZE_32MB                    (14)            /* memory size = 32MB   */
#define SC_SYS_MEM_SIZE_64MB                    (15)            /* memory size = 64MB   */
#define SC_SYS_MEM_SIZE_128MB                   (16)            /* memory size = 128MB  */
#define SC_SYS_MEM_SIZE_256MB                   (17)            /* memory size = 256MB  */
#define SC_SYS_MEM_SIZE_512MB                   (18)            /* memory size = 512MB  */
#define SC_SYS_MEM_SIZE_1GB                     (19)            /* memory size = 1GB    */
#define SC_SYS_MEM_SIZE_2GB                     (20)            /* memory size = 2GB    */
#define SC_SYS_MEM_SIZE_4GB                     (21)            /* memory size = 4GB    */

/* Protection of Linux Managed Area */
#define SC_SYS_MEM_PROTECT_ON                   (0x00000001)    /* protection on  */
#define SC_SYS_MEM_PROTECT_OFF                  (0x00000002)    /* protection off */

/*-----------------------------------------------------------------------------
 * CoreFW Common Parameter(Log Info)
 *---------------------------------------------------------------------------*/
#define SC_SYS_LOG_OFF                          (0x00000000)    /* Log off          */
#define SC_SYS_LOG_NORMAL_ON                    (0x00000001)    /* Normal Log on    */
#define SC_SYS_LOG_TRACE_ON                     (0x00000002)    /* Trace Log on     */
#define SC_SYS_LOG_BUF_MSK                      (0x0000001F)    /* Mask of Log size */

/*-----------------------------------------------------------------------------
 * CoreFW Common Parameter(Peripheral IO Info)
 *---------------------------------------------------------------------------*/
#define SC_SYS_USE_UART0                        (unsigned char)(0x01)      /* use UART0 */
#define SC_SYS_USE_SSP1                         (unsigned char)(0x10)      /* use SSP1  */
#define SC_SYS_NOUSE                            (unsigned char)(0xFF)      /* not use   */

#define SC_SYS_CAM_GPIO_NUM                     (2)                        /* GPIO port num for CamFW   */
#define SC_SYS_VIDEO_GPIO_NUM                   (1)                        /* GPIO port num for VideoFW */
#define SC_SYS_DBG_GPIO_NUM                     (6)                        /* GPIO port num for Debug   */
#define SC_SYS_DBG_GPIO_OFF                     (unsigned char)(0x00)      /* Debug GPIO off            */
#define SC_SYS_DBG_GPIO_ON                      (unsigned char)(0x01)      /* Debug GPIO on             */

#define SC_SYS_GPIO_INVALID                     (unsigned char)(0x00)      /* not use      */
#define SC_SYS_GPIO_PORTA                       (unsigned char)(0x10)      /* GPIO PORTA   */
#define SC_SYS_GPIO_PORTB                       (unsigned char)(0x20)      /* GPIO PORTB   */
#define SC_SYS_GPIO_PORTC                       (unsigned char)(0x30)      /* GPIO PORTC   */
                                                                           /* GPIO PORTD is not available */
#define SC_SYS_GPIO_PORTE                       (unsigned char)(0x40)      /* GPIO PORTE   */
#define SC_SYS_GPIO_BIT0                        (unsigned char)(0x00)      /* GPIO BIT0    */
#define SC_SYS_GPIO_BIT1                        (unsigned char)(0x01)      /* GPIO BIT1    */
#define SC_SYS_GPIO_BIT2                        (unsigned char)(0x02)      /* GPIO BIT2    */
#define SC_SYS_GPIO_BIT3                        (unsigned char)(0x03)      /* GPIO BIT3    */
#define SC_SYS_GPIO_BIT4                        (unsigned char)(0x04)      /* GPIO BIT4    */
#define SC_SYS_GPIO_BIT5                        (unsigned char)(0x05)      /* GPIO BIT5    */
#define SC_SYS_GPIO_BIT6                        (unsigned char)(0x06)      /* GPIO BIT6    */
#define SC_SYS_GPIO_BIT7                        (unsigned char)(0x07)      /* GPIO BIT7    */

#define SC_SYS_SSPCR0_SPH                       (0x00000080)    /* [7] SSPCLKOUT phase                */
#define SC_SYS_SSPCR0_SPO                       (0x00000040)    /* [6] SSPCLKOUT polarity             */
#define SC_SYS_SSPCR0_FRF_MASK                  (0x00000030)    /* [5-4] frame format Mask            */
#define SC_SYS_SSPCR0_FRF_MSPIF                 (0x00000000)    /* Motorola SPI frame format          */
#define SC_SYS_SSPCR0_FRF_TISF                  (0x00000010)    /* TI synchronous serial frame format */
#define SC_SYS_SSPCR0_FRF_NMF                   (0x00000020)    /* National Microwire frame format    */

/*-----------------------------------------------------------------------------
 * CoreFW Common Parameter(Memory Detail Info)
 *---------------------------------------------------------------------------*/
#define SC_SYS_MEM_DETAIL_THREAD_PICT_NUM       (4)             /* EZoom/Mask/OSD Thread Num */

/*-----------------------------------------------------------------------------
 * CoreFW Common Parameter(Video Info)
 *---------------------------------------------------------------------------*/
/* startup VideoOut Format */
#define SC_SYS_VIDEO_VOUT_FMT_OFF                                 (0x00)    /* VideoOut off */
#define SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5                 (0x01)    /* 720x486@59.94i(ITU-R BT.656-5)    */
#define SC_SYS_VIDEO_VOUT_FMT_720_486_59I_CEA861E                 (0x02)    /* 720x486@59.94i(CEA-861-E-like)    */
#define SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5                 (0x03)    /* 720x576@50i(ITU-R BT.656-5)       */
#define SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E                 (0x04)    /* 720x576@50i(CEA-861-E)            */
#define SC_SYS_VIDEO_VOUT_FMT_RESERVED_00	                  (0x05)
#define SC_SYS_VIDEO_VOUT_FMT_RESERVED_01		          (0x06)
#define SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5                 (0x07)    /* 720x480@59.94i(ITU-R BT.656-5)    */
#define SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E                 (0x08)    /* 720x480@59.94i(CEA-861-E)         */
#define SC_SYS_VIDEO_VOUT_FMT_RESERVED_02			  (0x09)
#define SC_SYS_VIDEO_VOUT_FMT_RESERVED_03		          (0x0a)
#define SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1                (0x0b)    /* 720x480@59.94p(ITU-R BT.1358-1)   */
#define SC_SYS_VIDEO_VOUT_FMT_720_480_59P_CEA861E                 (0x0c)    /* 720x480@59.94p(CEA-861-E)         */
#define SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1                (0x0d)    /* 720x576@50p(ITU-R BT.1358-1)      */
#define SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E                 (0x0e)    /* 720x576@50p(CEA-861-E)            */
#define SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_PAR_10_11       (0x0f)    /* 720x486@59.94i(ITU-R BT.656-5 PAR10:11)      */
#define SC_SYS_VIDEO_VOUT_FMT_720_486_59I_CEA861E_PAR_10_11       (0x10)    /* 720x486@59.94i(CEA-861-E-like PAR10:11)      */
#define SC_SYS_VIDEO_VOUT_FMT_RESERVED_04			  (0x11)
#define SC_SYS_VIDEO_VOUT_FMT_RESERVED_05			  (0x12)
#define SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_PAR_10_11       (0x13)    /* 720x480@59.94i(ITU-R BT.656-5-like PAR10:11) */
#define SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E_PAR_10_11       (0x14)    /* 720x480@59.94i(CEA-861-E PAR10:11)           */
#define SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_59_54       (0x15)    /* 720x576@50io(ITU-R BT.656-5 PAR59:54)     */
#define SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_PAR_59_54       (0x16)    /* 720x576@50io(CEA-861-E PAR59:54)          */
#define SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_12_11       (0x17)    /* 720x576@50io(ITU-R BT.656-5 PAR12:11)     */
#define SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_PAR_12_11       (0x18)    /* 720x576@50io(CEA-861-E PAR12:11)          */

#define SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M              (0x21)    /* 1280x720@60p(SMPTE 296M)          */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E                (0x22)    /* 1280x720@60p(CEA-861-E)           */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M              (0x23)    /* 1280x720@59.94p(SMPTE 296M)       */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E                (0x24)    /* 1280x720@59.94p(CEA-861-E)        */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_SMPTE296M              (0x25)    /* 1280x720@30p(SMPTE 296M)          */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_CEA861E                (0x26)    /* 1280x720@30p(CEA-861-E)           */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_SMPTE296M              (0x27)    /* 1280x720@29.97p(SMPTE 296M)       */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_CEA861E                (0x28)    /* 1280x720@29.97p(CEA-861-E)        */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M              (0x29)    /* 1280x720@50p(SMPTE 296M)          */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E                (0x2a)    /* 1280x720@50p(CEA-861-E)           */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_SMPTE296M              (0x2b)    /* 1280x720@25p(SMPTE 296M)          */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_CEA861E                (0x2c)    /* 1280x720@25p(CEA-861-E)           */

#define SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M_CROP_FULL    (0x31)  /* 1280x720@60p(SMPTE 296M Crop+Full)       */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E_CROP_FULL      (0x32)  /* 1280x720@60p(CEA-861-E Crop+Full)        */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M_CROP_FULL    (0x33)  /* 1280x720@59.94p(SMPTE 296M Crop+Full)    */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E_CROP_FULL      (0x34)  /* 1280x720@59.94p(CEA-861-E Crop+Full)     */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_SMPTE296M_CROP_FULL    (0x35)  /* 1280x720@30p(SMPTE 296M Crop+Full)       */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_CEA861E_CROP_FULL      (0x36)  /* 1280x720@30p(CEA-861-E)                  */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_SMPTE296M_CROP_FULL    (0x37)  /* 1280x720@29.97p(SMPTE 296M Crop+Full)    */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_CEA861E_CROP_FULL      (0x38)  /* 1280x720@29.97p(CEA-861-E Crop+Full)     */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M_CROP_FULL    (0x39)  /* 1280x720@50p(SMPTE 296M Crop+Full)       */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E_CROP_FULL      (0x3a)  /* 1280x720@50p(CEA-861-E Crop+Full)        */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_SMPTE296M_CROP_FULL    (0x3b)  /* 1280x720@25p(SMPTE 296M Crop+Full)       */
#define SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_CEA861E_CROP_FULL      (0x3c)  /* 1280x720@25p(CEA-861-E Crop+Full)        */

#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_BT1120_7              (0x41)    /* 1920x1080@60p(ITU-R BT.1120-7)    */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_CEA861E               (0x42)    /* 1920x1080@60p(CEA-861-E)          */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_59P_BT1120_7              (0x43)    /* 1920x1080@59.94p(ITU-R BT.1120-7) */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_59P_CEA861E               (0x44)    /* 1920x1080@59.94p(CEA-861-E)       */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7              (0x45)    /* 1920x1080@30p(ITU-R BT.1120-7)    */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E               (0x46)    /* 1920x1080@30p(CEA-861-E)          */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7              (0x47)    /* 1920x1080@29.97p(ITU-R BT.1120-7) */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E               (0x48)    /* 1920x1080@29.97p(CEA-861-E)       */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_BT1120_7              (0x49)    /* 1920x1080@50p(ITU-R BT.1120-7)    */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_CEA861E               (0x4a)    /* 1920x1080@50p(CEA-861-E)          */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7              (0x4b)    /* 1920x1080@25p(ITU-R BT.1120-7)    */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E               (0x4c)    /* 1920x1080@25p(CEA-861-E)          */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7              (0x4d)    /* 1920x1080@60i(ITU-R BT.1120-7)    */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_CEA861E               (0x4e)    /* 1920x1080@60i(CEA-861-E)          */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7              (0x4f)    /* 1920x1080@59.94i(ITU-R BT.1120-7) */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_CEA861E               (0x50)    /* 1920x1080@59.94i(CEA-861-E)       */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7              (0x51)    /* 1920x1080@50i(ITU-R BT.1120-7)    */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_CEA861E               (0x52)    /* 1920x1080@50i(CEA-861-E)          */

#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_BT1120_7_CROP_FULL    (0x61)  /* 1920x1080@60p(ITU-R BT.1120-7 Crop+Full)    */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_CEA861E_CROP_FULL     (0x62)  /* 1920x1080@60p(CEA-861-E Crop+Full)          */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_59P_BT1120_7_CROP_FULL    (0x63)  /* 1920x1080@59.94p(ITU-R BT.1120-7 Crop+Full) */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_59P_CEA861E_CROP_FULL     (0x64)  /* 1920x1080@59.94p(CEA-861-E Crop+Full)       */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7_CROP_FULL    (0x65)  /* 1920x1080@30p(ITU-R BT.1120-7 Crop+Full)    */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E_CROP_FULL     (0x66)  /* 1920x1080@30p(CEA-861-E)                    */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7_CROP_FULL    (0x67)  /* 1920x1080@29.97p(ITU-R BT.1120-7 Crop+Full) */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E_CROP_FULL     (0x68)  /* 1920x1080@29.97p(CEA-861-E Crop+Full)       */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_BT1120_7_CROP_FULL    (0x69)  /* 1920x1080@50p(ITU-R BT.1120-7 Crop+Full)    */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_CEA861E_CROP_FULL     (0x6a)  /* 1920x1080@50p(CEA-861-E Crop+Full)          */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7_CROP_FULL    (0x6b)  /* 1920x1080@25p(ITU-R BT.1120-7 Crop+Full)    */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E_CROP_FULL     (0x6c)  /* 1920x1080@25p(CEA-861-E Crop+Full)          */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7_CROP_FULL    (0x6d)  /* 1920x1080@60i(ITU-R BT.1120-7 Crop+Full)    */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_CEA861E_CROP_FULL     (0x6e)  /* 1920x1080@60i(CEA-861-E Crop+Full)          */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7_CROP_FULL    (0x6f)  /* 1920x1080@59.94i(ITU-R BT.1120-7 Crop+Full) */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_CEA861E_CROP_FULL     (0x70)  /* 1920x1080@59.94i(CEA-861-E Crop+Full)       */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7_CROP_FULL    (0x71)  /* 1920x1080@50i(ITU-R BT.1120-7 Crop+Full)    */
#define SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_CEA861E_CROP_FULL     (0x72)  /* 1920x1080@50i(CEA-861-E Crop+Full)          */

#define SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_FULL            (0x81)    /* 720x486@59.94i(ITU-R BT.656-5 Full)    */
#define SC_SYS_VIDEO_VOUT_FMT_720_486_59I_CEA861E_FULL            (0x82)    /* 720x486@59.94i(CEA-861-E-like Full)    */
#define SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_FULL            (0x83)    /* 720x576@50i(ITU-R BT.656-5 Full)       */
#define SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_FULL            (0x84)    /* 720x576@50i(CEA-861-E Full)            */
#define SC_SYS_VIDEO_VOUT_FMT_RESERVED_06			  (0x85)
#define SC_SYS_VIDEO_VOUT_FMT_RESERVED_07			  (0x86)
#define SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_FULL            (0x87)    /* 720x480@59.94i(ITU-R BT.656-5 Full)    */
#define SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E_FULL            (0x88)    /* 720x480@59.94i(CEA-861-E Full)         */
#define SC_SYS_VIDEO_VOUT_FMT_RESERVED_08			  (0x89)
#define SC_SYS_VIDEO_VOUT_FMT_RESERVED_09			  (0x8a)
#define SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1_FULL           (0x8b)    /* 720x480@59.94p(ITU-R BT.1358-1 Full)   */
#define SC_SYS_VIDEO_VOUT_FMT_720_480_59P_CEA861E_FULL            (0x8c)    /* 720x480@59.94p(CEA-861-E Full)         */
#define SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1_FULL           (0x8d)    /* 720x576@50p(ITU-R BT.1358-1 Full)      */
#define SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E_FULL            (0x8e)    /* 720x576@50p(CEA-861-E Full)            */

#define SC_SYS_VIDEO_VOUT_FMT_UNUSED                              (0xff)    /* not set */

/* mute status of startup VideoOut */
#define SC_SYS_VIDEO_VOUT_UNMUTE                (0x00)          /* mute off */
#define SC_SYS_VIDEO_VOUT_MUTE                  (0x01)          /* mute on  */

/* aspect of startup VideoOut */
#define SC_SYS_VIDEO_VOUT_ASPECT_1_1            (0x00)          /* 1:1 */
#define SC_SYS_VIDEO_VOUT_ASPECT_FULL           (0x01)          /* full */
#define SC_SYS_VIDEO_VOUT_ASPECT_10_11          (0x02)          /* 10:11(only on 720x486@59.94iA720x480@59.94iA720x480@60i) */
#define SC_SYS_VIDEO_VOUT_ASPECT_59_54          (0x03)          /* 59:54(only on 720x576@50i) */
#define SC_SYS_VIDEO_VOUT_ASPECT_12_11          (0x04)          /* 12:11(only on 720x576@50i) */

/* colorimetry of startup VideoOut */
#define SC_SYS_VIDEO_VOUT_COLOR_THRU            (0x00)          /* Colorimetry:Through                         */
#define SC_SYS_VIDEO_VOUT_COLOR_AUTO_1          (0x01)          /* Colorimetry:Auto Y:16-235, C:16-240(Normal) */
#define SC_SYS_VIDEO_VOUT_COLOR_AUTO_2          (0x02)          /* Colorimetry:Auto Y:16-235, C:16-240(Clip)   */
#define SC_SYS_VIDEO_VOUT_COLOR_AUTO_3          (0x03)          /* Colorimetry:Auto Y:16-254, C:1-254(Clip)    */

/* thread number of startup VideoOut */
#define SC_SYS_VIDEO_THREAD_0                   (0x00)          /* EZoom/Mask/OSD Thread 0 */
#define SC_SYS_VIDEO_THREAD_1                   (0x01)          /* EZoom/Mask/OSD Thread 1 */
#define SC_SYS_VIDEO_THREAD_2                   (0x02)          /* EZoom/Mask/OSD Thread 2 */
#define SC_SYS_VIDEO_THREAD_3                   (0x03)          /* EZoom/Mask/OSD Thread 3 */
#define SC_SYS_VIDEO_THREAD_RAW                 (0x04)          /* Raw Thread              */
#define SC_SYS_VIDEO_THREAD_SUB_DS              (0x05)          /* SubDS Thread            */

/* stabilize */
#define SC_SYS_VIDEO_STABILIZE_OFF              (0x00)          /* stabilize off */
#define SC_SYS_VIDEO_STABILIZE_ON               (0x01)          /* stabilize on  */

/* reversing */
#define SC_SYS_VIDEO_INPUT_WRRVS_OFF            (0x00)          /* reversing off                                                                 */
#define SC_SYS_VIDEO_INPUT_WRRVS_Y              (0x01)          /* vertical(Y) axis is mainly reversed                                           */
#define SC_SYS_VIDEO_INPUT_WRRVS_X              (0x02)          /* horizontal(X) axis is mainly reversed                                         */
#define SC_SYS_VIDEO_INPUT_WRRVS_XY             (0x03)          /* 1.horizontal(X) axis is mainly reversed 2.vertical(Y) axis is mainly reversed */

/* startup MASK/OSD */
#define SC_SYS_VIDEO_OUTPUT_OSD                 (0x00000001)    /* startup OSD on  */
#define SC_SYS_VIDEO_OUTPUT_MASK                (0x00000002)    /* startup MASK on */
#define SC_SYS_VIDEO_OUTPUT_VOUT_PROG           (0x00000004)    /* startup VOUT ProgresiveMode on */

/* colorimetry of CodecOut */
#define SC_SYS_VIDEO_COLOR_THRU                         (0)     /* Through                                      */
#define SC_SYS_VIDEO_COLOR_BT601_FULL                   (1)     /* BT.601, Y: 0-255, C: 0-255                   */
#define SC_SYS_VIDEO_COLOR_BT601_LIMITED_SCL            (2)     /* BT.601, Y:16-235, C:16-240(Range Scaling)    */
#define SC_SYS_VIDEO_COLOR_BT601_LIMITED_CLIP1          (3)     /* BT.601, Y:16-235, C:16-240(Range Clipping)   */
#define SC_SYS_VIDEO_COLOR_BT601_LIMITED_CLIP2          (4)     /* BT.601, Y:16-254, C: 1-254(Range Clipping)   */
#define SC_SYS_VIDEO_COLOR_BT709_FULL                   (5)     /* BT.709, Y: 0-255, C: 0-255                   */
#define SC_SYS_VIDEO_COLOR_BT709_LIMITED_SCL            (6)     /* BT.709, Y:16-235, C:16-240(Range Scaling)    */
#define SC_SYS_VIDEO_COLOR_BT709_LIMITED_CLIP1          (7)     /* BT.709, Y:16-235, C:16-240(Range Clipping)   */
#define SC_SYS_VIDEO_COLOR_BT709_LIMITED_CLIP2          (8)     /* BT.709, Y:16-254, C: 1-254(Range Clipping)   */

/*-----------------------------------------------------------------------------
 * CoreFW Common Parameter(Audio Info)
 *---------------------------------------------------------------------------*/
#define SC_SYS_AUDIO_DISABLE                    (0x00)          /* disable Audio Function */
#define SC_SYS_AUDIO_ENABLE                     (0x01)          /* enable Audio Function  */

#define SC_SYS_AUDIO_I2S_MODE_MASTER            (0x01)          /* I2S mode = Master */
#define SC_SYS_AUDIO_I2S_MODE_SLAVE             (0x02)          /* I2S mode = Slave  */

#define SC_SYS_AUDIO_I2S_FMT_LEFT               (0x01)          /* I2S serial format = left justified  */
#define SC_SYS_AUDIO_I2S_FMT_RIGHT              (0x02)          /* I2S serial format = right justified */
#define SC_SYS_AUDIO_I2S_FMT_I2S                (0x03)          /* I2S serial format = I2S justified   */

#define SC_SYS_AUDIO_I2S_BCK_32FS               (0x01)          /* I2S BCK = 32fs */
#define SC_SYS_AUDIO_I2S_BCK_64FS               (0x02)          /* I2S BCK = 64fs */

#ifdef __cplusplus
extern "C" {
#endif  /* __cplusplus */


/*-----------------------------------------------------------------------------
 * Structure
 *---------------------------------------------------------------------------*/
/* Boot Information */
typedef struct tagSSc_Sys_BootInfo {
    unsigned long  bootType;                           /* boot type(BootMode, use Mailbox or not) */
    unsigned long  responseCode;                       /* response code                           */
    unsigned long  enableRotation;                     /* rotation                                */
    unsigned char  reserve[20];                        /* reserved area                           */
} SSc_Sys_BootInfo;

/* Boot Parameter Version Information */
typedef struct tagSSc_Sys_VerInfo {
    unsigned char  version[32];                        /* boot parameter version(ASCII) */
} SSc_Sys_VerInfo;

/* Memory Information */
typedef struct tagSSc_Sys_MemInfo {
    unsigned char  ddr1MemSize;                        /* DDR1 size                      */
    unsigned char  reserve1[3];                        /* reserved area                  */
    unsigned long  ddr0MemSize;                        /* DDR0 size                      */
    unsigned long  ddr0MemProtect;                     /* DDR0 protection flag           */
    unsigned long  linuxMemAdr;                        /* Linux managed area top address */
    unsigned long  linuxMemSize;                       /* Linux managed area size        */
    unsigned char  reserved2[12];                      /* reserved area                  */
} SSc_Sys_MemInfo;

/* Log Information */
typedef struct tagSSc_Sys_LogInfo {
    unsigned long  outputType;                         /* output type            */
    unsigned long  logBufAdr;                          /* Normal Log top address */
    unsigned long  logBufSize;                         /* Normal Log size        */
    unsigned long  traceBufAdr;                        /* Trace Log top address  */
    unsigned long  traceBufSize;                       /* Trace Log size         */
    unsigned char  reserve[12];                        /* reserved area          */
} SSc_Sys_LogInfo;

/* Peripheral IO Information */
typedef struct tagSSc_Sys_PeriIoInfo {
    unsigned char  uartSspSel;                         /* use UART/SSP or not   */
    unsigned char  dbgGpioFlag;                        /* use Debug GPIO or not */
    unsigned char  camGpio[SC_SYS_CAM_GPIO_NUM];       /* GPIO port for CamFW   */
    unsigned long  sspInfo;                            /* SSP information       */
    unsigned char  videoGpio[SC_SYS_VIDEO_GPIO_NUM];   /* GPIO port for VideoFw */
    unsigned char  reserve[23];                        /* reserved area         */
} SSc_Sys_PeriIoInfo;

/* Video Information */
typedef struct tagSSc_Sys_VideoInfo {
    unsigned char  vOutFormat;                         /* VideoOut format                                            */
    unsigned char  mute;                               /* mute status of startup VideoOut                            */
    unsigned char  vOutThreadNum;                      /* thread number of startup VideoOut                          */
    unsigned char  vOutStabi;                          /* stabilize of startup VideoOut                              */
    unsigned long  muteColor;                          /* mute color(YUV) of startup VideoOut                        */
    unsigned char  vOutClock;                          /* clock status of startup VideoOut                           */
    unsigned char  vOutAspect;                         /* aspect of startup VideoOut                                 */
    unsigned char  vOutColorType;                      /* colorimetry of startup VideoOut                            */
    unsigned char  reserve1[1];                        /* reserved area                                              */
    unsigned long  outputType;                         /* startup OSD/MASK                                           */
    unsigned long  paletteAddress;                     /* OSD Color Palette top address                              */
    unsigned long  osdAddress;                         /* startup OSD top address                                    */
    unsigned long  maskAddress;                        /* startup Mask top address                                   */
    unsigned char  inputWrRvs;                         /* reversing                                                  */
    unsigned char  codecColorType;                     /* colorimetry of CodecOut                                    */
    unsigned short stabiMinZmRate;                     /* stabilize minimum zoom rate(Q12 format[decimal:bit`0-11`]) */
} SSc_Sys_VideoInfo;

/* Audio Information */
typedef struct tagSSc_Sys_AudioInfo {
    unsigned char  useFlag;                            /* use Audio Function or not */
    unsigned char  i2sMode;                            /* I2S mode                  */
    unsigned char  i2sFormat;                          /* I2S serial format         */
    unsigned char  i2sBck;                             /* I2S bit clock             */
    unsigned char  reserve[28];                        /* reserved area             */
} SSc_Sys_AudioInfo;

/* Memory Buffer for Picture */
typedef struct tagSSc_Sys_PictBuffInfo {
    unsigned long  field0MemAdr;                       /* "field:0" area top address */
    unsigned long  field0MemSize;                      /* "field:0" area size        */
    unsigned long  field1MemAdr;                       /* "field:1" area top address */
    unsigned long  field1MemSize;                      /* "field:1" area size        */
    unsigned long  field2MemAdr;                       /* "field:2" area top address */
    unsigned long  field2MemSize;                      /* "field:2" area size        */
    unsigned long  field3MemAdr;                       /* "field:3" area top address */
    unsigned long  field3MemSize;                      /* "field:3" area size        */
} SSc_Sys_PictBuffInfo;

/* Memory DetailInformation */
typedef struct tagSSc_Sys_MemDetailInfo {
    SSc_Sys_PictBuffInfo  rawPict;                     /* Raw Picture                                  */
    SSc_Sys_PictBuffInfo  subDsPict;                   /* SubDS Picture                                */
    SSc_Sys_PictBuffInfo  threadPict[SC_SYS_MEM_DETAIL_THREAD_PICT_NUM];  /* Thread Picture            */
    unsigned long  camIpcMemAdr;                       /* Cam I/P Convert area top address             */
    unsigned long  camIpcMemSize;                      /* Cam I/P Convert area size                    */
    unsigned long  wd3dnrMemAdr;                       /* WD 3DNR area top address                     */
    unsigned long  wd3dnrMemSize;                      /* WD 3DNR area size                            */
    unsigned long  artdMemAdr;                         /* ARTD area top address                        */
    unsigned long  artdMemSize;                        /* ARTD area size                               */
    unsigned long  stabiInfoMemAdr;                    /* StabilizeInfo shared memory top address      */
    unsigned long  stabiInfoMemSize;                   /* StabilizeInfo shared memory size             */
    unsigned long  camInfoMemAdr;                      /* CamInfo shared memory top address            */
    unsigned long  camInfoMemSize;                     /* CamInfo shared memory size                   */
    unsigned char  reserve[56];                        /* reserved area                                */
} SSc_Sys_MemDetailInfo;

/* CPU3 Information */
typedef struct tagSSc_Sys_Cpu3Info {
    unsigned long  timeOut;                            /* CPU3 Boot Time Out[msec] 0:Infinity */
    unsigned char  reserve[28];                        /* reserved area                       */
} SSc_Sys_Cpu3Info;

/* CoreFW Information */
typedef struct tagSSc_Sys_CoreFwInfo {
    unsigned long  debugInfoA;                         /* DebugInfo A POST Warning Num */
    unsigned long  debugInfoB;                         /* DebugInfo B Thread Info */
    unsigned long  debugInfoC;                         /* DebugInfo C RM Info Adr */
    unsigned long  debugInfoD;                         /* DebugInfo D RM Info Size */
    unsigned long  debugInfoE;                         /* DebugInfo E postCap(GetLimitŔf) */
    unsigned long  debugInfoF;                         /* DebugInfo F reserve */
    unsigned long  debugInfoG;                         /* DebugInfo G reserve */
    unsigned long  debugInfoH;                         /* DebugInfo H reserve */
} SSc_Sys_CoreFwInfo;


/* CoreFW Common Parameter */
typedef struct tagSSc_Sys_CoreFwParam {
    SSc_Sys_BootInfo      bootInfo;                    /* boot information         */
    SSc_Sys_VerInfo       verInfo;                     /* boot parameter version   */
    SSc_Sys_MemInfo       memInfo;                     /* memory information       */
    SSc_Sys_LogInfo       logInfo;                     /* log infomation           */
    SSc_Sys_PeriIoInfo    periIoInfo;                  /* Peripheral IO infomation */
    SSc_Sys_VideoInfo     videoInfo;                   /* Video infomation         */
    SSc_Sys_AudioInfo     audioInfo;                   /* Audio infomation         */
    SSc_Sys_MemDetailInfo memDetailInfo;               /* memory detail infomation */
    SSc_Sys_Cpu3Info      cpu3Info;                    /* CPU3 infomation          */
    unsigned char         reserve0[32];                /* reserved area            */
    unsigned char         reserve1[32];                /* reserved area            */
    unsigned char         reserve2[32];                /* reserved area            */
    unsigned char         reserve3[32];                /* reserved area            */
    unsigned char         reserve4[32];                /* reserved area            */
    unsigned char         reserve5[32];                /* reserved area            */
    SSc_Sys_CoreFwInfo    coreFwInfo;                  /* core fw information      */
} SSc_Sys_CoreFwParam;

#ifdef __cplusplus
}
#endif /* __cplusplus */


/* Video Out format ID for U-Boot */
#define VOUT_ID_ALL_NUM				(49)
#define VOUT_ID_SD_NUM				(19)
#define VOUT_ID_HD_NUM				(30)
#define VOUT_ID_720_486_5994I			(0x00)          /* 720x486@59.94i   */
#define VOUT_ID_720_576_50I			(0x01)          /* 720x576@50i      */
#define VOUT_ID_RESERVED_00			(0x02) 
#define VOUT_ID_720_480_5994I			(0x03)          /* 720x480@59.94i   */
#define VOUT_ID_RESERVED_01			(0x04) 
#define VOUT_ID_720_480_5994P			(0x05)          /* 720x480@59.94p   */
#define VOUT_ID_720_576_50P			(0x06)          /* 720x576@50p      */
#define VOUT_ID_720_486_5994I_PAR_10_11		(0x07)          /* 720x486@59.94i PAR10:11 */
#define VOUT_ID_RESERVED_02			(0x08)    
#define VOUT_ID_720_480_5994I_PAR_10_11		(0x09)          /* 720x480@59.94iPAR10:11  */
#define VOUT_ID_720_576_50I_PAR_59_54		(0x0a)          /* 720x576@50i PAR59:54    */
#define VOUT_ID_720_576_50I_PAR_12_11		(0x0b)          /* 720x576@50i PAR12:11    */
#define VOUT_ID_720_486_5994I_FULL		(0x0c)          /* 720x486@59.94i (Full)   */
#define VOUT_ID_720_576_50I_FULL		(0x0d)          /* 720x576@50i (Full)      */
#define VOUT_ID_RESERVED_03			(0x0e) 
#define VOUT_ID_720_480_5994I_FULL		(0x0f)          /* 720x480@59.94i (Full)   */
#define VOUT_ID_RESERVED_04			(0x10)
#define VOUT_ID_720_480_5994P_FULL		(0x11)          /* 720x480@59.94p (Full)   */
#define VOUT_ID_720_576_50P_FULL		(0x12)          /* 720x576@50p (Full)      */
#define VOUT_ID_1280_720_60P			(0x13)          /* 1280x720@60p     */
#define VOUT_ID_1280_720_5994P			(0x14)          /* 1280x720@59.94p  */
#define VOUT_ID_1280_720_30P			(0x15)          /* 1280x720@30p     */
#define VOUT_ID_1280_720_2997P			(0x16)          /* 1280x720@29.97p  */
#define VOUT_ID_1280_720_50P			(0x17)          /* 1280x720@50p     */
#define VOUT_ID_1280_720_25P			(0x18)          /* 1280x720@25p     */
#define VOUT_ID_1920_1080_60P			(0x19)          /* 1920x1080@60p    */
#define VOUT_ID_1920_1080_5994P			(0x1a)          /* 1920x1080@59.94p */
#define VOUT_ID_1920_1080_30P			(0x1b)          /* 1920x1080@30p    */
#define VOUT_ID_1920_1080_2997P			(0x1c)          /* 1920x1080@29.97p */
#define VOUT_ID_1920_1080_50P			(0x1d)          /* 1920x1080@50p    */
#define VOUT_ID_1920_1080_25P			(0x1e)          /* 1920x1080@25p    */
#define VOUT_ID_1920_1080_60I			(0x1f)          /* 1920x1080@60i    */
#define VOUT_ID_1920_1080_5994I			(0x20)          /* 1920x1080@59.94i */
#define VOUT_ID_1920_1080_50I			(0x21)          /* 1920x1080@50i    */
#define VOUT_ID_1280_720_60P_CROP_FULL		(0x22)          /* 1280x720@60p (Crop Full)     */
#define VOUT_ID_1280_720_5994P_CROP_FULL	(0x23)          /* 1280x720@59.94p (Crop Full)  */
#define VOUT_ID_1280_720_30P_CROP_FULL		(0x24)          /* 1280x720@30p (Crop Full)     */
#define VOUT_ID_1280_720_2997P_CROP_FULL	(0x25)          /* 1280x720@29.97p (Crop Full)  */
#define VOUT_ID_1280_720_50P_CROP_FULL		(0x26)          /* 1280x720@50p (Crop Full)     */
#define VOUT_ID_1280_720_25P_CROP_FULL		(0x27)          /* 1280x720@25p (Crop Full)     */
#define VOUT_ID_1920_1080_60P_CROP_FULL		(0x28)          /* 1920x1080@60p (Crop Full)    */
#define VOUT_ID_1920_1080_5994P_CROP_FULL	(0x29)          /* 1920x1080@59.94p (Crop Full) */
#define VOUT_ID_1920_1080_30P_CROP_FULL		(0x2a)          /* 1920x1080@30p (Crop Full)    */
#define VOUT_ID_1920_1080_2997P_CROP_FULL	(0x2b)          /* 1920x1080@29.97p (Crop Full) */
#define VOUT_ID_1920_1080_50P_CROP_FULL		(0x2c)          /* 1920x1080@50p (Crop Full)    */
#define VOUT_ID_1920_1080_25P_CROP_FULL		(0x2d)          /* 1920x1080@25p (Crop Full)    */
#define VOUT_ID_1920_1080_60I_CROP_FULL		(0x2e)          /* 1920x1080@60i (Crop Full)    */
#define VOUT_ID_1920_1080_5994I_CROP_FULL	(0x2f)          /* 1920x1080@59.94i (Crop Full) */
#define VOUT_ID_1920_1080_50I_CROP_FULL		(0x30)          /* 1920x1080@50i (Crop Full)    */
#define VOUT_ID_OFF				(0xFF)          /* video out off    */

#define VOUT_FMT_SYNC_NUM	(2)
#define VOUT_FMT_SYNC_TRC	(0x00)
#define VOUT_FMT_SYNC_HV	(0x01)
#define VOUT_FMT_SYNC_OFF	(0xFF)

#define VOUT_FMT_RES_SD		(0x00)
#define VOUT_FMT_RES_HD		(0x01)
#define VOUT_FMT_RES_OFF	(0xFF)

#define VOUT_FMT_IP_INTERLACE	(0x00)
#define VOUT_FMT_IP_PROGRESSIVE (0x01)
#define VOUT_FMT_IP_OFF		(0xFF)

#endif /* COREFW_BOOT_PARAM_H */

