/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <common.h>
#include <flash.h>
#include "boot_param.h"
#include "corefw_boot_param.h"
#include "corefw_comm_param.h"
#include "command_exec.h"
#include "xarina_bool.h"
#include "cam_set_param.h"

#define DDR_MEM_ADDR_MASK	0xF0000000
#define DDR_MEM_ADDR_DDR0	0x60000000
#define MEM_FIELD_NUM		4
#define THREAD_TYPE_NUM		3	/* 0:Thread 0-3, 1:Thread 4, 2: Thread 5 */
#define THREAD_TYPE_03		0
#define THREAD_TYPE_RAW		1
#define THREAD_TYPE_SUB_DS	2
#define CAP_TABLE_DELIMITER	0xFF


struct vout_cap_table {
	unsigned char videosize;
	unsigned char cap[THREAD_TYPE_NUM][VOUT_ID_ALL_NUM];

};

static const unsigned char vout_fmt_table[][2] = {
	{SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5,			SC_SYS_VIDEO_VOUT_FMT_720_486_59I_CEA861E},		/* 720x486@59.94i   */
	{SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5,			SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E},		/* 720x576@50i      */
	{SC_SYS_VIDEO_VOUT_FMT_RESERVED_00,				SC_SYS_VIDEO_VOUT_FMT_RESERVED_01},
	{SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5,			SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E},		/* 720x480@59.94i   */
	{SC_SYS_VIDEO_VOUT_FMT_RESERVED_02,				SC_SYS_VIDEO_VOUT_FMT_RESERVED_03},
	{SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1,			SC_SYS_VIDEO_VOUT_FMT_720_480_59P_CEA861E},		/* 720x480@59.94p   */
	{SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1,			SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E},		/* 720x576@50p      */
	{SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_PAR_10_11,		SC_SYS_VIDEO_VOUT_FMT_720_486_59I_CEA861E_PAR_10_11},	/* 720x486@59.94i PAR10:11 */
	{SC_SYS_VIDEO_VOUT_FMT_RESERVED_04,				SC_SYS_VIDEO_VOUT_FMT_RESERVED_05},
	{SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_PAR_10_11,		SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E_PAR_10_11},	/* 720x480@59.94i PAR10:11 */
	{SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_59_54,		SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_PAR_59_54},	/* 720x576@50i    PAR59:54 */
	{SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_12_11,		SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_PAR_12_11},	/* 720x576@50i    PAR12:11 */
	{SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_FULL,		SC_SYS_VIDEO_VOUT_FMT_720_486_59I_CEA861E_FULL},	/* 720x486@59.94i (Full)   */
	{SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_FULL,		SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_FULL},	/* 720x576@50i (Full)      */
	{SC_SYS_VIDEO_VOUT_FMT_RESERVED_06,				SC_SYS_VIDEO_VOUT_FMT_RESERVED_07},
	{SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_FULL,		SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E_FULL},	/* 720x480@59.94i (Full)   */
	{SC_SYS_VIDEO_VOUT_FMT_RESERVED_08,				SC_SYS_VIDEO_VOUT_FMT_RESERVED_09},
	{SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1_FULL,		SC_SYS_VIDEO_VOUT_FMT_720_480_59P_CEA861E_FULL},	/* 720x480@59.94p (Full)   */
	{SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1_FULL,		SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E_FULL},	/* 720x576@50p (Full)      */
	{SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M,			SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E},		/* 1280x720@60p     */
	{SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M,			SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E},		/* 1280x720@59.94p  */
	{SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_SMPTE296M,			SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_CEA861E},		/* 1280x720@30p     */
	{SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_SMPTE296M,			SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_CEA861E},		/* 1280x720@29.97p  */
	{SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M,			SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E},		/* 1280x720@50p     */
	{SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_SMPTE296M,			SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_CEA861E},		/* 1280x720@25p     */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_BT1120_7,			SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_CEA861E},		/* 1920x1080@60p    */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_59P_BT1120_7,			SC_SYS_VIDEO_VOUT_FMT_1920_1080_59P_CEA861E},		/* 1920x1080@59.94p */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7,			SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E},		/* 1920x1080@30p    */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7,			SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E},		/* 1920x1080@29.97p */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_BT1120_7,			SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_CEA861E},		/* 1920x1080@50p    */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7,			SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E},		/* 1920x1080@25p    */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7,			SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_CEA861E},		/* 1920x1080@60i    */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7,			SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_CEA861E},		/* 1920x1080@59.94i */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7,			SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_CEA861E},		/* 1920x1080@50i    */
	{SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M_CROP_FULL,	SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E_CROP_FULL},	/* 1280x720@60p Crop Full     */
	{SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M_CROP_FULL,	SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E_CROP_FULL},	/* 1280x720@59.94p Crop Full  */
	{SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_SMPTE296M_CROP_FULL,	SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_CEA861E_CROP_FULL},	/* 1280x720@30p Crop Full     */
	{SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_SMPTE296M_CROP_FULL,	SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_CEA861E_CROP_FULL},	/* 1280x720@29.97p Crop Full  */
	{SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M_CROP_FULL,	SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E_CROP_FULL},	/* 1280x720@50p Crop Full     */
	{SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_SMPTE296M_CROP_FULL,	SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_CEA861E_CROP_FULL},	/* 1280x720@25p Crop Full     */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_BT1120_7_CROP_FULL,	SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_CEA861E_CROP_FULL},	/* 1920x1080@60p Crop Full    */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_59P_BT1120_7_CROP_FULL,	SC_SYS_VIDEO_VOUT_FMT_1920_1080_59P_CEA861E_CROP_FULL},	/* 1920x1080@59.94p Crop Full */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7_CROP_FULL,	SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E_CROP_FULL},	/* 1920x1080@30p Crop Full    */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7_CROP_FULL,	SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E_CROP_FULL},	/* 1920x1080@29.97p Crop Full */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_BT1120_7_CROP_FULL,	SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_CEA861E_CROP_FULL},	/* 1920x1080@50p Crop Full    */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7_CROP_FULL,	SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E_CROP_FULL},	/* 1920x1080@25p Crop Full    */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7_CROP_FULL,	SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_CEA861E_CROP_FULL},	/* 1920x1080@60i Crop Full    */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7_CROP_FULL,	SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_CEA861E_CROP_FULL},	/* 1920x1080@59.94i Crop Full */
	{SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7_CROP_FULL,	SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_CEA861E_CROP_FULL},	/* 1920x1080@50i Crop Full    */


};

static const struct vout_cap_table cap_table[] = {

	{0x01,	{{	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	}}},
	{0x02,	{{	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	}}},
	{0x03,	{{	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	}}},
	{0x04,	{{	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	}}},
	{0x05,	{{	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	}}},
	{0x10,	{{	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	}}},
	{0x11,	{{	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	}}},
	{0x20,	{{	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	}}},
	{0x21,	{{	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	FALSE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	}}},
	{0x22,	{{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	}}},
	{0x23,	{{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	},
		{	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	TRUE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	FALSE,	}}},

};




unsigned char corefw_param_vout_fmt_get(unsigned char id, unsigned char sync)
{
	if ((VOUT_ID_ALL_NUM <= id) || (VOUT_FMT_SYNC_NUM <= sync)) {
		return SC_SYS_VIDEO_VOUT_FMT_OFF;
	}

	return vout_fmt_table[id][sync];

}

unsigned char corefw_param_vout_sync_type_get(unsigned char fmt)
{
	int i, j;

	for (i = 0; i < VOUT_ID_ALL_NUM; i++) {
		for (j = 0; j < VOUT_FMT_SYNC_NUM; j++) {
			if (fmt == vout_fmt_table[i][j]) {
				return j;
			}
		}
	}

	return VOUT_FMT_SYNC_OFF;
}
unsigned char corefw_param_vout_id_get(unsigned char fmt)
{
	int i, j;

	for (i = 0; i < VOUT_ID_ALL_NUM; i++) {
		for (j = 0; j < VOUT_FMT_SYNC_NUM; j++) {
			if (fmt == vout_fmt_table[i][j]) {
				return i;
			}
		}
	}


	return VOUT_ID_OFF;
}

unsigned char corefw_param_vout_res_type_get(unsigned char fmt)
{
	int i, j;

	for (i = 0; i < VOUT_ID_ALL_NUM; i++) {
		for (j = 0; j < VOUT_FMT_SYNC_NUM; j++) {
			if (fmt == vout_fmt_table[i][j]) {
				if (i < VOUT_ID_SD_NUM) {
					return VOUT_FMT_RES_SD;
				} else {
					return VOUT_FMT_RES_HD;
				}
			}
		}
	}

	return VOUT_FMT_RES_OFF;
}

unsigned char corefw_param_vout_ip_type_get(unsigned char fmt)
{
	int i, j;
	char ret = VOUT_FMT_IP_OFF;

	for (i = 0; i < VOUT_ID_ALL_NUM; i++) {
		for (j = 0; j < VOUT_FMT_SYNC_NUM; j++) {
			if (fmt == vout_fmt_table[i][j]) {
				switch (i) {
				case VOUT_ID_720_486_5994I:
				case VOUT_ID_720_576_50I:
				case VOUT_ID_720_480_5994I:
				case VOUT_ID_720_486_5994I_PAR_10_11:
				case VOUT_ID_720_480_5994I_PAR_10_11:
				case VOUT_ID_720_576_50I_PAR_59_54:
				case VOUT_ID_720_576_50I_PAR_12_11:
				case VOUT_ID_1920_1080_60I:
				case VOUT_ID_1920_1080_5994I:
				case VOUT_ID_1920_1080_50I:
				case VOUT_ID_720_486_5994I_FULL:
				case VOUT_ID_720_576_50I_FULL:
				case VOUT_ID_720_480_5994I_FULL:
				case VOUT_ID_1920_1080_60I_CROP_FULL:
				case VOUT_ID_1920_1080_5994I_CROP_FULL:
				case VOUT_ID_1920_1080_50I_CROP_FULL:

					ret = VOUT_FMT_IP_INTERLACE;
					break;
				default:
					ret = VOUT_FMT_IP_PROGRESSIVE;
					break;

				}

			}
		}
	}

	return ret;
}

/************************************/
/* parameter member operation       */
/************************************/
unsigned char corefw_param_uart0_spi1_select_get(void)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	return p_param->periIoInfo.uartSspSel;
}



void corefw_param_tvout_type_set(unsigned long data)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	p_param->videoInfo.vOutFormat = data;
	return;
}

void corefw_param_reverse_type_set(unsigned long data)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	p_param->videoInfo.inputWrRvs = data;
	return;
}

void corefw_param_audio_use_set(unsigned char flag)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	p_param->audioInfo.useFlag = flag;
}

void corefw_param_audio_i2s_mode_set(unsigned char mode)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	p_param->audioInfo.i2sMode = mode;
}

void corefw_param_audio_i2s_bck_set(unsigned char bit_clock)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	p_param->audioInfo.i2sBck = bit_clock;
}

unsigned char corefw_param_tvout_type_get(void)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	return p_param->videoInfo.vOutFormat;
}

unsigned long corefw_param_osd_mask_output_status_get(void)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	return p_param->videoInfo.outputType;
}

void  corefw_param_osd_mask_output_status_set(unsigned long data)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	p_param->videoInfo.outputType = data;
	return;
}

unsigned long corefw_param_osd_color_plt_addr_get(void)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	return p_param->videoInfo.paletteAddress;
}

unsigned long corefw_param_osd_data_addr_get(void)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	return p_param->videoInfo.osdAddress;
}

unsigned long corefw_param_mask_data_addr_get(void)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	return p_param->videoInfo.maskAddress;
}

unsigned char corefw_param_audio_use_get(void)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	return p_param->audioInfo.useFlag;
}
unsigned char corefw_param_audio_i2s_mode_get(void)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	return p_param->audioInfo.i2sMode;
}
unsigned char corefw_param_audio_i2s_format_get(void)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	return p_param->audioInfo.i2sFormat;
}
unsigned char corefw_param_vout_thread_number_get(void)
{
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	return p_param->videoInfo.vOutThreadNum;
}




int is_corefw_comm_param_uart0_spi1_select(void)
{
	unsigned char select;
	int ret = FALSE;

	select = corefw_param_uart0_spi1_select_get();

	switch(select) {
	case SC_SYS_USE_UART0:
		/* FALLTHROUGH */
	case SC_SYS_USE_SSP1:
		ret = TRUE;
		break;
	default:
		break;
	}

	return ret;
}



static int check_ddr_memory_for_vout(SSc_Sys_PictBuffInfo *p_info)
{
	unsigned long *tmp = (unsigned long *)p_info;
	int ret = FALSE;
	int i;

	for (i = 0; i < MEM_FIELD_NUM; i++) {
		if ((*(tmp + i * 2) & DDR_MEM_ADDR_MASK) == DDR_MEM_ADDR_DDR0) {
			ret = TRUE;
			break;
		}
	}

	return ret;

}

int corefw_param_vout_hd_limit_check(void)
{
	int ret = FALSE;
	struct  tagSSc_Sys_CoreFwParam *p_param = 
		(struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();
	
	/* rotation enable check */
	if (p_param->bootInfo.enableRotation == SC_SYS_ROTATION_DISABLE) {
		return FALSE;
	}

	/*  */
	switch (p_param->videoInfo.vOutThreadNum) {
	case SC_SYS_VIDEO_THREAD_0:	/* fall through */
	case SC_SYS_VIDEO_THREAD_1:	/* fall through */
	case SC_SYS_VIDEO_THREAD_2:	/* fall through */
	case SC_SYS_VIDEO_THREAD_3:
		ret = check_ddr_memory_for_vout(&(p_param->memDetailInfo.threadPict[p_param->videoInfo.vOutThreadNum]));
		break;
	case SC_SYS_VIDEO_THREAD_RAW:
		ret = check_ddr_memory_for_vout(&(p_param->memDetailInfo.rawPict));
		break;
	case SC_SYS_VIDEO_THREAD_SUB_DS:
		ret = check_ddr_memory_for_vout(&(p_param->memDetailInfo.subDsPict));
		break;
	default:
		/* do nothing */
		break;

	}

	return ret;


}

int corefw_param_vout_capability_check(unsigned long id) 
{
	int i;
	unsigned char video_size;
	unsigned char th_type;


	video_size = cam_param_videosize_ddr_get();

	/* argument check */
	if (VOUT_ID_ALL_NUM <= id) {
		return FALSE;
	}

	/* thread check */
	switch (corefw_param_vout_thread_number_get()) {
	case SC_SYS_VIDEO_THREAD_0:	/* fall through */
	case SC_SYS_VIDEO_THREAD_1:	/* fall through */
	case SC_SYS_VIDEO_THREAD_2:	/* fall through */
	case SC_SYS_VIDEO_THREAD_3:
		th_type = THREAD_TYPE_03;
		break;
	case SC_SYS_VIDEO_THREAD_RAW:
		th_type = THREAD_TYPE_RAW;
		break;
	case SC_SYS_VIDEO_THREAD_SUB_DS:
		th_type = THREAD_TYPE_SUB_DS;
		break;
	default:	/* error */
		return FALSE;

	}


	for (i = 0; i < (sizeof(cap_table) / sizeof(cap_table[0])); i++) {
		if (cap_table[i].videosize == video_size) {
			return (int)cap_table[i].cap[th_type][id];
		}
	}

	return FALSE;

}


int corefw_param_capability_check_total(void)
{
	unsigned int id, fmt;
	int ret = FALSE;

	/* current video out format get */
	fmt = corefw_param_tvout_type_get();
    if (fmt == SC_SYS_VIDEO_VOUT_FMT_UNUSED) {
        return TRUE;
    }
    
    /* video id get from format */
	id = corefw_param_vout_id_get(fmt);
	/* capability check */
	if (corefw_param_vout_capability_check(id) == FALSE) {
		ret = TRUE;
	}
	/* check whether HD resolution is effective */
	if ((corefw_param_vout_res_type_get(fmt) == VOUT_FMT_RES_HD) && 
		(corefw_param_vout_hd_limit_check() == TRUE)) {
			ret = TRUE;
	}

	return ret; 
}



