/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <asm/arch/hw_reg.h>
#include <asm/arch/xarina.h>
#include "ddrcommon.h"
#include "reg_ddr.h"

#define READ_BDL_STEP          8
#define READ_BDL_MARGIN        8
#define MARGIN_MUL             1              
#define MARGIN_DIV             4



static void wait_apb_10_cycle(void)
{
	volatile int wait;
	for(wait = 0 ; wait < 40 ; wait++)
		;
}

static void polling_idone(unsigned long addr)
{
	while((REG_DDRPHY_010_PGSR0(addr).w & 0x00000001) == 0)
	  ;
}

static void init_delayline_reg(unsigned int base, unsigned int lane0readtap, unsigned int lane1readtap)
{
	REG_DDRPHY_1CC_DX0BDLR0 (base).w = DDRTOP_DX0BDLR0_DATA;
	REG_DDRPHY_1D0_DX0BDLR1 (base).w = DDRTOP_DX0BDLR1_DATA;
	REG_DDRPHY_1D4_DX0BDLR2 (base).w = DDRTOP_DX0BDLR2_DATA | (lane0readtap << 18) | (lane0readtap << 12);
	REG_DDRPHY_1D8_DX0BDLR3 (base).w = (lane0readtap << 24) | (lane0readtap << 18) | (lane0readtap << 12) | (lane0readtap << 6) | lane0readtap;
	REG_DDRPHY_1DC_DX0BDLR4 (base).w =                        (lane0readtap << 18) | (lane0readtap << 12) | (lane0readtap << 6) | lane0readtap;
	REG_DDRPHY_20C_DX1BDLR0 (base).w = DDRTOP_DX1BDLR0_DATA;
	REG_DDRPHY_210_DX1BDLR1 (base).w = DDRTOP_DX1BDLR1_DATA;
	REG_DDRPHY_214_DX1BDLR2 (base).w = DDRTOP_DX1BDLR2_DATA | (lane1readtap << 18) | (lane1readtap << 12);
	REG_DDRPHY_218_DX1BDLR3 (base).w = (lane1readtap << 24) | (lane1readtap << 18) | (lane1readtap << 12) | (lane1readtap << 6) | lane1readtap;
	REG_DDRPHY_21C_DX1BDLR4 (base).w =                        (lane1readtap << 18) | (lane1readtap << 12) | (lane1readtap << 6) | lane1readtap;

}

static void do_phy_init_to_dqs_gate(unsigned int phy_addr, unsigned int lane0, unsigned int lane1, S_DQS_GATE_RESULT *ret)
{
	REG_DDRPHY_004_PIR(phy_addr).w  = 0x08000000;	/* clear Status Register */
	init_delayline_reg(phy_addr, lane0, lane1);
	REG_DDRPHY_004_PIR(phy_addr).w = 0x7F3;		/* Initialization to DQS gating */ 
	wait_apb_10_cycle();
	polling_idone(phy_addr);

	ret->sl[0]   = REG_DDRPHY_1F0_DX0GTR(phy_addr).w & 0x7;		/* SL, lane0 */
	ret->sl[1]   = REG_DDRPHY_230_DX1GTR(phy_addr).w & 0x7;		/* SL, lane1 */

	ret->lcdl[0] = REG_DDRPHY_1E8_DX0LCDLR2(phy_addr).w & 0xff;	/* LCDL, lane0 */
	ret->lcdl[1] = REG_DDRPHY_228_DX1LCDLR2(phy_addr).w & 0xff;	/* LCDL, lane1 */
}
static int check_diff_of_2results(unsigned int phy_addr, S_DQS_GATE_RESULT r[])
{
	unsigned int lane;
	unsigned int error = 0;

	for (lane=0; lane < 2; lane++) {
	unsigned int dqs_period;
	int diff; /* signed */
	int margin;

	if (lane == 0) {
		dqs_period = (REG_DDRPHY_1C4_DX0GSR0(phy_addr).w >> 16) & 0xff;
	} else {
		dqs_period = (REG_DDRPHY_204_DX1GSR0(phy_addr).w >> 16) & 0xff;
	}
	    
	diff =((r[0].sl[lane] - r[1].sl[lane]) * 2 * dqs_period) + r[0].lcdl[lane] - r[1].lcdl[lane];
	margin = dqs_period * MARGIN_MUL / MARGIN_DIV;
	  
	if ( (diff < ((READ_BDL_STEP * 2) - margin)) || (((READ_BDL_STEP * 2) + margin) < diff))
		error++;
	}

	return error;
}

/* setup ddrcon */
#ifdef CONFIG_XARINA_VSS
static	int do_init_ddrtop(unsigned int phy_addr, AggregatedDdrId ddr_id)
#else
static	int do_init_ddrtop(unsigned int phy_addr)
#endif // CONFIG_XARINA_VSS
{
	unsigned int stage;

	/* poling until PGSR0 bit3 is set to 1 */
	while((REG_DDRPHY_010_PGSR0(phy_addr).w & 0x00000008) == 0)
	;

	/* clear DSGCR bit0 and bit 5 to 0*/
	REG_DDRPHY_040_DSGCR(phy_addr).w &= 0xffffffde;

	/* set PIR */  
	REG_DDRPHY_004_PIR(phy_addr).w = 0x80000000;

	/* poling until PGSR0 bit0 is set to 1 */
	polling_idone(phy_addr);

	REG_DDRPHY_00C_PGCR1  (phy_addr).w = DDRTOP_PGCR1_0_DATA;
	REG_DDRPHY_018_PLLCR  (phy_addr).w = DDRTOP_PLLCR_DATA;
	REG_DDRPHY_01C_PTR0   (phy_addr).w = DDRTOP_PTR0_DATA;
	REG_DDRPHY_020_PTR1   (phy_addr).w = DDRTOP_PTR1_DATA;
	REG_DDRPHY_024_PTR2   (phy_addr).w = DDRTOP_PTR2_DATA;
	REG_DDRPHY_028_PTR3   (phy_addr).w = DDRTOP_PTR3_DATA;
	REG_DDRPHY_02C_PTR4   (phy_addr).w = DDRTOP_PTR4_DATA;
	REG_DDRPHY_040_DSGCR  (phy_addr).w = DDRTOP_DSGCR_DATA;
	REG_DDRPHY_044_DCR    (phy_addr).w = DDRTOP_DCR_DATA;
	REG_DDRPHY_048_DTPR0  (phy_addr).w = DDRTOP_DTP0_DATA;
	REG_DDRPHY_04C_DTPR1  (phy_addr).w = DDRTOP_DTP1_DATA;
	REG_DDRPHY_050_DTPR2  (phy_addr).w = DDRTOP_DTP2_DATA;
	REG_DDRPHY_054_MR0    (phy_addr).w = DDRTOP_MR0_DATA;
	REG_DDRPHY_058_MR1    (phy_addr).w = DDRTOP_MR1_DATA;
	REG_DDRPHY_05C_MR2    (phy_addr).w = DDRTOP_MR2_DATA;
	REG_DDRPHY_060_MR3    (phy_addr).w = DDRTOP_MR3_DATA;
	REG_DDRPHY_068_DTCR   (phy_addr).w = DDRTOP_DTCR_DATA;
#ifdef CONFIG_XARINA_VSS
    if ((ddr_id == DDR_ID_1_SYSRAM) || (ddr_id == DDR_ID_2_VRAM)) {
        REG_DDRPHY_184_ZQ0CR1 (phy_addr).w = DDRTOP_ZQ0CR1_DATA_MOD_1;
    }
    else if ((ddr_id == DDR_ID_2_SYSRAM) || (ddr_id == DDR_ID_3_VRAM)) {
        REG_DDRPHY_184_ZQ0CR1 (phy_addr).w = DDRTOP_ZQ0CR1_DATA_MOD_2;
    }
    else {
        REG_DDRPHY_184_ZQ0CR1 (phy_addr).w = DDRTOP_ZQ0CR1_DATA;
    }
#else
	REG_DDRPHY_184_ZQ0CR1 (phy_addr).w = DDRTOP_ZQ0CR1_DATA;
#endif // CONFIG_XARINA_VSS
	REG_DDRPHY_1C0_DX0GCR (phy_addr).w = DDRTOP_DX0GCR_DATA;
	REG_DDRPHY_200_DX1GCR (phy_addr).w = DDRTOP_DX1GCR_DATA;

	/* Pull Up/Down setting 500ohms / 0 : OPEN / 1 : 650ohms) */
	REG_DDRPHY_03C_DXCCR  (phy_addr).w = ((REG_DDRPHY_03C_DXCCR  (phy_addr).w) & 0xFFFFF11F) | (0x00000220);


	/*
	Apply WA or not
	stage  0   1   2   3
	lane0  No  Yes Yes No
	lann1  No  Yes No  Yes
	*/

	for (stage = 0 ; stage < 4 ; stage++) {
		S_DQS_GATE_RESULT r[2]; /* 2 trials */
		unsigned int offset[2];

		/* set offset */
		if ((stage == 1) || (stage == 2)) {
		  offset[0] = READ_BDL_STEP;
		} else {
		  offset[0] = 0;
		}

		/* calc offset */
		if ((stage == 1) || (stage == 3)) {
		  offset[1] = READ_BDL_STEP;
		} else {
		  offset[1] = 0;
		}

		/* check DQS's position */ 
		do_phy_init_to_dqs_gate(phy_addr, (READ_BDL_STEP * 2) + offset[0], (READ_BDL_STEP * 2) + offset[1], &r[0]);
		do_phy_init_to_dqs_gate(phy_addr, offset[0], offset[1], &r[1]);                                            

		/* check result */
		if(check_diff_of_2results(phy_addr, r) == 0) {
			break;
		}
	}

	if (stage == 4) {
		return STATUS_INIT_FAILED;
	}

	/* do the rest stages of training */
	REG_DDRPHY_004_PIR(phy_addr).w = DDRTOP_PIR_3_DATA;
	wait_apb_10_cycle();
	polling_idone(phy_addr);

	if (REG_DDRPHY_010_PGSR0(phy_addr).w & 0x7FFF0000) {
		return STATUS_INIT_FAILED;
	}

	/* set DSGCR bit5 to 1 (Read-Modify-Write) */
	REG_DDRPHY_040_DSGCR(phy_addr).w |= 0x00000020;

	return STATUS_INIT_SUCCEEDED;
}

#ifdef CONFIG_XARINA_VSS
void init_ddrtop_with_ddr_id(unsigned long *result, AggregatedDdrId ddr_id)
{
	int err;

#if defined(CONFIG_XARINA_SPL)
	err = do_init_ddrtop(DDRTOP0_BASE, ddr_id);
#else
	err = do_init_ddrtop(DDRTOP1_BASE, ddr_id);
#endif

	if (err == STATUS_INIT_SUCCEEDED) {
		*result = XARINA_DDR_INIT_SUCCESS;
	} else {
		*result = XARINA_DDR_INIT_FAILURE;
	}
}
#else
void init_ddrtop(unsigned long *result)
{
	int err;

#if defined(CONFIG_XARINA_SPL)
	err = do_init_ddrtop(DDRTOP0_BASE);
#else
	err = do_init_ddrtop(DDRTOP1_BASE);
#endif

	if (err == STATUS_INIT_SUCCEEDED) {
		*result = XARINA_DDR_INIT_SUCCESS;
	} else {
		*result = XARINA_DDR_INIT_FAILURE;
	}
}
#endif // CONFIG_XARINA_VSS

