/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include "hw_reg_ctrl.h"
#include "gpio_vss.h"

static void gpio_set_direction(unsigned long base, unsigned long data)
{
    unsigned long val = readl(base + GPIO_DATA_DIRECTION);
    val &= ~(GPIO_DIRECTION_MASK);
    val |= (data & GPIO_DIRECTION_MASK);
    writel(val, (base + GPIO_DATA_DIRECTION));

    return;
}

void gpio_init(void)
{
    unsigned long data;

    /* Pull Up: ABC */
    data = SYSREG1_RIN(SYSREG1_GPIO_ABCD_PC);
    data &= ~(SYSREG1_GPIO_PA_PC_MASK
              | SYSREG1_GPIO_PB_PC_MASK
              | SYSREG1_GPIO_PC_PC_MASK);
    data |= 0x00000089; /* Up: PA7,3,0 */
    data |= 0x0000F800; /* Up: PB7-3   */
    data |= 0x00FF0000; /* Up: PC7-0   */
    SYSREG1_ROUT(SYSREG1_GPIO_ABCD_PC, data);

    /* Pull Up: EFGH */
    data = 0;
    data |= 0x000000E1; /* Up: PE7-5,0 */
    data |= 0x00001F00; /* Up: PF4-0   */
    data |= 0x008E0000; /* Up: PG7,3-1 */
    data |= 0xFF000000; /* Up: PH7-0   */
    SYSREG1_ROUT(SYSREG1_GPIO_EFGH_PC, data);

    /* Pull Up: I */
    data = SYSREG1_RIN(SYSREG1_GPIO_I_PC);
    data &= ~(SYSREG1_GPIO_PI_PC_MASK);
    data |= 0x0000003F; /* Up: PI5-0 */
    SYSREG1_ROUT(SYSREG1_GPIO_I_PC, data);

    /* Direction: A */
    gpio_set_direction(GPIO_A_BASE, 0x000000F6); /* Out: PA7-4,2-1 */

    /* Direction: E */
    gpio_set_direction(GPIO_E_BASE, 0x0000009E); /* Out: PE7,4-1 */

    /* Direction: F */
    gpio_set_direction(GPIO_F_BASE, 0x000000E0); /* Out: PF7-5 */

    /* Direction: G */
    gpio_set_direction(GPIO_G_BASE, 0x00000051); /* Out: PG6,4,0 */

    /* Direction: I */
    gpio_set_direction(GPIO_I_BASE, 0x0000007D); /* Out: PI6-2,0 */

    return;
}

int gpio_read(unsigned long ch, unsigned long bit)
{
    const unsigned long addr = 1 << (bit + 2);

    unsigned long val = readl(ch + addr);

    return (val >> bit);
}

int gpio_write(unsigned long ch, unsigned long bit, unsigned long val)
{
    const unsigned long addr = 1 << (bit + 2);
    const unsigned long data = val << bit;

    writel(data, (ch + addr));

    return 0;
}
