/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#ifndef CONFIG_XARINA_SPL
#include <version.h>
#include <common.h>
#include "cam_comm_param.h"
#include "corefw_boot_param.h"
#include "init_i2c_audio.h"
#endif
#include <asm/arch/hw_reg.h>
#include <asm/arch/xarina.h>
#include "hw_reg_ctrl.h"
#include "pinmux.h"

/* PLL2 PLL7 setting */
#define CIS_108_0_PLL7		(0x00000040)
#define CIS_108_0_PLL2		(0x00031f02)
#define CIS_108_1_PLL7		(0x007C5A12)
#define CIS_108_1_PLL2		(0x00035702)
#define CIS_74_25_0_PLL7	(0x00000040)
#define CIS_74_25_0_PLL2	(0x00072b02)
#define CIS_74_25_1_PLL7	(0x00333102)
#define CIS_74_25_1_PLL2	(0x00062702)
#define CCD_NTSC_PLL7		(0x00000040)
#define CCD_NTSC_PLL2		(0x00082f02)
#define CCD_PAL_PLL7		(0x001A1C02)
#define CCD_PAL_PLL2		(0x00071F02)
#define CCD_SVGA_PLL7		(0x00000040)
#define CCD_SVGA_PLL2		(0x000e2902)
#define CCD_YC_5M_PLL7		(0x00000040)
#define CCD_YC_5M_PLL2		(0x00081D02)

extern void udelay (unsigned long usec);

/************************************/
/* SYSREG1                          */
/************************************/
void xarina_hw_pinmux_set(unsigned long *p_reg)
{
	unsigned long data;

	/* set GPIO A */
	data = SYSREG1_RIN(SYSREG1_CAM_GPIOA_SEL);
	data &= ~SYSREG1_PINSEL_PA_MASK;
	data |= p_reg[PINMUX_GPIO_A];
	SYSREG1_ROUT(SYSREG1_CAM_GPIOA_SEL, data);
	
	/* set GPIO B E*/
	data = SYSREG1_RIN(SYSREG1_GPIO_BE_SEL);
	data &= ~(SYSREG1_PINSEL_PB_MASK | SYSREG1_PINSEL_PE_MASK);
	data |= (p_reg[PINMUX_GPIO_B] << SYSREG1_PINSEL_PB_SHIFT);
	data |= (p_reg[PINMUX_GPIO_E] << SYSREG1_PINSEL_PE_SHIFT);
	SYSREG1_ROUT(SYSREG1_GPIO_BE_SEL, data);
	
	/* set GPIO C*/
	data = SYSREG1_RIN(SYSREG1_VIDEO_GPIO_SVI_SEL);
	data &= ~SYSREG1_PINSEL_PC_MASK;
	data |= (p_reg[PINMUX_GPIO_C] << SYSREG1_PINSEL_PC_SHIFT);
	SYSREG1_ROUT(SYSREG1_VIDEO_GPIO_SVI_SEL, data);

	/* set GPIO G*/
	data = SYSREG1_RIN(SYSREG1_GPIOFG_SEL);
	data &= ~SYSREG1_PINSEL_PG_MASK;
	data |= (p_reg[PINMUX_GPIO_G] << SYSREG1_PINSEL_PG_SHIFT);
	SYSREG1_ROUT(SYSREG1_GPIOFG_SEL, data);

	/* set GPIO H I*/
	data = SYSREG1_RIN(SYSREG1_GPIOHI_SEL);
	data &= ~(SYSREG1_PINSEL_PH_MASK | SYSREG1_PINSEL_PI_MASK);
	data |= (p_reg[PINMUX_GPIO_H] << SYSREG1_PINSEL_PH_SHIFT);
	data |= (p_reg[PINMUX_GPIO_I0] << SYSREG1_PINSEL_PI_SHIFT);
	SYSREG1_ROUT(SYSREG1_GPIOHI_SEL, data);

	/* set GPIO I*/
	data = SYSREG1_RIN(SYSREG1_NAND_GPIOI_SEL);
	data &= ~SYSREG1_NAND_PI_MASK;
	data |= p_reg[PINMUX_GPIO_I1];
	SYSREG1_ROUT(SYSREG1_NAND_GPIOI_SEL, data);

	return;
}

int xarina_hw_jtag_connection_check(void)
{
	int ret = 0;
	
	switch (SYSREG1_RIN(SYSREG1_PORT_MON) & SYSREG1_A9JTAGSEL) {
	case SYSREG1_A9JTAGSEL_CPU1:
		/* Fall through */
	case SYSREG1_A9JTAGSEL_CPU2:
		/* Fall through */
	case SYSREG1_A9JTAGSEL_CPU3:
		/* Fall through */
	case SYSREG1_A9JTAGSEL_DAISYCHAIN:
		ret = 1;
		break;
	default:
		/* do nothing */
		break;
	}
	
	return ret;
}

void xarina_hw_video_clock_select(unsigned long clock)
{
	unsigned long data;
	
	data = SYSREG1_RIN(SYSREG1_VIDEO_GPIO_SVI_SEL);
	data &= ~SYSREG1_VIDEOCLK_SEL_MASK;
	data |= (clock & SYSREG1_VIDEOCLK_SEL_MASK);
	SYSREG1_ROUT(SYSREG1_VIDEO_GPIO_SVI_SEL, data);
	
}


void xarina_hw_bootram_remap(void)
{
	SYSREG1_ROUT(SYSREG1_REMAP_CTR, SYSREG1_BOTR_BOOTREMAP_BIT);
}


void xarina_hw_nfif_clock_set(unsigned long clock)
{
	unsigned long data;
	
	data = SYSREG1_RIN(SYSREG1_CLK_CTL);
	data &= ~SYSREG1_NFIF_CKSEL_MASK;
	data |= clock << SYSREG1_NFIF_CKSEL_SHIFT;
	SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
}


void xarina_hw_sdif_clock_en(void)
{
	unsigned long data;

	data = SYSREG1_RIN(SYSREG1_CLK_EN);

	data |= SYSREG1_SDIF_CKON;

	SYSREG1_ROUT(SYSREG1_CLK_EN, data);
}

int xarina_hw_gbe_mode_get(void)
{
	unsigned long data;
	int ret;

	data = SYSREG1_RIN(SYSREG1_PORT_MON) & 0x00000001;

	switch (data) {
	case SYSREG1_CKSEL_GCLK_25MHZ:
		ret = XARINA_GBE_MII;
		break;
	case SYSREG1_CKSEL_GCLK_125MHZ:
	default:
		ret = XARINA_GBE_RGMII;
		break;
	}

#if defined(CONFIG_XARINA_EVBRD_PRE)
	ret = XARINA_GBE_RGMII;
#endif	/* CONFIG_XARINA_EVBRD_PRE */

	return ret;

}

int xarina_hw_nand_cont_type_get(void)
{
	unsigned long data;
	int ret;

	data = SYSREG1_RIN(SYSREG1_PORT_MON) & 0x00000040;

	switch (data) {
	case SYSREG1_NAND_CONTROLLER1:
		ret = XARINA_NAND_CONTROLLER1;
		break;
	case SYSREG1_NAND_CONTROLLER2:
	default:
		ret = XARINA_NAND_CONTROLLER2;
		break;
	}

	return ret;
}


#if !defined(CONFIG_NAND_SPL)
void xarina_hw_gbe_init(void)
{
	unsigned long data;

	data = SYSREG1_RIN(SYSREG1_CLK_CTL);
	data &= ~SYSREG1_GBE_CKSEL_MASK;

	/* GbE mode & clock setting */
	if (xarina_hw_gbe_mode_get() == XARINA_GBE_RGMII) {
		xarina_hw_pll5_select();
		SYSREG1_ROUT(SYSREG1_MODE_SEL, SYSREG1_GBEMODE_RGMII);
		data |= (SYSREG1_GBE_CKSEL_RGMII1000M << SYSREG1_GBE_CKSEL_SHIFT);
	} else {
		SYSREG1_ROUT(SYSREG1_MODE_SEL, SYSREG1_GBEMODE_MII);
		data |= (SYSREG1_GBE_CKSEL_MII10M_100M << SYSREG1_GBE_CKSEL_SHIFT);
	}

	SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
	
	/* GbE clock enable */
	data = SYSREG1_RIN(SYSREG1_CLK_EN);
	data |= (SYSREG1_GBE_CKON | SYSREG1_GBEEXT_EN); 
	SYSREG1_ROUT(SYSREG1_CLK_EN, data);

	/* GbE unreset */
	data = SYSREG1_RIN(SYSREG1_RST_CTL);	
	data |= SYSREG1_RST_CTL_GBE_RST_CTL_N;
	SYSREG1_ROUT(SYSREG1_RST_CTL, data);
	
	udelay(5);

	return;
}

void xarina_hw_pll5_select(void)
{
	unsigned long data;
	
	/* PLL5 setting */
	SYSREG1_ROUT(SYSREG1_PLL5, SYSREG1_PLL5_SMAC_FREQ);
	/* PLL5 Power On */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_567);
	data |= SYSREG1_PLL5_POWER_ON;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_567, data);
	
	/* wait PLL5 ready */
	while(!(SYSREG1_RIN(SYSREG1_PLL_READY) & SYSREG1_PLL5_READY));

	/* PLL5 output enable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_567);
	data |= (SYSREG1_PLL5_POWER_ON | SYSREG1_PLL5_OUT_EN);
	SYSREG1_ROUT(SYSREG1_PLL_POWER_567, data);
	udelay(5);	
}

#if defined(CONFIG_XARINA_SPL)

void xarina_hw_gbe_disable(void)
{
	unsigned long data;

	/* GbE clock disable */
	data = SYSREG1_RIN(SYSREG1_CLK_EN);
	data &= ~SYSREG1_GBE_CKON;
	data &= ~SYSREG1_GBEEXT_EN;
	SYSREG1_ROUT(SYSREG1_CLK_EN, data);
	
	/* GbE reset */
	data = SYSREG1_RIN(SYSREG1_RST_CTL);	
	data &= ~SYSREG1_RST_CTL_GBE_RST_CTL_N;
	SYSREG1_ROUT(SYSREG1_RST_CTL, data);

	return;
}


#else /* defined(CONFIG_XARINA_SPL) */ 

void xarina_hw_pll6_select(void)
{
	unsigned long data;
	
	/* PLL6 Power On */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_567);
	data |= SYSREG1_PLL6_POWER_ON;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_567, data);
	
	/* wait PLL6 ready */
	while(!(SYSREG1_RIN(SYSREG1_PLL_READY) & SYSREG1_PLL6_READY));

	/* PLL6 output enable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_567);
	data |= (SYSREG1_PLL6_POWER_ON | SYSREG1_PLL6_OUT_EN);
	SYSREG1_ROUT(SYSREG1_PLL_POWER_567, data);
	udelay(5);	
}

static void xarina_hw_pll2_select(unsigned long pll2)
{
	unsigned long data;
	
	SYSREG1_ROUT(SYSREG1_PLL2, pll2);
	/* PLL2 Power On */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data |= SYSREG1_PLL2_POWER_ON;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);
	/* wait PLL2 ready */
	while(!(SYSREG1_RIN(SYSREG1_PLL_READY) & SYSREG1_PLL2_READY));
	/* PLL2 Clock Enable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data |= SYSREG1_PLL2_OUT_EN;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);
}

static void xarina_hw_pll7_select(unsigned long pll7)
{
	unsigned long data;
	
	SYSREG1_ROUT(SYSREG1_PLL7, pll7);
	/* PLL7 Power On */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_567);
	data |= SYSREG1_PLL7_POWER_ON;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_567, data);
	/* wait PLL7 ready */
	while(!(SYSREG1_RIN(SYSREG1_PLL_READY) & SYSREG1_PLL7_READY));
	/* PLL7 Clock Enable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_567);
	data |= SYSREG1_PLL7_OUT_EN;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_567, data);
}

void xarina_hw_pll2_pll7_unselect(void)
{
	unsigned long data;

	/* PLL2 Power Off and Clock Disable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data &= ~(SYSREG1_PLL2_POWER_ON | SYSREG1_PLL2_OUT_EN);
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);
	
	/* PLL7 Power Off and Clock Disable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_567);
	data &= ~(SYSREG1_PLL7_POWER_ON | SYSREG1_PLL7_OUT_EN);
	SYSREG1_ROUT(SYSREG1_PLL_POWER_567, data);
	

}




void xarina_hw_pll2_pll7_select(int clock_type)
{
	switch(clock_type) {
	case CAM_PARAM_PLL2_CLK_216MHZ:
		SYSREG1_ROUT(SYSREG1_PLL7, CIS_108_0_PLL7);
		xarina_hw_pll2_select(CIS_108_0_PLL2);
		break;
	case CAM_PARAM_PLL2_CLK_216_216MHZ:
		xarina_hw_pll7_select(CIS_108_1_PLL7);
		xarina_hw_pll2_select(CIS_108_1_PLL2);
		break;
	case CAM_PARAM_PLL2_CLK_148_5MHZ:
		SYSREG1_ROUT(SYSREG1_PLL7, CIS_74_25_0_PLL7);
		xarina_hw_pll2_select(CIS_74_25_0_PLL2);
		break;
	case CAM_PARAM_PLL2_CLK_148_352MHZ:
		xarina_hw_pll7_select(CIS_74_25_1_PLL7);
		xarina_hw_pll2_select(CIS_74_25_1_PLL2);
		break;
	case CAM_PARAM_PLL2_CLK_144MHZ:
		SYSREG1_ROUT(SYSREG1_PLL7, CCD_NTSC_PLL7);
		xarina_hw_pll2_select(CCD_NTSC_PLL2);
		break;
	case CAM_PARAM_PLL2_CLK_116MHZ:
		xarina_hw_pll7_select(CCD_PAL_PLL7);
		xarina_hw_pll2_select(CCD_PAL_PLL2);
		break;
	case CAM_PARAM_PLL2_CLK_75_6MHZ:
		SYSREG1_ROUT(SYSREG1_PLL7, CCD_SVGA_PLL7);
		xarina_hw_pll2_select(CCD_SVGA_PLL2);
		break;
	case CAM_PARAM_PLL2_CLK_90MHZ:
		SYSREG1_ROUT(SYSREG1_PLL7, CCD_YC_5M_PLL7);
		xarina_hw_pll2_select(CCD_YC_5M_PLL2);
		break;
	default:
		printf("error: unknown PLL2 PLL7 type!!\n");
		/* do nothing */
		break;
	}
}




void xarina_hw_cpu1_reset(void)
{
	unsigned long data;
	
	data = SYSREG1_RIN(SYSREG1_RST_CTL);
	data &= ~SYSREG1_AVCC_ARM9_RST_CTL_N;
	SYSREG1_ROUT(SYSREG1_RST_CTL, data);
	udelay(5);

	return;
}


void xarina_hw_cpu1_unreset(void)
{	

	unsigned long data;

	/* AVARM9 clock enable */	
	data = SYSREG1_RIN(SYSREG1_CLK_EN);
	data |= SYSREG1_AVA9_CKON;
	SYSREG1_ROUT(SYSREG1_CLK_EN, data);

	/* AVARM9 unreset */
	data = SYSREG1_RIN(SYSREG1_RST_CTL);
	data |= SYSREG1_AVCC_ARM9_RST_CTL_N;
	SYSREG1_ROUT(SYSREG1_RST_CTL, data);

	udelay(5);

	return;
}


int xarina_hw_devid_get(void)
{
	unsigned long data;
	int ret;

	data = SYSREG1_RIN(SYSREG1_DEVICE_ID);

	switch (data) {
	case SYSREG1_DEVICE_ID_ES1:
		ret = XARINA_DEVICE_ES1;
		break;
	case SYSREG1_DEVICE_ID_ES2:
		ret = XARINA_DEVICE_ES2;
		break;
	case SYSREG1_DEVICE_ID_ES3:
		ret = XARINA_DEVICE_ES3;
		break;
	default:
		ret = XARINA_DEVICE_UNKNOWN;
		break;
	}

	return ret;

}

int xarina_hw_boot_type_get(void)
{
	return (int)*(unsigned char*)XARINA_BOOT_TYPE_ADDR;
}

unsigned char xarina_hw_evr_type_get(void)
{
	unsigned char ret;
	unsigned long evr_type;

	evr_type = SYSREG1_RIN(SYSREG1_PORT_MON) & SYSREG1_EVRINIT_MASK;

	switch(evr_type) {
	case SYSREG1_EVRINIT_SCI:
		ret = CAM_PARAM_EVR_TYPE_SCI;
		break;
	case SYSREG1_EVRINIT_SPI:
		ret = CAM_PARAM_EVR_TYPE_SPI;
		break;
	default:
		ret = CAM_PARAM_EVR_TYPE_SCI;
		break;
	}

	return ret;
	
}

int xarina_hw_system_type_get(void)
{
	unsigned long data;
	int ret;

	/* read efuse */
	data = SYSREG1_RIN(SYSREG1_DEVICE_ID_CTRL);
	data &= ~(0x1);
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, data);
	data |= 0x04;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, data);
	data &= ~(0xFF00);
	data |= 0x0900;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, data);
	data |= 0x02;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, data);
	data &= ~(0x02);
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, data);
	data |= 0x04;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, data);
	data &= ~(0x01);
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, data);
	data = SYSREG1_RIN(SYSREG1_DEVICE_ID_1108);
	data &= ~(0xFFFF00FF);

	switch(data) {
	case SYSREG1_DEVICE_ID_MODULE_H:
		ret = XARINA_SYSTEM_MODULE_H;
		break;
	case SYSREG1_DEVICE_ID_MODULE_S:
		ret = XARINA_SYSTEM_MODULE_S;
		break;
	case SYSREG1_DEVICE_ID_MODULE_H2:
		ret = XARINA_SYSTEM_MODULE_H2;
		break;
	case SYSREG1_DEVICE_ID_MODULE_S2:
		ret = XARINA_SYSTEM_MODULE_S2;
		break;
	default:
		ret = XARINA_SYSTEM_PACKAGE;
		break;
	}

	return ret;
}

void xarina_hw_bus_config_set(int ddr_num)
{
	switch (ddr_num) {
	case 1:
		/* 1 DDR memory */
		/* BUS Config 0 setting */
		BUS_CONFIG0_ROUT(BUS_CONFIG0_00_REG, BUS_CONFIG0_1DDR_00_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_01_REG, BUS_CONFIG0_1DDR_01_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_02_REG, BUS_CONFIG0_1DDR_02_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_03_REG, BUS_CONFIG0_1DDR_03_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_04_REG, BUS_CONFIG0_1DDR_04_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_05_REG, BUS_CONFIG0_1DDR_05_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_06_REG, BUS_CONFIG0_1DDR_06_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_07_REG, BUS_CONFIG0_1DDR_07_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_08_REG, BUS_CONFIG0_1DDR_08_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_09_REG, BUS_CONFIG0_1DDR_09_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_10_REG, BUS_CONFIG0_1DDR_10_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_11_REG, BUS_CONFIG0_1DDR_11_DATA);
		/* BUS Config 1 setting */
		BUS_CONFIG1_ROUT(BUS_CONFIG1_00_REG, BUS_CONFIG1_1DDR_00_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_01_REG, BUS_CONFIG1_1DDR_01_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_02_REG, BUS_CONFIG1_1DDR_02_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_03_REG, BUS_CONFIG1_1DDR_03_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_04_REG, BUS_CONFIG1_1DDR_04_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_05_REG, BUS_CONFIG1_1DDR_05_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_06_REG, BUS_CONFIG1_1DDR_06_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_07_REG, BUS_CONFIG1_1DDR_07_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_08_REG, BUS_CONFIG1_1DDR_08_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_09_REG, BUS_CONFIG1_1DDR_09_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_10_REG, BUS_CONFIG1_1DDR_10_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_11_REG, BUS_CONFIG1_1DDR_11_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_12_REG, BUS_CONFIG1_1DDR_12_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_13_REG, BUS_CONFIG1_1DDR_13_DATA);
		break;
	case 2:
	default:
		/* 2 DDR memories */
		/* BUS Config 0 setting */
		BUS_CONFIG0_ROUT(BUS_CONFIG0_00_REG, BUS_CONFIG0_2DDR_00_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_01_REG, BUS_CONFIG0_2DDR_01_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_02_REG, BUS_CONFIG0_2DDR_02_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_03_REG, BUS_CONFIG0_2DDR_03_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_04_REG, BUS_CONFIG0_2DDR_04_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_05_REG, BUS_CONFIG0_2DDR_05_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_06_REG, BUS_CONFIG0_2DDR_06_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_07_REG, BUS_CONFIG0_2DDR_07_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_08_REG, BUS_CONFIG0_2DDR_08_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_09_REG, BUS_CONFIG0_2DDR_09_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_10_REG, BUS_CONFIG0_2DDR_10_DATA);
		BUS_CONFIG0_ROUT(BUS_CONFIG0_11_REG, BUS_CONFIG0_2DDR_11_DATA);
		/* BUS Config 1 setting */
		BUS_CONFIG1_ROUT(BUS_CONFIG1_00_REG, BUS_CONFIG1_2DDR_00_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_01_REG, BUS_CONFIG1_2DDR_01_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_02_REG, BUS_CONFIG1_2DDR_02_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_03_REG, BUS_CONFIG1_2DDR_03_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_04_REG, BUS_CONFIG1_2DDR_04_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_05_REG, BUS_CONFIG1_2DDR_05_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_06_REG, BUS_CONFIG1_2DDR_06_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_07_REG, BUS_CONFIG1_2DDR_07_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_08_REG, BUS_CONFIG1_2DDR_08_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_09_REG, BUS_CONFIG1_2DDR_09_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_10_REG, BUS_CONFIG1_2DDR_10_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_11_REG, BUS_CONFIG1_2DDR_11_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_12_REG, BUS_CONFIG1_2DDR_12_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_13_REG, BUS_CONFIG1_2DDR_13_DATA);
		break;
	}
}

void xarina_hw_address_tiling_set(unsigned char sel)
{
	switch (sel) {
	case CAM_PARAM_TILING_SEL_TYPE1:
		BUS_CONFIG1_ROUT(BUS_CONFIG1_14_REG, BUS_CONFIG1_TILING_TYPE1_14_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_15_REG, BUS_CONFIG1_TILING_TYPE1_15_DATA);
		break;
	case CAM_PARAM_TILING_SEL_TYPE2:
		BUS_CONFIG1_ROUT(BUS_CONFIG1_14_REG, BUS_CONFIG1_TILING_TYPE2_14_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_15_REG, BUS_CONFIG1_TILING_TYPE2_15_DATA);
		break;
	case CAM_PARAM_TILING_SEL_OFF:
		default:
		BUS_CONFIG1_ROUT(BUS_CONFIG1_14_REG, BUS_CONFIG1_TILING_OFF_14_DATA);
		BUS_CONFIG1_ROUT(BUS_CONFIG1_15_REG, BUS_CONFIG1_TILING_OFF_15_DATA);
		break;
	}
}



void xarina_hw_audio_clock_input_sel(void)
{
	unsigned long data;

	data = SYSREG1_RIN(SYSREG1_FUNC_SEL1);
	#if defined(CONFIG_XARINA_VSS)
	if (is_i2c_audio_clock_master()) {
		data &= ~SYSREG1_AUDIO_MCLK_OE_N;
	}
	else {
		data |= SYSREG1_AUDIO_MCLK_OE_N;
	}
	#else
	data |= SYSREG1_AUDIO_MCLK_OE_N;
	#endif /* defined(CONFIG_XARINA_VSS) */

	SYSREG1_ROUT(SYSREG1_FUNC_SEL1, data);
}


/************************************/
/* SYSREG0                          */
/************************************/
/* SDout clock enable */
void xarina_hw_sdout_clock_enable(void)
{
	unsigned long data;
	
	data = SYSREG0_RIN(SYSREG0_CLK_EN);
	data |= SYSREG0_SLICEVIDEO_CKON;
	SYSREG0_ROUT(SYSREG0_CLK_EN, data);

	return;
}


#endif /* defined(CONFIG_XARINA_SPL) */ 
#endif /* !defined(CONFIG_NAND_SPL) */
