/*
 * (C) Copyright 2007, 2011, 2013 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <asm/byteorder.h>
#include <asm/arch/hw_reg.h>
#include <asm/arch/xarina.h>
#include "hw_reg_init.h"
#include "hw_reg_init_tables.h"
#include "ddrcommon.h"


/***********************************/
/* function prototype              */
/***********************************/
/* functions called from external */
extern void xa_pll1_sel_start(void);
extern void xa_hw_vram_init(void);
extern void xa_hw_sysram_init(void);
extern void xarina_init_unreset(void);

/* file-internal functions */
static void set_ddr_ctl_registers(const u32 base, const u32 * const ddr_ctl_data_ver);
#ifdef CONFIG_XARINA_VSS
static void xa_hw_ddrtop_init(AggregatedDdrId ddr_id);
#else
static void xa_hw_ddrtop_init(void);
#endif // CONFIG_XARINA_VSS
static u32 xa_hw_ddrcon_init(const u32 ddr_base_addr);

/* external function calls */
void se2p_i2c_init(void);
void se2p_i2c_eeprom_rd(u32 dst, u32 src, u32 size);


/***********************************/
/* initial unreset                 */
/***********************************/
#if defined(CONFIG_XARINA_NOR_BOOT)
void xarina_init_unreset(void)
{
	register u32 r1;

	/* AVCC Block clock enable */
	r1 = SYSREG1_RIN(SYSREG1_CLK_EN);
	r1 |= SYSREG1_AVCAMCTL_CKON;
	SYSREG1_ROUT(SYSREG1_CLK_EN, r1);
	/* wait */
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");

	/* AVCC Block unreset */
	r1 = SYSREG1_RIN(SYSREG1_RST_CTL);
	r1 |= SYSREG1_AVCC_RST_CTL_N;
	SYSREG1_ROUT(SYSREG1_RST_CTL, r1);
	/* wait */
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");

	/* EMC-SM2 CS1 16bitmode */
	r1 = EMC_SM2_PHY_AM_DW_16BIT;
	EMCSM2_ROUT(EMC_SM2_PHY_AM1_CFG, r1);
	/* wait */
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");

	return;
}
#endif /* defined(CONFIG_XARINA_NOR_BOOT) */


/***********************************/
/* pll1 select                     */
/***********************************/
void xa_pll1_sel_start(void)
{
	register u32 r1;

	/* PLL1 power on */
	r1 = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	r1 |= SYSREG1_PLL1_POWER_ON;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, r1);

	/* pll1_wait_loop */
	while(!(SYSREG1_RIN(SYSREG1_PLL_READY) & SYSREG1_PLL1_READY));

	/* select system clock */
	r1 = SYSREG1_RIN(SYSREG1_CLK_CTL);
	r1 |= SYSREG1_SYS_CKSEL_PLL1;
	SYSREG1_ROUT(SYSREG1_CLK_CTL, r1);

	/* wait */
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");

	return;
}


/***********************************/
/* VRAM (DDR1 REG) init            */
/***********************************/
#ifdef CONFIG_XARINA_VSS
AggregatedDdrId getDDRIdFromEEPROM(u32 eeprom_address)
{
    u16 ddr_data;
    u32 dst, src, size;

    /* init */
    se2p_i2c_init();

    /* read DDR ID */
    dst = (u32)&ddr_data;
    src = eeprom_address;
    size = sizeof(ddr_data);
    se2p_i2c_eeprom_rd(dst, src, size);

    switch(be16_to_cpu(ddr_data)) { /* converting endianness */
        case 0:
            return (eeprom_address == EEPROM_DDR1_ADDR) ? DDR_ID_0_VRAM : DDR_ID_0_SYSRAM;
        case 1:
            return (eeprom_address == EEPROM_DDR1_ADDR) ? DDR_ID_1_VRAM : DDR_ID_1_SYSRAM;
		case 2:
            return (eeprom_address == EEPROM_DDR1_ADDR) ? DDR_ID_2_VRAM : DDR_ID_2_SYSRAM;
        case 3:
            return (eeprom_address == EEPROM_DDR1_ADDR) ? DDR_ID_3_VRAM : DDR_ID_MODULE_SYSRAM;
        default:
            break;
    }
    return (eeprom_address == EEPROM_DDR1_ADDR) ? DDR_ID_MODULE_VRAM : DDR_ID_MODULE_SYSRAM;
}
#endif // CONFIG_XARINA_VSS

void xa_hw_vram_init(void)
{
	register u32 r1;
#ifdef CONFIG_XARINA_VSS
    AggregatedDdrId ddr_id;
#endif // CONFIG_XARINA_VSS

#ifdef CONFIG_XARINA_EVBRD_PRE

	/* VRAM unreset */
	r1 = SYSREG1_RIN(SYSREG1_RST_CTL);
	r1 |= SYSREG1_RST_CTL_DDR1_RST_CTL_N;
	SYSREG1_ROUT(SYSREG1_RST_CTL, r1);

	/* VRAM clock enable */
	r1 = SYSREG1_RIN(SYSREG1_CLK_EN);
	r1 |= SYSREG1_DDR1_CKON;
	SYSREG1_ROUT(SYSREG1_CLK_EN, r1);

	/* xa_hw_vram_controller_init */
	/* do nothing if the controller has already configured & kicked */
	if(DDR1_RIN(DDR1_DB_CTL07_REG) & DDR_START)	goto vram_init_end;

	/* #####DataBahn(vram) Setting##### */
	{ u32 i;
		for(r1 = DDR1_DB_CTL00_REG, i = 0; r1 <= DDR1_DB_CTL46_REG; r1 += 4, i++) {
			DDR1_ROUT_VAL(r1, ddr1_ctl_data[i]);
		}
	}

	/* set interrupt mask */
	DDR1_ROUT(DDR1_DB_CTL22_REG, DDR1_CTL22_DATA | DDR_INTERRUPT_MASK);

	/* set START bit */
	DDR1_ROUT(DDR1_DB_CTL07_REG, DDR1_CTL07_DATA | DDR_START);

	/* vram_init_loop */
	while(!(DDR1_RIN(DDR1_DB_CTL22_REG) & DDR_INT_STATUS));

	/* clear all interrupts */
	DDR1_ROUT(DDR1_DB_CTL17_REG, DDR1_CTL17_DATA | DDR_INTERRUPT_CLEAR);

#else /* CONFIG_XARINA_EVBRD_PRE */

	/* DDR1 clock select */
	r1 = SYSREG1_RIN(SYSREG1_CLK_CTL);
	r1 &= ~SYSREG1_DDR1_CKSEL;
	SYSREG1_ROUT(SYSREG1_CLK_CTL, r1);

	/* DDR1 clock enable */
	r1 = SYSREG1_RIN(SYSREG1_CLK_EN);
	r1 |= SYSREG1_DDR1_CKON;
	SYSREG1_ROUT(SYSREG1_CLK_EN, r1);

	/* DDR1 unreset */
	r1 = SYSREG1_RIN(SYSREG1_RST_CTL);
	r1 |= SYSREG1_RST_CTL_DDR1_RST_CTL_N;
	SYSREG1_ROUT(SYSREG1_RST_CTL, r1);

	/* vram_unreset_loop */
	for(r1 = 0x5000; r1 != 0; r1--) {
		asm("nop");
	}

	/* do nothing if the controller has already configured & kicked */
	if(DDR1_RIN(DDR_CTL_00_REG) & 0x1)	goto vram_init_end;

#ifdef CONFIG_XARINA_VSS
    /* before performing DDR TOP confiuration, we would need to get to know
     * whether Xarina package is used with its DDR ID   */
    ddr_id = getDDRIdFromEEPROM(EEPROM_DDR1_ADDR);
	xa_hw_ddrtop_init(ddr_id);
#else
	xa_hw_ddrtop_init();
#endif // CONFIG_XARINA_VSS

	if(*(volatile u32 *)XARINA_DDRTOP_INIT_RESULT_ADDR != XARINA_DDR_INIT_SUCCESS)	goto vram_init_end_failure;

	/* vram_top_init_end */
	/* check chip type */
	/* read efuse */
	r1 = SYSREG1_RIN(SYSREG1_DEVICE_ID_CTRL);
	r1 &= ~0x01;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, r1);
	r1 |= 0x04;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, r1);
	r1 &= ~0xFF00;
	r1 |= 0x0900;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, r1);
	r1 |= 0x02;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, r1);
	r1 &= ~0x02;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, r1);
	r1 |= 0x04;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, r1);
	r1 &= ~0x01;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, r1);

	r1 = SYSREG1_RIN(SYSREG1_DEVICE_ID_1108);
	r1 &= ~0xFFFF00FF;
	if((r1 == SYSREG1_DEVICE_ID_MODULE_S) ||
	   (r1 == SYSREG1_DEVICE_ID_MODULE_S2) ||
	   (r1 == SYSREG1_DEVICE_ID_MODULE_H) ||
	   (r1 == SYSREG1_DEVICE_ID_MODULE_H2)) {
		/* DDR is initialized with constant values */
		set_ddr_ctl_registers(DDR1_DB_BASE, ddr1_ctl_data_ver00);
	}
	else {
  #if defined(CONFIG_XARINA_VSS)
        switch (ddr_id) {
            case DDR_ID_0_VRAM:
            case DDR_ID_1_VRAM:
            case DDR_ID_2_VRAM:
            case DDR_ID_3_VRAM:
            case DDR_ID_MODULE_VRAM:
			    set_ddr_ctl_registers(DDR1_DB_BASE, ddr1_ctl_data_ver00); break;
            default:
                break;
        }
  #elif defined(CONFIG_XARINA_DDR_I2C_DATA_GET)
		/* If the board type is EVB, DDR is initialized with values in EEPROM */
		r1 = SYSREG1_RIN(SYSREG1_PORT_MON) & SYSREG1_BOARD_TYPE;
		if((r1 == SYSREG1_BOARD_TYPE_EVB_1_0) ||
		   (r1 == SYSREG1_BOARD_TYPE_EVB_1_1) ||
		   (r1 == SYSREG1_BOARD_TYPE_EVB_1_0_AF) ||
		   (r1 == SYSREG1_BOARD_TYPE_EVB_1_1_AF)) {
			u32 err;
			err = xa_hw_ddrcon_init(DDR1_DB_BASE);
			if(err) {
				set_ddr_ctl_registers(DDR1_DB_BASE, ddr1_ctl_data_ver00);
			}
		}
		else {
			/* If the borard type is RD, DDR is initialized with constant values */
			set_ddr_ctl_registers(DDR1_DB_BASE, ddr1_ctl_data_ver00);
		}
  #endif
	}

	r1 = DDR1_RIN(DDR_CTL_00_REG);
	r1 |= 0x1;
	DDR1_ROUT(DDR_CTL_00_REG, r1);

	/* vram_init_end_loop0 */
	while(!(DDR1_RIN(DDR_CTL_32_REG) & 0x20));

	/* vram_init_end_loop1 */
	while((DDR1_RIN(DDR_CTL_32_REG) & 0xFF) != 0x20);

#endif /* !CONFIG_XARINA_EVBRD_PRE */


vram_init_end:
	*(volatile u32 *)XARINA_VRAM_INIT_RESULT_ADDR = XARINA_DDR_INIT_SUCCESS;
	return;

vram_init_end_failure:
	*(volatile u32 *)XARINA_VRAM_INIT_RESULT_ADDR = XARINA_DDR_INIT_FAILURE;
	return;
}


/***********************************/
/* SysRAM (DDR0 REG) init          */
/***********************************/
void xa_hw_sysram_init(void)
{
	register u32 r1;
#ifdef CONFIG_XARINA_VSS
    AggregatedDdrId ddr_id;
#endif // CONFIG_XARINA_VSS

#ifdef CONFIG_XARINA_EVBRD_PRE

	/* do nothing if the controller has already configured & kicked */
	if(DDR0_RIN(DDR0_DB_CTL07_REG) & DDR_START) goto sysram_init_end;

	/* #####DataBahn(sysram) Setting##### */
	{ u32 i;
		for(r1 = DDR0_DB_CTL00_REG, i = 0; r1 <= DDR0_DB_CTL40_REG; r1 += 4, i++) {
			DDR0_ROUT_VAL(r1, ddr0_ctl_data[i]);
		}
	}

	/* set interrupt mask */
	DDR0_ROUT(DDR0_DB_CTL22_REG, DDR0_CTL22_DATA | DDR_INTERRUPT_MASK);

	/* set START bit */
	DDR0_ROUT(DDR0_DB_CTL07_REG, DDR0_CTL07_DATA | DDR_START);

	/* sysram_init_loop */
	while(!(DDR0_RIN(DDR0_DB_CTL22_REG) & DDR_INT_STATUS));

	/* clear all interrupts */
	DDR0_ROUT(DDR0_DB_CTL17_REG, DDR0_CTL17_DATA | DDR_INTERRUPT_CLEAR);

#else /* CONFIG_XARINA_EVBRD_PRE */

	/* DDR0 clock select */
	r1 = SYSREG1_RIN(SYSREG1_CLK_CTL);
	r1 &= ~SYSREG1_DDR0_CKSEL;
	SYSREG1_ROUT(SYSREG1_CLK_CTL, r1);

	/* DDR0 clock enable */
	r1 = SYSREG1_RIN(SYSREG1_CLK_EN);
	r1 |= SYSREG1_DDR0_CKON;
	SYSREG1_ROUT(SYSREG1_CLK_EN, r1);

#ifdef CONFIG_XARINA_VSS
    /* before performing DDR TOP confiuration, we would need to get to know
     * whether Xarina package is used with its DDR ID   */
    ddr_id = getDDRIdFromEEPROM(EEPROM_DDR0_ADDR);
	xa_hw_ddrtop_init(ddr_id);
#else
	xa_hw_ddrtop_init();
#endif // CONFIG_XARINA_VSS

	if(*(volatile u32 *)XARINA_DDRTOP_INIT_RESULT_ADDR != XARINA_DDR_INIT_SUCCESS)	goto sysram_init_end_failure;

	/* sysram_top_init_end */
	/* check chip type */
	/* read efuse */
	r1 = SYSREG1_RIN(SYSREG1_DEVICE_ID_CTRL);
	r1 &= ~0x01;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, r1);
	r1 |= 0x04;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, r1);
	r1 &= ~0xFF00;
	r1 |= 0x0900;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, r1);
	r1 |= 0x02;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, r1);
	r1 &= ~0x02;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, r1);
	r1 |= 0x04;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, r1);
	r1 &= ~0x01;
	SYSREG1_ROUT(SYSREG1_DEVICE_ID_CTRL, r1);

	r1 = SYSREG1_RIN(SYSREG1_DEVICE_ID_1108);
	r1 &= ~0xFFFF00FF;
	if((r1 == SYSREG1_DEVICE_ID_MODULE_S) ||
	   (r1 == SYSREG1_DEVICE_ID_MODULE_S2) ||
	   (r1 == SYSREG1_DEVICE_ID_MODULE_H) ||
	   (r1 == SYSREG1_DEVICE_ID_MODULE_H2)) {
		/* DDR is initialized with constant values */
		set_ddr_ctl_registers(DDR0_DB_BASE, ddr0_ctl_data_ver00);
	}
	else {
  #if defined(CONFIG_XARINA_VSS)
        switch (ddr_id) {
            case DDR_ID_0_SYSRAM:
            case DDR_ID_1_SYSRAM:
            case DDR_ID_2_SYSRAM:
            case DDR_ID_MODULE_SYSRAM:
			    set_ddr_ctl_registers(DDR0_DB_BASE, ddr0_ctl_data_ver00); break;
            default:
                break;
        }
  #elif defined(CONFIG_XARINA_DDR_I2C_DATA_GET)
		/* If the board type is EVB, DDR is initialized with values in EEPROM */
		r1 = SYSREG1_RIN(SYSREG1_PORT_MON) & SYSREG1_BOARD_TYPE;
		if((r1 == SYSREG1_BOARD_TYPE_EVB_1_0) ||
		   (r1 == SYSREG1_BOARD_TYPE_EVB_1_1) ||
		   (r1 == SYSREG1_BOARD_TYPE_EVB_1_0_AF) ||
		   (r1 == SYSREG1_BOARD_TYPE_EVB_1_1_AF)) {
			u32 err;
			err = xa_hw_ddrcon_init(DDR0_DB_BASE);
			if(err) {
				set_ddr_ctl_registers(DDR0_DB_BASE, ddr0_ctl_data_ver00);
			}
		}
		else {
			/* If the borard type is RD, DDR is initialized with constant values */
			set_ddr_ctl_registers(DDR0_DB_BASE, ddr0_ctl_data_ver00);
		}
  #endif
	}

	r1 = DDR0_RIN(DDR_CTL_00_REG);
	r1 |= 0x1;
	DDR0_ROUT(DDR_CTL_00_REG, r1);

	/* sysram_init_end_loop0 */
	while(!(DDR0_RIN(DDR_CTL_32_REG) & 0x20));

	/* sysram_init_end_loop1 */
	while((DDR0_RIN(DDR_CTL_32_REG) & 0xFF) != 0x20);

#endif /* !CONFIG_XARINA_EVBRD_PRE */


sysram_init_end:
	*(volatile u32 *)XARINA_SYSRAM_INIT_RESULT_ADDR = XARINA_DDR_INIT_SUCCESS;
	return;

sysram_init_end_failure:
	*(volatile u32 *)XARINA_SYSRAM_INIT_RESULT_ADDR = XARINA_DDR_INIT_FAILURE;
	return;
}


/***********************************/
/* DataBahn Setting                */
/***********************************/
static void set_ddr_ctl_registers(const u32 base, const u32 * const ddr_ctl_data_ver)
{
	u32 i, offset;
	for(offset = DDR_CTL_00_REG, i = 0; offset <= DDR_CTL_80_REG; offset += 4, i++) {
		REG_WRITE_VAL((volatile u32 *)base, offset, ddr_ctl_data_ver[i]);
	}
	return;
}


/***********************************/
/* ddrtop init                     */
/* (prototype)                     */
/***********************************/
#ifdef CONFIG_XARINA_VSS
static void xa_hw_ddrtop_init(AggregatedDdrId ddr_id)
{
	init_ddrtop_with_ddr_id((unsigned long *)XARINA_DDRTOP_INIT_RESULT_ADDR, ddr_id);
}
#else
static void xa_hw_ddrtop_init(void)
{
	init_ddrtop((unsigned long *)XARINA_DDRTOP_INIT_RESULT_ADDR);
}
#endif // CONFIG_XARINA_VSS


/***********************************/
/* ddrcon init (for databahn)      */
/* r0: set register base addr      */
/***********************************/
static u32 xa_hw_ddrcon_init(const u32 ddr_base_addr)
{
	u32 dst, src, size;
	u32 magic_number_from_eeprom;
	u32 read_base_addr, data_num;
	volatile u32 * const data_buf = (volatile u32 *)XARINA_E2PROM_DATA_BUF_ADDR;
	volatile u32 * base_addr;
	u32 i;

	/* init */
	se2p_i2c_init();

	/* magic number read */
	dst = XARINA_E2PROM_DATA_BUF_ADDR;
	src = 0;
	size = XARINA_E2PROM_MAGIC_BYTES;
	se2p_i2c_eeprom_rd(dst, src, size);
	src += XARINA_E2PROM_MAGIC_BYTES;

	/* magic number get */
	magic_number_from_eeprom = data_buf[0];
	if(magic_number_from_eeprom != XARINA_E2PROM_MAGIC_NUM) {
		return 1;
	}

ddrcon_init_header_read:
	/* base addr & data num read */
	size = XARINA_E2PROM_HEADER_BYTES;
	se2p_i2c_eeprom_rd(dst, src, size);
	src += XARINA_E2PROM_HEADER_BYTES;

	/* base addr & data num get */
	read_base_addr = data_buf[0];
	data_num = data_buf[1];

	if(read_base_addr != ddr_base_addr) {
		/* if base address and data num is 0, data is end */
		if((read_base_addr == 0) && (data_num == 0)) {
//			while(1);	/* The data which you should set are not found. ERROR. */ /* original */
			return 1;	/* The data which you should set are not found. ERROR. */ /* modified */
		}
		else {
			src += data_num * 4;
			goto ddrcon_init_header_read;
		}
	}
	else {
		/* ddrcon_init_data_get */
		/* data get */
		size = data_num * 4;
		se2p_i2c_eeprom_rd(dst, src, size);

		/* set register */
		base_addr = (volatile u32 *)read_base_addr;
		for(i = 0; i < data_num; i++) {
			base_addr[i] = data_buf[i];
		}
	}

	return 0;
}

