/*
 * (C) Copyright 2007, 2011, 2013 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef HW_REG_INIT_H
#define HW_REG_INIT_H

#include <asm/arch/hw_reg.h>
#include <asm/types.h>

#ifdef __GNUC__ /* gcc */
#define INLINE inline __attribute__((always_inline))
#else
#define INLINE inline
#endif


/***********************************/
/* read/write macro                */
/***********************************/
/* str #data, [base + ofs] */
INLINE void REG_WRITE_VAL(volatile u32 *base, u32 ofs, u32 data)
{
	*(base + ofs/4) = data;
}

/* str reg, [base + ofs] */
INLINE void REG_WRITE(volatile u32 *base, u32 ofs, register u32 reg)
{
	*(base + ofs/4) = reg;
}

/* ldr reg, [base + ofs] */
INLINE u32 REG_READ(volatile u32 *base, u32 ofs)
{
	return *(base + ofs/4);
}


/***********************************/
/* read/write macro                */
/***********************************/
/* str #data, [DDR0_DB_BASE + ofs] */
INLINE void DDR0_ROUT_VAL(u32 ofs, u32 data)
{
	REG_WRITE_VAL((volatile u32 *)DDR0_DB_BASE, ofs, data);
}

/* str reg, [DDR0_DB_BASE + ofs] */
INLINE void DDR0_ROUT(u32 ofs, register u32 reg)
{
	REG_WRITE((volatile u32 *)DDR0_DB_BASE, ofs, reg);
}

/* ldr reg, [DDR0_DB_BASE + ofs] */
INLINE u32 DDR0_RIN(u32 ofs)
{
	return REG_READ((volatile u32 *)DDR0_DB_BASE, ofs);
}


/***********************************/
/* read/write macro                */
/***********************************/
/* str #data, [DDR1_DB_BASE + ofs] */
INLINE void DDR1_ROUT_VAL(u32 ofs, u32 data)
{
	REG_WRITE_VAL((volatile u32 *)DDR1_DB_BASE, ofs, data);
}

/* str reg, [DDR1_DB_BASE + ofs] */
INLINE void DDR1_ROUT(u32 ofs, register u32 reg)
{
	REG_WRITE((volatile u32 *)DDR1_DB_BASE, ofs, reg);
}

/* ldr reg, [DDR1_DB_BASE + ofs] */
INLINE u32 DDR1_RIN(u32 ofs)
{
	return REG_READ((volatile u32 *)DDR1_DB_BASE, ofs);
}


/***********************************/
/* SYSREG0                         */
/***********************************/
/* str #data, [SYSREG0_REG_BASE + ofs] */
INLINE void SYSREG0_ROUT_VAL(u32 ofs, u32 data)
{
	REG_WRITE_VAL((volatile u32 *)SYSREG0_REG_BASE, ofs, data);
}

/* str reg, [SYSREG0_REG_BASE + ofs] */
INLINE void SYSREG0_ROUT(u32 ofs, register u32 reg)
{
	REG_WRITE((volatile u32 *)SYSREG0_REG_BASE, ofs, reg);
}

/* ldr reg, [SYSREG0_REG_BASE + ofs] */
INLINE u32 SYSREG0_RIN(u32 ofs)
{
	return REG_READ((volatile u32 *)SYSREG0_REG_BASE, ofs);
}


/***********************************/
/* SYSREG1                         */
/***********************************/
/* str #data, [SYSREG1_REG_BASE + ofs] */
INLINE void SYSREG1_ROUT_VAL(u32 ofs, u32 data)
{
	REG_WRITE_VAL((volatile u32 *)SYSREG1_REG_BASE, ofs, data);
}

/* str reg, [SYSREG1_REG_BASE + ofs] */
INLINE void SYSREG1_ROUT(u32 ofs, register u32 reg)
{
	REG_WRITE((volatile u32 *)SYSREG1_REG_BASE, ofs, reg);
}

/* ldr reg, [SYSREG1_REG_BASE + ofs] */
INLINE u32 SYSREG1_RIN(u32 ofs)
{
	return REG_READ((volatile u32 *)SYSREG1_REG_BASE, ofs);
}


/***********************************/
/* DDRTOP0                         */
/***********************************/
/* str #data, [SYSREG0_REG_BASE + ofs] */
INLINE void DDRTOP0_ROUT_VAL(u32 ofs, u32 data)
{
	REG_WRITE_VAL((volatile u32 *)DDRTOP0_BASE, ofs, data);
}

/* str reg, [SYSREG0_REG_BASE + ofs] */
INLINE void DDRTOP0_ROUT(u32 ofs, register u32 reg)
{
	REG_WRITE((volatile u32 *)DDRTOP0_BASE, ofs, reg);
}

/* ldr reg, [SYSREG0_REG_BASE + ofs] */
INLINE u32 DDRTOP0_RIN(u32 ofs)
{
	return REG_READ((volatile u32 *)DDRTOP0_BASE, ofs);
}


/***********************************/
/* DDRTOP1                         */
/***********************************/
/* str #data, [SYSREG1_REG_BASE + ofs] */
INLINE void DDRTOP1_ROUT_VAL(u32 ofs, u32 data)
{
	REG_WRITE_VAL((volatile u32 *)DDRTOP1_BASE, ofs, data);
}

/* str reg, [SYSREG1_REG_BASE + ofs] */
INLINE void DDRTOP1_ROUT(u32 ofs, register u32 reg)
{
	REG_WRITE((volatile u32 *)DDRTOP1_BASE, ofs, reg);
}

/* ldr reg, [SYSREG1_REG_BASE + ofs] */
INLINE u32 DDRTOP1_RIN(u32 ofs)
{
	return REG_READ((volatile u32 *)DDRTOP1_BASE, ofs);
}


/***********************************/
/* EMC_SM2                         */
/***********************************/
/* str #data, [EMC_SM2_REG_BASE + ofs] */
INLINE void EMCSM2_ROUT_VAL(u32 ofs, u32 data)
{
	REG_WRITE_VAL((volatile u32 *)EMC_SM2_REG_BASE, ofs, data);
}

/* str reg, [EMC_SM2_REG_BASE + ofs] */
INLINE void EMCSM2_ROUT(u32 ofs, register u32 reg)
{
	REG_WRITE((volatile u32 *)EMC_SM2_REG_BASE, ofs, reg);
}

/* ldr reg, [EMC_SM2_REG_BASE + ofs] */
INLINE u32 EMCSM2_RIN(u32 ofs)
{
	return REG_READ((volatile u32 *)EMC_SM2_REG_BASE, ofs);
}


#endif /* HW_REG_INIT_H */
