/*
 * (C) Copyright 2007, 2011, 2013 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */


#include <config.h>
#include "hw_reg_pll3_pll4.h"

#if defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL)
#include "hw_reg_vss.h"
#include "init_i2c_audio.h"
#endif /* defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL) */

/************************************/
/* PLL3/PLL4 setting value          */
/************************************/
#define PLL3_148_352MHZ		(0x00333102)
#define PLL4_148_352MHZ		(0x00062702)
#define PLL4_148_5MHZ		(0x00072b02)
#define PLL4_297MHZ		(0x00032b02)
#define PLL3_74_176MHZ		(0x00333102)
#define PLL4_74_176MHZ		(0x000D2702)
#define PLL3_49_152MHZ		(0x004A1F02)
#define PLL4_49_152MHZ		(0x001D7F02)

static void xarina_hw_pll3_select(unsigned long pll3)
{
	unsigned long data;
	
	SYSREG1_ROUT(SYSREG1_PLL3, pll3);
	/* PLL3 Power On */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data |= SYSREG1_PLL3_POWER_ON;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);
	/* wait PLL3 ready */
	while(!(SYSREG1_RIN(SYSREG1_PLL_READY) & SYSREG1_PLL3_READY));
	/* PLL3 Clock Enable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data |= SYSREG1_PLL3_OUT_EN;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);
}

static void xarina_hw_pll4_select(unsigned long pll4)
{
	unsigned long data;
	
	SYSREG1_ROUT(SYSREG1_PLL4, pll4);
	/* PLL4 Power On */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data |= SYSREG1_PLL4_POWER_ON;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);
	/* wait PLL4 ready */
	while(!(SYSREG1_RIN(SYSREG1_PLL_READY) & SYSREG1_PLL4_READY));
	/* PLL4 Clock Enable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data |= SYSREG1_PLL4_OUT_EN;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);
}

void xarina_hw_pll3_pll4_unselect(void)
{
	unsigned long data;

	/* PLL3 and PLL4 Power Off and Clock Disable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data &= ~(SYSREG1_PLL3_POWER_ON | SYSREG1_PLL3_OUT_EN 
		| SYSREG1_PLL4_POWER_ON | SYSREG1_PLL4_OUT_EN );
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);

}

int xarina_hw_board_type_get(void)
{
	unsigned long data;
	int ret;
    #if defined(CONFIG_XARINA_VSS)
    ret = xarina_hw_vss_board_type_get();
    #else /* defined(CONFIG_XARINA_VSS) */
	data = SYSREG1_RIN(SYSREG1_PORT_MON) & SYSREG1_BOARD_TYPE;

	switch (data) {

	case SYSREG1_BOARD_TYPE_EVB_1_0:	/* fall through */
	case SYSREG1_BOARD_TYPE_EVB_1_0_AF:
		ret = XARINA_BOARD_EVB_1_0;
		break;
	case SYSREG1_BOARD_TYPE_EVB_1_1:	/* fall through */
	case SYSREG1_BOARD_TYPE_EVB_1_1_AF:
		ret = XARINA_BOARD_EVB_1_1;
		break;
	case SYSREG1_BOARD_TYPE_RD_IMX122:
		ret = XARINA_BOARD_RD_IMX122;
		break;
	case SYSREG1_BOARD_TYPE_RD_ICX692:
		ret = XARINA_BOARD_RD_ICX692;
		break;
	case SYSREG1_BOARD_TYPE_RD_IMX122AF:
		ret = XARINA_BOARD_RD_IMX122AF;
		break;
	case SYSREG1_BOARD_TYPE_RD_IMX139:
		ret = XARINA_BOARD_RD_IMX139;
		break;
	case SYSREG1_BOARD_TYPE_RD_IMX140:
		ret = XARINA_BOARD_RD_IMX140;
		break;
	case SYSREG1_BOARD_TYPE_RD_IMX139AF:
		ret = XARINA_BOARD_RD_IMX139AF;
		break;
	case SYSREG1_BOARD_TYPE_RD_IMX140AF:
		ret = XARINA_BOARD_RD_IMX140AF;
		break;
	case SYSREG1_BOARD_TYPE_RD_IMX140LA:
		ret = XARINA_BOARD_RD_IMX140LA;
		break;
	case SYSREG1_BOARD_TYPE_RD_ICX693:
		ret = XARINA_BOARD_RD_ICX693;
		break;
	default:
		ret = XARINA_BOARD_UNKNOWN;
		break;
	}
    #endif /* defined(CONFIG_XARINA_VSS) */

	return ret;

}

int xarina_hw_board_type_print(void)
{
	int ret;
    #if defined(CONFIG_XARINA_VSS)
    ret = xarina_hw_vss_board_type_print();
    #else
    ret = 0;
    #endif /* defined(CONFIG_XARINA_VSS) */

	return ret;
}

void xarina_hw_pll3_pll4_select(int clock_type)
{
	unsigned long data;
	
	data = SYSREG1_RIN(SYSREG1_CLK_CTL);
	data &= ~(SYSREG1_VCLK_CKSEL_MASK | SYSREG1_AUDIO_CKSEL_MASK);

	switch(clock_type) {
	case SC_SYS_VIDEO_VOUT_FMT_UNUSED:
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5:			/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_CEA861E:			/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5:			/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E:			/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_00: 			/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_01: 			/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5:			/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E:			/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_PAR_10_11:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_CEA861E_PAR_10_11:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_04:				/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_05:				/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_PAR_10_11:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E_PAR_10_11:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_59_54:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_PAR_59_54:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_12_11:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_PAR_12_11:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_FULL:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_CEA861E_FULL:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_FULL:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_FULL:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_06: 			/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_07: 			/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_FULL:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E_FULL:			
		data |= SYSREG1_VCLK_CKSEL_13_5MHZ_27MHZ;
		#if defined(CONFIG_XARINA_VSS)
		if (is_i2c_audio_clock_master()) {
			data &= ~SYSREG1_AUDIO_CKSEL_SLAVE;
			}
		else {
			data |= SYSREG1_AUDIO_CKSEL_SLAVE;
			}
		#else /* defined(CONFIG_XARINA_VSS) */
		if (xarina_hw_board_type_get() == XARINA_BOARD_EVB_1_1) {
			data |= SYSREG1_AUDIO_CKSEL_SLAVE;
		}
        #endif /* defined(CONFIG_XARINA_VSS) */
		SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
		xarina_hw_pll3_select(PLL3_49_152MHZ);
		xarina_hw_pll4_select(PLL4_49_152MHZ);
		break;
    #if !defined(CONFIG_XARINA_VSS)
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_02: 		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_03: 		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1: 	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59P_CEA861E: 	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1: 	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_08: 		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_09: 		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1_FULL: 	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59P_CEA861E_FULL: 	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1_FULL: 	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E_FULL:
		data |= SYSREG1_VCLK_CKSEL_27MHZ_27MHZ;
		if (xarina_hw_board_type_get() == XARINA_BOARD_EVB_1_1) {
			data |= SYSREG1_AUDIO_CKSEL_SLAVE;
		}
		SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
		xarina_hw_pll3_select(PLL3_49_152MHZ);
		xarina_hw_pll4_select(PLL4_49_152MHZ);
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_CEA861E:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_CEA861E_CROP_FULL:
		data |= SYSREG1_AUDIO_CKSEL_SLAVE;
		data |= SYSREG1_VCLK_CKSEL_74_175MHZ_74_175MHZ;
		SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
		xarina_hw_pll3_select(PLL3_148_352MHZ);
		xarina_hw_pll4_select(PLL4_148_352MHZ);
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_CEA861E:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_CEA861E:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_CEA861E_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_CEA861E_CROP_FULL:
		data |= SYSREG1_AUDIO_CKSEL_SLAVE;
		data |= SYSREG1_VCLK_CKSEL_74_25MHZ_74_25MHZ_PLL4;
		SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
		xarina_hw_pll4_select(PLL4_148_5MHZ);
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_BT1120_7:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_CEA861E:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_BT1120_7:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_CEA861E:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_BT1120_7_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_CEA861E_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_BT1120_7_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_CEA861E_CROP_FULL:
		data |= SYSREG1_AUDIO_CKSEL_SLAVE;
		data |= SYSREG1_VCLK_CKSEL_74_25MHZ_74_25MHZ_PLL4;
		SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
		xarina_hw_pll4_select(PLL4_297MHZ);
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_SMPTE296M:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_CEA861E:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_SMPTE296M:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_CEA861E:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_SMPTE296M_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_CEA861E_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_SMPTE296M_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_CEA861E_CROP_FULL:
		data |= SYSREG1_AUDIO_CKSEL_SLAVE;
		data |= SYSREG1_VCLK_CKSEL_74_25MHZ_74_25MHZ_PLL4;
		SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
		xarina_hw_pll4_select(PLL4_148_5MHZ);
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_SMPTE296M:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_CEA861E:		/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_SMPTE296M_CROP_FULL:	/* fall through */
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_CEA861E_CROP_FULL:
		data |= SYSREG1_AUDIO_CKSEL_SLAVE;
		data |= SYSREG1_VCLK_CKSEL_74_175MHZ_74_175MHZ;
		SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
		xarina_hw_pll3_select(PLL3_148_352MHZ);
		xarina_hw_pll4_select(PLL4_148_352MHZ);
		break;
    #endif /* !defined(CONFIG_XARINA_VSS) */
	case SC_SYS_VIDEO_VOUT_FMT_OFF:
		data |= SYSREG1_VCLK_CKSEL_74_25MHZ_74_25MHZ_PLL7;
		#if defined(CONFIG_XARINA_VSS)
		if (is_i2c_audio_clock_master()) {
			data &= ~SYSREG1_AUDIO_CKSEL_SLAVE;
		}
		else {
			data |= SYSREG1_AUDIO_CKSEL_SLAVE;
		}
		#else /* defined(CONFIG_XARINA_VSS) */
		if (xarina_hw_board_type_get() == XARINA_BOARD_EVB_1_1) {
			data |= SYSREG1_AUDIO_CKSEL_SLAVE;
		}
        #endif /* defined(CONFIG_XARINA_VSS) */
		SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
		xarina_hw_pll3_select(PLL3_49_152MHZ);
		xarina_hw_pll4_select(PLL4_49_152MHZ);

		break;
	default:
		/* do nothing */
		printf("error: unknown PLL3 PLL4 type!!\n");
		break;
	}
	
	
}
