/**
   hw_reg_vss.c
   
   Copyright (C) 2011- Sony Corporation.
*/

#include <config.h>
#include <asm/arch/hw_reg.h>
#include <common.h>
#include <i2c.h>

#if defined(CONFIG_ARCH_XARINA)
#include <mmc.h>
#endif /* defined(CONFIG_ARCH_XARINA) */

#if defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL)

#include <asm/arch/xarina.h>
#include <asm/arch/xarina_vss.h>
#include "hw_reg_ctrl.h"
#include "hw_reg_vss.h"

int xarina_hw_vss_board_type_print(void)
{
    int ret = -1;

    #ifdef CONFIG_CMD_EEPROM
    uchar buffer[VSS_EEPROM_BOARD_TYPE_SIZE + 1];
    int devid;
    unsigned int bus_id;

    devid = xarina_hw_devid_get();

    switch (devid) {
    case XARINA_DEVICE_ES1:
        bus_id = VSS_EEPROM_BOARD_TYPE_I2C_BUS0;
        break;
    case XARINA_DEVICE_ES2:
    case XARINA_DEVICE_ES3:
        bus_id = VSS_EEPROM_BOARD_TYPE_I2C_BUS1;
        break;
    default:
        /* unknown */
        bus_id = VSS_EEPROM_BOARD_TYPE_I2C_BUS0;
        break;
    }
    ret = i2c_set_bus_num(bus_id);
    if (ret) {
        error("i2c_set_bus_num");
        return -1;
    }
        
    memset(buffer, 0, sizeof(buffer));
    ret = eeprom_read(CONFIG_SYS_I2C_EEPROM_ADDR, VSS_EEPROM_BOARD_TYPE_OFFSET,
                      buffer, VSS_EEPROM_BOARD_TYPE_SIZE);
    if (ret) {
        error("eeprom_read");
        ret = -1;
        return -1;
    } else {
        printf("%s\n", buffer);
    }
    #else
    #error "XARINA VSS must support EEPROM"
    #endif
    
    return ret;
}

int xarina_hw_vss_board_type_get(void)
{
    int ret = XARINA_BOARD_VSS_TYPE;
    
    return ret;
}

#endif /* defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL) */

#if defined(CONFIG_ARCH_XARINA)
static struct mmc* find_xarina_sd(int* dev_num)
{
#define VSS_SD_NAME "XARINA SD"

    static const int DEV_NUM_MAX = 8;
    struct mmc* m = NULL;

    int dev;
    for (dev = 0; dev < DEV_NUM_MAX; dev++) {
        m = find_mmc_device(dev);
        if (m != NULL) {
            debug("vss found %s\n", m->name);
            printf("print vss found %s\n", m->name);
            if (strncmp(m->name, VSS_SD_NAME, strlen(VSS_SD_NAME)) == 0) {
                debug("vss found");
                *dev_num = dev;
                return m;
            }
        }
    }

    return NULL;
}

int xarina_hw_vss_sd_read(ulong *dst_addr, const ulong size)
{
    if (size == 0) {
       debug("size 0, do nothing\n");
       return 0;
    }

    int dev = 0;
    struct mmc* m = find_xarina_sd(&dev);
    if (m == NULL) {
        return 1;
    }

    mmc_init(m);

    const ulong cnt = (size / 512) + 1;
    const ulong n = m->block_dev.block_read(dev, 1, cnt, (void*)dst_addr); // mmc_bread

    flush_cache((unsigned long)dst_addr, cnt * 512);

    return (n == cnt) ? 0 : 1;
}

int xarina_hw_vss_sd_write(const ulong *src_addr, const ulong size)
{
    if (size == 0) {
       debug("size 0, do nothing\n");
       return 0;
    }

    int dev = 0;
    struct mmc* m = find_xarina_sd(&dev);
    if (m == NULL) {
        return 1;
    }

    if(m->write_bl_len == 0) {
       debug("m->write_bl_len invalid\n");
       return 1;
    }

    mmc_init(m);

    const ulong cnt = (size / m->write_bl_len) + 1;
    const ulong n = m->block_dev.block_write(dev, 1, cnt, (void*)src_addr); // mmc_bwrite

    return (n == cnt) ? 0 : 1;
}
#endif /* defined(CONFIG_ARCH_XARINA) */
