/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <config.h>
#include <asm/arch/xarina.h>
#include "init_i2c_audio.h"
#include "corefw_comm_param.h"
#include "corefw_boot_param.h"
#include "i2c.h"

#define CHIP_ID_AK4645 0x12
#define CHIP_ID_AK4953 0x13

static int audio_dev_id = I2C_AUDIO_DEV_NONE;

static void init_i2c_audio_ak4645()
{
    int ret = 0;
    uint i = 0;
    uchar data[] = {
        0x4d, 0x00, 0x10, 0x00, 0x03, 0x00, 0x00, 0x00,
            0xe1, 0x91, 0x18, 0x00, 0x91, 0x18, 0x11, 0x08,
    };
    uint len = sizeof(data) / sizeof(data[0]);

    for (i = 0; i < len; i++) {
        ret = i2c_write(CHIP_ID_AK4645, i, 1, &data[i], 1);
        if (ret != 0) {
            puts ("failed to initialize audio device AK4645. disable audio feature.\n");
            corefw_param_audio_use_set(0);
            return;
        }
    }

    audio_dev_id = I2C_AUDIO_DEV_AK4645;
    corefw_param_audio_i2s_mode_set(SC_SYS_AUDIO_I2S_MODE_MASTER);
    corefw_param_audio_i2s_bck_set(SC_SYS_AUDIO_I2S_BCK_64FS);
    corefw_param_audio_use_set(1);

    puts ("AK4645 initialized.\n");
}

static void init_i2c_audio_ak4953()
{
    int ret = 0;
    uint i = 0;

    struct init_sequence {
        uint  index;
        uchar data;
    } seq[] = {
        { 5, 0x02 },
        { 6, 0x02 },
        { 0, 0x40 }
    };
    uchar data = 0x01;

    uint len = sizeof(seq) / sizeof(seq[0]);

    for (i = 0; i < len; i++) {
        ret = i2c_write(CHIP_ID_AK4953, seq[i].index, 1, &seq[i].data, 1);
        if ( ret != 0 ) {
            puts ("failed to initialize audio device AK4953. disable audio feature.\n");
            corefw_param_audio_use_set(0);
            return;
        }
    }
    udelay(5 * 1000);
    ret = i2c_write(CHIP_ID_AK4953, 0x01, 1, &data, 1);
    if ( ret != 0 ) {
        puts ("failed to initialize audio device AK4953. disable audio feature.\n");
        corefw_param_audio_use_set(0);
        return;
    }

    audio_dev_id = I2C_AUDIO_DEV_AK4953;
    corefw_param_audio_i2s_mode_set(SC_SYS_AUDIO_I2S_MODE_MASTER);
    corefw_param_audio_i2s_bck_set(SC_SYS_AUDIO_I2S_BCK_32FS);
    corefw_param_audio_use_set(1);

    puts ("AK4953 initialized.\n");
}

void init_i2c_audio()
{
    unsigned int bus_id = 0;
    int ret = 0;

    ret = i2c_set_bus_num(bus_id);
    if (ret) {
        error("i2c_set_bus_num");
        return;
    }

    if (!i2c_probe(CHIP_ID_AK4645)) {
        init_i2c_audio_ak4645();
    }
    else {
        i2c_reg_write(CHIP_ID_AK4953, 0x00, 0x00);

        if (!i2c_probe(CHIP_ID_AK4953)) {
            init_i2c_audio_ak4953();
        }
        else {
            puts ("detect no audio device. disable audio feature.\n");
            corefw_param_audio_use_set(0);
        }
    }
}

int get_i2c_audio_dev()
{
    return audio_dev_id;
}

int is_i2c_audio_clock_master()
{
    return audio_dev_id != I2C_AUDIO_DEV_AK4645;
}
