/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <config.h>
#include <asm/arch/xarina.h>
#include "corefw_boot_param.h"
#include "corefw_comm_param.h"
#include "cam_comm_param.h"
#include "cam_set_param.h"
#include "hw_reg_ctrl.h"
#include "gpio_vss.h"
#include "init_vss.h"
#include "init_i2c_audio.h"
#include "i2c.h"
#include "cmd_corefw.h"

DECLARE_GLOBAL_DATA_PTR;

#define OFF 0
#define ON 1
#define NTSC 1
#define PAL 0
#define SLOC_VIDEO_RESET 1

enum Gpio_A_Def
{
    FACTORY_RESET=0,
    CAMAFVD,
    LDAF_RESET,
    NTSC_PAL,
    POWER_LED,
    NET_LED_GREEN,
    NET_LED_ORANGE,
    SELF_RESET,
};

enum Gpio_F_Def
{
    SD_POW_CNT=5,
    ETH_PHY_RESET_CNT,
    VIDEO_MODE_CNT,
};

static void msleep(int cunt)
{
    int i;
    for (i = 0; i<cunt; i++) {
        udelay(1000);
    }
}

static void init_i2c_bus_speed(void)
{
    int ret = 0;
    ret = i2c_set_bus_num(AUDIO_CODEC_I2C_DEV);
    if (ret) {
        printf("Failure changing bus number (%d)\n", ret);
    }
    else {
        ret = i2c_set_bus_speed(CONFIG_SYS_I2C_SPEED);
        if (ret) {
            printf("Failure changing bus speed (%d)\n", ret);
        }
    }

    return;
}

static void init_ram_size(void)
{
    if (gd->bd->bi_dram[0].size == PHYS_SDRAM_1_SIZE_2G) {
        setenv("mem_size","128M");
        setenv("kernel_path","uImage2");
        setenv("initramfs_addr","69000000");
    } else {
        setenv("mem_size","64M");
        setenv("kernel_path","uImage");
        setenv("initramfs_addr","65000000");
    }
}

static void init_video_encoder_hard(void)
{
    if (corefw_param_tvout_type_get() != SC_SYS_VIDEO_VOUT_FMT_OFF) {
        if (!(SYSREG1_RIN(SYSREG1_PORT_MON) & SYSREG1_BOARD_CONFIG_NTSC_PAL)) {
            setenv("video_signal","PAL");
            gpio_write(GPIO_F_BASE, VIDEO_MODE_CNT, PAL);
            corefw_param_tvout_type_set(SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_FULL);
        } else {
            setenv("video_signal", "NTSC");
            gpio_write(GPIO_F_BASE, VIDEO_MODE_CNT, NTSC);
            corefw_param_tvout_type_set(SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_FULL);
        }
    } else {
        setenv("video_signal", "NTSC");
        #if 1 /* debug for C-model  */
        if (!(SYSREG1_RIN(SYSREG1_PORT_MON) & SYSREG1_BOARD_CONFIG_NTSC_PAL)) {
            setenv("video_signal","PAL");
            gpio_write(GPIO_F_BASE, VIDEO_MODE_CNT, PAL);
            corefw_param_tvout_type_set(SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_59_54);
        } else {
            setenv("video_signal", "NTSC");
            gpio_write(GPIO_F_BASE, VIDEO_MODE_CNT, NTSC);
            corefw_param_tvout_type_set(SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_PAR_10_11);
        }
        #endif
    }
}

static void init_video_encoder_soft(void)
{
    /* TODO: get from frame rate and configure actual video mode */
    setenv("video_signal", "NTSC");
    gpio_write(GPIO_F_BASE, VIDEO_MODE_CNT, NTSC);
    corefw_param_tvout_type_set(SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_FULL);
}

static void init_video_encoder(void)
{
//    setenv("video_signal", "UNUSED");
    gpio_write(GPIO_F_BASE, VIDEO_MODE_CNT, NTSC);
    corefw_param_tvout_type_set(SC_SYS_VIDEO_VOUT_FMT_UNUSED);

    if (cam_param_senssystem_ddr_get() & CAM_PARAM_SENSSYS_VIDEOIN_MASK) {
//        init_video_encoder_soft();
        setenv("video_signal", "NTSC");
    } else {
//        init_video_encoder_hard();
        if (!(SYSREG1_RIN(SYSREG1_PORT_MON) & SYSREG1_BOARD_CONFIG_NTSC_PAL)) {
            setenv("video_signal","PAL");
        } else {
            setenv("video_signal", "NTSC");
        }
    }
}

void init_video_out_device_vss(void)
{
    /* video_reset */
    gpio_write(GPIO_E_BASE, SLOC_VIDEO_RESET, OFF);
    msleep(100);
    gpio_write(GPIO_E_BASE, SLOC_VIDEO_RESET, ON);
}

void init_vss(void)
{
    gpio_init();

    /* LED on */
    gpio_write(GPIO_A_BASE, POWER_LED, ON);
    gpio_write(GPIO_A_BASE, NET_LED_GREEN, ON);

    /* isfactoryreset */
    if (!(SYSREG1_RIN(SYSREG1_PORT_MON) & SYSREG1_BOARD_CONFIG_FACTORY_RESET)) {
        setenv("factory_reset", "on");
        gpio_write(GPIO_A_BASE, NET_LED_GREEN, OFF);
        gpio_write(GPIO_A_BASE, NET_LED_ORANGE, ON);
    }

    /* NTSC or PAL */
    init_video_encoder();

    /* SD card power on */
    gpio_write(GPIO_F_BASE, SD_POW_CNT, ON);

    /* phy_reset */
    gpio_write(GPIO_F_BASE, ETH_PHY_RESET_CNT, OFF);
    msleep(100);
    gpio_write(GPIO_F_BASE, ETH_PHY_RESET_CNT, ON);

    init_i2c_bus_speed();

    init_i2c_audio();

    init_ram_size();
}

