/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <common.h>
#include <command.h>
#include "cmd_menu.h"
#include "menu_top.h"
#include "menu_input.h"
#include "corefw_boot_param.h"
#include "corefw_comm_param.h"
#include "cam_set_param.h"
#include "cam_comm_param.h"
#include "boot_param.h"
#include "xarina_bool.h"
#include "vin.h"

#define INPUT_CHANGE_MAX_PAGE	(16)
#define INPUT_CHANGE_MAX_ITEM	(9)
extern char console_buffer[];	/* readline buffer */
extern char cmd_buff[];

struct menu_item item_in_cgm[INPUT_CHANGE_MAX_ITEM] ;
static int menu_input_page;
static char menu_input_title[50];

/**************************************/
/* menu item code                     */
/**************************************/
static void print_input_info(void)
{
	char tmp[256];
	int ret;
	unsigned char sysmiris;
	
	ret = getenv_f(CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM, tmp, 256);

	if (ret == -1) {
		sprintf(tmp, "none."); 
	}
	printf("[Current setting]\n");
	printf("  File: %s\n", tmp);

	if (ret == -1) {
		return;
	}

	if (cam_param_senssystem_ddr_get() & CAM_PARAM_SENSSYS_VIDEOIN_MASK) {
		sprintf(tmp, "Video");
	} else {
		sprintf(tmp, "Sensor");
	}

	printf("  Input type: %s\n", tmp);

	if ((cam_param_senssystem_ddr_get() & CAM_PARAM_SENSSYS_VIDEOIN_MASK) == 0) {
		sysmiris = cam_param_sysmiris_ddr_get();
		if (sysmiris == CAM_PARAM_MIRIS_SYSMIRIS_NONE) {
			sprintf(tmp, "Off");
		} else if (sysmiris == CAM_PARAM_MIRIS_SYSMIRIS_YES) {
			sprintf(tmp, "On");
		} else {
			sprintf(tmp, "Unknown");
		}
		printf("  Miris: %s\n", tmp);
	}

}

static int menu_in_out(struct menu_info *p_info, struct menu_proc *p_proc)
{
	print_input_info();

	return 0;
}

static int menu_in_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	menu_input_page = 0;
	sprintf(menu_input_title, "input change page %02d / %02d", 
		menu_input_page + 1, INPUT_CHANGE_MAX_PAGE);
	return 0;
}


/*------------------------------------*/
/* main                               */
/*------------------------------------*/
/*------------------------------------*/
/* factory default                    */
/*------------------------------------*/
static int menu_in_def_restore(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	ret = boot_param_camfw_param_restore();

	if (ret == FALSE) {
		p_item->next = &info_input;
		return 0;
	}

	if (cam_param_senssystem_ddr_get() & CAM_PARAM_SENSSYS_VIDEOIN_MASK) {
		p_item->next = &info_input;
	} else {
		p_item->next = &info_in_cg_ms;
	}
	
	return 0;
}


/*------------------------------------*/
/* input_change                       */
/*------------------------------------*/

static int menu_in_cgm_next(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	menu_input_page++;
	
	if (menu_input_page >= INPUT_CHANGE_MAX_PAGE) {
		menu_input_page = 0;
	}
	
	sprintf(menu_input_title, "input change page %02d / %02d", 
		menu_input_page + 1, INPUT_CHANGE_MAX_PAGE);
	return 0;
}

static int menu_in_cgm_back(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	
	if (menu_input_page == 0) {
		menu_input_page = INPUT_CHANGE_MAX_PAGE - 1;
	} else {
		menu_input_page--;
	}
	
	sprintf(menu_input_title, "input change page %02d / %02d", 
		menu_input_page + 1, INPUT_CHANGE_MAX_PAGE);
	return 0;
}

static int menu_in_cgm_select(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	char env[64];
	char file_name[256];
	int ret;
	struct s_param_data_image_header *p_header = 
		(struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;

	sprintf(env, "menu_input_%02d_%02d_file", menu_input_page + 1, p_proc->select_num);
	
	/* load saved parameter */
	ret = boot_param_load_file_env(env, CONFIG_SYS_LOAD_ADDR, 0);

	printf("load Cam FW set param ... ");

	if (ret != TRUE) {
		printf("not found!\n");
		return 0;
	}
			
	if (boot_param_data_expansion(p_header, CAM_SET_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done!\n");
	} else {
		printf("invalid file!\n");
		return 0;
	}

	ret = getenv_f(env, file_name, 256);
	if (ret == -1) {
		return 0;
	}
	sprintf(cmd_buff, "setenv %s %s", 
			CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM, file_name);
	run_command(cmd_buff, 0);
	
	if (cam_param_senssystem_ddr_get() & CAM_PARAM_SENSSYS_VIDEOIN_MASK) {
		item_in_cgm[p_proc->select_num - 1].next = &info_input;
	} else {
		item_in_cgm[p_proc->select_num - 1].next = &info_in_cg_ms;
	}

	return 0;
}

static int menu_in_cgm_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	char env[30];
	char file_name[256];
	char cur_file_name[256];
	int i;
	int ret0, ret1;
	
	for (i = 0; i < INPUT_CHANGE_MAX_ITEM; i++) {
		sprintf(env, "menu_input_%02d_%02d_str", menu_input_page + 1, i + 1);
		item_in_cgm[i].p_str = getenv(env);
		if (item_in_cgm[i].p_str != NULL) {
			p_proc->b_disable[i] = FALSE;
		} else {
			p_proc->b_disable[i] = TRUE;
		}
		
		sprintf(env, "menu_input_%02d_%02d_file", menu_input_page + 1, i + 1);
		ret0 = getenv_f(env, file_name, 256);
		ret1 = getenv_f(CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM, cur_file_name, 256);
		if((ret0 != -1) && (ret1 != -1) && !strncmp(file_name, cur_file_name, 256)) {
			p_proc->b_choice[i] = TRUE;
		} else {
			p_proc->b_choice[i] = FALSE;
		}

		item_in_cgm[i].p_func = menu_in_cgm_select;
		item_in_cgm[i].next = &info_in_cgm;
	}

	return 0;
}

/*------------------------------------*/
/* MIRIS select                       */
/*------------------------------------*/
static int menu_in_cg_ms_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	unsigned char sysmiris;
	
	sysmiris = cam_param_sysmiris_ddr_get();
	
	if (sysmiris == CAM_PARAM_MIRIS_SYSMIRIS_NONE) {
		p_proc->b_choice[0] = TRUE;
	} else if (sysmiris == CAM_PARAM_MIRIS_SYSMIRIS_YES) {
		p_proc->b_choice[1] = TRUE;
	} else {
		p_proc->b_choice[0] = FALSE;
		p_proc->b_choice[1] = FALSE;
	}
	
	return 0;
}
static int menu_in_cg_ms(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	cam_param_sysmiris_ddr_set((unsigned char)p_item->info);

	return 0;
}

/*------------------------------------*/
/* video-in dev                       */
/*------------------------------------*/
static int menu_in_vid_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	char env[256];
	char str[4];
	int ret;

	ret = getenv_f(CONFIG_XARINA_ENV_VIDEOIN_DEV, env, 256);
	
	if (ret == -1) {
		sprintf(str,"%x", VIN_DEV_DEFAULT);
		setenv(CONFIG_XARINA_ENV_VIDEOIN_DEV, str);
		set_vin_dev(VIN_DEV_DEFAULT);
		p_proc->b_choice[0] = TRUE;
		return 0;
	}

	switch(simple_strtoul(env, NULL, 16)) {
	case VIN_DEV_NONE:
		p_proc->b_choice[0] = TRUE;
		break;
	case VIN_DEV_VDEC:
		p_proc->b_choice[1] = TRUE;
		break;
	default:
		/* do nothing */
		break;
	}

	return 0;
}

static int menu_in_vid(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	char str[10];

	sprintf(str,"%lx", p_item->info);
	setenv(CONFIG_XARINA_ENV_VIDEOIN_DEV, str);
	set_vin_dev((int)(p_item->info));

	return 0;
}

/**************************************/
/* menu table                         */
/**************************************/


/*------------------------------------*/
/* factory default                    */
/*------------------------------------*/
struct menu_item item_in_def_restore = {
	"restore", menu_in_def_restore, &info_input, 0, 0,
};
struct menu_item item_in_def_quit = {
	"quit", NULL, &info_input, 0, 0,
};

struct menu_info info_in_default= {
	"factory default", NULL, NULL,
	NULL,
	1,
	{	&item_in_def_restore,
	},
	&item_in_def_quit, NULL,
	0,
};


/*------------------------------------*/
/* MIRIS select                       */
/*------------------------------------*/
struct menu_item item_in_cg_ms_off = {
	"off", menu_in_cg_ms, &info_input, 0, CAM_PARAM_MIRIS_SYSMIRIS_NONE,
};
struct menu_item item_in_cg_ms_on = {
	"on", menu_in_cg_ms, &info_input, 0, CAM_PARAM_MIRIS_SYSMIRIS_YES,
};
struct menu_info info_in_cg_ms = {
	"MIRIS select",
	menu_in_cg_ms_pre, NULL,
	NULL,
	2,
	{&item_in_cg_ms_off, &item_in_cg_ms_on},
	NULL, NULL,
	0,
};

/*------------------------------------*/
/* input change 		      */
/*------------------------------------*/

struct menu_item item_in_cgm_quit = {
	"quit", NULL, &info_input, 0, 0,
};
struct menu_item item_in_cgm_next = {
	"next >>", menu_in_cgm_next, &info_in_cgm, 0, 0,
};
struct menu_item item_in_cgm_back = {
	"<< back", menu_in_cgm_back, &info_in_cgm, 0, 0,
};
struct menu_info info_in_cgm = {
	menu_input_title,
	menu_in_cgm_pre, NULL,
	"**NOTE**\nIf you select any input mode, \nthe setting such as miris or video range will be overwritten.",
	11,
	{	&item_in_cgm[0], &item_in_cgm[1],
		&item_in_cgm[2], &item_in_cgm[3],
		&item_in_cgm[4], &item_in_cgm[5],
		&item_in_cgm[6], &item_in_cgm[7], &item_in_cgm[8],
		&item_in_cgm_next, &item_in_cgm_back
	},
	&item_in_cgm_quit, NULL,
	0,
};

/*------------------------------------*/
/* save                               */
/*------------------------------------*/
static int menu_in_save(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	run_command("saveenv", 0);
	return 0;
}

/*------------------------------------*/
/* input change by file name          */
/*------------------------------------*/
static int menu_in_cgf(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	struct s_param_data_image_header *p_header = 
		(struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;
	
	ret = readline("FILENAME: ");
	if (ret <= 0) {
		return FALSE;
	}

	ret = boot_param_load_file_name(console_buffer, CONFIG_SYS_LOAD_ADDR, 0);

	printf("load Cam FW set param ... ");

	if (ret == 0) {
		printf("not found!\n");
		return FALSE;
	}
	if (boot_param_data_expansion(p_header, CAM_SET_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done!\n");
		setenv(CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM, console_buffer);
	} else {
		printf("invalid file!\n");
		p_item->next = &info_input;
		return FALSE;
	}
	
	if (cam_param_senssystem_ddr_get() & CAM_PARAM_SENSSYS_VIDEOIN_MASK) {
		p_item->next = &info_input;
	} else {
		p_item->next = &info_in_cg_ms;
	}
		

	return 0;
}

/*------------------------------------*/
/* video-in device	              */
/*------------------------------------*/
struct menu_item item_vid_none = {
	"none", menu_in_vid, &info_input, 0, VIN_DEV_NONE,
};
struct menu_item item_vid_vdec = {
	"VIDEO DECODER", menu_in_vid, &info_input, 0, VIN_DEV_VDEC,
};
struct menu_item item_vid_quit = {
	"quit", NULL, &info_input, 0, 0,
};
struct menu_info info_in_vid = {
	"video-in device",
	menu_in_vid_pre, NULL,
	NULL,
	2,
	{&item_vid_none, &item_vid_vdec},
	&item_vid_quit, NULL,
	0,
};


/*------------------------------------*/
/* main                               */
/*------------------------------------*/
struct menu_item item_in_default = {
	"factory default", NULL, &info_in_default, 0, 0,
};
struct menu_item item_in_change_menu = {
	"input change menu", NULL, &info_in_cgm, 0, 0,
};
struct menu_item item_in_change_file = {
	"input change by file name", menu_in_cgf, NULL, 0, 0,
};
struct menu_item item_in_vid = {
	"video-in device", NULL, &info_in_vid, 0, 0,
};
struct menu_item item_in_save = {
	"save", menu_in_save, NULL, 0, 0,
};
struct menu_item item_in_quit = {
	"quit", NULL, &info_top, 0, 0,
};

struct menu_info info_input = {
	"input setting",
	menu_in_pre, menu_in_out,
	NULL,
	5,
	{	&item_in_default, &item_in_change_menu, 
		&item_in_change_file, &item_in_vid, &item_in_save
	},
	&item_in_quit, NULL,
	0,
};


