/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <common.h>
#include <command.h>
#include "cmd_menu.h"
#include "menu_top.h"
#include "menu_output.h"
#include "command_exec.h"
#include "corefw_boot_param.h"
#include "corefw_comm_param.h"
#include "boot_param.h"
#include "xarina_bool.h"
#include "menu_update.h"
#include "vout.h"


extern char console_buffer[];	/* readline buffer */
extern char cmd_buff[CONFIG_SYS_CBSIZE];
extern int data_down_load(unsigned int type, unsigned long addr);

int (*check_dev_support[])(unsigned char type) = {
	check_venc_support,
	check_hdmi_support
};

static unsigned char dev_default_tbl[] = {
	VENC_DEFAULT_TYPE,
	HDMI_DEFAULT_TYPE
};

static unsigned char vout_id = VOUT_ID_720_486_5994I;


/**************************************/
/* menu item code                     */
/**************************************/
/*------------------------------------*/
/* output menu                        */
/*------------------------------------*/
static int menu_output_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{

	if (boot_param_check_load_corefw_param() == FALSE) {
		p_proc->b_disable[1] = TRUE;	
		p_proc->b_disable[2] = TRUE;	
		p_proc->b_disable[3] = TRUE;	
	} else {
		p_proc->b_disable[1] = FALSE;
		p_proc->b_disable[2] = FALSE;	
		p_proc->b_disable[3] = FALSE;	
	}

	if (corefw_param_capability_check_total()) {
		corefw_param_tvout_type_set(SC_SYS_VIDEO_VOUT_FMT_OFF);
	}


	switch (get_vout_dev()) {
	case VOUT_DEV_VENC:
		if (check_venc_support(corefw_param_tvout_type_get()) == FALSE) {
			corefw_param_tvout_type_set(VENC_DEFAULT_TYPE);
		}
		break;
	case VOUT_DEV_HDMI:
		if (check_hdmi_support(corefw_param_tvout_type_get()) == FALSE) {
			corefw_param_tvout_type_set(HDMI_DEFAULT_TYPE);
		}
		break;
	default:
		/* do nothing */
		break;
	}



	return 0;
}
/*------------------------------------*/
/* factory default                    */
/*------------------------------------*/
static int menu_out_def_restore(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	boot_param_corefw_param_restore();
	return 0;
}
/*------------------------------------*/
/* resolution select                  */
/*------------------------------------*/
static int menu_out_cg_rs_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	unsigned long type;

	type = corefw_param_tvout_type_get();
	switch(type) {
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5:
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E: 
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_00: 
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_01: 
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5: 
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E: 
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_02:
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_03:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_PAR_10_11:
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_CEA861E_PAR_10_11:
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_04:
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_05:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_PAR_10_11:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E_PAR_10_11:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_59_54:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_PAR_59_54:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_12_11:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_PAR_12_11:
		p_proc->b_choice[0] = TRUE;
		break;
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_CEA861E_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_FULL: 
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_06: 
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_07: 
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_FULL: 
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E_FULL: 
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_08:
	case SC_SYS_VIDEO_VOUT_FMT_RESERVED_09:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59P_CEA861E_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E_FULL:
		p_proc->b_choice[1] = TRUE;
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_SMPTE296M:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_SMPTE296M:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_CEA861E:  
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E:  
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_SMPTE296M:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_CEA861E:  
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_CEA861E: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59P_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59P_CEA861E: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_CEA861E: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_CEA861E: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_CEA861E: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_CEA861E: 
		p_proc->b_choice[2] = TRUE;
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_SMPTE296M_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_CEA861E_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_SMPTE296M_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_CEA861E_CROP_FULL:  
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E_CROP_FULL:  
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_SMPTE296M_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_CEA861E_CROP_FULL:  
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_BT1120_7_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_CEA861E_CROP_FULL: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59P_BT1120_7_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59P_CEA861E_CROP_FULL: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E_CROP_FULL: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E_CROP_FULL: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_BT1120_7_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_CEA861E_CROP_FULL: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E_CROP_FULL: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_CEA861E_CROP_FULL: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_CEA861E_CROP_FULL: 
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7_CROP_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_CEA861E_CROP_FULL: 
		p_proc->b_choice[3] = TRUE;
		break;
	default:
		p_proc->b_choice[4] = TRUE;
		break;

	}

	/* corefw limit check */
	p_proc->b_disable[2] = p_proc->b_disable[3] = corefw_param_vout_hd_limit_check();
	


	return 0;
}
/*------------------------------------*/
/* resolution select SD-HD common pre */
/*------------------------------------*/

static int check_dev_support_id(unsigned long id)
{
	int dev, ret = FALSE;
	dev = get_vout_dev();


	if (check_dev_support[dev](corefw_param_vout_fmt_get(id, VOUT_FMT_SYNC_HV)) ||
		check_dev_support[dev](corefw_param_vout_fmt_get(id, VOUT_FMT_SYNC_TRC))) {
		ret = TRUE;
	}

	return ret;

}

static int menu_out_cg_rs_sdhd_common_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	unsigned char type, id;
	int i;

	type = corefw_param_tvout_type_get();
	id = corefw_param_vout_id_get(type);

	for (i = 0; i < p_info->item_num; i++) {
		/* selected item check */
		if (id == p_info->p_item[i]->info) {
			p_proc->b_choice[i] = TRUE;
		}

		/* device support check */
		if (check_dev_support_id(p_info->p_item[i]->info) == FALSE) {
			p_proc->b_disable[i] = TRUE;
		}

		/* corefw capability check */
		if (corefw_param_vout_capability_check(p_info->p_item[i]->info) == FALSE) {
			p_proc->b_disable[i] = TRUE;
		}

	}

	return 0;
}


/*------------------------------------*/
/* video-out dev pre                 */
/*------------------------------------*/
static int menu_out_vod_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	char *env;
	char str[4];
	
	env = getenv(CONFIG_XARINA_ENV_VIDEOOUT_DEV);
	
	if (!env) {
		sprintf(str,"%x", VOUT_DEV_DEFAULT);
		setenv(CONFIG_XARINA_ENV_VIDEOOUT_DEV, str);
		set_vout_dev(VOUT_DEV_DEFAULT);
		p_proc->b_choice[0] = TRUE;
		return 0;
	}

	switch(simple_strtoul(env, NULL, 16)) {
	case VOUT_DEV_VENC:
		p_proc->b_choice[0] = TRUE;
		break;
	case VOUT_DEV_HDMI:
		p_proc->b_choice[1] = TRUE;
		break;
	default:
		/* do nothing */
		break;
	}
	
	return 0;
}

/*------------------------------------*/
/* sync type select pre                 */
/*------------------------------------*/
static int menu_out_cg_st_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	unsigned char type, sync;
	int dev, i;

	dev = get_vout_dev();

	type = corefw_param_tvout_type_get();
	sync = corefw_param_vout_sync_type_get(type);

	if (sync < VOUT_FMT_SYNC_NUM) {
		p_proc->b_choice[sync] = TRUE;
	}

	for (i = 0; i < p_info->item_num; i++) {
		if (check_dev_support[dev](corefw_param_vout_fmt_get(vout_id, p_info->p_item[i]->info)) == FALSE) {
			p_proc->b_disable[i] = TRUE;
		}
	}
		
	return 0;
}

/*------------------------------------*/
/* output change                      */
/*------------------------------------*/
static int menu_out_cg(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	if (p_item->info == VOUT_ID_OFF) {
		corefw_param_tvout_type_set(SC_SYS_VIDEO_VOUT_FMT_OFF);
	} else {
		vout_id = p_item->info;
	}
	return 0;
}

/*------------------------------------*/
/* sync type select                   */
/*------------------------------------*/
static int menu_out_cg_st(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	unsigned char fmt;

	fmt = corefw_param_vout_fmt_get(vout_id, (unsigned char)p_item->info);
	corefw_param_tvout_type_set(fmt);

	return 0;
}

/*------------------------------------*/
/* interlace setting                  */
/*------------------------------------*/
static int menu_out_int_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	unsigned long data;
	data = corefw_param_osd_mask_output_status_get();

	if (data & SC_SYS_VIDEO_OUTPUT_VOUT_PROG) {
		p_proc->b_choice[1] = TRUE;
	} else {
		p_proc->b_choice[0] = TRUE;
	}

	return 0;
}

static int menu_out_int_nlock(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	unsigned long data;

	data = corefw_param_osd_mask_output_status_get();
	corefw_param_osd_mask_output_status_set(data & ~SC_SYS_VIDEO_OUTPUT_VOUT_PROG);

	return 0;
}
static int menu_out_int_lock(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	unsigned long data;

	data = corefw_param_osd_mask_output_status_get();
	corefw_param_osd_mask_output_status_set(data | SC_SYS_VIDEO_OUTPUT_VOUT_PROG);

	return 0;
}

/*------------------------------------*/
/* parameter load                     */
/*------------------------------------*/
static int menu_out_ld_param(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	struct s_param_data_image_header *p_header
		= (struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;
		
	/* not create filesize variable when tftp download size 0 file */
	run_command("setenv filesize", 0);	/* delete filesize */

	ret = data_down_load(p_item->info, (unsigned long)p_header);
	if (ret <= 0 || getenv("filesize") == NULL) {
		return 0;
	}

	printf("load CoreFW comm param ... ");
	if (boot_param_data_expansion(p_header, COREFW_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done!\n");
	} else {
		printf("invalid file!\n");
		return 0;
	}
	
	return ret;
}
/*------------------------------------*/
/* video-out dev                      */
/*------------------------------------*/
static int menu_out_vod(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	char str[16];

	sprintf(str,"%lx", p_item->info);
	setenv(CONFIG_XARINA_ENV_VIDEOOUT_DEV, str);
	set_vout_dev((int)(p_item->info));

	if (p_item->info < VOUT_DEV_NUM) {
		if (check_dev_support[p_item->info](corefw_param_tvout_type_get()) == FALSE) {
			printf("video out type is changed.\n");
			corefw_param_tvout_type_set(dev_default_tbl[p_item->info]);
		}
	}

	return 0;
}
static int menu_out_vod_save(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	run_command("saveenv", 0);
	return 0;
}
/**************************************/
/* menu table                         */
/**************************************/
/*------------------------------------*/
/* select sync type                   */
/*------------------------------------*/
struct menu_item item_out_cg_st_trc = {
	"TRC", menu_out_cg_st, &info_output, 0, VOUT_FMT_SYNC_TRC,
};
struct menu_item item_out_cg_st_hv = {
	"HV", menu_out_cg_st, &info_output, 0, VOUT_FMT_SYNC_HV,
};

struct menu_info info_out_cg_sync_type = {
	"sync type select",
	menu_out_cg_st_pre, NULL,
	NULL,
	2,
	{&item_out_cg_st_trc, &item_out_cg_st_hv},
	NULL, NULL,
	0,
};

/*------------------------------------*/
/* factory default                    */
/*------------------------------------*/
struct menu_item item_out_def_restore = {
	"restore", menu_out_def_restore, &info_output, 0, 0,
};
struct menu_item item_out_def_quit = {
	"quit", NULL, &info_output, 0, 0,
};

struct menu_info info_out_default = {
	"default setting",
	NULL, NULL,
	NULL,
	1,
	{	&item_out_def_restore
	},
	&item_out_def_quit, NULL,
	0,
};

/*------------------------------------*/
/* select resolution HD               */
/*------------------------------------*/
struct menu_item item_out_cg_1280x720_60p = {
	"1280x720p@60Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1280_720_60P,
};
struct menu_item item_out_cg_1280x720_59p = {
	"1280x720p@59.94Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1280_720_5994P,
};
struct menu_item item_out_cg_1280x720_30p = {
	"1280x720p@30Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1280_720_30P,
};
struct menu_item item_out_cg_1280x720_29p = {
	"1280x720p@29.97Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1280_720_2997P,
};
struct menu_item item_out_cg_1280x720_50p = {
	"1280x720p@50Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1280_720_50P,
};
struct menu_item item_out_cg_1280x720_25p = {
	"1280x720p@25Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1280_720_25P,
};
struct menu_item item_out_cg_1920x1080_60p = {
	"1920x1080p@60Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_60P,
};
struct menu_item item_out_cg_1920x1080_59i = {
	"1920x1080i@59.94Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_5994I,
};
struct menu_item item_out_cg_1920x1080_30p = {
	"1920x1080p@30Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_30P,
};
struct menu_item item_out_cg_1920x1080_29p = {
	"1920x1080p@29.97Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_2997P,
};
struct menu_item item_out_cg_1920x1080_50p = {
	"1920x1080p@50Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_50P,
};
struct menu_item item_out_cg_1920x1080_25p = {
	"1920x1080p@25Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_25P,
};
struct menu_item item_out_cg_1920x1080_60i = {
	"1920x1080i@60Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_60I,
};
struct menu_item item_out_cg_1920x1080_50i = {
	"1920x1080i@50Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_50I,
};
struct menu_item item_out_cg_rs_hd_quit = {
	"quit", NULL, &info_out_cg_rs, 0, 0,
};

struct menu_info info_out_cg_rs_hd = {
	"output change (HD)",
	menu_out_cg_rs_sdhd_common_pre, NULL,
	NULL,
	14,
	{	&item_out_cg_1280x720_60p,	&item_out_cg_1280x720_59p,
		&item_out_cg_1280x720_30p, 	&item_out_cg_1280x720_29p,
		&item_out_cg_1280x720_50p,	&item_out_cg_1280x720_25p, 
		&item_out_cg_1920x1080_60p,	&item_out_cg_1920x1080_59i,
		&item_out_cg_1920x1080_30p, 	&item_out_cg_1920x1080_29p,
		&item_out_cg_1920x1080_50p,	&item_out_cg_1920x1080_25p, 	
		&item_out_cg_1920x1080_60i,	&item_out_cg_1920x1080_50i
	},
	&item_out_cg_rs_hd_quit, NULL,
	0,
};

/*------------------------------------*/
/* select resolution HD full screen   */
/*------------------------------------*/
struct menu_item item_out_cg_1280x720_60p_cf = {
	"1280x720p@60Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1280_720_60P_CROP_FULL,
};
struct menu_item item_out_cg_1280x720_59p_cf = {
	"1280x720p@59.94Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1280_720_5994P_CROP_FULL,
};
struct menu_item item_out_cg_1280x720_30p_cf = {
	"1280x720p@30Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1280_720_30P_CROP_FULL,
};
struct menu_item item_out_cg_1280x720_29p_cf = {
	"1280x720p@29.97Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1280_720_2997P_CROP_FULL,
};
struct menu_item item_out_cg_1280x720_50p_cf = {
	"1280x720p@50Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1280_720_50P_CROP_FULL,
};
struct menu_item item_out_cg_1280x720_25p_cf = {
	"1280x720p@25Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1280_720_25P_CROP_FULL,
};
struct menu_item item_out_cg_1920x1080_60p_cf = {
	"1920x1080p@60Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_60P_CROP_FULL,
};
struct menu_item item_out_cg_1920x1080_59i_cf = {
	"1920x1080i@59.94Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_5994I_CROP_FULL,
};
struct menu_item item_out_cg_1920x1080_30p_cf = {
	"1920x1080p@30Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_30P_CROP_FULL,
};
struct menu_item item_out_cg_1920x1080_29p_cf = {
	"1920x1080p@29.97Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_2997P_CROP_FULL,
};
struct menu_item item_out_cg_1920x1080_50p_cf = {
	"1920x1080p@50Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_50P_CROP_FULL,
};
struct menu_item item_out_cg_1920x1080_25p_cf = {
	"1920x1080p@25Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_25P_CROP_FULL,
};
struct menu_item item_out_cg_1920x1080_60i_cf = {
	"1920x1080i@60Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_60I_CROP_FULL,
};
struct menu_item item_out_cg_1920x1080_50i_cf = {
	"1920x1080i@50Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_1920_1080_50I_CROP_FULL,
};
struct menu_item item_out_cg_rs_hd_fs_quit = {
	"quit", NULL, &info_out_cg_rs, 0, 0,
};

struct menu_info info_out_cg_rs_hd_fs = {
	"output change (HD full screen)",
	menu_out_cg_rs_sdhd_common_pre, NULL,
	NULL,
	14,
	{	&item_out_cg_1280x720_60p_cf,	&item_out_cg_1280x720_59p_cf,
		&item_out_cg_1280x720_30p_cf, 	&item_out_cg_1280x720_29p_cf,
		&item_out_cg_1280x720_50p_cf,	&item_out_cg_1280x720_25p_cf, 
		&item_out_cg_1920x1080_60p_cf,	&item_out_cg_1920x1080_59i_cf,
		&item_out_cg_1920x1080_30p_cf, 	&item_out_cg_1920x1080_29p_cf,
		&item_out_cg_1920x1080_50p_cf,	&item_out_cg_1920x1080_25p_cf, 	
		&item_out_cg_1920x1080_60i_cf,	&item_out_cg_1920x1080_50i_cf
	},
	&item_out_cg_rs_hd_fs_quit, NULL,
	0,
};



/*------------------------------------*/
/* select resolution SD               */
/*------------------------------------*/
struct menu_item item_out_cg_720x486_59i = {
	"720x486i@59.94Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_720_486_5994I,
};
struct menu_item item_out_cg_720x576_50i = {
	"720x576i@50Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_720_576_50I,
};
struct menu_item item_out_cg_720x480_59i = {
	"720x480i@59.94Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_720_480_5994I,
};
struct menu_item item_out_cg_720x480_59p = {
	"720x480p@59.94Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_720_480_5994P,
};
struct menu_item item_out_cg_720x576_50p = {
	"720x576p@50Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_720_576_50P,
};
struct menu_item item_out_cg_720x486_59i_10_11 = {
	"720x486i@59.94Hz [10:11]", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_720_486_5994I_PAR_10_11,
};
struct menu_item item_out_cg_720x480_59i_10_11 = {
	"720x480i@59.94Hz [10:11]", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_720_480_5994I_PAR_10_11,
};
struct menu_item item_out_cg_720x576_50i_59_54 = {
	"720x576i@50Hz    [59:54]", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_720_576_50I_PAR_59_54,
};
struct menu_item item_out_cg_720x576_50i_12_11 = {
	"720x576i@50Hz    [12:11]", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_720_576_50I_PAR_12_11,
};

struct menu_item item_out_cg_rs_sd_quit = {
	"quit", NULL, &info_out_cg_rs, 0, 0,
};

struct menu_info info_out_cg_rs_sd = {
	"output change (SD)",
	menu_out_cg_rs_sdhd_common_pre, NULL,
	NULL,
    #if defined(CONFIG_XARINA_VSS)
	9,
	{	&item_out_cg_720x486_59i, &item_out_cg_720x576_50i,
		&item_out_cg_720x480_59i,
		&item_out_cg_720x486_59i_10_11,	&item_out_cg_720x480_59i_10_11,
		&item_out_cg_720x576_50i_59_54,	&item_out_cg_720x576_50i_12_11
	},
    #else /* defined(CONFIG_XARINA_VSS) */
	9,
	{	&item_out_cg_720x486_59i,	&item_out_cg_720x576_50i,
		&item_out_cg_720x480_59i,	&item_out_cg_720x480_59p,
		&item_out_cg_720x576_50p,	&item_out_cg_720x486_59i_10_11,
		&item_out_cg_720x480_59i_10_11, &item_out_cg_720x576_50i_59_54,	
		&item_out_cg_720x576_50i_12_11
	},
    #endif /* defined(CONFIG_XARINA_VSS) */
	&item_out_cg_rs_sd_quit, NULL,
	0,
};


/*------------------------------------*/
/* select resolution SD full screen   */
/*------------------------------------*/
struct menu_item item_out_cg_720x486_59i_f = {
	"720x486i@59.94Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_720_486_5994I_FULL,
};
struct menu_item item_out_cg_720x576_50i_f = {
	"720x576i@50Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_720_576_50I_FULL,
};
struct menu_item item_out_cg_720x480_59i_f = {
	"720x480i@59.94Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_720_480_5994I_FULL,
};
struct menu_item item_out_cg_720x480_59p_f = {
	"720x480p@59.94Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_720_480_5994P_FULL,
};
struct menu_item item_out_cg_720x576_50p_f = {
	"720x576p@50Hz", menu_out_cg, &info_out_cg_sync_type, 0, VOUT_ID_720_576_50P_FULL,
};

struct menu_item item_out_cg_rs_sd_fs_quit = {
	"quit", NULL, &info_out_cg_rs, 0, 0,
};

struct menu_info info_out_cg_rs_sd_fs = {
	"output change (SD full screen)",
	menu_out_cg_rs_sdhd_common_pre, NULL,
	NULL,
	5,
	{	&item_out_cg_720x486_59i_f,	&item_out_cg_720x576_50i_f,
		&item_out_cg_720x480_59i_f,	&item_out_cg_720x480_59p_f,
		&item_out_cg_720x576_50p_f
	},
	&item_out_cg_rs_sd_fs_quit, NULL,
	0,
};


/*------------------------------------*/
/* select resolution                  */
/*------------------------------------*/
struct menu_item item_out_cg_rs_sd = {
	"SD", NULL, &info_out_cg_rs_sd, 0, 0,
};
struct menu_item item_out_cg_rs_sd_fs = {
	"SD full screen", NULL, &info_out_cg_rs_sd_fs, 0, 0,
};
struct menu_item item_out_cg_rs_hd = {
	"HD", NULL, &info_out_cg_rs_hd, 0, 0,
};
struct menu_item item_out_cg_rs_hd_fs = {
	"HD full screen", NULL, &info_out_cg_rs_hd_fs, 0, 0,
};
struct menu_item item_out_cg_rs_stop = {
	"output stop", menu_out_cg, &info_output, 0, VOUT_ID_OFF,
};
struct menu_item item_out_cg_rs_quit = {
	"quit", NULL, &info_output, 0, 0,
};

struct menu_info info_out_cg_rs = {
	"resolution select SD/HD",
	menu_out_cg_rs_pre, NULL,
	NULL,
    #if defined(CONFIG_XARINA_VSS)
	2,
	{	&item_out_cg_rs_sd,
		&item_out_cg_rs_stop
	},
    #else /* defined(CONFIG_XARINA_VSS) */
	5,
	{	&item_out_cg_rs_sd, &item_out_cg_rs_sd_fs,
		&item_out_cg_rs_hd, &item_out_cg_rs_hd_fs,
		&item_out_cg_rs_stop
	},
    #endif /* defined(CONFIG_XARINA_VSS) */
	&item_out_cg_rs_quit, NULL,
	0,
};


/*------------------------------------*/
/* parameter load                     */
/*------------------------------------*/
struct menu_item item_out_ld_xmodem = {
	"load over serial (xmodem)", menu_out_ld_param, &info_output, 0, DOWNLOAD_TYPE_LOADX,
};
struct menu_item item_out_ld_tftp = {
	"load over ether (tftp)", menu_out_ld_param, &info_output, 0, DOWNLOAD_TYPE_TFTP,
};
struct menu_item item_out_ld_quit = {
	"quit", NULL, &info_output, 0, 0,
};

struct menu_info info_out_load = {
	"corefw comm parameter load",
	NULL, NULL,
	NULL,
	2,
	{	&item_out_ld_xmodem, &item_out_ld_tftp
	},
	&item_out_ld_quit, NULL,
	0,
};


/*------------------------------------*/
/* video-out device                   */
/*------------------------------------*/
struct menu_item item_out_vod_venc = {
	"VIDEO ENCODER", menu_out_vod, &info_out_vod, 0, VOUT_DEV_VENC,
};
struct menu_item item_out_vod_hdmi = {
	"HDMI", menu_out_vod, &info_out_vod, 0, VOUT_DEV_HDMI,
};
struct menu_item item_out_vod_save = {
	"save", menu_out_vod_save, &info_output, 0, 0,
};
struct menu_item item_out_vod_quit = {
	"quit", NULL, &info_output, 0, 0,
};

struct menu_info info_out_vod = {
	"video-out device",
	menu_out_vod_pre, NULL,
	NULL,
	3,
	{	&item_out_vod_venc, &item_out_vod_hdmi, &item_out_vod_save
	},
	&item_out_vod_quit, NULL,
	0,
};

/*------------------------------------*/
/* interlace setting                  */
/*------------------------------------*/
struct menu_item item_out_int_nolock = {
	"frame NO lock", menu_out_int_nlock, &info_output, 0, 0,
};
struct menu_item item_out_int_lock = {
	"frame lock", menu_out_int_lock, &info_output, 0, 0,
};
struct menu_item item_out_int_quit = {
	"quit", NULL, &info_output, 0, 0,
};

struct menu_info info_out_int = {
	"interlace setting",
	menu_out_int_pre, NULL,
	"**NOTE**\nThis setting is effective\nonly if output type is interlace and input frame rate is 60/59.94/50fps.",
	2,
	{	&item_out_int_nolock, &item_out_int_lock
	},
	&item_out_int_quit, NULL,
	0,
};



/*------------------------------------*/
/* main                               */
/*------------------------------------*/
struct menu_item item_out_default = {
	"factory default", NULL, &info_out_default, 0, 0,
};
struct menu_item item_out_change = {
	"output change", NULL, &info_out_cg_rs, 0, 0,
};
struct menu_item item_out_vod = {
	"video-out device", NULL, &info_out_vod, 0, 0,
};
struct menu_item item_out_int = {
	"interlace setting", NULL, &info_out_int, 0, 0,
};
struct menu_item item_out_load = {
	"corefw comm param load (XMODEM/TFTP)", NULL, &info_out_load, 0, 0,
};

struct menu_item item_out_quit = {
	"quit", NULL, &info_top, 0, 0,
};

struct menu_info info_output = {
	"output setting",
	menu_output_pre, NULL,
	NULL,
    #if defined(CONFIG_XARINA_VSS)
	3,
	{	&item_out_default, &item_out_change, &item_out_load
	},
    #else /* defined(CONFIG_XARINA_VSS) */
	5,
	{	&item_out_default, &item_out_change, &item_out_vod, &item_out_int, &item_out_load
	},
    #endif /* defined(CONFIG_XARINA_VSS) */
	&item_out_quit, NULL,
	0,
};

