/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <common.h>
#include <command.h>
#include "cmd_menu.h"
#include "menu_top.h"
#include "menu_pinmux.h"
#include "xarina_bool.h"
#include "pinmux.h"
#include "corefw_comm_param.h"
#include "corefw_boot_param.h"
#include "hw_reg_ctrl.h"

extern char console_buffer[];	/* readline buffer */

static int menu_pm_10_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	int status;

	/* GPIO A or AFVD */
	status = pinmux_gpioa_afvd_select_val_get();
	switch(status) {
	case PINMUX_GPIOA_SELECT:
		/* GPIO A */
		p_proc->b_choice[0] = TRUE;
		break;
	case PINMUX_AFVD_SELECT:
		/* AFVD */
		p_proc->b_choice[1] = TRUE;
		break;
	default:
		/* GPIO A */
		p_proc->b_choice[0] = TRUE;
		break;
	}

	return 0;
}


static int menu_pm_10_01(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpioa_afvd_select(PINMUX_GPIOA_SELECT);
	return 0;
}

static int menu_pm_10_02(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpioa_afvd_select(PINMUX_AFVD_SELECT);
	return 0;
}

static int menu_pm_01_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	int status;

	/* GPIO E or SPI0 */
	status = pinmux_gpioe_spi0_select_val_get();
	switch(status) {
	case PINMUX_GPIOE_SELECT:
		/* GPIO E */
		p_proc->b_choice[0] = TRUE;
		break;
	case PINMUX_SPI0_SELECT:
		/* SPI0 */
		p_proc->b_choice[1] = TRUE;
		break;
	default:
		/* SPI0 */
		p_proc->b_choice[1] = TRUE;
		break;
	}

	return 0;
}

static int menu_pm_01_02(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpioe_spi0_select(PINMUX_SPI0_SELECT);
	return 0;
}

static int menu_pm_01_01(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpioe_spi0_select(PINMUX_GPIOE_SELECT);
	return 0;
}

static int menu_pm_02_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	int status;

	status = pinmux_gpioe_uart0_select_val_get();
	switch(status) {
	case PINMUX_GPIOE_SELECT:
		/* GPIO E */
		p_proc->b_choice[0] = TRUE;
		break;
	case PINMUX_UART0_SELECT:
		/* UART */
		p_proc->b_choice[1] = TRUE;
		break;
	default:
		/* GPIO E */
		p_proc->b_choice[0] = TRUE;
		break;
	}
	
	return 0;
}

static int menu_pm_02_02(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpioe_uart0_select(PINMUX_UART0_SELECT);
	return 0;
}

static int menu_pm_02_01(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpioe_uart0_select(PINMUX_GPIOE_SELECT);
	return 0;
}
static int menu_pm_03_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	int status;

	/* GPIO G or UART1 or SPI2 */
	status = pinmux_gpiog_uart1_spi2_select_val_get();
	switch(status) {
	case PINMUX_GPIOG_SELECT:
		/* GPIO G */
		p_proc->b_choice[0] = TRUE;
		break;
	case PINMUX_UART1_SELECT:
		/* UART1 */
		p_proc->b_choice[1] = TRUE;
		break;
	case PINMUX_SPI2_SELECT:
		/* SPI2 */
		p_proc->b_choice[2] = TRUE;
		break;
	default:
		/* default */
		/* UART1 */
		p_proc->b_choice[1] = TRUE;
		break;
	}

	return 0;
}

static int menu_pm_03_03(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpiog_uart1_spi2_select(PINMUX_SPI2_SELECT);
	return 0;
}

static int menu_pm_03_02(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpiog_uart1_spi2_select(PINMUX_UART1_SELECT);
	return 0;
}

static int menu_pm_03_01(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpiog_uart1_spi2_select(PINMUX_GPIOG_SELECT);
	return 0;
}
static int menu_pm_04_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	int status;

	/* GPIO G or UART2 */
	status = pinmux_gpiog_uart2_val_get();
	switch(status) {
	case PINMUX_GPIOG_SELECT:
		/* GPIO G */
		p_proc->b_choice[0] = TRUE;
		break;
	case PINMUX_UART2_SELECT:
		/* UART2 */
		p_proc->b_choice[1] = TRUE;
		break;
	default:
		/* default */
		/* GPIO G */
		p_proc->b_choice[0] = TRUE;
		break;
	}

	return 0;
}

static int menu_pm_04_02(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpiog_uart2_select(PINMUX_UART2_SELECT);
	return 0;
}

static int menu_pm_04_01(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpiog_uart2_select(PINMUX_GPIOG_SELECT);
	return 0;
}


static int menu_pm_05_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	int status;

	status = pinmux_gpioh_sdif_val_get();
	switch(status) {
	case PINMUX_GPIOH_SELECT:
		/* GPIO H */
		p_proc->b_choice[0] = TRUE;
		break;
	case PINMUX_SDIF_SELECT:
		/* SDIF */
		p_proc->b_choice[1] = TRUE;
		break;
	default:
		/* GPIO H */
		p_proc->b_choice[0] = TRUE;
		break;
	}

	return 0;
}

static int menu_pm_05_02(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpioh_sdif_select(PINMUX_SDIF_SELECT);
	return 0;
}

static int menu_pm_05_01(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpioh_sdif_select(PINMUX_GPIOH_SELECT);
	return 0;
}




static int menu_pm_06_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	int status;

	/* GPIO I or SPI3 */
	status = pinmux_gpioi_spi3_val_get();
	switch(status) {
	case PINMUX_GPIOI_SELECT:
		/* GPIO I */
		p_proc->b_choice[0] = TRUE;
		break;
	case PINMUX_SPI3_SELECT:
		/* SPI3 */
		p_proc->b_choice[1] = TRUE;
		break;
	default:
		/* GPIO I */
		p_proc->b_choice[0] = TRUE;
		break;
	}

	return 0;
}

static int menu_pm_06_02(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpioi_spi3_select(PINMUX_SPI3_SELECT);
	return 0;
}

static int menu_pm_06_01(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpioi_spi3_select(PINMUX_GPIOI_SELECT);
	return 0;
}
static int menu_pm_07_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	int status;

	/* GPIO I or PWM */
	status = pinmux_gpioi_pwm_val_get();
	switch(status) {
	case PINMUX_GPIOI_SELECT:
		/* GPIO I */
		p_proc->b_choice[0] = TRUE;
		break;
	case PINMUX_PWM_SELECT:
		/* PWM */
		p_proc->b_choice[1] = TRUE;
		break;
	default:
		/* GPIO I */
		p_proc->b_choice[0] = TRUE;
		break;
	}

	return 0;
}

static int menu_pm_07_02(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpioi_pwm_select(PINMUX_PWM_SELECT);
	return 0;
}

static int menu_pm_07_01(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpioi_pwm_select(PINMUX_GPIOI_SELECT);
	return 0;
}
static int menu_pm_08_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	int status;

	/* GPIO I or NCE1 */
	status = pinmux_gpioi_nce1_val_get();
	switch(status) {
	case PINMUX_GPIOI_SELECT:
		/* GPIO I */
		p_proc->b_choice[0] = TRUE;
		break;
	case PINMUX_NCE1_SELECT:
		/* NCE1 */
		p_proc->b_choice[1] = TRUE;
		break;
	default:
		/* GPIO I */
		p_proc->b_choice[0] = TRUE;
		break;
	}

	return 0;
}

static int menu_pm_08_02(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpioi_nce1_select(PINMUX_NCE1_SELECT);
	return 0;
}

static int menu_pm_08_01(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_gpioi_nce1_select(PINMUX_GPIOI_SELECT);
	return 0;
}

static int menu_pm_09_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	int status;

	/* VIDEO OUT */
	status = pinmux_video_out_val_get();
	switch(status) {
	case PINMUX_VOUT_DESELECT:
		/* VIDEO OUT INVALID */
		p_proc->b_choice[0] = TRUE;
		break;
	case PINMUX_VOUT_SELECT:
		/* VIDEO OUT */
		p_proc->b_choice[1] = TRUE;
		break;
#if defined(CONFIG_FOR_DEVELOPMENT)
	case PINMUX_SVI_SELECT:
		/* SVI */
		p_proc->b_choice[2] = TRUE;
		break;
#endif /* defined(CONFIG_FOR_DEVELOPMENT) */
	default:
		/* VIDEO OUT */
		p_proc->b_choice[1] = TRUE;
		break;
	}

	return 0;
}

#if defined(CONFIG_FOR_DEVELOPMENT)
static int menu_pm_09_03(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_video_out_select(PINMUX_SVI_SELECT);
	return 0;
}
#endif /* defined(CONFIG_FOR_DEVELOPMENT) */

static int menu_pm_09_02(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_video_out_select(PINMUX_VOUT_SELECT);
	return 0;
}
static int menu_pm_09_01(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_video_out_select(PINMUX_VOUT_DESELECT);
	return 0;
}


static int menu_pm_save(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	run_command("saveenv", 0);
	return 0;
}

static int menu_pm_def_restore(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	pinmux_default_set();
	return 0;
}


static int menu_pinmux_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	int vout;
	unsigned char fmt;
	
	if (is_corefw_comm_param_uart0_spi1_select()) {
		/* UART0 or SPI1 */
		p_proc->b_disable[3] = TRUE;
	} else {
		/* other */
		p_proc->b_disable[3] = FALSE;
	}
	
	if (xarina_hw_jtag_connection_check()) {
		/* use JTAG for CPU1-3 */
		p_proc->b_disable[7] = TRUE;
		p_proc->b_disable[8] = TRUE;
	}


	/* VIDEO OUT */
	vout = pinmux_video_out_val_get();
	fmt = corefw_param_tvout_type_get();

	switch(vout) {
	case PINMUX_VOUT_DESELECT:
		break;
	case PINMUX_VOUT_SELECT:
		/* VIDEO OUT */
		p_proc->b_disable[5] = TRUE;
		if (corefw_param_vout_id_get(fmt) != VOUT_ID_OFF) {
			if (corefw_param_vout_res_type_get(fmt) != VOUT_FMT_RES_SD ||
				corefw_param_vout_ip_type_get(fmt) != VOUT_FMT_IP_INTERLACE) {
					p_proc->b_disable[6] = TRUE;
			}
		}
		if (corefw_param_vout_sync_type_get(fmt) == VOUT_FMT_SYNC_HV) {
			p_proc->b_disable[8] = TRUE;
		}
		break;
#if defined(CONFIG_FOR_DEVELOPMENT)
	case PINMUX_SVI_SELECT:
		/* SVI */
		p_proc->b_disable[5] = TRUE;
		p_proc->b_disable[6] = TRUE;
		break;
#endif /* defined(CONFIG_FOR_DEVELOPMENT) */
	default:
		break;
	}


	return 0;
}

/**************************************/
/* GPIO A/AFVD table                  */
/**************************************/
struct menu_item item_pm_10_01 = {
	"GPIO A bit1", menu_pm_10_01, &info_pinmux, 0, 0,
};
struct menu_item item_pm_10_02 = {
	"AFVD", menu_pm_10_02, &info_pinmux, 0, 0,
};
struct menu_item item_pm_10_quit = {
	"quit", NULL, &info_pinmux, 0, 0,
};

struct menu_info info_pm_10 = {
	"GPIO A bit1/AFVD",
	menu_pm_10_pre, NULL,
	NULL,
	2,
	{	&item_pm_10_01, &item_pm_10_02,
	},
	&item_pm_10_quit, NULL,
	0,
};

/**************************************/
/* GPIO E/SPI0 table             */
/**************************************/
struct menu_item item_pm_01_01 = {
	"GPIO E bit0-3", menu_pm_01_01, &info_pinmux, 0, 0,
};
struct menu_item item_pm_01_02 = {
	"SPI0", menu_pm_01_02, &info_pinmux, 0, 0,
};
struct menu_item item_pm_01_quit = {
	"quit", NULL, &info_pinmux, 0, 0,
};

struct menu_info info_pm_01 = {
	"GPIO E bit0-3/SPI0",
	menu_pm_01_pre, NULL,
	NULL,
	2,
	{	&item_pm_01_01, &item_pm_01_02,
	},
	&item_pm_01_quit, NULL,
	0,
};

/**************************************/
/* GPIO E/UART0 menu table            */
/**************************************/
struct menu_item item_pm_02_01 = {
	"GPIO E bit4-7", menu_pm_02_01, &info_pinmux, 0, 0,
};
struct menu_item item_pm_02_02 = {
	"UART0", menu_pm_02_02, &info_pinmux, 0, 0,
};
struct menu_item item_pm_02_quit = {
	"quit", NULL, &info_pinmux, 0, 0,
};

struct menu_info info_pm_02 = {
	"GPIO E bit4-7/UART0",
	menu_pm_02_pre, NULL,
	NULL,
	2,
	{	&item_pm_02_01, &item_pm_02_02,
	},
	&item_pm_02_quit, NULL,
	0,
};


/**************************************/
/* GPIO G/UART1/SPI2 menu table       */
/**************************************/
struct menu_item item_pm_03_01 = {
	"GPIO G bit0-3", menu_pm_03_01, &info_pinmux, 0, 0,
};
struct menu_item item_pm_03_02 = {
	"UART1", menu_pm_03_02, &info_pinmux, 0, 0,
};
struct menu_item item_pm_03_03 = {
	"SPI2", menu_pm_03_03, &info_pinmux, 0, 0,
};
struct menu_item item_pm_03_quit = {
	"quit", NULL, &info_pinmux, 0, 0,
};

struct menu_info info_pm_03 = {
	"GPIO G bit0-3/UART1/SPI2",
	menu_pm_03_pre, NULL,
	NULL,
	3,
	{	&item_pm_03_01, &item_pm_03_02, &item_pm_03_03,
	},
	&item_pm_03_quit, NULL,
	0,
};
/**************************************/
/* GPIOG UART2 menu table  */
/**************************************/
struct menu_item item_pm_04_01 = {
	"GPIO G bit6-7", menu_pm_04_01, &info_pinmux, 0, 0,
};
struct menu_item item_pm_04_02 = {
	"UART2", menu_pm_04_02, &info_pinmux, 0, 0,
};
struct menu_item item_pm_04_quit = {
	"quit", NULL, &info_pinmux, 0, 0,
};

struct menu_info info_pm_04 = {
	"GPIO G bit6-7/UART2",
	menu_pm_04_pre, NULL,
	NULL,
	2,
	{	&item_pm_04_01, &item_pm_04_02,
	},
	&item_pm_04_quit, NULL,
	0,
};

/**************************************/
/* GPIOH SDIF menu table                     */
/**************************************/
struct menu_item item_pm_05_01 = {
	"GPIO H bit0-7", menu_pm_05_01, &info_pinmux, 0, 0,
};
struct menu_item item_pm_05_02 = {
	"SDIF", menu_pm_05_02, &info_pinmux, 0, 0,
};
struct menu_item item_pm_05_quit = {
	"quit", NULL, &info_pinmux, 0, 0,
};

struct menu_info info_pm_05 = {
	"GPIO H bit0-7/SDIF",
	menu_pm_05_pre, NULL,
	NULL,
	2,
	{	&item_pm_05_01, &item_pm_05_02,
	},
	&item_pm_05_quit, NULL,
	0,
};


/**************************************/
/* GPIO I/SPI3 menu table            */
/**************************************/
struct menu_item item_pm_06_01 = {
	"GPIO I bit0-3", menu_pm_06_01, &info_pinmux, 0, 0,
};
struct menu_item item_pm_06_02 = {
	"SPI3", menu_pm_06_02, &info_pinmux, 0, 0,
};
struct menu_item item_pm_06_quit = {
	"quit", NULL, &info_pinmux, 0, 0,
};

struct menu_info info_pm_06 = {
	"GPIO I bit0-3/SPI3",
	menu_pm_06_pre, NULL,
	NULL,
	2,
	{	&item_pm_06_01, &item_pm_06_02,
	},
	&item_pm_06_quit, NULL,
	0,
};
/**************************************/
/* GPIO I/PWM menu table            */
/**************************************/
struct menu_item item_pm_07_01 = {
	"GPIO I bit4-5", menu_pm_07_01, &info_pinmux, 0, 0,
};
struct menu_item item_pm_07_02 = {
	"PWM", menu_pm_07_02, &info_pinmux, 0, 0,
};
struct menu_item item_pm_07_quit = {
	"quit", NULL, &info_pinmux, 0, 0,
};

struct menu_info info_pm_07 = {
	"GPIO I bit4-5/PWM",
	menu_pm_07_pre, NULL,
	NULL,
	2,
	{	&item_pm_07_01, &item_pm_07_02,
	},
	&item_pm_07_quit, NULL,
	0,
};
/**************************************/
/* GPIO I/NCE1 menu table            */
/**************************************/
struct menu_item item_pm_08_01 = {
	"GPIO I bit6", menu_pm_08_01, &info_pinmux, 0, 0,
};
struct menu_item item_pm_08_02 = {
	"NCE1", menu_pm_08_02, &info_pinmux, 0, 0,
};
struct menu_item item_pm_08_quit = {
	"quit", NULL, &info_pinmux, 0, 0,
};

struct menu_info info_pm_08 = {
	"GPIO I bit6/NCE1",
	menu_pm_08_pre, NULL,
	NULL,
	2,
	{	&item_pm_08_01, &item_pm_08_02,
	},
	&item_pm_08_quit, NULL,
	0,
};

/**************************************/
/*  VIDEO OUT menu table            */
/**************************************/
struct menu_item item_pm_09_01 = {
	"VIDEO OUT OFF", menu_pm_09_01, &info_pinmux, 0, 0,
};
struct menu_item item_pm_09_02 = {
	"VIDEO OUT ON", menu_pm_09_02, &info_pinmux, 0, 0,
};
#if defined(CONFIG_FOR_DEVELOPMENT)
struct menu_item item_pm_09_03 = {
	"SVI", menu_pm_09_03, &info_pinmux, 0, 0,
};
#endif

struct menu_item item_pm_09_quit = {
	"quit", NULL, &info_pinmux, 0, 0,
};

#if defined(CONFIG_FOR_DEVELOPMENT)
struct menu_info info_pm_09 = {
	"VIDEO OUT/SVI",
	menu_pm_09_pre, NULL,
	NULL,
	3,
	{	&item_pm_09_01, 
		&item_pm_09_02,
		&item_pm_09_03
	},
	&item_pm_09_quit, NULL,
	0,
};
#else /* defined(CONFIG_FOR_DEVELOPMENT) */
struct menu_info info_pm_09 = {
	"VIDEO OUT",
	menu_pm_09_pre, NULL,
	NULL,
	2,
	{	&item_pm_09_01, 
		&item_pm_09_02,
	},
	&item_pm_09_quit, NULL,
	0,
};
#endif /* defined(CONFIG_FOR_DEVELOPMENT) */

/**************************************/
/* default setting                    */
/**************************************/
struct menu_item item_pm_def_restore = {
	"restore", menu_pm_def_restore, &info_pinmux, 0, 0,
};
struct menu_item item_pm_def_quit = {
	"quit", NULL, &info_pinmux, 0, 0,
};

struct menu_info info_pm_def = {
	"factory default",
	NULL, NULL,
	"Only the setting that can be changed from pinmux\n"
	"setting menu is returned to default.",
	1,
	{	&item_pm_def_restore,
	},
	&item_pm_def_quit, NULL,
	0,
};

/**************************************/
/* pinmux menu table                  */
/**************************************/
struct menu_item item_pm_def = {
	"factory default", NULL, &info_pm_def, 0, 0,
};
struct menu_item item_pm_10 = {
	"GPIO A bit1/AFVD", NULL, &info_pm_10, 0, 0,
};
struct menu_item item_pm_01 = {
	"GPIO E bit0-3/SPI0", NULL, &info_pm_01, 0, 0,
};
struct menu_item item_pm_02 = {
	"GPIO E bit4-7/UART0", NULL, &info_pm_02, 0, 0,
};
struct menu_item item_pm_03 = {
	"GPIO G bit0-3/UART1/SPI2", NULL, &info_pm_03, 0, 0,
};
struct menu_item item_pm_04 = {
	"GPIO G bit6-7/UART2", NULL, &info_pm_04, 0, 0,
};
struct menu_item item_pm_05 = {
	"GPIO H bit0-7/SDIF", NULL, &info_pm_05, 0, 0,
};
struct menu_item item_pm_06 = {
	"GPIO I bit0-3/SPI3", NULL, &info_pm_06, 0, 0,
};
struct menu_item item_pm_07 = {
	"GPIO I bit4-5/PWM", NULL, &info_pm_07, 0, 0,
};
struct menu_item item_pm_08 = {
	"GPIO I bit6/NCE1", NULL, &info_pm_08, 0, 0,
};
#if defined(CONFIG_FOR_DEVELOPMENT)
struct menu_item item_pm_09 = {
	"VIDEO OUT/SVI", NULL, &info_pm_09, 0, 0,
};
#else /* defined(CONFIG_FOR_DEVELOPMENT) */
struct menu_item item_pm_09 = {
	"VIDEO OUT", NULL, &info_pm_09, 0, 0,
};
#endif /* defined(CONFIG_FOR_DEVELOPMENT) */
 
struct menu_item item_pm_save = {
	"save", menu_pm_save, NULL, 0, 0,
};
struct menu_item item_pm_quit = {
	"quit", NULL, &info_top, 0, 0,
};

struct menu_info info_pinmux = {
	"pinmux setting",
	menu_pinmux_pre, NULL,
	NULL,
	12,
	{	&item_pm_def, 	&item_pm_10,	&item_pm_01, 	
		&item_pm_02,	&item_pm_03, 	&item_pm_04, 	
		&item_pm_05,	&item_pm_06, 	&item_pm_07, 	
		&item_pm_08,	&item_pm_09, 	&item_pm_save,
	},
	&item_pm_quit, NULL,
	0,
};
