/*
 * (C) Copyright 2007, 2011, 2013 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <common.h>
#include <command.h>
#include <asm/arch/xarina.h>
#include "cmd_menu.h"
#include "menu_top.h"
#include "menu_uboot.h"
#include "menu_output.h"
#include "menu_input.h"
#include "menu_update.h"
#include "menu_pinmux.h"
#include "menu_flash.h"
#include "menu_version.h"
#include "menu_preset.h"
#include "xarina_bool.h"
#include "hw_reg_ctrl.h"
#include "hw_reg_pll3_pll4.h"

extern char console_buffer[];	/* readline buffer */


static int menu_top_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
#ifdef CONFIG_XARINA_VSS
    p_proc->b_disable[2] = TRUE;            /* output setting */
    p_proc->b_disable[3] = TRUE;            /* input setting */
    p_proc->b_disable[5] = TRUE;            /* pinmux setting */
    p_proc->b_disable[6] = TRUE;            /* load data and nand r/w */
    p_proc->b_disable[8] = TRUE;            /* pre-set */
#else
	int type;

	type = xarina_hw_board_type_get();
	if (type == XARINA_BOARD_RD_IMX122 ||
        type == XARINA_BOARD_RD_ICX692 ||
	    type == XARINA_BOARD_RD_IMX122AF ||
	    type == XARINA_BOARD_RD_IMX139 ||
	    type == XARINA_BOARD_RD_IMX140 ||
	    type == XARINA_BOARD_RD_IMX139AF ||
	    type == XARINA_BOARD_RD_IMX140AF ||
        type == XARINA_BOARD_RD_IMX140LA ||
        type == XARINA_BOARD_RD_ICX693) {
		p_proc->b_disable[5] = TRUE;
	}
#endif

	return 0;
}

/**************************************/
/* boot linux                         */
/**************************************/
static int menu_boot(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	char *s;

	/* corefw boot */
	run_command("bootcofw", 0);

	/* bootcmd exec */
	s = getenv ("bootcmd");
	if (s != NULL) {
		run_command (s, 0);
	} else {
		printf(	"error: no \"bootcmd\" set.\n"
			"       please set \"bootcmd\" in command line mode\n");
 	}

	return 0;
}


/**************************************/
/* main menu                          */
/**************************************/
struct menu_item item_top_boot = {
	"boot linux",  menu_boot, NULL, 0, 0,
};
struct menu_item item_top_uboot = {
	"u-boot setting", NULL, &info_uboot, 0, 0,
};
struct menu_item item_top_output = {
	"output setting", NULL, &info_output, 0, 0,
};
struct menu_item item_top_input = {
	"input setting", NULL, &info_input, 0, 0,
};
struct menu_item item_top_update = {
	"update", NULL, &info_update, 0, 0,
};
struct menu_item item_top_pinmux = {
	"pinmux setting", NULL, &info_pinmux, 0, 0,
};
struct menu_item item_top_flash = {
	"load data and nand r/w", NULL, &info_flash, 0, 0,
};
struct menu_item item_top_ver = {
	"version", NULL, &info_ver, 0, 0,
};
struct menu_item item_top_preset = {
	"pre-set", NULL, &info_preset, 0, 0,
};
struct menu_item item_top_cmdline = {
	"commandline mode", NULL, NULL, 0, 0,
};
struct menu_info info_top = {
	"main menu",
	menu_top_pre, NULL,
	NULL,
	9,
	{	&item_top_boot, &item_top_uboot, &item_top_output, &item_top_input, 
		&item_top_update, &item_top_pinmux, &item_top_flash, &item_top_ver,
		&item_top_preset
	},
	NULL, &item_top_cmdline,
	0,
};

