/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <common.h>
#include <command.h>
#include "cmd_menu.h"
#include "menu_top.h"
#include "menu_update.h"
#include "command_exec.h"
#include "xarina_bool.h"


extern int pushyesno(int c);
extern char console_buffer[];	/* readline buffer */
extern char cmd_buff[CONFIG_SYS_CBSIZE];


/**************************************/
/* menu item code                     */
/**************************************/
int data_down_load(unsigned int type, unsigned long addr)
{
	int b_autostart = FALSE;
#ifdef CONFIG_AUTOSCRIPT
	int b_autoscript = FALSE;
#endif /* CONFIG_AUTOSCRIPT */
	char *s;
	int ret;

	switch(type) {
	case DOWNLOAD_TYPE_LOADS:
		break;
	case DOWNLOAD_TYPE_LOADX:
#ifdef CONFIG_AUTOSCRIPT
		if (((s = getenv("autoscript")) != NULL) && (strcmp(s,"yes") == 0)) {
			run_command("setenv autoscript", 0);	/* delete autoscript */
			b_autoscript = TRUE;
		}
#endif /* CONFIG_AUTOSCRIPT */

		ret =  loadx_command(addr);

#ifdef CONFIG_AUTOSCRIPT
		if (b_autoscript) {
			setenv ("autoscript", "yes");	/* set autoscript */
		}
#endif /* CONFIG_AUTOSCRIPT */
		break;
	case DOWNLOAD_TYPE_TFTP:
		ret = readline("FILENAME: ");
		if (ret <= 0) {
			return ret;
		}

		/* The bootm command operates after executing the tftpboot command when autostart is set to yes. */
		/* Autostart is deleted because there is an obstacle in the update operation when this is done once. */
		if (((s = getenv("autostart")) != NULL) && (strcmp(s,"yes") == 0)) {
			run_command("setenv autostart", 0);	/* delete autostart */
			b_autostart = TRUE;
		}
#ifdef CONFIG_AUTOSCRIPT
		if (((s = getenv("autoscript")) != NULL) && (strcmp(s,"yes") == 0)) {
			run_command("setenv autoscript", 0);	/* delete autoscript */
			b_autoscript = TRUE;
		}
#endif /* CONFIG_AUTOSCRIPT */

		tftp_command(addr, console_buffer);

		if (b_autostart) {
			setenv ("autostart", "yes");	/* set autostart */
		}
#ifdef CONFIG_AUTOSCRIPT
		if (b_autoscript) {
			setenv ("autoscript", "yes");	/* set autoscript */
		}
#endif /* CONFIG_AUTOSCRIPT */

		if (had_ctrlc()) {
			return 0;
		}
		break;

#if defined(CONFIG_ARCH_XARINA)
    case DOWNLOAD_TYPE_MMC:
        {
            char dev[7];
            char addr_str[16];
            char fw_path[128];
            char * const argv[6] = { "fatload", "mmc", dev, addr_str, fw_path, NULL };

            /* Get name of firmware directory */
            const cmd_tbl_t* bcmd = find_cmd("fatload");
            if (!bcmd) {
                printf("Error - 'fatload' command not present.\n");
                return 1;
            }

            const char* file_name = getenv(CONFIG_XARINA_ENV_UPDATE_MMC_FILE_NAME);
            if (file_name) {
                sprintf(fw_path, "%s", file_name);
            }
            else {
                ret = readline("FILENAME: ");
                if (ret <= 0) {
                    return ret;
                }
                sprintf(fw_path, "%s", console_buffer);
            }

            const int devno = 0;
            const int partno = 1;
            const unsigned int addr = 0x60c00000;

            sprintf(dev, "%d:%d", devno, partno);
            sprintf(addr_str, "%lx", addr);

            if (do_fat_fsload(bcmd, 0, 5, argv) != 0) {
                return 0;
            }
        }
        break;
#endif /* defined(CONFIG_ARCH_XARINA) */

	default:
		break;
	}

	return 1;
}


static int menu_ud_uboot(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	char *file_size;
	/* not create filesize variable when tftp download size 0 file */
	run_command("setenv filesize", 0);	/* delete filesize */

	ret = data_down_load(p_item->info, CONFIG_SYS_LOAD_ADDR);
	if (ret <= 0) {
		return 0;
	}
	
	file_size = getenv("filesize");
	if (file_size == NULL) {
		return 0;
	}
	
	sprintf(cmd_buff, "update u-boot %x %s", CONFIG_SYS_LOAD_ADDR, file_size);
	
	ret = run_command(cmd_buff, 0);

	return ret;
}

static int menu_ud_spl(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	char *file_size;
	/* not create filesize variable when tftp download size 0 file */
	run_command("setenv filesize", 0);	/* delete filesize */

	ret = data_down_load(p_item->info, CONFIG_SYS_LOAD_ADDR);
	if (ret <= 0) {
		return 0;
	}
	
	file_size = getenv("filesize");
	if (file_size == NULL) {
		return 0;
	}
	
	sprintf(cmd_buff, "update spl %x %s", CONFIG_SYS_LOAD_ADDR, file_size);
	
	ret = run_command(cmd_buff, 0);

	return ret;
}

static int menu_ud_ubifs_fw(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	char *file_size;
	char part_name[256];
	char vol_name[256];

	ret = getenv_f(CONFIG_XARINA_ENV_UBI_PART, part_name, 256);
	if (ret == -1) {
		printf("error: env \"%s\" is not found.\n", CONFIG_XARINA_ENV_UBI_PART);
		return 1;
	}

	ret = getenv_f(CONFIG_XARINA_ENV_UBI_VOLUME, vol_name, 256);

	if (ret == -1) {
		printf("error: env \"%s\" is not found.\n", CONFIG_XARINA_ENV_UBI_VOLUME);
		return 1;
	}

	/* not create filesize variable when tftp download size 0 file */
	run_command("setenv filesize", 0);	/* delete filesize */

	ret = data_down_load(p_item->info, CONFIG_SYS_LOAD_ADDR);

	if (ret <= 0 || getenv("filesize") == NULL) {
		printf("data download error\n");
		return 1;
	}
	sprintf(cmd_buff, "nand erase %s", part_name);

	printf("%s\n",cmd_buff);
	ret = run_command(cmd_buff, 0);
	if (ret < 0) {
		printf("nand erase failure\n");
		return 1;
	}

	sprintf(cmd_buff, "ubi part %s", part_name);
	ret = run_command(cmd_buff, 0);
	if (ret < 0) {
		printf("ubi attach failure\n");
		return 1;
	}

	sprintf(cmd_buff, "ubi create %s", vol_name);
	ret = run_command(cmd_buff, 0);
	
	file_size = getenv("filesize");
	if (file_size == NULL) {
		return 1;
	}
	sprintf(cmd_buff, "ubi write %x %s %s",CONFIG_SYS_LOAD_ADDR, vol_name	, file_size);
	ret = run_command(cmd_buff, 0);
	if (ret < 0) {
		printf("ubi write failure\n");
		return 1;
	}
	
	sprintf(cmd_buff, "ubifsmount %s", vol_name);
	ret = run_command(cmd_buff, 0);

	if (ret < 0) {
		printf("ubifs mount failure\n");
		return 1;
	}

	printf("\nubifs update is done!\n\n");


	return ret;
}

static int menu_ud_ubifs_rootfs(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	char *file_size;
	char part_name[256];
	char vol_name[256];


	ret = getenv_f(CONFIG_XARINA_ENV_ROOTFS_UBI_PART, part_name, 256);
	if (ret == -1) {
		printf("error: env \"%s\" is not found.\n", CONFIG_XARINA_ENV_ROOTFS_UBI_PART);
		return 1;
	}

	ret = getenv_f(CONFIG_XARINA_ENV_ROOTFS_UBI_VOLUME, vol_name, 256);
	if (ret == -1) {
		printf("error: env \"%s\" is not found.\n", CONFIG_XARINA_ENV_ROOTFS_UBI_VOLUME);
		return 1;
	}


	/* not create filesize variable when tftp download size 0 file */
	run_command("setenv filesize", 0);	/* delete filesize */

	ret = data_down_load(p_item->info, CONFIG_SYS_LOAD_ADDR);
	if (ret <= 0 || getenv("filesize") == NULL) {
		return 1;
	}
	
	
	sprintf(cmd_buff, "nand erase %s", part_name);
	ret = run_command(cmd_buff, 0);
	if (ret < 0) {
		printf("nand erase failure\n");
		return 1;
	}

	sprintf(cmd_buff, "ubi part %s", part_name);
	ret = run_command(cmd_buff, 0);
	if (ret < 0) {
		printf("ubi attach failure\n");
		return 1;
	}

	sprintf(cmd_buff, "ubi create %s", vol_name);
	ret = run_command(cmd_buff, 0);
	
	file_size = getenv("filesize");
	if (file_size == NULL) {
		return 1;
	}
	sprintf(cmd_buff, "ubi write %x %s %s",CONFIG_SYS_LOAD_ADDR, vol_name, file_size);
	ret = run_command(cmd_buff, 0);
	if (ret < 0) {
		printf("ubi write failure\n");
		return 1;
	}
	
	
	/* mount UBIFS for u-boot */
	ret = getenv_f(CONFIG_XARINA_ENV_UBI_PART, part_name, 256);
	if (ret == -1) {
		printf("error: env \"%s\" is not found.\n", CONFIG_XARINA_ENV_UBI_PART);
		return 1;
	}

	sprintf(cmd_buff, "ubi part %s", part_name);
	ret = run_command(cmd_buff, 0);
	if (ret < 0) {
		printf("ubi attach failure\n");
		return 1;
	}

	ret = getenv_f(CONFIG_XARINA_ENV_UBI_VOLUME, vol_name, 256);
	if (ret == -1) {
		printf("error: env \"%s\" is not found.\n", CONFIG_XARINA_ENV_UBI_VOLUME);
		return 1;
	}
	
	sprintf(cmd_buff, "ubifsmount %s", vol_name);
	ret = run_command(cmd_buff, 0);

	if (ret < 0) {
		printf("ubifs mount failure\n");
		return 1;
	}

	printf("\nrootfs update is done!\n\n");


	return ret;
}

#if defined(CONFIG_ARCH_XARINA)
static int xarina_fw_ubifs_update_mmc(void)
{
    struct menu_item item = {
        NULL,               /* char *p_str            : item name */
        NULL,               /* int (*p_func)          : function to execute if item select */
        NULL,               /* struct menu_info *next : next menu if item select */
        0,                  /* unsigned long option   : menu item option */
        DOWNLOAD_TYPE_MMC   /* unsigned long info     : some kind of item original information */
    };

    return menu_ud_ubifs_fw(NULL, &item, NULL);
}

static int xarina_rootfs_ubifs_update_mmc(void)
{
    struct menu_item item = {
        NULL,               /* char *p_str            : item name */
        NULL,               /* int (*p_func)          : function to execute if item select */
        NULL,               /* struct menu_info *next : next menu if item select */
        0,                  /* unsigned long option   : menu item option */
        DOWNLOAD_TYPE_MMC   /* unsigned long info     : some kind of item original information */
    };

    return menu_ud_ubifs_rootfs(NULL, &item, NULL);
}

static int do_update_fw_mmc(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    const int E_CMD_FW_UPDATE_UBIFS_ARG_NUM = -1;

    if (argc != 1 ) {
        printf("invalid arg num.\n");
        return E_CMD_FW_UPDATE_UBIFS_ARG_NUM;
    }

    printf("do_update_fw_mmc\n");

    return xarina_fw_ubifs_update_mmc();
}

static int do_update_rootfs_mmc(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    const int E_CMD_ROOTFS_UPDATE_UBIFS_ARG_NUM = -1;

    if (argc != 1 ) {
        printf("invalid arg num.\n");
        return E_CMD_ROOTFS_UPDATE_UBIFS_ARG_NUM;
    }

    printf("do_update_rootfs_mmc\n");

    return xarina_rootfs_ubifs_update_mmc();
}

U_BOOT_CMD(
	update_fw_mmc, 1, 0,	do_update_fw_mmc,
	NULL,
	NULL
);

U_BOOT_CMD(
	update_rootfs_mmc, 1, 0,	do_update_rootfs_mmc,
	NULL,
	NULL
);
#endif // defined(CONFIG_ARCH_XARINA)

/**************************************/
/* menu table                         */
/**************************************/
/*------------------------------------*/
/* u-boot update                  */
/*------------------------------------*/
struct menu_item item_ud_ub_nd_loadx = {
	"load over serial (xmodem)", menu_ud_uboot, &info_update, 0, DOWNLOAD_TYPE_LOADX,
};
struct menu_item item_ud_ub_nd_tftp = {
	"load over ether (tftp)", menu_ud_uboot, &info_update, 0, DOWNLOAD_TYPE_TFTP,
};
struct menu_item item_ud_ub_nd_quit = {
	"quit", NULL, &info_update, 0, 0,
};
struct menu_info info_ud_uboot = {
	"u-boot update", NULL, NULL,
	NULL,
	2,
	{	&item_ud_ub_nd_loadx, &item_ud_ub_nd_tftp,
	},
	&item_ud_ub_nd_quit, NULL,
	0,
};
/*------------------------------------*/
/* ubifs (fw) update                */
/*------------------------------------*/
struct menu_item item_ud_ubifs_fw_loadx = {
	"load over serial (xmodem)", menu_ud_ubifs_fw, &info_update, 0, DOWNLOAD_TYPE_LOADX,
};
struct menu_item item_ud_ubifs_fw_tftp = {
	"load over ether (tftp)", menu_ud_ubifs_fw, &info_update, 0, DOWNLOAD_TYPE_TFTP,
};
struct menu_item item_ud_ubifs_fw_quit = {
	"quit", NULL, &info_update, 0, 0,
};

struct menu_info info_ud_ubifs_fw = {
	"ubifs (fw) update", NULL, NULL,
	NULL,
	2,
	{	&item_ud_ubifs_fw_loadx, &item_ud_ubifs_fw_tftp,
	},
	&item_ud_ubifs_fw_quit, NULL,
	0,
};
/*------------------------------------*/
/* ubifs (rootfs) update              */
/*------------------------------------*/
struct menu_item item_ud_ubifs_rootfs_loadx = {
	"load over serial (xmodem)", menu_ud_ubifs_rootfs, &info_update, 0, DOWNLOAD_TYPE_LOADX,
};
struct menu_item item_ud_ubifs_rootfs_tftp = {
	"load over ether (tftp)", menu_ud_ubifs_rootfs, &info_update, 0, DOWNLOAD_TYPE_TFTP,
};
struct menu_item item_ud_ubifs_rootfs_quit = {
	"quit", NULL, &info_update, 0, 0,
};

struct menu_info info_ud_ubifs_rootfs = {
	"ubifs (rootfs) update", NULL, NULL,
	NULL,
	2,
	{	&item_ud_ubifs_rootfs_loadx, &item_ud_ubifs_rootfs_tftp,
	},
	&item_ud_ubifs_rootfs_quit, NULL,
	0,
};

/*------------------------------------*/
/* spl update 		              */
/*------------------------------------*/
struct menu_item item_ud_spl_loadx = {
	"load over serial (xmodem)", menu_ud_spl, &info_update, 0, DOWNLOAD_TYPE_LOADX,
};
struct menu_item item_ud_spl_tftp = {
	"load over ether (tftp)", menu_ud_spl, &info_update, 0, DOWNLOAD_TYPE_TFTP,
};
struct menu_item item_ud_spl_quit = {
	"quit", NULL, &info_update, 0, 0,
};
struct menu_info info_ud_spl = {
	"spl update", NULL, NULL,
	NULL,
	2,
	{	&item_ud_spl_loadx, &item_ud_spl_tftp,
	},
	&item_ud_spl_quit, NULL,
	0,
};


/*------------------------------------*/
/* main                               */
/*------------------------------------*/
struct menu_item item_ud_uboot = {
	"u-boot", NULL, &info_ud_uboot, 0, 0,
};
struct menu_item item_ud_ubifs_fw = {
	"ubifs (fw)", NULL, &info_ud_ubifs_fw, 0, 0,
};
struct menu_item item_ud_ubifs_rootfs = {
	"ubifs (rootfs)", NULL, &info_ud_ubifs_rootfs, 0, 0,
};
struct menu_item item_ud_spl = {
	"spl", NULL, &info_ud_spl, 0, 0,
};
struct menu_item item_ud_quit = {
	"quit", NULL, &info_top, 0, 0,
};

struct menu_info info_update = {
	"update",
	NULL, NULL,
	NULL,
	4,
	{	&item_ud_uboot, &item_ud_spl, &item_ud_ubifs_fw, &item_ud_ubifs_rootfs
	},
	&item_ud_quit, NULL,
	0,
};

