/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <common.h>
#include <command.h>
#include "cmd_menu.h"
#include "cmd_corefw.h"
#include "menu_top.h"
#include "menu_version.h"
#include "corefw_comm_param.h"
#include "cam_set_param.h"
#include "xarina_bool.h"
#include "boot_param.h"

extern char console_buffer[];	/* readline buffer */

static char cmd_buff[CONFIG_SYS_CBSIZE];

/**************************************/
/* menu item code                     */
/**************************************/
static int menu_vr_uboot(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	run_command("version", 0);

	return 0;
}

static int menu_vr_corefw(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	run_command("cofwinfo", 0);

	return 0;
}

static int menu_vr_kernel(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	
	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_KERNEL_IMAGE, CONFIG_SYS_LOAD_ADDR, 0);
	
	if (!ret) {
		printf("not found!\n");
		return 0;
	}
	
	sprintf(cmd_buff, "iminfo %x", CONFIG_SYS_LOAD_ADDR);
	run_command(cmd_buff, 0);

	return 0;
}

static int menu_vr_corefwparam(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	char tmp[64];
	struct s_param_data_image_header *p_header 
		= (struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;
	
	/* load saved parameter */
	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_COREFW_COMM_PARAM,
				CONFIG_SYS_LOAD_ADDR, 0);
	printf("load CoreFW comm param ... ");
	if (!ret) {
		printf("not found!\n");
		return 0;
	}
	
	printf("\n");
	printf("## CoreFW comm param at %08x ...\n", (unsigned int)p_header->addr);
	printf("   size:    0x%08lx\n", p_header->size);
	memcpy(tmp, p_header->date, sizeof(p_header->date)); tmp[sizeof(p_header->date)] = '\0';
	printf("   date:    %s\n", tmp);
	memcpy(tmp, p_header->version, sizeof(p_header->version)); tmp[sizeof(p_header->version)] = '\0';
	printf("   version: %s\n", tmp);
	memcpy(tmp, p_header->information, sizeof(p_header->information)); tmp[sizeof(p_header->information)] = '\0';
	printf("   info:    %s\n", tmp);
	printf("\n");	


	return 0;
}

static int menu_vr_camfwparam(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	char tmp[64];
	struct s_param_data_image_header *p_header 
		= (struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;
	
	/* load saved parameter */
	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM,
				CONFIG_SYS_LOAD_ADDR, 0);
	printf("load CamFW setting param ... ");
	if (!ret) {
		printf("not found!\n");
		return 0;
	}
	
	printf("\n");
	printf("## CamFW setting param at %08x ...\n", (unsigned int)p_header->addr);
	printf("   size:    0x%08lx\n", p_header->size);
	memcpy(tmp, p_header->date, sizeof(p_header->date)); tmp[sizeof(p_header->date)] = '\0';
	printf("   date:    %s\n", tmp);
	memcpy(tmp, p_header->version, sizeof(p_header->version)); tmp[sizeof(p_header->version)] = '\0';
	printf("   version: %s\n", tmp);
	memcpy(tmp, p_header->information, sizeof(p_header->information)); tmp[sizeof(p_header->information)] = '\0';
	printf("   info:    %s\n", tmp);
	printf("\n");	

	return 0;
}

static int menu_vr_sensfw(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	sensfw_info_out_all();
	return 0;
}
static int menu_vr_all(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	printf("\n*******U-Boot********\n");
	menu_vr_uboot(p_info, p_item, p_proc);
	printf("\n*******CoreFW********\n");
	menu_vr_corefw(p_info, p_item, p_proc);
	printf("\n*******SensorFW********\n");
	menu_vr_sensfw(p_info, p_item, p_proc);
	printf("\n*******Kernel********\n");
	menu_vr_kernel(p_info, p_item, p_proc);
	printf("\n*******CoreFW common parameter********\n");
	menu_vr_corefwparam(p_info, p_item, p_proc);
	printf("\n*******CAM FW setting parameter********\n");
	menu_vr_camfwparam(p_info, p_item, p_proc);

	return 0;
}


/**************************************/
/* menu table                         */
/**************************************/
struct menu_item item_vr_uboot = {
	"u-boot", menu_vr_uboot, NULL, 0, 0,
};
struct menu_item item_vr_corefw = {
	"corefw", menu_vr_corefw, NULL, 0, 0,
};
struct menu_item item_vr_sensfw = {
	"sensfw", menu_vr_sensfw, NULL, 0, 0,
};
struct menu_item item_vr_kernel = {
	"linux kernel", menu_vr_kernel, NULL, 0, 0,
};
struct menu_item item_vr_corefwparam = {
	"corefw comm param", menu_vr_corefwparam, NULL, 0, 0,
};
struct menu_item item_vr_camfwparam = {
	"cam fw set param", menu_vr_camfwparam, NULL, 0, 0,
};
struct menu_item item_vr_all = {
	"all", menu_vr_all, NULL, 0, 0,
};
struct menu_item item_vr_quit = {
	"quit", NULL, &info_top, 0, 0,
};

struct menu_info info_ver = {
	"version",
	NULL, NULL,
	NULL,
	7,
	{	&item_vr_uboot, &item_vr_corefw, &item_vr_sensfw, &item_vr_kernel,
		&item_vr_corefwparam, &item_vr_camfwparam, &item_vr_all,
	},
	&item_vr_quit, NULL,
	0,
};

