/*
 * (C) Copyright 2007, 2011, 2013 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <config.h>
#include <i2c.h>
#include <asm/arch/hw_reg.h>
#include <asm/arch/xarina.h>
#include "pinmux.h"
#if defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL)
#include "pinmux_vss.h"
#endif /* defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL) */
#include "hw_reg_ctrl.h"
#if !defined(CONFIG_XARINA_SPL)
#include "corefw_boot_param.h"
#include "corefw_comm_param.h"
#include "xarina_bool.h"
#include "hw_reg_pll3_pll4.h"
#endif /* !defined(CONFIG_XARINA_SPL) */

#if !defined(CONFIG_XARINA_SPL)
static int g_pinmux_gpioa_afvd;
static int g_pinmux_gpioe_spi0;
static int g_pinmux_gpioe_uart0;
static int g_pinmux_gpiog_uart1_spi2;
static int g_pinmux_gpiog_uart2;
static int g_pinmux_gpioh_sdif;
static int g_pinmux_gpioi_spi3;
static int g_pinmux_gpioi_pwm;
static int g_pinmux_gpioi_nce1;
static int g_pinmux_video_out;




int pinmux_gpioa_afvd_select_val_get(void)
{
	return g_pinmux_gpioa_afvd;
}
int pinmux_gpioe_uart0_select_val_get(void)
{
	return g_pinmux_gpioe_uart0;
}
int pinmux_gpioe_spi0_select_val_get(void)
{
	return g_pinmux_gpioe_spi0;
}
int pinmux_gpiog_uart1_spi2_select_val_get(void)
{
	return g_pinmux_gpiog_uart1_spi2;
}
int pinmux_gpiog_uart2_val_get(void)
{
	return g_pinmux_gpiog_uart2;
}
int pinmux_gpioh_sdif_val_get(void)
{
	return g_pinmux_gpioh_sdif;
}
int pinmux_gpioi_spi3_val_get(void)
{
	return g_pinmux_gpioi_spi3;
}
int pinmux_gpioi_pwm_val_get(void)
{
	return g_pinmux_gpioi_pwm;
}
int pinmux_gpioi_nce1_val_get(void)
{
	return g_pinmux_gpioi_nce1;
}
int pinmux_video_out_val_get(void)
{
	return g_pinmux_video_out;
}


static int pinmux_pa_set(unsigned long *p_reg)
{
	unsigned long reg = SYSREG1_PINSEL_PA_GPIO;	/* GPIO A */

	/* GPIO A or AFVD */
	if (g_pinmux_gpioa_afvd == PINMUX_AFVD_SELECT) {
		reg = SYSREG1_PINSEL_PA_AFVD;
	}
	*p_reg = reg;
	
	return 0;
}

static int pinmux_pb_set(unsigned long *p_reg)
{
	unsigned long reg = SYSREG1_PINSEL_PB_GPIO;	/* GPIO B */
	unsigned char audio;

	/* GPIO B or AudioIF */
	audio = corefw_param_audio_use_get();
	if (audio == SC_SYS_AUDIO_ENABLE) {
		reg |= SYSREG1_PINSEL_PB_AUDIOIF;
	}
	
	*p_reg = reg;
	
	return 0;
}
static int pinmux_pc_set(unsigned long *p_reg)
{
	unsigned long reg = SYSREG1_PINSEL_PC_GPIO;	/* GPIO C */

	switch(g_pinmux_video_out) {
	case PINMUX_VOUT_DESELECT:
		/* Do Nothing */
		break;
#if defined(CONFIG_FOR_DEVELOPMENT)
	case PINMUX_SVI_SELECT:
		reg |= SYSREG1_PINSEL_PC_SVI;
		break;
#endif /* defined(CONFIG_FOR_DEVELOPMENT) */
	default:
		/* default */
		reg |= SYSREG1_PINSEL_PC_VIDEO;
		break;
	}
	
	*p_reg = reg;
	
	return 0;
}
static int pinmux_pe_set(unsigned long *p_reg)
{
	unsigned long reg = SYSREG1_PINSEL_PE_GPIO;	/* GPIO E */
	unsigned char select;

	/* GPIO E or UART0 or SPI1 */
	select = corefw_param_uart0_spi1_select_get();

	switch(select) {
	case SC_SYS_USE_UART0:
		reg |= SYSREG1_PINSEL_PE_UART0;
		break;
	case SC_SYS_USE_SSP1:
		reg |= SYSREG1_PINSEL_PE_SPI1;
		break;
	case SC_SYS_NOUSE:
		/* Do Nothing */
		/* default */
		/* GPIO E */
		break;
	default:
		/* Do Nothing */
		/* default */
		/* GPIO E */
		break;
	}

	if (reg == SYSREG1_PINSEL_PE_GPIO) {
		/* CoreFW param doesn't select UART0 or SPI1. */

		/* GPIO E or UART */
		switch(g_pinmux_gpioe_uart0) {
		case PINMUX_GPIOE_SELECT:
			/* Do Nothing */
			/* default */
			/* GPIO E */
			break;
		case PINMUX_UART0_SELECT:
			reg |= SYSREG1_PINSEL_PE_UART0;
			break;
		default:
			/* Do Nothing */
			/* default */
			/* GPIO E */
			break;
		}
	}
	
	/* GPIO E or SPI0 */
	switch(g_pinmux_gpioe_spi0) {
	case PINMUX_GPIOE_SELECT:
		/* Do Nothing */
		break;
	case PINMUX_SPI0_SELECT:
		reg |= SYSREG1_PINSEL_PE_SPI0;
		break;
	default:
		/* default */
		reg |= SYSREG1_PINSEL_PE_SPI0;
		break;
	}

	*p_reg = reg;
	
	return 0;
}
static int pinmux_pg_set(unsigned long *p_reg)
{
	unsigned long reg = SYSREG1_PINSEL_PG_UART3;	/* UART3 */

	/* GPIO G or UART1 or SPI2 */
	switch(g_pinmux_gpiog_uart1_spi2) {
	case PINMUX_GPIOG_SELECT:
		/* Do Nothing */
		break;
	case PINMUX_UART1_SELECT:
		reg |= SYSREG1_PINSEL_PG_UART1;
		break;
	case PINMUX_SPI2_SELECT:
		reg |= SYSREG1_PINSEL_PG_SPI2;
		break;
	default:
		/* Do Nothing */
		break;
	}

	/* GPIO G or UART2 or VIDEO or SVI*/
	switch (g_pinmux_video_out) { 
	case PINMUX_VOUT_DESELECT:
		switch(g_pinmux_gpiog_uart2) {
		case PINMUX_GPIOG_SELECT:
			/* Do Nothing */
			break;
		case PINMUX_UART2_SELECT:
			reg |= SYSREG1_PINSEL_PG_UART2;
			break;
		default:
			/* Do Nothing */
			break;
		}
		break;
	case PINMUX_VOUT_SELECT:
		reg |= SYSREG1_PINSEL_PG_VIDEO;
		break;
#if defined(CONFIG_FOR_DEVELOPMENT)
	case PINMUX_SVI_SELECT:
		reg |= SYSREG1_PINSEL_PG_SVI;
		break;
#endif
	default:
		/* Do Nothing */
		break;
	}
		
	*p_reg = reg;
	
	return 0;
}
static int pinmux_ph_set(unsigned long *p_reg)
{
	unsigned long reg = SYSREG1_PINSEL_PH_GPIO;	/* GPIO H */
	unsigned char fmt;

	fmt = corefw_param_tvout_type_get();

	/* GPIO H or SDIF or VIDEO or SVI*/
	switch (g_pinmux_video_out) { 
	case PINMUX_VOUT_DESELECT:
		switch(g_pinmux_gpioh_sdif) {
		case PINMUX_GPIOH_SELECT:
			/* Do Nothing */
			break;
		case PINMUX_SDIF_SELECT:
			reg |= SYSREG1_PINSEL_PH_SDIF;
			break;
		default:
			/* Do Nothing */
			break;
		}
		break;
	case PINMUX_VOUT_SELECT:
		if ((corefw_param_vout_res_type_get(fmt) == VOUT_FMT_RES_SD && 
		    corefw_param_vout_ip_type_get(fmt) == VOUT_FMT_IP_INTERLACE) || 
		    corefw_param_vout_id_get(fmt) == VOUT_ID_OFF) {
			switch(g_pinmux_gpioh_sdif) {
			case PINMUX_GPIOH_SELECT:
				/* Do Nothing */
				break;
			case PINMUX_SDIF_SELECT:
				reg |= SYSREG1_PINSEL_PH_SDIF;
				break;
			default:
				/* Do Nothing */
				break;
			}
		} else {
			reg |= SYSREG1_PINSEL_PH_VIDEO;
		}
		break;
#if defined(CONFIG_FOR_DEVELOPMENT)
	case PINMUX_SVI_SELECT:
		reg |= SYSREG1_PINSEL_PH_SVI;
		break;
#endif
	default:
		/* Do Nothing */
		break;
	}
		
	*p_reg = reg;
	
	return 0;
}
static int pinmux_pi0_set(unsigned long *p_reg)
{
	unsigned long reg = SYSREG1_PINSEL_PI_GPIO;	/* GPIO I */
	unsigned char fmt;

	if (!xarina_hw_jtag_connection_check()) {

		/* GPIO G or SPI3 */
		switch(g_pinmux_gpioi_spi3) {
		case PINMUX_GPIOI_SELECT:
			/* Do Nothing */
			break;
		case PINMUX_SPI3_SELECT:
			reg |= SYSREG1_PINSEL_PI_SPI3;
			break;
		default:
			/* Do Nothing */
			break;
		}


		/* GPIO I or PWM or VIDEO */
		fmt = corefw_param_tvout_type_get();
		if ((g_pinmux_video_out == PINMUX_VOUT_DESELECT) ||
			(corefw_param_vout_sync_type_get(fmt) == VOUT_FMT_SYNC_TRC) ||
			(corefw_param_vout_sync_type_get(fmt) == VOUT_FMT_SYNC_OFF)) {
			switch(g_pinmux_gpioi_pwm) {
			case PINMUX_GPIOI_SELECT:
				/* Do Nothing */
				break;
			case PINMUX_PWM_SELECT:
				reg |= SYSREG1_PINSEL_PI_PWM;
				break;
			default:
				/* Do Nothing */
				break;
			}

		} else {
			reg |= SYSREG1_PINSEL_PI_VIDEO;
		}

	}
	
	*p_reg = reg;
	
	return 0;
}

static int pinmux_pi1_set(unsigned long *p_reg)
{
	unsigned long reg = SYSREG1_NAND_PI_GPIO;	/* GPIO I */

	/* GPIO G or NCE1 */
	switch(g_pinmux_gpioi_nce1) {
	case PINMUX_GPIOI_SELECT:
		/* Do Nothing */
		break;
	case PINMUX_NCE1_SELECT:
		reg |= SYSREG1_PINSEL_PI_NCE1;
		break;
	default:
		/* Do Nothing */
		break;
	}

	*p_reg = reg;
	
	return 0;
}


static void pinmux_ioexpander_set(unsigned long* reg)
{
	int ret;
	unsigned char port[3];

	/* i2c device set */
	ret = i2c_set_bus_num(IOEX_I2C_DEV);
	if (ret < 0) {
		return;
	}

	/* enable pins as output */
#if !defined(CONFIG_XARINA_EVBRD_PRE)
	i2c_reg_write(IOEX_I2C_SLAVE_ADDR, IOEX_CONFIG_PORT0_ADDR, IOEX_CONFIG_OUTPUT);
	i2c_reg_write(IOEX_I2C_SLAVE_ADDR, IOEX_CONFIG_PORT1_ADDR, IOEX_CONFIG_OUTPUT);
	i2c_reg_write(IOEX_I2C_SLAVE_ADDR, IOEX_CONFIG_PORT2_ADDR, IOEX_CONFIG_OUTPUT);
#endif
	port[0] = port[1] = port[2] = 0xff;

	/* PB */
	switch(reg[PINMUX_GPIO_B]) {
	case SYSREG1_PINSEL_PB_AUDIOIF:
		port[0] &= ~IOEX_PINSEL_PB_AUDIOIF;
		break;
	default:
		port[0] &= ~IOEX_PINSEL_PB_GPIO;
		break;
	}

	/* PC */
	switch(reg[PINMUX_GPIO_C]) {
	case SYSREG1_PINSEL_PC_GPIO:
		port[0] &= ~IOEX_PINSEL_PC_GPIO;
		break;
	case SYSREG1_PINSEL_PC_VIDEO:
		port[0] &= ~IOEX_PINSEL_PC_VIDEO;
		break;
#if defined(CONFIG_FOR_DEVELOPMENT)
	case SYSREG1_PINSEL_PC_SVI:
		port[0] &= ~IOEX_PINSEL_PC_SVI;
		break;
#endif /* defined(CONFIG_FOR_DEVELOPMENT) */
	default:
		port[0] &= ~IOEX_PINSEL_PC_VIDEO;
		break;
	}

	/* PE0-3 */
	switch(reg[PINMUX_GPIO_E] & 0xF) {
	case SYSREG1_PINSEL_PE_SPI0:
		port[0] &= ~IOEX_PINSEL_PE_SPI0;
		break;
	default:
		port[0] &= ~IOEX_PINSEL_PE_GPIO_03;
		break;
	}

	/* PE4-7 */
	switch(reg[PINMUX_GPIO_E] & 0xFF00) {
	case SYSREG1_PINSEL_PE_UART0:
		port[0] &= ~IOEX_PINSEL_PE_UART0;
		break;
	case SYSREG1_PINSEL_PE_SPI1:
		port[0] &= ~IOEX_PINSEL_PE_SPI1;
		break;
	default:
		port[0] &= ~IOEX_PINSEL_PE_GPIO_47;
		break;
	}

	/* PF */
	port[0] |= IOEX_PINSEL_PF_GPIO;


	/* PG0-3 */
	switch(reg[PINMUX_GPIO_G] & 0xFF) {
	case SYSREG1_PINSEL_PG_UART1:
		port[1] &= ~IOEX_PINSEL_PG_UART1;
		break;
	case SYSREG1_PINSEL_PG_SPI2:
		port[1] &= ~IOEX_PINSEL_PG_SPI2;
		break;
	default:
		port[1] &= ~IOEX_PINSEL_PG_GPIO_03;
		break;
	}

	/* PG4-5 */
	switch(reg[PINMUX_GPIO_G] & 0x300) {
	case SYSREG1_PINSEL_PG_UART3:
		port[1] &= ~IOEX_PINSEL_PG_UART3;
		break;
	default:
		port[1] &= ~IOEX_PINSEL_PG_GPIO_45;
		break;
	}

	/* PG6-7 */
	switch(reg[PINMUX_GPIO_G] & 0x3C00) {
	case SYSREG1_PINSEL_PG_UART2:
		port[1] &= ~IOEX_PINSEL_PG_UART2;
		break;
	case SYSREG1_PINSEL_PG_VIDEO:
		port[1] &= ~IOEX_PINSEL_PG_VIDEO;
		break;
	case SYSREG1_PINSEL_PG_SVI:
		port[1] &= ~IOEX_PINSEL_PG_SVI;
		break;
	default:
		port[1] &= ~IOEX_PINSEL_PG_GPIO_67;
		break;
	}


	/* PH0-7 */
	switch(reg[PINMUX_GPIO_H] & 0xFFFF) {
	case SYSREG1_PINSEL_PH_SDIF:
		port[1] &= ~IOEX_PINSEL_PH_SDIF;
		break;
	case SYSREG1_PINSEL_PH_VIDEO:
		port[1] &= ~IOEX_PINSEL_PH_VIDEO;
		break;
	case SYSREG1_PINSEL_PH_SVI:
		port[1] &= ~IOEX_PINSEL_PH_SVI;
		break;
	default:
		port[1] &= ~IOEX_PINSEL_PH_GPIO;
		break;
	}

	/* PI0-3 */
	switch(reg[PINMUX_GPIO_I0] & 0xFF) {
	case SYSREG1_PINSEL_PI_SPI3:
		port[2] &= ~IOEX_PINSEL_PI_SPI3;
		break;
	default:
		if (xarina_hw_jtag_connection_check()) {
			port[2] &= ~IOEX_PINSEL_PI_JTAG_03;
		} else {
			port[2] &= ~IOEX_PINSEL_PI_GPIO_03;
		}
		break;
	}

	/* PI4-5 */
	switch(reg[PINMUX_GPIO_I0] & 0xF00) {
	case SYSREG1_PINSEL_PI_VIDEO:
		port[2] &= ~IOEX_PINSEL_PI_VIDEO;
		break;
	case SYSREG1_PINSEL_PI_GPIO:
		if (xarina_hw_jtag_connection_check()) {
			port[2] &= ~IOEX_PINSEL_PI_JTAG_45;
		} else {
			port[2] &= ~IOEX_PINSEL_PI_GPIO_45;
		}
		break;
	default:
		port[2] &= ~IOEX_PINSEL_PI_PWM;	
		break;
	}

	/* PI6 */
	switch(reg[PINMUX_GPIO_I1] & 0x1) {
	case SYSREG1_PINSEL_PI_NCE1:
		port[2] &= ~IOEX_PINSEL_PI_NCE1;
		break;
	default:
		port[2] &= ~IOEX_PINSEL_PI_GPIO_6;	
		break;
	}
#if !defined(CONFIG_XARINA_EVBRD_PRE)
	i2c_reg_write(IOEX_I2C_SLAVE_ADDR, IOEX_OUTPUT_PORT0_ADDR, port[0]);
	i2c_reg_write(IOEX_I2C_SLAVE_ADDR, IOEX_OUTPUT_PORT1_ADDR, port[1]);
	i2c_reg_write(IOEX_I2C_SLAVE_ADDR, IOEX_OUTPUT_PORT2_ADDR, port[2]);
#endif

#if defined(PINMUX_DEBUG)
	printf("port[0] = %02x\n", port[0]);
	printf("port[1] = %02x\n", port[1]);
	printf("port[2] = %02x\n", port[2]);
#endif
	return;



}

static void pinmux_rd_default_set(unsigned long *reg)
{
	reg[PINMUX_GPIO_A] = SYSREG1_PINSEL_PA_AFVD;
	reg[PINMUX_GPIO_B] = SYSREG1_PINSEL_PB_AUDIOIF;
	reg[PINMUX_GPIO_C] = SYSREG1_PINSEL_PC_VIDEO;
	reg[PINMUX_GPIO_D] = 0;
	reg[PINMUX_GPIO_E] = SYSREG1_PINSEL_PE_GPIO | SYSREG1_PINSEL_PE_UART0;
	reg[PINMUX_GPIO_F] = 0;
	reg[PINMUX_GPIO_G] = SYSREG1_PINSEL_PG_UART3 | SYSREG1_PINSEL_PG_VIDEO;
	reg[PINMUX_GPIO_H] = SYSREG1_PINSEL_PH_SDIF;
	reg[PINMUX_GPIO_I0] = SYSREG1_PINSEL_PI_SPI3 | SYSREG1_PINSEL_PI_PWM;
	reg[PINMUX_GPIO_I1] = SYSREG1_NAND_PI_GPIO;

	xarina_hw_video_clock_select(SYSREG1_VIDEO_HCLK);
}


int pinmux_set(void)
{
    #if defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL)
	/* video clock setting */
    xarina_hw_video_clock_select(SYSREG1_VIDEO_HCLK);

	/* audio clock setting */
    xarina_hw_audio_clock_input_sel();

    pinmux_vss_initialize();
    #else /* defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL) */
	unsigned long reg[PINMUX_REG_NUM];
	int type;
	
	pinmux_pa_set(&reg[PINMUX_GPIO_A]);
	pinmux_pb_set(&reg[PINMUX_GPIO_B]);
	pinmux_pc_set(&reg[PINMUX_GPIO_C]);
	reg[PINMUX_GPIO_D] = 0;
	pinmux_pe_set(&reg[PINMUX_GPIO_E]);
	reg[PINMUX_GPIO_F] = 0;
	pinmux_pg_set(&reg[PINMUX_GPIO_G]);
	pinmux_ph_set(&reg[PINMUX_GPIO_H]);
	pinmux_pi0_set(&reg[PINMUX_GPIO_I0]);
	pinmux_pi1_set(&reg[PINMUX_GPIO_I1]);
	
	/* video clock setting */
	switch(g_pinmux_video_out) {
	case PINMUX_VOUT_SELECT:
		xarina_hw_video_clock_select(SYSREG1_VIDEO_HCLK);
		break;
	case PINMUX_SVI_SELECT:
		xarina_hw_video_clock_select(SYSREG1_SVICLK);
		break;
	default:
		/* do nothinig */
		break;
	}

	/* audio clock setting */
	if (xarina_hw_board_type_get() == XARINA_BOARD_EVB_1_1) {
		xarina_hw_audio_clock_input_sel();
	}

	pinmux_gpio_uart3_set();

	if (!strcmp(getenv(CONFIG_XARINA_ENV_PINUMX_IOEX_USE), "yes")) {
		if (xarina_hw_board_type_get() == XARINA_BOARD_EVB_1_0 ||
                    xarina_hw_board_type_get() == XARINA_BOARD_EVB_1_1 ) {
			pinmux_ioexpander_set(reg); 
		}
	}

	type = xarina_hw_board_type_get();

	if (type == XARINA_BOARD_RD_IMX122 ||
        type == XARINA_BOARD_RD_ICX692 ||
	    type == XARINA_BOARD_RD_IMX122AF ||
	    type == XARINA_BOARD_RD_IMX139 ||
	    type == XARINA_BOARD_RD_IMX140 ||
	    type == XARINA_BOARD_RD_IMX139AF ||
	    type == XARINA_BOARD_RD_IMX140AF ||
	    type == XARINA_BOARD_RD_IMX140LA ||
	    type == XARINA_BOARD_RD_ICX693) {
		pinmux_rd_default_set(reg); 
	}

	/* pinmux set */
	xarina_hw_pinmux_set(reg);
    #endif /* defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL) */

	return 0;
}


static int pinmux_env_vari_val_set(char *p_vari_name, int status)
{
	char str[PINMUX_STATUS_STR_NUM];

	sprintf(str, "%x", status);
	setenv(p_vari_name, str);

	return 0;
}

int pinmux_gpioa_afvd_select(int status)
{
	g_pinmux_gpioa_afvd = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOA_AFVD, status);
	pinmux_set();

	return 0;
}


int pinmux_gpioe_spi0_select(int status)
{
	g_pinmux_gpioe_spi0 = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOE_SPI0, status);
	pinmux_set();

	return 0;
}

int pinmux_gpioe_uart0_select(int status)
{
	g_pinmux_gpioe_uart0 = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOE_UART0, status);
	pinmux_set();

	return 0;
}


int pinmux_gpiog_uart1_spi2_select(int status)
{
	g_pinmux_gpiog_uart1_spi2 = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOG_UART1_SPI2, status);
	pinmux_set();

	return 0;
}

int pinmux_gpiog_uart2_select(int status)
{
	g_pinmux_gpiog_uart2 = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOG_UART2, status);
	pinmux_set();

	return 0;
}

int pinmux_gpioh_sdif_select(int status)
{
	g_pinmux_gpioh_sdif = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOH_SDIF, status);
	pinmux_set();

	return 0;
}

int pinmux_gpioi_spi3_select(int status)
{
	g_pinmux_gpioi_spi3 = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOI_SPI3, status);
	pinmux_set();

	return 0;
}

int pinmux_gpioi_pwm_select(int status)
{
	g_pinmux_gpioi_pwm = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOI_PWM, status);
	pinmux_set();

	return 0;
}


int pinmux_gpioi_nce1_select(int status)
{
	g_pinmux_gpioi_nce1 = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOI_NCE1, status);
	pinmux_set();

	return 0;
}
int pinmux_video_out_select(int status)
{
	g_pinmux_video_out = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_VIDEO_OUT, status);
	
	pinmux_set();

	return 0;
}


int pinmux_default_set(void)
{
	pinmux_video_out_select(PINMUX_DEFAULT_VIDEO_OUT);
	pinmux_gpioa_afvd_select(PINMUX_DEFAULT_GPIOA_AFVD);
	pinmux_gpioe_spi0_select(PINMUX_DEFAULT_GPIOE_SPI0);
	pinmux_gpioe_uart0_select(PINMUX_DEFAULT_GPIOE_UART0);
	pinmux_gpiog_uart1_spi2_select(PINMUX_DEFAULT_GPIOG_UART1_SPI2);
	pinmux_gpiog_uart2_select(PINMUX_DEFAULT_GPIOG_UART2);
	pinmux_gpioh_sdif_select(PINMUX_DEFAULT_GPIOH_SDIF);
	pinmux_gpioi_spi3_select(PINMUX_DEFAULT_GPIOI_SPI3);
	pinmux_gpioi_pwm_select(PINMUX_DEFAULT_GPIOI_PWM);
	pinmux_gpioi_nce1_select(PINMUX_DEFAULT_GPIOI_NCE1);

	return 0;
}


int pinmux_env_vari_val_all_get(void)
{
    #if !defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL)
	char *s;

	/* VIDEO OUT */
	s = getenv(PINMUX_ENV_VARI_NAME_VIDEO_OUT);
	if (s) {
		g_pinmux_video_out = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_video_out = PINMUX_DEFAULT_VIDEO_OUT;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_VIDEO_OUT, g_pinmux_video_out);
	}

	/* GPIO A or AFVD */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOA_AFVD);
	if (s) {
		g_pinmux_gpioa_afvd = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpioa_afvd = PINMUX_GPIOA_SELECT;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOA_AFVD, g_pinmux_gpioa_afvd);
	}

	/* GPIO E or SPI0 */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOE_SPI0);
	if (s) {
		g_pinmux_gpioe_spi0 = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpioe_spi0 = PINMUX_DEFAULT_GPIOE_SPI0;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOE_SPI0, g_pinmux_gpioe_spi0);
	}

	/* GPIO E or UART0 */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOE_UART0);
	if (s) {
		g_pinmux_gpioe_uart0 = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpioe_uart0 = PINMUX_DEFAULT_GPIOE_UART0;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOE_UART0, g_pinmux_gpioe_uart0);
	}

	/* GPIO G or UART1 or SPI2 */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOG_UART1_SPI2);
	if (s) {
		g_pinmux_gpiog_uart1_spi2 = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpiog_uart1_spi2 = PINMUX_DEFAULT_GPIOG_UART1_SPI2;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOG_UART1_SPI2, g_pinmux_gpiog_uart1_spi2);
	}

	/* GPIO G or UART2 */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOG_UART2);
	if (s) {
		g_pinmux_gpiog_uart2 = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpiog_uart2 = PINMUX_DEFAULT_GPIOG_UART2;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOG_UART2, g_pinmux_gpiog_uart2);
	}

	/* GPIO H or SDIF */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOH_SDIF);
	if (s) {
		g_pinmux_gpioh_sdif = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpioh_sdif = PINMUX_DEFAULT_GPIOH_SDIF;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOH_SDIF, g_pinmux_gpioh_sdif);
	}

	/* GPIO I or SPI3 */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOI_SPI3);
	if (s) {
		g_pinmux_gpioi_spi3 = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpioi_spi3 = PINMUX_DEFAULT_GPIOI_SPI3;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOI_SPI3, g_pinmux_gpioi_spi3);
	}

	/* GPIO I or PWM */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOI_PWM);
	if (s) {
		g_pinmux_gpioi_pwm = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpioi_pwm = PINMUX_DEFAULT_GPIOI_PWM;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOI_PWM, g_pinmux_gpioi_pwm);
	}


	/* GPIO I or NCE1 */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOI_NCE1);
	if (s) {
		g_pinmux_gpioi_nce1 = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpioi_nce1 = PINMUX_DEFAULT_GPIOI_NCE1;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOI_NCE1, g_pinmux_gpioi_nce1);
	}
    #endif /* !defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL) */

	return 0;
}
#endif /* !defined(CONFIG_XARINA_SPL) */


int pinmux_gpio_uart3_sdif_set(void)
{
#if !defined(CONFIG_XARINA_VSS)
	unsigned long reg[PINMUX_REG_NUM];

	reg[PINMUX_GPIO_A] = SYSREG1_PINSEL_PA_GPIO;
	reg[PINMUX_GPIO_B] = SYSREG1_PINSEL_PB_GPIO;
	reg[PINMUX_GPIO_C] = SYSREG1_PINSEL_PC_GPIO;
	reg[PINMUX_GPIO_D] = 0;
	reg[PINMUX_GPIO_E] = SYSREG1_PINSEL_PE_GPIO;
	reg[PINMUX_GPIO_F] = 0;
	reg[PINMUX_GPIO_G] = SYSREG1_PINSEL_PG_UART3;
	reg[PINMUX_GPIO_H] = SYSREG1_PINSEL_PH_SDIF;
	reg[PINMUX_GPIO_I0] = SYSREG1_PINSEL_PI_GPIO;
	reg[PINMUX_GPIO_I1] = SYSREG1_NAND_PI_GPIO;
	
	xarina_hw_pinmux_set(reg);
#endif /* !defined(CONFIG_XARINA_VSS) */

	return 0;
}

int pinmux_gpio_uart3_set(void)
{
#if !defined(CONFIG_XARINA_VSS)
	unsigned long reg[PINMUX_REG_NUM];

	reg[PINMUX_GPIO_A] = SYSREG1_PINSEL_PA_GPIO;
	reg[PINMUX_GPIO_B] = SYSREG1_PINSEL_PB_GPIO;
	reg[PINMUX_GPIO_C] = SYSREG1_PINSEL_PC_GPIO;
	reg[PINMUX_GPIO_D] = 0;
	reg[PINMUX_GPIO_E] = SYSREG1_PINSEL_PE_GPIO;
	reg[PINMUX_GPIO_F] = 0;
	reg[PINMUX_GPIO_G] = SYSREG1_PINSEL_PG_UART3;
	reg[PINMUX_GPIO_H] = SYSREG1_PINSEL_PH_GPIO;
	reg[PINMUX_GPIO_I0] = SYSREG1_PINSEL_PI_GPIO;
	reg[PINMUX_GPIO_I1] = SYSREG1_NAND_PI_GPIO;
	
	xarina_hw_pinmux_set(reg);
#endif /* !defined(CONFIG_XARINA_VSS) */

	return 0;
}

int pinmux_uart3_sdif_set(void)
{
	unsigned long reg[PINMUX_REG_NUM];

	reg[PINMUX_GPIO_A] = 0;
	reg[PINMUX_GPIO_B] = 0;
	reg[PINMUX_GPIO_C] = SYSREG1_PINSEL_PC_VIDEO;
	reg[PINMUX_GPIO_D] = 0;
	reg[PINMUX_GPIO_E] = 0;
	reg[PINMUX_GPIO_F] = 0;
	reg[PINMUX_GPIO_G] = SYSREG1_PINSEL_PG_UART3;
	reg[PINMUX_GPIO_H] = SYSREG1_PINSEL_PH_SDIF;
	reg[PINMUX_GPIO_I0] = 0;
	reg[PINMUX_GPIO_I1] = 0;
	
	xarina_hw_pinmux_set(reg);

	return 0;
}

int pinmux_uart3_set(void)
{
	unsigned long reg[PINMUX_REG_NUM];

	reg[PINMUX_GPIO_A] = 0;
	reg[PINMUX_GPIO_B] = 0;
	reg[PINMUX_GPIO_C] = SYSREG1_PINSEL_PC_VIDEO;
	reg[PINMUX_GPIO_D] = 0;
	reg[PINMUX_GPIO_E] = 0;
	reg[PINMUX_GPIO_F] = 0;
	reg[PINMUX_GPIO_G] = SYSREG1_PINSEL_PG_UART3;
	reg[PINMUX_GPIO_H] = 0;
	reg[PINMUX_GPIO_I0] = 0;
	reg[PINMUX_GPIO_I1] = 0;
	
	xarina_hw_pinmux_set(reg);

	return 0;
}

