/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef PINMUX_H
#define PINMUX_H

#if defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL)
#include "pinmux_vss.h"
#endif /* defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL) */

/* register index */
#define PINMUX_REG_NUM				(10)

#define PINMUX_STATUS_STR_NUM			(33)


/* for xarina */

#define PINMUX_GPIO_A				(0)
#define PINMUX_GPIO_B				(1)
#define PINMUX_GPIO_C				(2)
#define PINMUX_GPIO_D				(3)
#define PINMUX_GPIO_E				(4)
#define PINMUX_GPIO_F				(5)
#define PINMUX_GPIO_G				(6)
#define PINMUX_GPIO_H				(7)
#define PINMUX_GPIO_I0				(8)
#define PINMUX_GPIO_I1				(9)


/* select status */
#define PINMUX_GPIOA_SELECT			(0)
#define PINMUX_AFVD_SELECT			(1)
#define PINMUX_GPIOE_SELECT			(0)
#define PINMUX_SPI0_SELECT			(1)
#define PINMUX_UART0_SELECT			(1)
#define PINMUX_GPIOG_SELECT			(0)
#define PINMUX_UART1_SELECT			(1)
#define PINMUX_SPI2_SELECT			(2)
#define PINMUX_UART2_SELECT			(1)
#define PINMUX_GPIOH_SELECT			(0)
#define PINMUX_SDIF_SELECT			(1)
#define PINMUX_GPIOI_SELECT			(0)
#define PINMUX_SPI3_SELECT			(1)
#define PINMUX_PWM_SELECT			(1)
#define PINMUX_NCE1_SELECT			(1)
#define PINMUX_VOUT_DESELECT			(0)
#define PINMUX_VOUT_SELECT			(1)
#define PINMUX_SVI_SELECT			(2)


/* default status */
#define PINMUX_DEFAULT_GPIOA_AFVD		PINMUX_GPIOA_SELECT
#define PINMUX_DEFAULT_GPIOE_SPI0		PINMUX_SPI0_SELECT
#define PINMUX_DEFAULT_GPIOE_UART0		PINMUX_GPIOE_SELECT
#define PINMUX_DEFAULT_GPIOG_UART1_SPI2		PINMUX_UART1_SELECT
#define PINMUX_DEFAULT_GPIOG_UART2		PINMUX_GPIOG_SELECT
#define PINMUX_DEFAULT_GPIOH_SDIF		PINMUX_GPIOH_SELECT
#define PINMUX_DEFAULT_GPIOI_SPI3		PINMUX_SPI3_SELECT
#define PINMUX_DEFAULT_GPIOI_PWM		PINMUX_GPIOI_SELECT
#define PINMUX_DEFAULT_GPIOI_NCE1		PINMUX_GPIOI_SELECT
#define PINMUX_DEFAULT_VIDEO_OUT		PINMUX_VOUT_SELECT



/* enviroment variable name */
#define PINMUX_ENV_VARI_NAME_GPIOA_AFVD		"xarina_pinmux_pa"
#define PINMUX_ENV_VARI_NAME_GPIOE_SPI0		"xarina_pinmux_pe0"
#define PINMUX_ENV_VARI_NAME_GPIOE_UART0	"xarina_pinmux_pe1"
#define PINMUX_ENV_VARI_NAME_GPIOG_UART1_SPI2	"xarina_pinmux_pg0"
#define PINMUX_ENV_VARI_NAME_GPIOG_UART2	"xarina_pinmux_pg1"
#define PINMUX_ENV_VARI_NAME_GPIOH_SDIF		"xarina_pinmux_ph"
#define PINMUX_ENV_VARI_NAME_GPIOI_SPI3		"xarina_pinmux_pi0"
#define PINMUX_ENV_VARI_NAME_GPIOI_PWM		"xarina_pinmux_pi1"
#define PINMUX_ENV_VARI_NAME_GPIOI_NCE1		"xarina_pinmux_pi2"
#define PINMUX_ENV_VARI_NAME_VIDEO_OUT		"xarina_pinmux_vout"


/* I/O Expaner */
#define IOEX_I2C_SLAVE_ADDR	(0x22)
#define IOEX_I2C_DEV		(0x1)

#define IOEX_PINSEL_PB_GPIO	(0x01)
#define IOEX_PINSEL_PB_AUDIOIF	(0x00)

#define IOEX_PINSEL_PC_GPIO	(0x02)
#define IOEX_PINSEL_PC_VIDEO	(0x00)
#define IOEX_PINSEL_PC_SVI	(0x00)

#define IOEX_PINSEL_PE_GPIO_03	(0x04)
#define IOEX_PINSEL_PE_SPI0	(0x08)
#define IOEX_PINSEL_PE_GPIO_47	(0x10)
#define IOEX_PINSEL_PE_UART0	(0x00)
#define IOEX_PINSEL_PE_SPI1	(0x20)

#define IOEX_PINSEL_PF_GPIO	(0xC0)


#define IOEX_PINSEL_PG_GPIO_03	(0x01)
#define IOEX_PINSEL_PG_UART1	(0x00)
#define IOEX_PINSEL_PG_SPI2	(0x02)
#define IOEX_PINSEL_PG_GPIO_45	(0x04)
#define IOEX_PINSEL_PG_UART3	(0x00)
#define IOEX_PINSEL_PG_GPIO_67	(0x18)
#define IOEX_PINSEL_PG_UART2	(0x08)
#define IOEX_PINSEL_PG_VIDEO	(0x00)
#define IOEX_PINSEL_PG_SVI	(0x00)

#define IOEX_PINSEL_PH_GPIO	(0xC0)
#define IOEX_PINSEL_PH_SDIF	(0x40)
#define IOEX_PINSEL_PH_VIDEO	(0x00)
#define IOEX_PINSEL_PH_SVI	(0x00)

#define IOEX_PINSEL_PI_GPIO_03	(0x03)
#define IOEX_PINSEL_PI_JTAG_03	(0x01)
#define IOEX_PINSEL_PI_SPI3	(0x00)
#define IOEX_PINSEL_PI_PWM	(0x0C)
#define IOEX_PINSEL_PI_VIDEO	(0x00)
#define IOEX_PINSEL_PI_GPIO_45	(0x0C)
#define IOEX_PINSEL_PI_JTAG_45	(0x04)
#define IOEX_PINSEL_PI_GPIO_6	(0x10)
#define IOEX_PINSEL_PI_NCE1	(0x00)

#define IOEX_OUTPUT_PORT0_ADDR	(0x04)
#define IOEX_OUTPUT_PORT1_ADDR	(0x05)
#define IOEX_OUTPUT_PORT2_ADDR	(0x06)

#define IOEX_CONFIG_PORT0_ADDR	(0x0C)
#define IOEX_CONFIG_PORT1_ADDR	(0x0D)
#define IOEX_CONFIG_PORT2_ADDR	(0x0E)

#define IOEX_CONFIG_OUTPUT	(0x00)



extern int pinmux_gpioa_afvd_select_val_get(void);
extern int pinmux_gpioe_uart0_select_val_get(void);
extern int pinmux_gpioe_spi0_select_val_get(void);
extern int pinmux_gpiog_uart1_spi2_select_val_get(void);
extern int pinmux_gpiog_uart2_val_get(void);
extern int pinmux_gpioh_sdif_val_get(void);
extern int pinmux_gpioi_spi3_val_get(void);
extern int pinmux_gpioi_pwm_val_get(void);
extern int pinmux_gpioi_nce1_val_get(void);
extern int pinmux_video_out_val_get(void);

extern int pinmux_set(void);
extern int pinmux_gpio_uart3_set(void);
extern int pinmux_gpio_uart3_sdif_set(void);
extern int pinmux_gpioa_afvd_select(int status);
extern int pinmux_gpioe_uart0_select(int status);
extern int pinmux_gpioe_spi0_select(int status);
extern int pinmux_gpiog_uart1_spi2_select(int status);
extern int pinmux_gpiog_uart2_select(int status);
extern int pinmux_gpioh_sdif_select(int status);
extern int pinmux_gpioi_spi3_select(int status);
extern int pinmux_gpioi_pwm_select(int status);
extern int pinmux_gpioi_nce1_select(int status);
extern int pinmux_video_out_select(int status);
extern int pinmux_default_set(void);
extern int pinmux_env_vari_val_all_get(void);
extern int pinmux_uart3_set(void);
extern int pinmux_uart3_sdif_set(void);

#endif /* PINMUX_H */

