/**
   pinmux_vss.c
   
   Copyright (C) 2011- Sony Corporation.
*/

#include <config.h>
#include <asm/arch/hw_reg.h>
#include <common.h>
#include <i2c.h>
#include "hw_reg_ctrl.h"
#include "pinmux.h"
#include "pinmux_vss.h"
#include "hw_reg_pll3_pll4.h"

/* Change pinumx_set() to pinmux_vss_initialize() exclude in pinmux.c. */

#if defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL)

struct ioexp_data_tbl {
	unsigned char addr;
	unsigned char data;
};

#define IO_EXPANDER_I2C_DEV     1
#define IO_EXPANDER_I2C_ADDR1   0x09
#define IO_EXPANDER_I2C_ADDR2   0x0E

static const struct ioexp_data_tbl ioexp_data1[] = {
    { 0x08, 0xCF },
    { 0x11, 0x00 },
    { 0x12, 0x30 },
    { 0x13, 0x00 },
};

static const struct ioexp_data_tbl ioexp_data2[] = {
    { 0x08, 0x80 },
    { 0x11, 0x40 },
    { 0x12, 0x7F },
    { 0x13, 0x00 },
};


static unsigned long pinmux_vss_pi0_init_get(void)
{
    unsigned long reg = SYSREG1_PINSEL_PI_GPIO; 

    if (!xarina_hw_jtag_connection_check()) {
        reg |= (SYSREG1_PINSEL_PI_SPI3 | SYSREG1_PINSEL_PI_PWM);
    }

    return reg;
}

static void pinmux_vss_ioexpander_set(void)
{
    int ret = 0;
    int i;
	int limit = 0;

	ret = i2c_set_bus_num(IO_EXPANDER_I2C_DEV);
	if (ret < 0) {
		return;
	}

    ret = i2c_probe(IO_EXPANDER_I2C_ADDR1);
    if (!ret) {
		limit  = sizeof(ioexp_data1) / sizeof(ioexp_data1[0]);
		for (i = 0; i < limit; i++) {
			i2c_reg_write(IO_EXPANDER_I2C_ADDR1, ioexp_data1[i].addr, ioexp_data1[i].data);
		}
		printf("IO Expander1 initialized.\n");
    }

    ret = i2c_probe(IO_EXPANDER_I2C_ADDR2);
    if (!ret) {
		limit  = sizeof(ioexp_data2) / sizeof(ioexp_data2[0]);
		for (i = 0; i < limit; i++) {
			i2c_reg_write(IO_EXPANDER_I2C_ADDR2, ioexp_data2[i].addr, ioexp_data2[i].data);
		}
		printf("IO Expander2 initialized.\n");
    }
}

void pinmux_vss_initialize(void)
{
    unsigned long reg[PINMUX_REG_NUM];

    reg[PINMUX_GPIO_A]  = SYSREG1_PINSEL_PA_AFVD;
    reg[PINMUX_GPIO_B]  = SYSREG1_PINSEL_PB_AUDIOIF;
    reg[PINMUX_GPIO_C]  = SYSREG1_PINSEL_PC_VIDEO;
    reg[PINMUX_GPIO_D]  = 0;
    reg[PINMUX_GPIO_E]  = SYSREG1_PINSEL_PE_UART0_NOFC;
    reg[PINMUX_GPIO_F]  = 0;
    reg[PINMUX_GPIO_G]  = (SYSREG1_PINSEL_PG_UART1_NOFC | SYSREG1_PINSEL_PG_UART2 | SYSREG1_PINSEL_PG_UART3);
    reg[PINMUX_GPIO_H]  = SYSREG1_PINSEL_PH_SDIF;
    reg[PINMUX_GPIO_I0] = pinmux_vss_pi0_init_get();
    reg[PINMUX_GPIO_I1] = SYSREG1_NAND_PI_GPIO;

    xarina_hw_pinmux_set(reg);

    pinmux_vss_ioexpander_set();
}
#endif /* defined(CONFIG_XARINA_VSS) && !defined(CONFIG_XARINA_SPL) */
