/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */


#include <common.h>
#include <asm/arch/xarina.h>
#include <command.h>
#include "sdboot.h"
#include "xarina_bool.h"

#define MAX_NAME_LEN	255
#define MAX_EXT_LEN	3

extern char cmd_buff[];


int sdboot_extcmp(const char *file_name, const char *ext)
{
	char tmp_ext[MAX_EXT_LEN + 1];
	int i;

	memset(tmp_ext, 0, MAX_EXT_LEN + 1);

	for (i = 0; i < MAX_NAME_LEN; i++) {
		if (*(file_name + i) == '.') {
			strncpy(tmp_ext, file_name + i + 1, MAX_EXT_LEN);
		} else if (*(file_name + i) == '\0') {
			break;
		}
	}

	return strncmp(tmp_ext, ext, MAX_EXT_LEN);

}



static int script_core(unsigned char* addr, unsigned long size)
{
	int i, idx, ign;

	for (i = 0, idx = 0, ign = FALSE; i < size; i++) {
		if (*addr == '\n' || *addr == '\r') {
			if (idx != 0) {
				cmd_buff[idx] = '\0';
				printf("exec \"%s\"\n", cmd_buff);
				run_command(cmd_buff, 0);
			}
			idx = 0;
			ign = FALSE;
		} else if (*addr == '#') {
			ign = TRUE;
		} else {
			if (ign == FALSE && (' ' <= *addr && *addr <= '~' )) {
				cmd_buff[idx] = *addr;
				idx++;
				if (idx >= CONFIG_SYS_CBSIZE - 1) {
					ign = TRUE;
				}
			}
		}

		addr++;
	}
	if (idx != 0) {
		cmd_buff[idx] = '\0';
		printf("exec \"%s\"\n", cmd_buff);
		run_command(cmd_buff, 0);
	}
	
	return 0;
}

int do_script(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{

	unsigned char *addr;
	unsigned long size;
	
	if (argc != 3) {
		cmd_usage(cmdtp);
		return -1;	
	}
	
	addr = (unsigned char *)simple_strtoul(argv[1], NULL, 16);
	size = simple_strtoul(argv[2], NULL, 16);

	return script_core(addr, size);
}



U_BOOT_CMD(
	script, 3, 0,	do_script,
	"execute script ",
	"< addr > < size >\n"
	"    - execute script file in DDR memory. \n"
);


#if !defined(CONFIG_XARINA_SPL)
char boot_script_file[256];


void sdboot_linux_boot(void)
{
	int ret;
	char cmd[1088];
	char file_name_full[1028];
	unsigned char *addr;
	unsigned long size;


	strncpy(file_name_full, CONFIG_XARINA_SD_BOOTSCRIPT_DIR, 1028);

	/* enable SDIF */
#if !defined(CONFIG_XARINA_VSS)
	run_command("pinmux vout_invalid", 0);
	run_command("pinmux sdif", 0);
#endif /* if !defined(CONFIG_XARINA_VSS) */

	/* scan SD card */
	run_command("mmc rescan 0", 0);

	/* search script file */
	sprintf(cmd, "fatls mmc 0 %s", CONFIG_XARINA_SD_BOOTSCRIPT_DIR); 
	run_command(cmd, 0);

	if (boot_script_file[0] == '\0') {
		printf("boot script file is not found!\n");
		return;
	}
	/* load script file */
	strncat(file_name_full, "/", 1);
	strncat(file_name_full, (const char *)boot_script_file, 256);
	/* remove filesize */
	run_command("setenv filesize", 0);
	sprintf(cmd, "fatload mmc 0 %x %s", 
			CONFIG_XARINA_SD_BOOTSCRIPT_LOAD_ADDR, file_name_full);
	ret = run_command(cmd, 0);


	if (getenv("filesize") == NULL) {
		printf("boot script load error.\n");
		return ;
	}
	/* size check */
	size = simple_strtoul(getenv("filesize"), NULL, 16);
	if (size >= CONFIG_XARINA_SD_BOOTSCRIPT_MAX_SIZE) {
		printf("script file is too big\n");
		return ;
	}

	addr = (unsigned char *)CONFIG_XARINA_SD_BOOTSCRIPT_LOAD_ADDR;


	script_core(addr, size);


}



#endif

/******************************
 Routine:
 Description:for SD Boot
******************************/
#if defined(CONFIG_XARINA_SDSPL)
char uboot_file[256];

void sdboot_uboot_boot(void)
{
	int ret;
	char cmd[1088];
	char file_name_full[1024];

	strncpy(file_name_full, CONFIG_XARINA_SDSPL_UBOOT_DIR, 1024);

	run_command("mmc rescan 0", 0);

	sprintf(cmd, "fatls mmc 0 %s", CONFIG_XARINA_SDSPL_UBOOT_DIR); 
	run_command(cmd, 0);

	if (uboot_file[0] == '\0') {
		printf("u-boot file is not found!\n");
		return;
	}

	strncat(file_name_full, "/", 1);
	strncat(file_name_full, uboot_file, 256);
	
	sprintf(cmd, "fatload mmc 0 %x %s", CONFIG_SYS_LOAD_ADDR, file_name_full);
	ret = run_command(cmd, 0);

	if (!ret) {
		run_command("xarina_uboot", 0);
	}

}
#endif /* defined(CONFIG_XARINA_SDSPL) */


