/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <i2c.h>
#include "vin.h"
#include "cam_comm_param.h"
#if defined(CONFIG_XARINA_VSS)
#include "cam_set_param.h"
#include "boot_param.h"
#include "corefw_boot_param.h"
#include "gpio_vss.h"
#endif /* defined(CONFIG_XARINA_VSS) */

/* #define VDEC_DEBUG */

static int vin_dev = VIN_DEV_DEFAULT;

#if defined(CONFIG_XARINA_VSS)

#define GPIO_E_BOOT_MODE1                   0x02
#define GPIO_E_BOOT_MODE2                   0x03
#define GPIO_E_CAM_BLOCK_POWER_CTRL         0x07
#define GPIO_E_CAM_BLOCK_POWER_CTRL_ON      0x00
#define GPIO_E_CAM_BLOCK_POWER_CTRL_OFF     0x01

#else /* defined(CONFIG_XARINA_VSS) */

static const struct s_vdec_data_tbl vdec_data_ntsc[] = {
	{ 0x00, 0x00 },
	{ 0x03, 0x0D },
	{ 0x28, 0x0A },
};

static const struct s_vdec_data_tbl vdec_data_pal[] = {
	{ 0x00, 0x00 },
	{ 0x03, 0x0D },
	{ 0x28, 0x04 },
};

#endif /* defined(CONFIG_XARINA_VSS) */

void set_vin_dev(int dev)
{
	vin_dev = dev;
}

int get_vin_dev(void)
{
	return vin_dev;
}

#if defined(CONFIG_XARINA_VSS)

void init_vin_dev(void)
{
	if (cam_param_senssystem_ddr_get() & CAM_PARAM_SENSSYS_VIDEOIN_MASK) {
        struct  tagSSc_Sys_CoreFwParam *p_param = 
            (struct  tagSSc_Sys_CoreFwParam *)boot_param_core_addr_get();

        printf("Video Input Initialize.\n");
        set_vin_dev(VIN_DEV_VDEC);
        gpio_write(GPIO_E_BASE, GPIO_E_BOOT_MODE1, 0);
        gpio_write(GPIO_E_BASE, GPIO_E_BOOT_MODE2, 0);
    }
}

#else /* defined(CONFIG_XARINA_VSS) */

void init_vin_dev(void)
{
	char *env;
	char str[4];
	
	env = getenv(CONFIG_XARINA_ENV_VIDEOIN_DEV);
	
	if (!env) {
		sprintf(str,"%x", VIN_DEV_DEFAULT);
		setenv(CONFIG_XARINA_ENV_VIDEOIN_DEV, str);
		set_vin_dev(VIN_DEV_DEFAULT);
		return;
	}
	
	set_vin_dev((int)simple_strtoul(env, NULL, 16));
}

#endif /* defined(CONFIG_XARINA_VSS) */

#if defined(CONFIG_XARINA_EVBRD_PRE)

int vdec_init(unsigned long type)
{
	int limit = 0;
	int ret;
	int i;

	/* i2c device set */
	ret = i2c_set_bus_num(VDEC_I2C_DEV);
	if (ret < 0) {
		return ret;
	}

#ifdef VDEC_DEBUG
	/* Read data */
	printf("Befor register set.\n");
	for(i = 0; i < 0x100; i++) {
		printf("reg:%02x data:%02x\n", i, i2c_reg_read(VDEC_I2C_SLAVE_ADDR, i));
	}
#endif	/* VDEC_DEBUG */

	switch(type) {
	case VDEC_TYPE_NTSC:
		limit  = sizeof(vdec_data_ntsc)/sizeof(vdec_data_ntsc[0]);

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VDEC_I2C_SLAVE_ADDR, vdec_data_ntsc[i].addr, vdec_data_ntsc[i].data);
		}
		break;
	case VDEC_TYPE_PAL:
		limit  = sizeof(vdec_data_pal)/sizeof(vdec_data_pal[0]);

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VDEC_I2C_SLAVE_ADDR, vdec_data_pal[i].addr, vdec_data_pal[i].data);
		}
		break;
	default:
		return -1;
		break;
	}

#ifdef VDEC_DEBUG
	printf("\n" "After register set.\n");
	/* Read data */
	for(i = 0; i < 0x100; i++) {
		printf("reg:%02x data:%02x\n", i, i2c_reg_read(VDEC_I2C_SLAVE_ADDR, i));
	}
#endif	/* VDEC_DEBUG */

	return 0;
}
#else

#if defined(CONFIG_XARINA_VSS)

int vdec_init(unsigned long video_size, unsigned long video_frame)
{
	int ret = 0;

	return ret;
}

#else /* defined(CONFIG_XARINA_VSS) */

int vdec_init(unsigned long type)
{
	int limit = 0;
	int ret;
	int i;

	/* i2c device set */
	ret = i2c_set_bus_num(VDEC_I2C_DEV);
	if (ret < 0) {
		return ret;
	}

	switch(type) {
	case CAM_PARAM_VIDEOSIZE_NTSC:
		limit  = sizeof(vdec_data_ntsc)/sizeof(vdec_data_ntsc[0]);

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VDEC_I2C_SLAVE_ADDR, vdec_data_ntsc[i].addr, vdec_data_ntsc[i].data);
		}
		break;
	case CAM_PARAM_VIDEOSIZE_PAL:
		limit  = sizeof(vdec_data_pal)/sizeof(vdec_data_pal[0]);

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VDEC_I2C_SLAVE_ADDR, vdec_data_pal[i].addr, vdec_data_pal[i].data);
		}
		break;
	default:
		printf("VIDEO DECODER does not support this video size.\n");
		return -1;
	}



	return 0;
}

#endif /* defined(CONFIG_XARINA_VSS) */

#endif
