/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <common.h>
#include <i2c.h>
#include <asm/arch/xarina.h>
#include "xarina_bool.h"
#include "vout.h"
#include "corefw_boot_param.h"
#include "hw_reg_ctrl.h"
#include "hw_reg_pll3_pll4.h"
#include "vout_table.h"
#if defined(CONFIG_XARINA_VSS)
#include "init_vss.h"
#endif

/* #define VENC_DEBUG */

static int vout_dev;

#if defined(CONFIG_XARINA_VSS)

static unsigned char list_venc_vss_type[] =
{
	SC_SYS_VIDEO_VOUT_FMT_OFF,
	SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5,
	SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_PAR_10_11,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_PAR_10_11,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_59_54,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_12_11,
	SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_FULL,
	SC_SYS_VIDEO_VOUT_FMT_UNUSED,
};

#else /* !defined(CONFIG_XARINA_VSS) */

static unsigned char list_venc_evb_type[] =
{
	SC_SYS_VIDEO_VOUT_FMT_OFF,
	SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_PAR_10_11,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_PAR_10_11,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_59_54,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_12_11,
	SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_SMPTE296M,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_SMPTE296M,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_SMPTE296M,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_SMPTE296M_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_SMPTE296M_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_SMPTE296M_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7_CROP_FULL,
};

static unsigned char list_venc_rd_type[] =
{
	SC_SYS_VIDEO_VOUT_FMT_OFF,
	SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5,
	SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_PAR_10_11,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_PAR_10_11,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_59_54,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_12_11,
	SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_FULL,

};


static unsigned char list_hdmi_type[] =
{
	SC_SYS_VIDEO_VOUT_FMT_OFF,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_PAR_10_11,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E_PAR_10_11,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_59_54,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_PAR_59_54,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_12_11,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_PAR_12_11,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_480_59P_CEA861E_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1_FULL,
	SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_BT1120_7,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_BT1120_7,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7, 
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_CEA861E, 
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_CEA861E,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_BT1120_7_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_BT1120_7_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7_CROP_FULL, 
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_CEA861E_CROP_FULL, 
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_CEA861E_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7_CROP_FULL,
	SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_CEA861E_CROP_FULL

};
#endif /* defined(CONFIG_XARINA_VSS) */
#if defined(CONFIG_XARINA_EVBRD_PRE)
static const unsigned char venc_data_ntsc[] = {
	0x00,	/* 00 *//* read only. not write */
	0x00,	/* 01 */
	0x00,	/* 02 */
	0x00,	/* 03 */
	0x00,	/* 04 */
	0x00,	/* 05 */
	0x00,	/* 06 */
	0x00,	/* 07 */
	0x00,	/* 08 */
	0x00,	/* 09 */
	0x00,	/* 0A */
	0x00,	/* 0B */
	0x00,	/* 0C */
	0x00,	/* 0D */
	0x00,	/* 0E */
	0x00,	/* 0F */
	0x00,	/* 10 */
	0x00,	/* 11 */
	0x00,	/* 12 */
	0x00,	/* 13 */
	0x00,	/* 14 */
	0x00,	/* 15 */
	0x00,	/* 16 */
	0x00,	/* 17 */
	0x00,	/* 18 */
	0x00,	/* 19 */
	0x00,	/* 1A */
	0x00,	/* 1B */
	0x00,	/* 1C */
	0x00,	/* 1D */
	0x00,	/* 1E */
	0x00,	/* 1F */
	0x00,	/* 20 */
	0x00,	/* 21 */
	0x00,	/* 22 */
	0x00,	/* 23 */
	0x00,	/* 24 */
	0x00,	/* 25 */
	0x00,	/* 26 */
	0x00,	/* 27 */
	0x19,	/* 28 */
	0x1d,	/* 29 */
	0x00,	/* 2A */
	0x00,	/* 2B */
	0x00,	/* 2C */
	0x3f,	/* 2D */
	0x00,	/* 2E */
	0x00,	/* 2F */
	0x00,	/* 30 */
	0x00,	/* 31 */
	0x00,	/* 32 */
	0x00,	/* 33 */
	0x00,	/* 34 */
	0x00,	/* 35 */
	0x00,	/* 36 */
	0x00,	/* 37 */
	0x00,	/* 38 */
	0x00,	/* 39 */
	0x13,	/* 3A */
	0x00,	/* 3B */
	0x00,	/* 3C */
	0x00,	/* 3D */
	0x00,	/* 3E */
	0x00,	/* 3F */
	0x00,	/* 40 */
	0x00,	/* 41 */
	0x80,	/* 42 */
	0x80,	/* 43 */
	0x80,	/* 44 */
	0x80,	/* 45 */
	0x80,	/* 46 */
	0x80,	/* 47 */
	0x80,	/* 48 */
	0x80,	/* 49 */
	0x80,	/* 4A */
	0x80,	/* 4B */
	0x80,	/* 4C */
	0x80,	/* 4D */
	0x3f,	/* 4E */
	0x3f,	/* 4F */
	0x3f,	/* 50 */
	0x80,	/* 51 */
	0x80,	/* 52 */
	0x80,	/* 53 */
	0x3c,	/* 54 */
	0x00,	/* 55 */
	0x00,	/* 56 */
	0x00,	/* 57 */
	0x00,	/* 58 */
	0x00,	/* 59 */
	0x35,	/* 5A */
	0x76,	/* 5B */
	0xa5,	/* 5C */
	0x3f,	/* 5D */
	0x2e,	/* 5E */
	0x30,	/* 5F */
	0x00,	/* 60 */
	0x15,	/* 61 */
	0x3f,	/* 62 */
	0x1f,	/* 63 */
	0x7c,	/* 64 */
	0xf0,	/* 65 */
	0x21,	/* 66 */
	0x00,	/* 67 */
	0x00,	/* 68 */
	0x00,	/* 69 */
	0x00,	/* 6A */
	0xd2,	/* 6B */
	0xfa,	/* 6C */
	0x00,	/* 6D */
	0x90,	/* 6E */
	0x00,	/* 6F */
	0x49,	/* 70 */
	0x49,	/* 71 */
	0x00,	/* 72 */
	0x00,	/* 73 */
	0x00,	/* 74 */
	0x03,	/* 75 */
	0x00,	/* 76 */
	0x00,	/* 77 */
	0x00,	/* 78 */
	0x00,	/* 79 */
	0x00,	/* 7A */
	0x01,	/* 7B */
	0x40,	/* 7C */
	0x00,	/* 7D */
	0x00,	/* 7E */
	0x00,	/* 7F */
};


static const unsigned char venc_data_pal[] = {
	0x00,	/* 00 *//* read only. not write */
	0x00,	/* 01 */
	0x00,	/* 02 */
	0x00,	/* 03 */
	0x00,	/* 04 */
	0x00,	/* 05 */
	0x00,	/* 06 */
	0x00,	/* 07 */
	0x00,	/* 08 */
	0x00,	/* 09 */
	0x00,	/* 0A */
	0x00,	/* 0B */
	0x00,	/* 0C */
	0x00,	/* 0D */
	0x00,	/* 0E */
	0x00,	/* 0F */
	0x00,	/* 10 */
	0x00,	/* 11 */
	0x00,	/* 12 */
	0x00,	/* 13 */
	0x00,	/* 14 */
	0x00,	/* 15 */
	0x00,	/* 16 */
	0x00,	/* 17 */
	0x00,	/* 18 */
	0x00,	/* 19 */
	0x00,	/* 1A */
	0x00,	/* 1B */
	0x00,	/* 1C */
	0x00,	/* 1D */
	0x00,	/* 1E */
	0x00,	/* 1F */
	0x00,	/* 20 */
	0x00,	/* 21 */
	0x00,	/* 22 */
	0x00,	/* 23 */
	0x00,	/* 24 */
	0x00,	/* 25 */
	0x00,	/* 26 */
	0x00,	/* 27 */
	0x21,	/* 28 */
	0x1d,	/* 29 */
	0x00,	/* 2A */
	0x00,	/* 2B */
	0x00,	/* 2C */
	0x3f,	/* 2D */
	0x00,	/* 2E */
	0x00,	/* 2F */
	0x00,	/* 30 */
	0x00,	/* 31 */
	0x00,	/* 32 */
	0x00,	/* 33 */
	0x00,	/* 34 */
	0x00,	/* 35 */
	0x00,	/* 36 */
	0x00,	/* 37 */
	0x00,	/* 38 */
	0x00,	/* 39 */
	0x13,	/* 3A */
	0x00,	/* 3B */
	0x00,	/* 3C */
	0x00,	/* 3D */
	0x00,	/* 3E */
	0x00,	/* 3F */
	0x00,	/* 40 */
	0x00,	/* 41 */
	0x80,	/* 42 */
	0x80,	/* 43 */
	0x80,	/* 44 */
	0x80,	/* 45 */
	0x80,	/* 46 */
	0x80,	/* 47 */
	0x80,	/* 48 */
	0x80,	/* 49 */
	0x80,	/* 4A */
	0x80,	/* 4B */
	0x80,	/* 4C */
	0x80,	/* 4D */
	0x3f,	/* 4E */
	0x3f,	/* 4F */
	0x3f,	/* 50 */
	0x80,	/* 51 */
	0x80,	/* 52 */
	0x80,	/* 53 */
	0x3c,	/* 54 */
	0x00,	/* 55 */
	0x00,	/* 56 */
	0x00,	/* 57 */
	0x00,	/* 58 */
	0x00,	/* 59 */
	0x00,	/* 5A */
	0x76,	/* 5B */
	0xa5,	/* 5C */
	0x3f,	/* 5D */
	0x2e,	/* 5E */
	0x30,	/* 5F */
	0x00,	/* 60 */
	0x16,	/* 61 */
	0x2b,	/* 62 */
	0xcb,	/* 63 */
	0x8a,	/* 64 */
	0x09,	/* 65 */
	0x2a,	/* 66 */
	0x00,	/* 67 */
	0x00,	/* 68 */
	0x00,	/* 69 */
	0x00,	/* 6A */
	0xd2,	/* 6B */
	0x02,	/* 6C */
	0x20,	/* 6D */
	0xa0,	/* 6E */
	0x00,	/* 6F */
	0x49,	/* 70 */
	0x49,	/* 71 */
	0x00,	/* 72 */
	0x00,	/* 73 */
	0x00,	/* 74 */
	0x03,	/* 75 */
	0x00,	/* 76 */
	0x00,	/* 77 */
	0x00,	/* 78 */
	0x00,	/* 79 */
	0x1b,	/* 7A */
	0x30,	/* 7B */
	0x40,	/* 7C */
	0x00,	/* 7D */
	0x00,	/* 7E */
	0x00,	/* 7F */
};

#else

#endif


void set_vout_dev(int dev)
{
	vout_dev = dev;
}

int get_vout_dev(void)
{
	return vout_dev;
}

void init_vout_dev(void)
{
    #if defined(CONFIG_XARINA_VSS)
    set_vout_dev(VOUT_DEV_VENC);
    #else /* defined(CONFIG_XARINA_VSS */
	char *env;
	char str[4];
	
	env = getenv(CONFIG_XARINA_ENV_VIDEOOUT_DEV);
	
	if (!env) {
		sprintf(str,"%x", VOUT_DEV_DEFAULT);
		setenv(CONFIG_XARINA_ENV_VIDEOOUT_DEV, str);
		set_vout_dev(VOUT_DEV_DEFAULT);
		return;
	}
	
	set_vout_dev((int)simple_strtoul(env, NULL, 16));
    #endif /* defined(CONFIG_XARINA_VSS) */
	
}

#if defined(CONFIG_XARINA_EVBRD_PRE)

int venc_evb_init(unsigned long type)
{
	const unsigned char *p_data = NULL;
	int limit = 0;
	int ret;
	int i;

	/* i2c device set */
	ret = i2c_set_bus_num(VENC_I2C_DEV);
	if (ret < 0) {
		return ret;
	}

#ifdef VENC_DEBUG
	/* Read data */
	printf("Befor register set.\n");
	for(i = 0; i < 0x80; i++) {
		printf("reg:%02x data:%02x\n", i, i2c_reg_read(VENC_I2C_SLAVE_ADDR, i));
	}
#endif	/* VENC_DEBUG */

	switch(type) {
	case VENC_TYPE_NTSC:
		printf("VideoEnc set NTSC.\n");
		p_data = venc_data_ntsc;
		limit  = sizeof(venc_data_ntsc)/sizeof(venc_data_ntsc[0]);

		/* Write data */
		for(i = 1; i < limit; i++) {	/* subaddress 0 is read only register. not write. */
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, i, *(p_data + i));
		}
		break;
	case VENC_TYPE_PAL:
		printf("VideoEnc set PAL.\n");
		p_data = venc_data_pal;
		limit  = sizeof(venc_data_pal)/sizeof(venc_data_pal[0]);

		/* Write data */
		for(i = 1; i < limit; i++) {	/* subaddress 0 is read only register. not write. */
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, i, *(p_data + i));
		}

		break;
	case VENC_TYPE_OUT_STOP:
		/* output stop */
		printf("VideoEnc set Output stop.\n");

		i2c_reg_write(VENC_I2C_SLAVE_ADDR,
		              VENC_I2C_OUT_PORT_CTRL_SADDR,
		              VENC_I2C_OUT_PORT_CTRL_STOP);
		break;
	default:
		return -1;
		break;
	}

#ifdef VENC_DEBUG
	printf("\n" "After register set.\n");
#if 0
	/* Read data */
	for(i = 0; i < 0x80; i++) {
		printf("reg:%02x data:%02x\n", i, i2c_reg_read(VENC_I2C_SLAVE_ADDR, i));
	}
#else
	uchar buffer[0x80];
	i2c_read(VENC_I2C_SLAVE_ADDR, 0, 1, buffer, 0x80);
	for(i = 0; i < 0x80; i++) {
		printf("reg:%02x data:%02x\n", i, buffer[i]);
	}
#endif
#endif	/* VENC_DEBUG */

	return 0;
}

int venc_rd_init(unsigned long type) {
	/* dummy function */
	return 0;
}

int hdmi_init(unsigned long type) {
	/* dummy function */
	return 0;
}

#else
#if defined(CONFIG_XARINA_VSS)
int venc_vss_init(unsigned long fmt)
{
	int limit = 0;
	int ret;
	int i;

	/* i2c device set */
	ret = i2c_set_bus_num(VENC_I2C_DEV);
	if (ret < 0) {
		return ret;
	}

	if (check_venc_support(fmt) == FALSE) {
		fmt = VENC_DEFAULT_TYPE;
		corefw_param_tvout_type_set(fmt);
		printf("selected video out format is not supported. format is changed to default.\n");
	}

#ifdef VENC_DEBUG
	/* Read data */
	printf("Befor register set.\n");
	for(i = 0; i < 0x5; i++) {
		printf("reg:%02x data:%02x\n", i, i2c_reg_read(VENC_I2C_SLAVE_ADDR, i));
	}
#endif	/* VENC_DEBUG */

    init_video_out_device_vss();

    switch(fmt) {
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5:
    case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_PAR_10_11:
    case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_PAR_10_11:
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_FULL:
        printf("VideoEnc set NTSC. TRC mode.\n");
		limit  = sizeof(venc_vss_data_trc_ntsc) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_vss_data_trc_ntsc[i][0], venc_vss_data_trc_ntsc[i][1]);
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_FULL:
    case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_59_54:
    case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_12_11:
        printf("VideoEnc set PAL. TRC mode.\n");
		limit  = sizeof(venc_vss_data_trc_pal) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_vss_data_trc_pal[i][0], venc_vss_data_trc_pal[i][1]);
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_OFF:
	case SC_SYS_VIDEO_VOUT_FMT_UNUSED:
		printf("VideoEnc set output stop.\n");
		limit  = sizeof(venc_vss_data_trc_mute) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_vss_data_trc_mute[i][0], venc_vss_data_trc_mute[i][1]);
		}
		break;
	default:
		printf("VideoEnc set is not done. This output type is not supported.\n");
		return -1;
		break;
	}
	
#ifdef VENC_DEBUG
	printf("\n" "After register set.\n");
	/* Read data */
	for(i = 0; i < 0x05; i++) {
		printf("reg:%02x data:%02x\n", i, i2c_reg_read(VENC_I2C_SLAVE_ADDR, i));
	}
#endif	/* VENC_DEBUG */

	return 0;
}
#else /* defined(CONFIG_XARINA_VSS */

int venc_evb_init(unsigned long fmt)
{
	int limit = 0;
	int ret;
	int i;
	int debug = 0;

	/* i2c device set */
	ret = i2c_set_bus_num(VENC_I2C_DEV);
	if (ret < 0) {
		return ret;
	}

	if (check_venc_support(fmt) == FALSE) {
		fmt = VENC_DEFAULT_TYPE;
		corefw_param_tvout_type_set(fmt);
		printf("selected video out format is not supported. format is changed to default.\n");
	}

#ifdef VENC_DEBUG
	/* Read data */
	printf("Befor register set.\n");
	for(i = 0; i < 0x80; i++) {
		printf("reg:%02x data:%02x\n", i, i2c_reg_read(VENC_I2C_SLAVE_ADDR, i));
	}
#endif	/* VENC_DEBUG */

	if (getenv("debug") != NULL) {
		debug = !strncmp(getenv("debug"), "yes", 3);
	}

	switch(fmt) {
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5:
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_PAR_10_11:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_PAR_10_11:
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_FULL:
		printf("VideoEnc set NTSC. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_ntsc) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_ntsc[i][0], venc_evb_data_trc_ntsc[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_ntsc[i][0], venc_evb_data_trc_ntsc[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_59_54:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_12_11:
		printf("VideoEnc set PAL. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_pal) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_pal[i][0], venc_evb_data_trc_pal[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_pal[i][0], venc_evb_data_trc_pal[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1_FULL:
		printf("VideoEnc set 480p 59.94Hz. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_480p_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_480p_60hz[i][0], venc_evb_data_trc_480p_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_480p_60hz[i][0], venc_evb_data_trc_480p_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1_FULL:
		printf("VideoEnc set 576p 50Hz. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_576p_50hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_576p_50hz[i][0], venc_evb_data_trc_576p_50hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_576p_50hz[i][0], venc_evb_data_trc_576p_50hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E_FULL:
		printf("VideoEnc set 576p 50Hz. HV Sync mode.\n");
		limit  = sizeof(venc_evb_data_hv_576p_50hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_hv_576p_50hz[i][0], venc_evb_data_hv_576p_50hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_hv_576p_50hz[i][0], venc_evb_data_hv_576p_50hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M_CROP_FULL:
		printf("VideoEnc set 720p 60Hz. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_720p_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_720p_60hz[i][0], venc_evb_data_trc_720p_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_720p_60hz[i][0], venc_evb_data_trc_720p_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E_CROP_FULL:
		printf("VideoEnc set 720p 60Hz. HV Sync mode.\n");
		limit  = sizeof(venc_evb_data_hv_720p_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_hv_720p_60hz[i][0], venc_evb_data_hv_720p_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_hv_720p_60hz[i][0], venc_evb_data_hv_720p_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M_CROP_FULL:
		printf("VideoEnc set 720p 59.94Hz. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_720p_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_720p_60hz[i][0], venc_evb_data_trc_720p_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_720p_60hz[i][0], venc_evb_data_trc_720p_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E_CROP_FULL:
		printf("VideoEnc set 720p 59.94Hz. HV Sync mode.\n");
		limit  = sizeof(venc_evb_data_hv_720p_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_hv_720p_60hz[i][0], venc_evb_data_hv_720p_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_hv_720p_60hz[i][0], venc_evb_data_hv_720p_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_SMPTE296M:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_SMPTE296M_CROP_FULL:
		printf("VideoEnc set 720p 29.97Hz. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_720p_30hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_720p_30hz[i][0], venc_evb_data_trc_720p_30hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_720p_30hz[i][0], venc_evb_data_trc_720p_30hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_29P_CEA861E_CROP_FULL:
		printf("VideoEnc set 720p 29.97Hz. HV Sync mode.\n");
		limit  = sizeof(venc_evb_data_hv_720p_30hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_hv_720p_30hz[i][0], venc_evb_data_hv_720p_30hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_hv_720p_30hz[i][0], venc_evb_data_hv_720p_30hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_SMPTE296M:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_SMPTE296M_CROP_FULL:
		printf("VideoEnc set 720p 30Hz. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_720p_30hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_720p_30hz[i][0], venc_evb_data_trc_720p_30hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_720p_30hz[i][0], venc_evb_data_trc_720p_30hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_30P_CEA861E_CROP_FULL:
		printf("VideoEnc set 720p 30Hz. HV Sync mode.\n");
		limit  = sizeof(venc_evb_data_hv_720p_30hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_hv_720p_30hz[i][0], venc_evb_data_hv_720p_30hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_hv_720p_30hz[i][0], venc_evb_data_hv_720p_30hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M_CROP_FULL:
		printf("VideoEnc set 720p 50Hz. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_720p_50hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_720p_50hz[i][0], venc_evb_data_trc_720p_50hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_720p_50hz[i][0], venc_evb_data_trc_720p_50hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E_CROP_FULL:
		printf("VideoEnc set 720p 50Hz. HV Sync mode.\n");
		limit  = sizeof(venc_evb_data_hv_720p_50hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_hv_720p_50hz[i][0], venc_evb_data_hv_720p_50hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_hv_720p_50hz[i][0], venc_evb_data_hv_720p_50hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_SMPTE296M:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_SMPTE296M_CROP_FULL:
		printf("VideoEnc set 720p 25Hz. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_720p_25hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_720p_25hz[i][0], venc_evb_data_trc_720p_25hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_720p_25hz[i][0], venc_evb_data_trc_720p_25hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_25P_CEA861E_CROP_FULL:
		printf("VideoEnc set 720p 25Hz. HV Sync mode.\n");
		limit  = sizeof(venc_evb_data_hv_720p_25hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_hv_720p_25hz[i][0], venc_evb_data_hv_720p_25hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_hv_720p_25hz[i][0], venc_evb_data_hv_720p_25hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7_CROP_FULL:
		printf("VideoEnc set 1080p 30Hz. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_1080p_30hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_1080p_30hz[i][0], venc_evb_data_trc_1080p_30hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_1080p_30hz[i][0], venc_evb_data_trc_1080p_30hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E_CROP_FULL:
		printf("VideoEnc set 1080p 30Hz. HV Sync mode.\n");
		limit  = sizeof(venc_evb_data_hv_1080p_30hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_hv_1080p_30hz[i][0], venc_evb_data_hv_1080p_30hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_hv_1080p_30hz[i][0], venc_evb_data_hv_1080p_30hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7_CROP_FULL:
		printf("VideoEnc set 1080p 29.97Hz. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_1080p_30hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_1080p_30hz[i][0], venc_evb_data_trc_1080p_30hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_1080p_30hz[i][0], venc_evb_data_trc_1080p_30hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E_CROP_FULL:
		printf("VideoEnc set 1080p 29.97Hz. HV Sync mode.\n");
		limit  = sizeof(venc_evb_data_hv_1080p_30hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_hv_1080p_30hz[i][0], venc_evb_data_hv_1080p_30hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_hv_1080p_30hz[i][0], venc_evb_data_hv_1080p_30hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7_CROP_FULL:
		printf("VideoEnc set 1080p 25Hz. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_1080p_25hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_1080p_25hz[i][0], venc_evb_data_trc_1080p_25hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_1080p_25hz[i][0], venc_evb_data_trc_1080p_25hz[i][1]);
			}
		}
		break;	
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E_CROP_FULL:
		printf("VideoEnc set 1080p 25Hz. HV Sync mode.\n");
		limit  = sizeof(venc_evb_data_hv_1080p_25hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_hv_1080p_25hz[i][0], venc_evb_data_hv_1080p_25hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_hv_1080p_25hz[i][0], venc_evb_data_hv_1080p_25hz[i][1]);
			}
		}
		break;	
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7_CROP_FULL:
		printf("VideoEnc set 1080i 60Hz. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_1080i_60hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_1080i_60hz[i][0], venc_evb_data_trc_1080i_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_1080i_60hz[i][0], venc_evb_data_trc_1080i_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7_CROP_FULL:
		printf("VideoEnc set 1080i 59.94Hz. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_1080i_60hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_1080i_60hz[i][0], venc_evb_data_trc_1080i_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_1080i_60hz[i][0], venc_evb_data_trc_1080i_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7_CROP_FULL:
		printf("VideoEnc set 1080i 50Hz. TRC mode.\n");
		limit  = sizeof(venc_evb_data_trc_1080i_50hz) / 2;
		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_evb_data_trc_1080i_50hz[i][0], venc_evb_data_trc_1080i_50hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_evb_data_trc_1080i_50hz[i][0], venc_evb_data_trc_1080i_50hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_OFF:
		printf("VideoEnc set output stop.\n");
		break;
	default:
		printf("VideoEnc set is not done. This output type is not supported.\n");
		return -1;
	}
	
#ifdef VENC_DEBUG
	printf("\n" "After register set.\n");
#if 0
	/* Read data */
	for(i = 0; i < 0x80; i++) {
		printf("reg:%02x data:%02x\n", i, i2c_reg_read(VENC_I2C_SLAVE_ADDR, i));
	}
#else
	uchar buffer[0x80];
	i2c_read(VENC_I2C_SLAVE_ADDR, 0, 1, buffer, 0x80);
	for(i = 0; i < 0x80; i++) {
		printf("reg:%02x data:%02x\n", i, buffer[i]);
	}
#endif
#endif	/* VENC_DEBUG */

	return 0;
}
int venc_rd_init(unsigned long fmt)
{
	int limit = 0;
	int ret;
	int i;
	int debug = 0;

	/* i2c device set */
	ret = i2c_set_bus_num(VENC_I2C_DEV);
	if (ret < 0) {
		return ret;
	}

	if (check_venc_support(fmt) == FALSE) {
		fmt = VENC_DEFAULT_TYPE;
		corefw_param_tvout_type_set(fmt);
		printf("selected video out format is not supported. format is changed to default.\n");

	}

	if (getenv("debug") != NULL) {
		debug = !strncmp(getenv("debug"), "yes", 3);
	}

#ifdef VENC_DEBUG
	/* Read data */
	printf("Befor register set.\n");
	for(i = 0; i < 0x80; i++) {
		printf("reg:%02x data:%02x\n", i, i2c_reg_read(VENC_I2C_SLAVE_ADDR, i));
	}
#endif	/* VENC_DEBUG */

	switch(fmt) {
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5:
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_PAR_10_11:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_PAR_10_11:
	case SC_SYS_VIDEO_VOUT_FMT_720_486_59I_BT656_5_FULL:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_FULL:
		printf("VideoEnc set NTSC. TRC mode.\n");
		limit  = sizeof(venc_rd_data_trc_ntsc) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_rd_data_trc_ntsc[i][0], venc_rd_data_trc_ntsc[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_rd_data_trc_ntsc[i][0], venc_rd_data_trc_ntsc[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_59_54:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_12_11:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_FULL:
		printf("VideoEnc set PAL. TRC mode.\n");
		limit  = sizeof(venc_rd_data_trc_pal) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, venc_rd_data_trc_pal[i][0], venc_rd_data_trc_pal[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", venc_rd_data_trc_pal[i][0], venc_rd_data_trc_pal[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_OFF:
		printf("VideoEnc set output stop.\n");
		break;
	default:
		printf("VideoEnc set is not done. This output type is not supported.\n");
		return -1;
	}
	
#ifdef VENC_DEBUG
	printf("\n" "After register set.\n");
#if 0
	/* Read data */
	for(i = 0; i < 0x80; i++) {
		printf("reg:%02x data:%02x\n", i, i2c_reg_read(VENC_I2C_SLAVE_ADDR, i));
	}
#else
	uchar buffer[0x80];
	i2c_read(VENC_I2C_SLAVE_ADDR, 0, 1, buffer, 0x80);
	for(i = 0; i < 0x80; i++) {
		printf("reg:%02x data:%02x\n", i, buffer[i]);
	}
#endif
#endif	/* VENC_DEBUG */

	return 0;
}

int hdmi_init(unsigned long fmt)
{
	int limit = 0;
	int ret;
	int i;
	int debug = 0;

	/* i2c device set */
	ret = i2c_set_bus_num(HDMI_I2C_DEV);
	if (ret < 0) {
		return ret;
	}

	if (check_hdmi_support(fmt) == FALSE) {
		fmt = HDMI_DEFAULT_TYPE;
		corefw_param_tvout_type_set(fmt);
		printf("selected video out format is not supported. format is changed to default.\n");
	}

	if (getenv("debug") != NULL) {
		debug = !strncmp(getenv("debug"), "yes", 3);
	}

	switch(fmt) {
	case SC_SYS_VIDEO_VOUT_FMT_OFF:
		printf("HDMI output stop\n");
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_BT1120_7_CROP_FULL:
		printf("HDMI set 1080p 60Hz. TRC mode.\n");
		limit  = sizeof(hdmi_data_trc_1080p_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_trc_1080p_60hz[i][0], hdmi_data_trc_1080p_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_trc_1080p_60hz[i][0], hdmi_data_trc_1080p_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60P_CEA861E_CROP_FULL:
		printf("HDMI set 1080p 60Hz. HV Sync mode.\n");
		limit  = sizeof(hdmi_data_hv_1080p_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_hv_1080p_60hz[i][0], hdmi_data_hv_1080p_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_hv_1080p_60hz[i][0], hdmi_data_hv_1080p_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_BT1120_7_CROP_FULL:
		printf("HDMI set 1080p 50Hz. TRC mode.\n");
		limit  = sizeof(hdmi_data_trc_1080p_50hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_trc_1080p_50hz[i][0], hdmi_data_trc_1080p_50hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_trc_1080p_50hz[i][0], hdmi_data_trc_1080p_50hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50P_CEA861E_CROP_FULL:
		printf("HDMI set 1080p 50Hz. HV Sync mode.\n");
		limit  = sizeof(hdmi_data_hv_1080p_50hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_hv_1080p_50hz[i][0], hdmi_data_hv_1080p_50hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_hv_1080p_50hz[i][0], hdmi_data_hv_1080p_50hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_BT1120_7_CROP_FULL:
		printf("HDMI set 1080p 30Hz. TRC mode.\n");
		limit  = sizeof(hdmi_data_trc_1080p_30hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_trc_1080p_30hz[i][0], hdmi_data_trc_1080p_30hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_trc_1080p_30hz[i][0], hdmi_data_trc_1080p_30hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_30P_CEA861E_CROP_FULL:
		printf("HDMI set 1080p 30Hz. HV Sync mode.\n");
		limit  = sizeof(hdmi_data_hv_1080p_30hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_hv_1080p_30hz[i][0], hdmi_data_hv_1080p_30hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_hv_1080p_30hz[i][0], hdmi_data_hv_1080p_30hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_BT1120_7_CROP_FULL:
		printf("HDMI set 1080p 29.97Hz. TRC mode.\n");
		limit  = sizeof(hdmi_data_trc_1080p_30hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_trc_1080p_30hz[i][0], hdmi_data_trc_1080p_30hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_trc_1080p_30hz[i][0], hdmi_data_trc_1080p_30hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_29P_CEA861E_CROP_FULL:
		printf("HDMI set 1080p 29.97Hz. HV Sync mode.\n");
		limit  = sizeof(hdmi_data_hv_1080p_30hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_hv_1080p_30hz[i][0], hdmi_data_hv_1080p_30hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_hv_1080p_30hz[i][0], hdmi_data_hv_1080p_30hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_BT1120_7_CROP_FULL:
		printf("HDMI set 1080p 25Hz. TRC mode.\n");
		limit  = sizeof(hdmi_data_trc_1080p_25hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_trc_1080p_25hz[i][0], hdmi_data_trc_1080p_25hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_trc_1080p_25hz[i][0], hdmi_data_trc_1080p_25hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_25P_CEA861E_CROP_FULL:
		printf("HDMI set 1080p 25Hz. HV Sync mode.\n");
		limit  = sizeof(hdmi_data_hv_1080p_25hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_hv_1080p_25hz[i][0], hdmi_data_hv_1080p_25hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_hv_1080p_25hz[i][0], hdmi_data_hv_1080p_25hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_BT1120_7_CROP_FULL:
		printf("HDMI set 1080i 60Hz. TRC mode.\n");
		limit  = sizeof(hdmi_data_trc_1080i_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_trc_1080i_60hz[i][0], hdmi_data_trc_1080i_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_trc_1080i_60hz[i][0], hdmi_data_trc_1080i_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_60I_CEA861E_CROP_FULL:
		printf("HDMI set 1080i 60Hz. HV Sync mode.\n");
		limit  = sizeof(hdmi_data_hv_1080i_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_hv_1080i_60hz[i][0], hdmi_data_hv_1080i_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_hv_1080i_60hz[i][0], hdmi_data_hv_1080i_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_BT1120_7_CROP_FULL:
		printf("HDMI set 1080i 59.94Hz. TRC mode.\n");
		limit  = sizeof(hdmi_data_trc_1080i_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_trc_1080i_60hz[i][0], hdmi_data_trc_1080i_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_trc_1080i_60hz[i][0], hdmi_data_trc_1080i_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_59I_CEA861E_CROP_FULL:
		printf("HDMI set 1080i 59.94Hz. HV Sync mode.\n");
		limit  = sizeof(hdmi_data_hv_1080i_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_hv_1080i_60hz[i][0], hdmi_data_hv_1080i_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_hv_1080i_60hz[i][0], hdmi_data_hv_1080i_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_BT1120_7_CROP_FULL:
		printf("HDMI set 1080i 50Hz. TRC mode.\n");
		limit  = sizeof(hdmi_data_trc_1080i_50hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_trc_1080i_50hz[i][0], hdmi_data_trc_1080i_50hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_trc_1080i_50hz[i][0], hdmi_data_trc_1080i_50hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1920_1080_50I_CEA861E_CROP_FULL:
		printf("HDMI set 1080i 50Hz. HV Sync mode.\n");
		limit  = sizeof(hdmi_data_hv_1080i_50hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_hv_1080i_50hz[i][0], hdmi_data_hv_1080i_50hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_hv_1080i_50hz[i][0], hdmi_data_hv_1080i_50hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_SMPTE296M_CROP_FULL:
		printf("HDMI set 720p 60Hz. TRC mode.\n");
		limit  = sizeof(hdmi_data_trc_720p_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_trc_720p_60hz[i][0], hdmi_data_trc_720p_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_trc_720p_60hz[i][0], hdmi_data_trc_720p_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_60P_CEA861E_CROP_FULL:
		printf("HDMI set 720p 60Hz. HV Sync mode.\n");
		limit  = sizeof(hdmi_data_hv_720p_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_hv_720p_60hz[i][0], hdmi_data_hv_720p_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_hv_720p_60hz[i][0], hdmi_data_hv_720p_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_SMPTE296M_CROP_FULL:
		printf("HDMI set 720p 59.94Hz. TRC mode.\n");
		limit  = sizeof(hdmi_data_trc_720p_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_trc_720p_60hz[i][0], hdmi_data_trc_720p_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_trc_720p_60hz[i][0], hdmi_data_trc_720p_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_59P_CEA861E_CROP_FULL:
		printf("HDMI set 720p 59.94Hz. HV Sync mode.\n");
		limit  = sizeof(hdmi_data_hv_720p_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_hv_720p_60hz[i][0], hdmi_data_hv_720p_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_hv_720p_60hz[i][0], hdmi_data_hv_720p_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_SMPTE296M_CROP_FULL:
		printf("HDMI set 720p 50Hz. TRC mode.\n");
		limit  = sizeof(hdmi_data_trc_720p_50hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_trc_720p_50hz[i][0], hdmi_data_trc_720p_50hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_trc_720p_50hz[i][0], hdmi_data_trc_720p_50hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_1280_720_50P_CEA861E_CROP_FULL:
		printf("HDMI set 720p 50Hz. HV Sync mode.\n");
		limit  = sizeof(hdmi_data_hv_720p_50hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_hv_720p_50hz[i][0], hdmi_data_hv_720p_50hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_hv_720p_50hz[i][0], hdmi_data_hv_720p_50hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59P_BT1358_1_FULL:
		printf("HDMI set 480p 59.94Hz. TRC mode.\n");
		limit  = sizeof(hdmi_data_trc_480p_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_trc_480p_60hz[i][0], hdmi_data_trc_480p_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_trc_480p_60hz[i][0], hdmi_data_trc_480p_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59P_CEA861E_FULL:
		printf("HDMI set 480p 59.94Hz. HV Sync mode.\n");
		limit  = sizeof(hdmi_data_hv_480p_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_hv_480p_60hz[i][0], hdmi_data_hv_480p_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_hv_480p_60hz[i][0], hdmi_data_hv_480p_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_PAR_10_11:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_BT656_5_FULL:
		printf("HDMI set 480i 59.94Hz. TRC mode.\n");
		limit  = sizeof(hdmi_data_trc_480i_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_trc_480i_60hz[i][0], hdmi_data_trc_480i_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_trc_480i_60hz[i][0], hdmi_data_trc_480i_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E_PAR_10_11:
	case SC_SYS_VIDEO_VOUT_FMT_720_480_59I_CEA861E_FULL:
		printf("HDMI set 480i 59.94Hz. HV Sync mode.\n");
		limit  = sizeof(hdmi_data_hv_480i_60hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_hv_480i_60hz[i][0], hdmi_data_hv_480i_60hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_hv_480i_60hz[i][0], hdmi_data_hv_480i_60hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_BT1358_1_FULL:
		printf("HDMI set 576p 50Hz. TRC mode.\n");
		limit  = sizeof(hdmi_data_trc_576p_50hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_trc_576p_50hz[i][0], hdmi_data_trc_576p_50hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_trc_576p_50hz[i][0], hdmi_data_trc_576p_50hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50P_CEA861E_FULL:
		printf("HDMI set 576p 50Hz. HV Sync mode.\n");
		limit  = sizeof(hdmi_data_hv_576p_50hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_hv_576p_50hz[i][0], hdmi_data_hv_576p_50hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_hv_576p_50hz[i][0], hdmi_data_hv_576p_50hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_59_54:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_PAR_12_11:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_BT656_5_FULL:
		printf("HDMI set 576i 50Hz. TRC mode.\n");
		limit  = sizeof(hdmi_data_trc_576i_50hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_trc_576i_50hz[i][0], hdmi_data_trc_576i_50hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_trc_576i_50hz[i][0], hdmi_data_trc_576i_50hz[i][1]);
			}
		}
		break;
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_PAR_59_54:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_PAR_12_11:
	case SC_SYS_VIDEO_VOUT_FMT_720_576_50I_CEA861E_FULL:
		printf("HDMI set 576i 50Hz. HV Sync mode.\n");
		limit  = sizeof(hdmi_data_hv_576i_50hz) / 2;

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(HDMI_I2C_SLAVE_ADDR, hdmi_data_hv_576i_50hz[i][0], hdmi_data_hv_576i_50hz[i][1]);
			if (debug) {
				printf("0x%02x,0x%02x\n", hdmi_data_hv_576i_50hz[i][0], hdmi_data_hv_576i_50hz[i][1]);
			}
		}
		break;
	default:
		printf("HDMI set is not done. This output type is not supported.\n");
		return -1;
	}



	return 0;
}

#endif /* defined(CONFIG_XARINA_VSS) */

#endif


int check_venc_support(unsigned char fmt)
{
	int i;
	int cnt;

    #if defined(CONFIG_XARINA_VSS)
    cnt = (int)sizeof(list_venc_vss_type);
    for (i = 0; i < cnt; i++) {
        if (fmt == list_venc_vss_type[i]) {
            return TRUE;
        }
    }
    #else /* defined(CONFIG_XARINA_VSS) */
	if ((xarina_hw_board_type_get() == XARINA_BOARD_EVB_1_0) ||
	    (xarina_hw_board_type_get() == XARINA_BOARD_EVB_1_1)) {
		cnt = (int)sizeof(list_venc_evb_type);
		for (i = 0; i < cnt; i++) {
			if (fmt == list_venc_evb_type[i]) {
				return TRUE;
			}
		}
	} else {
		cnt = (int)sizeof(list_venc_rd_type);
		for (i = 0; i < cnt; i++) {
			if (fmt == list_venc_rd_type[i]) {
				return TRUE;
			}
		}
	}
    #endif /* defined(CONFIG_XARINA_VSS) */


	return FALSE;
}

int check_hdmi_support(unsigned char fmt)
{
    #if !defined(CONFIG_XARINA_VSS)
	int i;
	int cnt;

	cnt = (int)sizeof(list_hdmi_type);

	for (i = 0; i < cnt; i++) {
		if (fmt == list_hdmi_type[i]) {
			return TRUE;
		}
	}
    #endif /* defined(CONFIG_XARINA_VSS) */

	return FALSE;
}

/*
	I do not do a loop of i2c_reg_read, when I want to perform read at once:
		i2c_read(VENC_I2C_SLAVE_ADDR, 0, 1, buffer, 0x80);

	I do not do a loop of i2c_reg_read, when I want to perform write at once:
		i2c00 performs repeat start every 64 bytes transmission of a message.
		Because it includes slave address and subaddress to transmission of a message size, it is every 62 bytes for pure data.
		I have to appoint a subaddress again when I perform repeat start in a write.

		I put a subaddress to do a write next in every 62 bytes of write data.

		Or I cannot do it, I divide by 62 bytes
			i2c_write(VENC_I2C_SLAVE_ADDR, 0x01, 1, &p_data[0x01], 0x3E);
			i2c_write(VENC_I2C_SLAVE_ADDR, 0x3F, 1, &p_data[0x3F], 0x3E);
			i2c_write(VENC_I2C_SLAVE_ADDR, 0x7D, 1, &p_data[0x7D], 0x03);
*/

