/*
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * (C) Copyright 2002
 * David Mueller, ELSOFT AG, <d.mueller@elsoft.ch>
 *
 * (C) Copyright 2003
 * Texas Instruments, <www.ti.com>
 * Kshitij Gupta <Kshitij@ti.com>
 *
 * (C) Copyright 2004
 * ARM Ltd.
 * Philippe Robin, <philippe.robin@arm.com>
 *
 * (C) Copyright 2007, 2011, 2013 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <common.h>
#include <i2c.h>
#include <flash.h>
#include <nand.h>
#include <asm/arch/xarina.h>
#include "boot_param.h"
#include "corefw_comm_param.h"
#include "cam_comm_param.h"
#include "cam_set_param.h"
#include "pinmux.h"
#include "hw_reg_ctrl.h"
#include "xarina_bool.h"
#include "hw_reg_pll3_pll4.h"
#include "set_mac_addr.h"


DECLARE_GLOBAL_DATA_PTR;
#if !defined(CONFIG_SYS_NO_FLASH)
extern flash_info_t flash_info[];		/* FLASH chips info */
#endif /* !defind(ONFIG_SYS_NO_FLASH) */

#if defined(CONFIG_SHOW_BOOT_PROGRESS)
void show_boot_progress(int progress)
{
    printf("Boot reached stage %d\n", progress);
}
#endif

unsigned long xarina_env_offset;
unsigned long xarina_env_offset_redund;
unsigned long xarina_env_range;


static inline void delay (unsigned long loops)
{
	__asm__ volatile ("1:\n"
		"subs %0, %1, #1\n"
		"bne 1b":"=r" (loops):"0" (loops));
}

/*
 * Miscellaneous platform dependent initialisations
 */
#if !defined(CONFIG_XARINA_SPL)
void set_miris(void)
{
    if (cam_param_senssystem_ddr_get() & CAM_PARAM_SENSSYS_VIDEOIN_MASK) {
    } else {
        uchar data = 1;
        cam_param_sysmiris_ddr_set(data);
        printf("set MIRIS\n");
    }
}

void devid_out(void)
{
	int devid;

	printf("DevID: ");
	devid = xarina_hw_devid_get();

	switch (devid) {
	case XARINA_DEVICE_ES1:
		if (xarina_hw_system_type_get() == XARINA_SYSTEM_PACKAGE) {
			printf("CXD4135GG ES1\n");
		} else {
			printf("CXD4235GG ES1\n");
		}
		break;
	case XARINA_DEVICE_ES2:
		if (xarina_hw_system_type_get() == XARINA_SYSTEM_PACKAGE) {
			printf("CXD4135GG ES2\n");
		} else {
			printf("CXD4235GG ES2\n");
		}
		break;
	case XARINA_DEVICE_ES3:
		if (xarina_hw_system_type_get() == XARINA_SYSTEM_PACKAGE) {
			printf("CXD4135GG ES3\n");
		} else {
			printf("CXD4235GG ES3\n");
		}
		break;
	default:
		/* unknown */
		printf("Unknown ID\n");
		break;
	}

	return;
}
void board_type_out(void)
{
	int type;

	printf("Board: ");
	type = xarina_hw_board_type_get();
    #if defined(CONFIG_XARINA_VSS)
	switch (type) {
    case XARINA_BOARD_VSS_TYPE:
        xarina_hw_board_type_print();
		break;
	default:
		/* unknown */
		printf("Unknown Type: 0x%04x\n", type);
		break;
	}
    #else /* defined(CONFIG_XARINA_VSS) */
	switch (type) {
	case XARINA_BOARD_EVB_1_0:
		printf("EVB 1.0\n");
		break;
	case XARINA_BOARD_EVB_1_1:
		printf("EVB 1.1+\n");
		break;
	case XARINA_BOARD_RD_IMX122:
		printf("RD (IMX122)\n");
		break;
	case XARINA_BOARD_RD_ICX692:
		printf("RD (ICX692)\n");
		break;
	case XARINA_BOARD_RD_IMX122AF:
		printf("RD (IMX122AF)\n");
		break;
	case XARINA_BOARD_RD_IMX139:
		printf("RD (IMX139)\n");
		break;
	case XARINA_BOARD_RD_IMX140:
		printf("RD (IMX140)\n");
		break;
	case XARINA_BOARD_RD_IMX139AF:
		printf("RD (IMX139AF)\n");
		break;
	case XARINA_BOARD_RD_IMX140AF:
		printf("RD (IMX140AF)\n");
		break;
	case XARINA_BOARD_RD_IMX140LA:
		printf("RD (IMX140LA)\n");
		break;
	case XARINA_BOARD_RD_ICX693:
		printf("RD (ICX693)\n");
		break;
	default:
		/* unknown */
		printf("Unknown Type\n");
		break;
	}
    #endif /* defined(CONFIG_XARINA_VSS) */

	return;
}


void boot_type_out(void)
{
	int type;

	printf("Boot:  ");
	type = xarina_hw_boot_type_get();

	switch (type) {
	case XARINA_BOOT_NAND_BOOT:
		printf("NAND Boot\n");
		break;
	case XARINA_BOOT_UART_BOOT:
		printf("UART Boot\n");
		break;
	case XARINA_BOOT_SD_BOOT:
		printf("SD Boot\n");
		break;
	default:
		/* unknown */
		printf("Unknown\n");
		break;
	}


	return;
}

#endif /* !defined(CONFIG_XARINA_SPL) */

#if defined(CONFIG_MMC)
extern int xarina_sd_initialize(bd_t *bis);
int board_mmc_init(bd_t *bis)
{
	int rc = 0;

	rc = xarina_sd_initialize(bis);

	return rc;
}
#endif /* CONFIG_MMC */


int xarina_init_f(void)
{
	/* GbE init */
#if defined(CONFIG_SMAC)
	xarina_hw_gbe_init();
#endif
	/* PG is selected as UART3. */
	/* Other as GPIO */
#if !defined(CONFIG_XARINA_SDSPL)
#if defined(CONFIG_XARINA_VSS)
	pinmux_uart3_set();
#else
	pinmux_gpio_uart3_set();
#endif
#else
#if defined(CONFIG_XARINA_VSS)
	pinmux_uart3_sdif_set();
#else
	pinmux_gpio_uart3_sdif_set();
#endif
#endif
//#ifndef CONFIG_XARINA_DDR_TEST
	/* NFIF clock set*/
	xarina_hw_nfif_clock_set(SYSREG1_NFIF_CKSEL_80MHZ);

	/* enable SDIF clock */
	xarina_hw_sdif_clock_en();
//#endif
#if !defined(CONFIG_XARINA_SPL)
	if (xarina_hw_nand_cont_type_get() == XARINA_NAND_CONTROLLER1) {
		xarina_env_offset = CONFIG_XARINA_NAND1_ENV_OFFSET;
		xarina_env_offset_redund = CONFIG_XARINA_NAND1_ENV_OFFSET_REDUND;
		xarina_env_range = CONFIG_XARINA_NAND1_ENV_RANGE;
	} else {
		xarina_env_offset = CONFIG_XARINA_NAND2_ENV_OFFSET;
		xarina_env_offset_redund = CONFIG_XARINA_NAND2_ENV_OFFSET_REDUND;
		xarina_env_range = CONFIG_XARINA_NAND2_ENV_RANGE;
	}
#endif
	return 0;
}

/* from start.S */
extern unsigned long _ddr_vector_start;
extern unsigned long _ddr_vector_end;

int xarina_init_r(void)
{
	unsigned long vtbl_start_addr = (unsigned long)&_ddr_vector_start;
	unsigned long vtbl_end_addr   = (unsigned long)&_ddr_vector_end;
	size_t len = vtbl_end_addr - vtbl_start_addr;

	/* vector table for ddr copy to BootRAM */
	memcpy((void *)XARINA_BOOTRAM_BASE, (void *)vtbl_start_addr, len);

	asm ("mcr p15, 0, %0, c7, c10, 0": :"r" (0));
	/* invalidate both caches and flush btb */
	asm ("mcr p15, 0, %0, c7, c7, 0": :"r" (0));
	/* mem barrier to sync things */
	asm ("mcr p15, 0, %0, c7, c10, 4": :"r" (0));

	/* BootRAM remap */
	xarina_hw_bootram_remap();

	return 0;
}


int board_init (void)
{
#if !defined(CONFIG_XARINA_SPL)
    int sys_type;
    ulong phys_addr;

	/* arch number of Versatile Board */
	gd->bd->bi_arch_number = MACH_TYPE_XARINA;

    sys_type = xarina_hw_system_type_get();

	/* adress of boot parameters */
    switch (sys_type) {
    case XARINA_SYSTEM_MODULE_H2:
    case XARINA_SYSTEM_MODULE_S2:
#ifdef PHYS_SDRAM_1_2G
        phys_addr = PHYS_SDRAM_1_2G;
#else
        phys_addr = PHYS_SDRAM_1;
#endif
        break;
    case XARINA_SYSTEM_PACKAGE:
#ifdef PHYS_SDRAM_1_2G
        phys_addr = PHYS_SDRAM_1_2G;
#else
        phys_addr = PHYS_SDRAM_1;
#endif
        break;
    case XARINA_SYSTEM_MODULE_H:
    case XARINA_SYSTEM_MODULE_S:
    default:
        phys_addr = PHYS_SDRAM_1;
        break;
    }
	gd->bd->bi_boot_params = phys_addr + 0x100;
#endif /* !defined(CONFIG_XARINA_SPL) */
	gd->flags = 0;

	return 0;
}


int misc_init_r (void)
{

#if !defined(CONFIG_XARINA_SPL)
	unsigned long vram_init_result;

	vram_init_result = *(unsigned long *)XARINA_VRAM_INIT_RESULT_ADDR;

	if (vram_init_result != XARINA_DDR_INIT_FAILURE) {
		setenv(CONFIG_XARINA_ENV_DDR_NUM, "2");
		xarina_hw_bus_config_set(2);
	} else {
		setenv(CONFIG_XARINA_ENV_DDR_NUM, "1");
		xarina_hw_bus_config_set(1);
	}
#endif
	setenv("verify", "n");

	return (0);
}

#ifdef CONFIG_CMD_NET
#ifdef CONFIG_SMAC
extern int smac_rgmii_initialize(bd_t *bis);
extern int smac_mii_initialize(bd_t *bis);
#endif /* CONFIG_SMAC */

int board_eth_init(bd_t *bis)
{
	int rc = 0;

#if defined(CONFIG_SMAC)
	if (xarina_hw_gbe_mode_get() == XARINA_GBE_RGMII) {
		rc = smac_rgmii_initialize(bis);
	} else {
		rc = smac_mii_initialize(bis);
	}
#endif /* CONFIG_SMAC */
	return rc;
}
#endif /* CONFIG_CMD_NET */


#if defined(CONFIG_CMD_NAND)
extern void board_nand1_info_set(struct nand_chip *nand);
extern void board_nand2_info_set(struct nand_chip *nand);

int board_nand_init(struct nand_chip *nand)
{

	if (xarina_hw_nand_cont_type_get() == XARINA_NAND_CONTROLLER1) {
		board_nand1_info_set(nand);
	} else {
		board_nand2_info_set(nand);
	}
	return 0;
}

#endif /* defined(CONFIG_CMD_NAND) */


/******************************
 Routine:
 Description:
******************************/
int dram_init (void)
{
    ulong phys_addr;
    ulong ram_size;

#if !defined(CONFIG_XARINA_SPL)
    int sys_type;

    sys_type = xarina_hw_system_type_get();

    switch (sys_type) {
    case XARINA_SYSTEM_MODULE_H2:
    case XARINA_SYSTEM_MODULE_S2:
#ifdef PHYS_SDRAM_1_2G
        phys_addr = PHYS_SDRAM_1_2G;
#else
        phys_addr = PHYS_SDRAM_1;
#endif
#ifdef PHYS_SDRAM_1_SIZE_2G
        ram_size = PHYS_SDRAM_1_SIZE_2G;
#else
        ram_size = PHYS_SDRAM_1_SIZE;
#endif
        break;
    case XARINA_SYSTEM_PACKAGE:
#ifdef PHYS_SDRAM_1_2G
        phys_addr = PHYS_SDRAM_1_2G;
#else
        phys_addr = PHYS_SDRAM_1;
#endif
#ifdef PHYS_SDRAM_1_SIZE_2G
        ram_size = PHYS_SDRAM_1_SIZE_2G;
#else
        ram_size = PHYS_SDRAM_1_SIZE;
#endif
        break;
    case XARINA_SYSTEM_MODULE_H:
    case XARINA_SYSTEM_MODULE_S:
    default:
        phys_addr = PHYS_SDRAM_1;
        ram_size = PHYS_SDRAM_1_SIZE;
        break;
    }
#else
    phys_addr = PHYS_SDRAM_1;
    ram_size = PHYS_SDRAM_1_SIZE;
#endif
	gd->bd->bi_dram[0].start	= phys_addr;
	gd->bd->bi_dram[0].size		= ram_size;

	return 0;
}
