/*
 * (C) Copyright 2007 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <common.h>
#include <command.h>
#include <asm/arch/xarina.h>
#include "corefw_boot_param.h"
#include "corefw_comm_param.h"
#include "cam_set_param.h"
#include "cam_comm_param.h"
#include "boot_param.h"
#include "xarina_bool.h"
#include "hw_reg_ctrl.h"
#include "pinmux.h"
#include "menu_update.h"

#define PARAM_TYPE_COREFW_COMM		(0)
#define PARAM_TYPE_CAMFW_SET		(1)
#define PARAM_TYPE_CAMFW_BLEMISH	(2)
#define PARAM_TYPE_CAMFW_SHADING	(3)

extern char cmd_buff[];
extern char console_buffer[];	/* readline buffer */
extern int data_down_load(unsigned int type, unsigned long addr);

static unsigned long camfw_param_addr = CAMFW_PARAM_DEFAULT_ADDR;
static unsigned long corefw_param_addr = COREFW_PARAM_DEFAULT_ADDR;

unsigned long boot_param_cam_addr_get(void)
{
	return camfw_param_addr;
}

unsigned long boot_param_core_addr_get(void)
{
	return corefw_param_addr;
}


static int is_param_data_file(struct s_param_data_image_header *p_header, const char *identifier)
{

	size_t n = strlen(identifier);
	if (memcmp((const void *)p_header->identifier, (const void *)identifier, n) != 0) {
		return FALSE;
	}
	return TRUE;
}


int boot_param_data_expansion(struct s_param_data_image_header *p_header,
		const char *identifier)
{
	if (is_param_data_file(p_header, identifier)) {
		if(p_header->addr < CONFIG_XARINA_COREFW_BOOT_INFO_ADDR) {
			return E_CMD_EXPANSION_ADDR_ERROR;
		}
		if(p_header->size > CONFIG_XARINA_PARAM_DATA_MAX_SIZE) {
			return E_CMD_EXPANSION_SIZE_ERROR;
		}
		memcpy((void *)p_header->addr, 
			(const void *)p_header 
				+ sizeof(struct s_param_data_image_header),
			(size_t)p_header->size);			
	} else {
		return E_CMD_EXPANSION_ID_ERROR;
	}
	
	if (strcmp(identifier, CAM_SET_PARAM_FILE_IDENTIFIER) == 0) {
		camfw_param_addr = p_header->addr;
	} else if (strcmp(identifier, COREFW_PARAM_FILE_IDENTIFIER) == 0) {
		corefw_param_addr = p_header->addr;
	} else {
		/* do nothing */
	}
	
	return E_CMD_EXPANSION_SUCCESS;
}


int boot_param_load_file_env(char *env, u32 addr, u32 size)
{
	int ret;
	char *file_name;

	file_name = getenv(env);

	if (file_name == NULL) {
		return FALSE;
	}

	sprintf(cmd_buff, "ubifsload %x %s %x", addr, file_name, size);	
	ret = run_command(cmd_buff, 0);
		
	return !ret;
}
int boot_param_load_file_name(char *name, u32 addr, u32 size)
{
	int ret;

	if (name == NULL) {
		return FALSE;
	}

	sprintf(cmd_buff, "ubifsload %x %s %x", addr, name, size);	
	ret = run_command(cmd_buff, 0);
	
	return !ret;
}
static int load_corefw_param(void)
{
	int ret;
	struct s_param_data_image_header *p_header 
		= (struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;
	
	/* load saved parameter */
	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_COREFW_COMM_PARAM,
				CONFIG_SYS_LOAD_ADDR, 0);
	printf("load CoreFW comm param ... ");
	if (!ret) {
		printf("not found!\n");
		return FALSE;
	}
			
	if (boot_param_data_expansion(p_header, COREFW_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done! at 0x%08lx size 0x%08lx\n", p_header->addr, p_header->size);
	} else {
		printf("invalid file!\n");
		return FALSE;
	}
	

	return TRUE;

}
static int load_cam_set_param(void)
{
	int ret;

	struct s_param_data_image_header *p_header 
		= (struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;

	/* load saved parameter */
	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM,
				CONFIG_SYS_LOAD_ADDR, 0);

	printf("load CAM FW setting param ... ");
	if (!ret) {
		printf("not found!\n");
		return FALSE;
	}
			
	if (boot_param_data_expansion(p_header, CAM_SET_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done! at 0x%08lx size 0x%08lx\n", p_header->addr, p_header->size);
	} else {
		printf("invalid file!\n");
		return FALSE;
	}
	
	return TRUE;

}
static int load_cam_rev_data(void)
{
	int ret;
	struct s_param_data_image_header *p_header 
		= (struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;

	/* load saved parameter */
	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_CAMFW_BLEMISH_DATA,
				CONFIG_SYS_LOAD_ADDR,
				0);

	printf("load cam blemish correct data ... ");

	if (!ret) {
		printf("not found!\n");
		return FALSE;
	}
	if (boot_param_data_expansion(p_header, CAMFW_BLEMISH_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done! at 0x%08lx size 0x%08lx\n", p_header->addr, p_header->size);
	} else {
		printf("invalid file!\n");
		return FALSE;
	}


	return TRUE;

}

static int load_cam_shading_data(void)
{
	int ret;
	struct s_param_data_image_header *p_header 
		= (struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;

	/* load saved parameter */
	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_CAMFW_SHADING_DATA,
				CONFIG_SYS_LOAD_ADDR,
				0);

	printf("load cam shading correct data ... ");

	if (!ret) {
		printf("not found!\n");
		return FALSE;
	}
	if (boot_param_data_expansion(p_header, CAMFW_SHADING_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done! at 0x%08lx size 0x%08lx\n", p_header->addr, p_header->size);
	} else {
		printf("invalid file!\n");
		return FALSE;
	}


	return TRUE;

}

int boot_param_load(void)
{
	int ret_corefw;
	int ret_camfw;
	int ret_blemish;
	int ret_shading;
	int ret;

	if (xarina_hw_boot_type_get() == XARINA_BOOT_NAND_BOOT) {
		/* Load coreFW comm param */
		ret_corefw = load_corefw_param();
		/* Load cam set_param */
		ret_camfw = load_cam_set_param();
		/* Load cam rev data */
		ret_blemish = load_cam_rev_data();
		/* Load cam shading data */
		ret_shading = load_cam_shading_data();
	
	
		ret = (ret_corefw && ret_camfw && ret_blemish && ret_shading) ? 
			XARINA_BOOT_PARAM_LOAD_SUCCESS : 
			XARINA_BOOT_PARAM_LOAD_ERROR;
	} else {
		ret = XARINA_BOOT_PARAM_UART_BOOT;
	}	
#if 0
	boot_type = xarina_hw_boot_type_get();
	if (boot_type != XARINA_BOOT_UART_BOOT) {
		/* NOR Boot */

		/* load "corefwfw param" */
		b_top = boot_param_load_top();

		/* load "cam fw set param" & "cam fw rev data" */
		b_bottom = boot_param_load_bottom();

		if (b_top && b_bottom) {
			/* boot param load success */
			status = XARINA_BOOT_PARAM_LOAD_SUCCESS;
		} else {
			/* boot param load error */
			status = XARINA_BOOT_PARAM_LOAD_ERROR;
			printf("\nAutoboot is disable.\n");
		}
	} else {
		/* UART Boot */
		corefw_comm_param_dummydata_load();		/* dummy load */
		cam_set_param_dummydata_load();			/* dummy load */
		status = XARINA_BOOT_PARAM_UART_BOOT;
	}
#endif

	return ret;
}

#if 0
int boot_param_cam_video_size_frame_set(unsigned long type)
{
	unsigned char senssys, basicfps;
	unsigned char videosize, videoframe;
	int b_ret;

	b_ret = insel_videoin_info_get(type, &senssys, &basicfps, &videosize, &videoframe);
	if (b_ret) {
		cam_param_videosize_ddr_set(videosize);
		cam_param_videoframe_ddr_set(videoframe);
	}

	return b_ret;
}


static int boot_param_cam_default_param_load(	unsigned char senssys, unsigned char basicfps,
						unsigned char videosize, unsigned char videoframe)
{
	int b_ret = FALSE;

	switch (cam_set_senssys_type_check(senssys)) {
	case CAM_PARAM_SENSSYS_TYPE_SENSOR:
		b_ret = cam_set_default_param_load(senssys, basicfps);
		break;

	case CAM_PARAM_SENSSYS_TYPE_VIDEOIN:
		b_ret = cam_set_default_param_videoin_load();
		if (b_ret) {
			cam_param_senssystem_ddr_set(senssys);
			cam_param_basicfps_ddr_set(basicfps);
			cam_param_videosize_ddr_set(videosize);
			cam_param_videoframe_ddr_set(videoframe);
		}
		break;
	default:
		break;
	}

	if (!b_ret) {
		/* error */
		cam_set_param_dummydata_load();

		printf("error: There are not default data corresponding to sensor classification and\n"
		       "       the frame rate.\n"
		       "       Therefore CAM FW SET PARAMETER is not loaded.\n");

	}

	return b_ret;
}


int boot_param_cam_videoin_default_param_load(unsigned long type)
{
	unsigned char senssys, basicfps, videosize, videoframe;
	int b_ret = FALSE;

	b_ret = insel_videoin_info_get(type, &senssys, &basicfps, &videosize, &videoframe);
	if (!b_ret) {
		printf("unknown type(0x%08lX).\n", type);
		printf("system error.\n");
		return b_ret;
	}

	b_ret = boot_param_cam_default_param_load(senssys, basicfps, videosize, videoframe);

	return b_ret;
}


int boot_param_cam_sens_default_param_load(unsigned char senssys, unsigned char basicfps)
{
	int b_ret;

	b_ret = boot_param_cam_default_param_load(senssys, basicfps, 0xff, 0xff);

	return b_ret;
}


int boot_param_cam_restore(void)
{
	unsigned char senssys, basicfps;
	unsigned char videosize = 0xff, videoframe = 0xff;
	int b_ret;

	senssys    = cam_param_senssystem_ddr_get();
	basicfps   = cam_param_basicfps_ddr_get();
	if (cam_set_senssys_type_check(senssys) == CAM_PARAM_SENSSYS_TYPE_VIDEOIN) {
		videosize  = cam_param_videosize_ddr_get();
		videoframe = cam_param_videoframe_ddr_get();
		insel_videoin_info_def_change(&senssys, &basicfps, &videosize, &videoframe);
	}

	/***********************************/
	/* memory copy 'cam fw set param'  */
	/***********************************/
	b_ret = boot_param_cam_default_param_load(senssys, basicfps, videosize, videoframe);

	/*********************************************/
	/* memory copy 'cam fw defect revision data' */
	/*********************************************/
	cam_rev_data_load();

	return b_ret;
}

#endif
int boot_param_corefw_param_restore(void)
{
	return load_corefw_param();
}

int do_load_param(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int param_type = 0;
	int load_type = 0;
	int ret;
	struct s_param_data_image_header *p_header 
		= (struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;
	
	if (argc != 3) {
		cmd_usage(cmdtp);
		return 0;
	}

	if (strcmp(argv[1], "copa") == 0) {
		param_type = PARAM_TYPE_COREFW_COMM;
	} else if (strcmp(argv[1], "capa") == 0) {
		param_type = PARAM_TYPE_CAMFW_SET;	
	} else if (strcmp(argv[1], "blda") == 0) {
		param_type = PARAM_TYPE_CAMFW_BLEMISH;	
	} else {
		printf("arg[1] must be copa | capa | blda\n");
		return FALSE;  
	}
	
	if (strcmp(argv[2], "t") == 0) {
		load_type = DOWNLOAD_TYPE_TFTP;
	} else if (strcmp(argv[2], "x") == 0) {
		load_type = DOWNLOAD_TYPE_LOADX;	
	} else if (strcmp(argv[2], "u") == 0) {
		load_type = DOWNLOAD_TYPE_UBIFS;
	} else {
		printf("arg[2] must be t | x | u\n");
		return FALSE;  
	}

	if ((load_type == DOWNLOAD_TYPE_LOADX) |
	    (load_type == DOWNLOAD_TYPE_TFTP)) {	
		/* not create filesize variable when tftp download size 0 file */
		run_command("setenv filesize", 0);	/* delete filesize */
	
		ret = data_down_load(load_type, CONFIG_SYS_LOAD_ADDR);
		printf("load file ... ");
	
		if (ret <= 0 || getenv("filesize") == NULL) {
			printf("error\n");
			return FALSE;
		}
	} else {
		ret = readline("FILENAME: ");
		if (ret <= 0) {
			return FALSE;
		}
		ret = boot_param_load_file_name(console_buffer, CONFIG_SYS_LOAD_ADDR, 0);
		if (!ret) {
			printf("error\n");
			return FALSE;
		}
	}
	
	switch (param_type) {
	case PARAM_TYPE_COREFW_COMM:
	if (boot_param_data_expansion(p_header, COREFW_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
			printf("done!\n");
		} else {
			printf("invalid file!\n");
			return FALSE;
		}	
		break;
	case PARAM_TYPE_CAMFW_SET:
	if (boot_param_data_expansion(p_header, CAM_SET_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
			printf("done!\n");
		} else {
			printf("invalid file!\n");
			return FALSE;
		}	
		break;
	case PARAM_TYPE_CAMFW_BLEMISH:
	if (boot_param_data_expansion(p_header, CAMFW_BLEMISH_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
			printf("done!\n");
		} else {
			printf("invalid file!\n");
			return FALSE;
		}	
		break;
	default:
		/* do nothing */
		break;
	}


	return TRUE;
}


U_BOOT_CMD(
	loadprm, 3, 0,	do_load_param,
	"load paramter",
	"<parameter (copa | capa | blda)> <download type (t | x | u)>\n"
	"    - load 'parameter' by 'download type'\n"
	"    - 'download type' t is TFTP\n"
	"    - 'download type' x is XMODEM\n"
	"    - 'download type' u is UBIFS\n"
);




