/*
 * (C) Copyright 2007 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <command.h>
#include <common.h>
#include "cmd_corefw.h"
#include "corefw.h"
#include "corefw_boot_param.h"
#include "corefw_comm_param.h"
#include "hw_reg_ctrl.h"
#include "cam_comm_param.h"
#include "venc.h"
#include "vdec.h"
#include "pinmux.h"
#include "xarina_bool.h"
#include "cam_set_param.h"
#include "menu_update.h"
#include "boot_param.h"

#define LOAD_TYPE_UBIFS		(1)
#define LOAD_TYPE_TFTP		(2)
#define LOAD_TYPE_XMODEM	(3)

extern char cmd_buff[];

extern int data_down_load(unsigned int type, unsigned long addr);

static struct tagS_Arm9_BootResInfo *p_boot_info = (struct tagS_Arm9_BootResInfo *)CONFIG_XARINA_COREFW_BOOT_INFO_ADDR;
static struct s_input_table input_table[] =	{
							{CAM_PARAM_SENSSYS_IMX035, ENV_BIN_FILE_IMX035},
							{CAM_PARAM_SENSSYS_IMX122, ENV_BIN_FILE_IMX122},
							{CAM_PARAM_SENSSYS_ICX445, ENV_BIN_FILE_ICX445},
						};



static void boot_info_notifyflag_set(unsigned long data)
{
	p_boot_info->notifyFlag = data;
	return;
}
#if 0
static unsigned long boot_info_notifyflag_get(void)
{
	return p_boot_info->notifyFlag;
}
#endif
static void boot_info_responsecode_set(unsigned long data)
{
	p_boot_info->responseCode = data;
	return;
}

static unsigned long boot_info_responsecode_get(void)
{
	return p_boot_info->responseCode;
}


/* corefw boot check */
static int boot_check(unsigned long res, int b_complete)
{
	unsigned long mask = 0;
	int ret = E_CMD_BOOT_NOT_YET;

	if (!b_complete) {
		mask = ARM9_BOOT_RES_OK;
	} else {
		mask = ARM9_BOOT_RES_OK_COMP;
	}

	if (res != ARM9_BOOT_RES_WAIT) {
		if ((res & ARM9_BOOT_RES_TYPE_ERR) == ARM9_BOOT_RES_TYPE_ERR) {
			/* boot error */
			ret = E_CMD_BOOT_ERROR;
	
		} else if ((res & mask) == mask) {
			/* OK */
			if ((res & ARM9_BOOT_RES_WARNING_MASK) != 0) {
				ret = E_CMD_BOOT_WARNING;
			} else {
				ret = E_CMD_BOOT_SUCCESS;
			}
		}
	}

	return ret;
}


static void pll2_pll7_start(void)
{
	unsigned char clock_type = cam_param_pll2_clk_ddr_get();
	
	xarina_hw_pll2_pll7_unselect();

	xarina_hw_pll2_pll7_select(clock_type);
	
	return;
}

static void pll3_pll4_start(void)
{
	unsigned char clock_type = corefw_param_tvout_type_get();
	
	xarina_hw_pll3_pll4_unselect();

	xarina_hw_pll3_pll4_select(clock_type);
	
	return;
}

/* video output init */
static int video_out_init(void)
{
	/* sdout clock enable */
	xarina_hw_sdout_clock_enable();

	switch(corefw_param_tvout_type_get()) {
	case COREFW_VIDEO_TVOUT_720X486_59I:
		venc_init(VENC_TYPE_NTSC);
		printf("VideoEnc set NTSC.\n");
		break;
	case COREFW_VIDEO_TVOUT_720X576_50I:
		venc_init(VENC_TYPE_PAL);
		printf("VideoEnc set PAL.\n");
		break;
	default:
		venc_init(VENC_TYPE_OUT_STOP);
		printf("VideoEnc set Output Stop.\n");
		break;
	}

	return 0;
}

#if 0
/* video input init */
static int video_in_init(void)
{
	switch(cam_param_basicfps_ddr_get()) {
	case CAM_PARAM_BASICFPS_30FPS:
		/* 30fps - NTSC */
		vdec_init(VDEC_TYPE_NTSC);
		printf("VideoDec set NTSC.\n");
		break;
	case CAM_PARAM_BASICFPS_25FPS:
		/* 25fps - PAL */
		vdec_init(VDEC_TYPE_PAL);
		printf("VideoDec set PAL.\n");
		break;
	default:
		printf("Unknown BASICFPS. VideoDec don't set.\n");
		return -1;
	}

	return 0;
}
#endif 


static unsigned long osd_color_palette_load(void)
{
	int ret;
	struct s_param_data_image_header *p_header = 
		(struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;


	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_OSD_COLOR_PALETTE,
				CONFIG_SYS_LOAD_ADDR, 0);

	printf("OSD color palette load ... ");

	if (!ret) {
		printf("not found!\n");
		return 0;
	}
			
	if (boot_param_data_expansion(p_header, OSD_COLOR_PALETTE_FILE_IDENTIFIER) 
	 	== E_CMD_EXPANSION_SUCCESS) {
		printf("at %08lx ... done!\n", p_header->addr);
	} else {
		printf("invalid file!\n");
		return 0;
	}

	return p_header->addr;	/* load saved parameter */

}

static unsigned long osd_data_load(void)
{
	int ret;
	struct s_param_data_image_header *p_header = 
		(struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;

	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_OSD_DATA,
				CONFIG_SYS_LOAD_ADDR, 0);

	printf("OSD data load ... ");
	if (!ret) {
		printf("not found!\n");
		return 0;
	}
			
	if (boot_param_data_expansion(p_header, OSD_DATA_FILE_IDENTIFIER) 
	 	== E_CMD_EXPANSION_SUCCESS) {
		printf("at %08lx ... done!\n", p_header->addr);
	} else {
		printf("invalid file!\n");
		return 0;
	}
	return p_header->addr;
}	


static unsigned long mask_data_load(void)
{
	int ret;
	struct s_param_data_image_header *p_header = 
		(struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;
	

	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_MASK_DATA,
				CONFIG_SYS_LOAD_ADDR, 0);
	printf("MASK data load ... ");
	if (!ret) {
		printf("not found!\n");
		return 0;
	}
			
	if (boot_param_data_expansion(p_header, MASK_DATA_FILE_IDENTIFIER) 
	 	== E_CMD_EXPANSION_SUCCESS) {
		printf("at %08lx ... done!\n", p_header->addr);
	} else {
		printf("invalid file!\n");
		return 0;
	}

	return p_header->addr;
}


static int osd_mask_load(void)
{
	unsigned long outStatus;	/* OSD & Mask output setting */
	unsigned long osdColorPltAddr;	/* OSD Color Palette load address */
	unsigned long osdDataAddr;	/* OSD data load address */
	unsigned long maskDataAddr;	/* Mask data load address */

	/* osd data & mask data output setting get */
	outStatus = corefw_param_osd_mask_output_status_get();

	/* OSD output setting is ON */
	if ((outStatus & ARM9_VIDEO_OSD_OUTPUT) == ARM9_VIDEO_OSD_OUTPUT) {
		/* OSD Color Palette load */
		osdColorPltAddr = osd_color_palette_load();
		/* OSD data load */
		osdDataAddr = osd_data_load();
	}

	/* Mask output setting is ON */
	if ((outStatus & ARM9_VIDEO_MASK_OUTPUT) == ARM9_VIDEO_MASK_OUTPUT) {
		/* Mask data load */
		maskDataAddr = mask_data_load();
	}

	return 0;
}

static int is_corefw_bin_file(struct corefw_image_header *p_header)
{
	char *p_identifier = COREFW_IMAGE_IDENTIFIER;
	size_t n = strlen(COREFW_IMAGE_IDENTIFIER);

	if (memcmp((const void *)p_header->identifier, p_identifier, n) != 0) {
		return FALSE;
	}
	return TRUE;
}
static int is_sensfw_bin_file(struct sensfw_image_header *p_header)
{
	char *p_identifier = SENSFW_IMAGE_IDENTIFIER;
	size_t n = strlen(SENSFW_IMAGE_IDENTIFIER);

	if (memcmp((const void *)p_header->identifier, p_identifier, n) != 0) {
		return FALSE;
	}
	return TRUE;
}

void corefw_info_out(struct corefw_image_header *p_header)
{
	printf("\n");
	printf("## Core F/W at %08x ...\n", (unsigned int)p_header->corefw_addr);
	printf("   date:    %s\n", p_header->date);
	printf("   version: %s\n", p_header->version);
	printf("   info:    %s\n", p_header->info);
	printf("\n");

	return;
}

void sensfw_info_out(struct sensfw_image_header *p_header)
{
	printf("\n");
	printf("## Sensor F/W at %08x ...\n", (unsigned int)p_header->sensfw_addr);
	printf("   date:    %s\n", p_header->date);
	printf("   version: %s\n", p_header->version);
	printf("   info:    %s\n", p_header->info);
	printf("\n");

	return;
}

void sensfw_info_out_all(void)
{
	int i, ret;
	
	for (i = 0; i < INPUT_MAX_NUM; i++) {
		ret = boot_param_load_file_env(input_table[i].env, CONFIG_SYS_LOAD_ADDR, 0);
		if (!ret) {
			printf("not found!\n");
			continue;
		} else {
			sensfw_info_out((struct sensfw_image_header *)CONFIG_SYS_LOAD_ADDR);
		}
	}
	
	return;
}

static void corefw_image_expansion(struct corefw_image_header *p_header)
{

	void *src_addr;

	/* Load Vector Area */	
	src_addr = (void *)p_header
			+ sizeof(struct corefw_image_header);
	if ((void *)p_header->corefw_addr == NULL) {
		printf("FW addr is invalid\n");
		return;
	}
	memcpy((void *)p_header->vector_addr, (const void *)src_addr,
		(size_t)p_header->vector_size);
	/* Load CoreFW Program Area */
	src_addr += p_header->vector_size;
	memcpy((void *)p_header->corefw_addr, (const void *)src_addr,
		(size_t)p_header->corefw_size);
	/* Load Audio FW Program Area */
	src_addr += p_header->corefw_size;
	memcpy((void *)p_header->audiofw_addr, (const void *)src_addr,
		(size_t)p_header->audiofw_size);


	return;
}
static void sensfw_image_expansion(struct sensfw_image_header *p_header)
{

	void *src_addr;

	src_addr = (void *)p_header
			+ sizeof(struct sensfw_image_header);
	/* Load Program Area */
	
	if ((void*)p_header->sensfw_addr == NULL) {
		printf("FW addr is invalid\n");
		return;
	}
	/* Load Program Area */
	memcpy((void *)p_header->sensfw_addr, (const void *)src_addr,
		(size_t)p_header->sensfw_size);


	return;
}
int load_corefw_bin(void *addr, char *cfile, int load_type)
{
	int ret;
	struct corefw_image_header *p_header = (struct corefw_image_header *)addr;
	
	/* load saved parameter */
	if (load_type == LOAD_TYPE_UBIFS) {
		if (cfile == NULL) {
			ret = boot_param_load_file_env(CONFIG_XARINA_ENV_COREFW_BIN, (u32)addr, 0);
		} else {
			ret = boot_param_load_file_name(cfile, (u32)addr, 0);
		}
	} else if (load_type == LOAD_TYPE_TFTP) {
		printf("load CoreFW binary file\n");
		/* not create filesize variable when tftp download size 0 file */
		run_command("setenv filesize", 0);	/* delete filesize */
		ret = data_down_load(DOWNLOAD_TYPE_TFTP, (unsigned long)addr);
		if (ret <= 0) {
			return FALSE;
		}

	} else if (load_type == LOAD_TYPE_XMODEM) {
		printf("load CoreFW binary file\n");
		/* not create filesize variable when tftp download size 0 file */
		run_command("setenv filesize", 0);	/* delete filesize */
		ret = data_down_load(DOWNLOAD_TYPE_LOADX, (unsigned long)addr);
		if (ret <= 0) {
			return FALSE;
		}
	
	} else {
		printf("load type error\n");
		return FALSE;
	}
	
	printf("load CoreFW binary file ... ");
	if (!ret) {
		printf("not found!\n");
		return FALSE;
	}
			

	if (is_corefw_bin_file(p_header)) {
		printf("done! at 0x%08lx size 0x%08lx\n", 
			p_header->corefw_addr, p_header->corefw_size);
	} else {
		printf("invalid file!\n");
		return FALSE;
	}
	
	
	return TRUE;

}

static char* get_sensfw_file_name_env(unsigned char senssystem)
{
	int i;
	
	for (i = 0; i < INPUT_MAX_NUM; i++) {
		if (input_table[i].id == senssystem) {
			return input_table[i].env;
		}
	}

	return NULL;
}


int load_sensfw_bin(void *addr, char *sfile, int load_type)
{
	int ret;
	unsigned char senssystem;
	char *sensor_env;
	struct sensfw_image_header *p_header = (struct sensfw_image_header *)addr;
	
	/* load saved parameter */
	if (load_type == LOAD_TYPE_UBIFS) {
		if (sfile == NULL) {
			senssystem = cam_param_senssystem_ddr_get();
			sensor_env = get_sensfw_file_name_env(senssystem);			
			ret = boot_param_load_file_env(sensor_env, (u32)addr, 0);
		} else {
			ret = boot_param_load_file_name(sfile, (u32)addr, 0);
		}
	} else if (load_type == LOAD_TYPE_TFTP) {
		printf("load Sensor binary file\n");
		/* not create filesize variable when tftp download size 0 file */
		run_command("setenv filesize", 0);	/* delete filesize */
		ret = data_down_load(DOWNLOAD_TYPE_TFTP, (unsigned long)addr);
		if (ret <= 0) {
			return FALSE;
		}

	} else if (load_type == LOAD_TYPE_XMODEM) {
		printf("load Sensor binary file\n");
		/* not create filesize variable when tftp download size 0 file */
		run_command("setenv filesize", 0);	/* delete filesize */
		ret = data_down_load(DOWNLOAD_TYPE_LOADX, (unsigned long)addr);
		if (ret <= 0) {
			return FALSE;
		}
	
	} else {
		printf("load type error\n");
		return FALSE;
	}
	
	printf("load Sensor binary file ... ");
	if (!ret) {
		printf("not found!\n");
		return FALSE;
	}
			

	if (is_sensfw_bin_file(p_header)) {
		printf("done! at 0x%08lx size 0x%08lx\n", 
			p_header->sensfw_addr, p_header->sensfw_size);
	} else {
		printf("invalid file!\n");
		return FALSE;
	}
	
	
	return TRUE;

}
int do_corefwinfo(cmd_tbl_t *cmdtp, int flag, int argc, char *const argv[])
{
	struct corefw_image_header *p_cheader;	/* CoreFW image top address */
	struct sensfw_image_header *p_sheader;	/* SensorFW image top address */
	char *cfile = NULL;
	char *sfile = NULL;

	if (argc >= 2) {
		cfile = argv[1];
	}
	if (argc >= 3) {
		sfile = argv[2];
	}

	p_cheader = (struct corefw_image_header *)CONFIG_SYS_LOAD_ADDR;
	p_sheader = (struct sensfw_image_header *)(CONFIG_SYS_LOAD_ADDR +
			 CONFIG_XARINA_COREFW_BIN_MAX_SIZE);
	
	/* load coreFW file */
	if (!load_corefw_bin((void *)p_cheader, cfile, LOAD_TYPE_UBIFS)) {
		return E_CMD_BOOTARM9_FAILURE;
	};
	/* load sensorFW file */
	if (!(cam_param_senssystem_ddr_get() & CAM_PARAM_SENSSYS_VIDEOIN_MASK)) {
		if (!load_sensfw_bin((void *)p_sheader, sfile, LOAD_TYPE_UBIFS)) {
			return E_CMD_BOOTARM9_FAILURE;
		}
	}	
	
	/* put CoreFW info */
	corefw_info_out(p_cheader);

	/* put sensorFW info */
	if (!(cam_param_senssystem_ddr_get() & CAM_PARAM_SENSSYS_VIDEOIN_MASK)) {
		sensfw_info_out(p_sheader);
	}

	return 0;
}

static int boot_corefw(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[],
			 int b_complete, int load_type)
{
	struct corefw_image_header *p_cheader;	/* core f/w image top address */
	struct sensfw_image_header *p_sheader;	/* sensor f/w image top address */
	unsigned long 	res;			/* boot entry code */
	int 		err;			/* boot entry code */
	unsigned long	timeout;		/* timeout value */
	int 		i;
	int		b_tmout = FALSE;	/* timeout flag */
	char 		*cfile = NULL;		/* coreFW file name */
	char 		*sfile = NULL;		/* sensorFW file name */

	if (argc >= 2) {
		cfile = argv[1];
	}
	if (argc >= 3) {
		sfile = argv[2];
	}

	p_cheader = (struct corefw_image_header *)CONFIG_SYS_LOAD_ADDR;
	p_sheader = (struct sensfw_image_header *)(CONFIG_SYS_LOAD_ADDR +
			 CONFIG_XARINA_COREFW_BIN_MAX_SIZE);
	

	/* load  OSD Color Palette & OSD data & Mask data */
	osd_mask_load();

	/* load coreFW file */
	if (!load_corefw_bin((void *)p_cheader, cfile, load_type)) {
		return E_CMD_BOOTARM9_FAILURE;
	}
	
	/* load sensorFW file */
	if (!(cam_param_senssystem_ddr_get() & CAM_PARAM_SENSSYS_VIDEOIN_MASK)) {
		if (!load_sensfw_bin((void *)p_sheader, sfile, load_type)) {
			return E_CMD_BOOTARM9_FAILURE;
		}
	}	
	
	/* video device init */
	video_out_init();
	
	/* Put CoreFW info */
	corefw_info_out(p_cheader);
	
	/* Put SensorFW info */
	if (!(cam_param_senssystem_ddr_get() & CAM_PARAM_SENSSYS_VIDEOIN_MASK)) {
		sensfw_info_out(p_sheader);
	}
	printf("\nBoot Start.\n");


	timeout = CFG_ARM9FW_BOOT_TIMEOUT;

	/* pinmux */
	pinmux_set();
	printf("PinMux Set.\n");

	/* pll2/pll7 start */
	pll2_pll7_start();
	
	/* pll3/pll4 start */
	pll3_pll4_start();

	for (i = 0; i < CONFIG_XARINA_COREFW_BOOT_RETRY + 1; i++) {
	
		if (i != 0) {
			/* retry only */
			printf("\nCoreFW Boot Retry(%d).\n", i);
		}

		/* start entry type initialize */
		boot_info_notifyflag_set(ARM9_BOOT_NOTIFY_OFF);
		/* start entry code initialize */
		boot_info_responsecode_set(ARM9_BOOT_RES_WAIT);

		/************************/
		/* cpu1 reset           */
		/************************/
		xarina_hw_cpu1_reset();
		/************************/
		/* Image Expansion      */
		/************************/
		corefw_image_expansion(p_cheader);
		sensfw_image_expansion(p_sheader);

		/************************/
		/* cpu1 boot            */
		/************************/
		/* cpu1 unreset */
		xarina_hw_cpu1_unreset();
	
		reset_timer();
		for(;;) {
			/***********************/
			/* CoreFW boot check */
			/***********************/
			res = boot_info_responsecode_get();
			err = boot_check(res, b_complete);

			if (err == E_CMD_BOOT_SUCCESS) {
				/* boot ok */
				printf("CoreFW Boot Succcess.\n\n");
				return 0;
			} else if (err == E_CMD_BOOT_WARNING) {
				/* boot ok (warning) */
				printf("CoreFW Boot Succcess [Warning] (%08lX).\n\n", res);
				return 0;
			} else if (err == E_CMD_BOOT_ERROR) {
				/* boot ng */
				printf("CoreFW Boot Failure [Error] (%08lX).\n", res);
				b_tmout = FALSE;
				break;
			} else {
				/* Do Nothing */
			}

			/***********************/
			/* timeout check       */
			/***********************/
			if (timeout != 0) {
				if (get_timer(0) > timeout * get_tbclk()) {
					printf("ARM9 Boot Failure [Timeout] (%08lX)\n", res);
					b_tmout = TRUE;
					break;
				}
			}

		}
	}

	printf("\nCoreFW could not boot.\n\n");

	/* boot result check */
	if (b_tmout == TRUE) {
		return E_CMD_BOOTARM9_TMOUT;
	}

#if 0
	/* video-in only execute */
	input = cam_set_senssys_type_check(cam_param_senssystem_ddr_get());
	if (input == CAM_PARAM_SENSSYS_TYPE_VIDEOIN) {
		video_in_init();
	}

	/* load  OSD Color Palette & OSD data & Mask data */
	osd_mask_load();

	for(i = 0; i < (CFG_ARM9FW_BOOT_RETRY + 1); i++) {
		if (i != 0) {
			/* retry only */
			printf("\nARM9 Boot Retry(%d).\n", i);
		}

		/* start entry type initialize */
		boot_info_notifyflag_set(ARM9_BOOT_NOTIFY_OFF);
		/* start entry code initialize */
		boot_info_responsecode_set(ARM9_BOOT_RES_WAIT);

		/************************/
		/* arm9 reset           */
		/************************/
		xarina_hw_corefw_reset();

		/************************/
		/* Image Expansion      */
		/************************/
		/* F/W image expansion */
		arm9fw_image_expansion(p_header);
		printf("Image Expansion.\n");

		/************************/
		/* arm9 boot            */
		/************************/
		/* arm9 unreset */
		xarina_hw_corefw_unreset();

		reset_timer();
		for(;;) {
			/***********************/
			/* arm9 f/w boot check */
			/***********************/
			res = boot_info_responsecode_get();
			err = boot_check(res, b_complete);
			if (err == E_CMD_BOOT_SUCCESS) {
				/* boot ok */
				printf("ARM9 Boot Succcess.\n\n");
				return 0;
			} else if (err == E_CMD_BOOT_WARNING) {
				/* boot ok (warning) */
				printf("ARM9 Boot Succcess [Warning] (%08lX).\n\n", res);
				return 0;
			} else if (err == E_CMD_BOOT_ERROR) {
				/* boot ng */
				printf("ARM9 Boot Failure [Error] (%08lX).\n", res);
				b_tmout = FALSE;
				break;
			} else {
				/* Do Nothing */
			}

			/***********************/
			/* timeout check       */
			/***********************/
			if (timeout != 0) {
				if (get_timer(0) > timeout * get_tbclk()) {
					printf("ARM9 Boot Failure [Timeout] (%08lX)\n", res);
					b_tmout = TRUE;
					break;
				}
			}

		}
	}

	printf("\nARM9 could not boot.\n\n");

	/* boot result check */
	if (b_tmout == TRUE) {
		return E_CMD_BOOTARM9_TMOUT;
	}
#endif



	return E_CMD_BOOTARM9_FAILURE;
}	

#if defined(CONFIG_FOR_DEVELOPMENT)
static int unreset_cpu1(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[],
			 int b_complete, int load_type)
{
	
	/* video device init */
	video_out_init();

	/************************/
	/* cpu1 boot            */
	/************************/
	/* cpu1 unreset */
	xarina_hw_cpu1_unreset();
	

	return E_CMD_BOOTARM9_FAILURE;
}
int do_unreset_cpu1(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	return unreset_cpu1(cmdtp, flag, argc, argv, TRUE, LOAD_TYPE_UBIFS);
}
U_BOOT_CMD(
	bootcpu1, 2, 0,	do_unreset_cpu1,
	"bootcpu1- only boot cpu1",
	"    - boot cpu1\n"
);
#endif	
int do_bootcorefw_w(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	return boot_corefw(cmdtp, flag, argc, argv, TRUE, LOAD_TYPE_UBIFS);
}


int do_bootcorefw_l(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int load_type;
	
	if (argc != 2) {
		cmd_usage(cmdtp);
		return 0;	
	}
	
	if (strcmp(argv[1] , "t") == 0) {
		load_type = LOAD_TYPE_TFTP;
	} else if (strcmp(argv[1] , "x") == 0) {
		load_type = LOAD_TYPE_XMODEM;
	} else {
		cmd_usage(cmdtp);
		return 0;	
	}
	
	return boot_corefw(cmdtp, flag, argc, argv, FALSE, load_type);
}

int do_bootcorefw(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	return boot_corefw(cmdtp, flag, argc, argv, FALSE, LOAD_TYPE_UBIFS);
}


U_BOOT_CMD(
	cofwinfo, 3, 1,	do_corefwinfo,
	"cofwinfo-print header information for coreFW image ",
	"[ cfile ] [ sfile ]\n"
	"    - print header information for \n"
	" coreFW binary file 'cfile' and sensorFW binary file 'sfile'\n"
);

U_BOOT_CMD(
	bootcofw_w, 3, 0,	do_bootcorefw_w,
	"bootcofw_w- boot coreFW image (wait for boot)",
	"[ cfile ] [ sfile ]\n"
	"    - boot coreFW (wait for boot)"
	" using coreFW binary file 'cfile' and sensorFW binary file 'sfile'\n"
);
U_BOOT_CMD(
	bootcofw_l, 2, 0,	do_bootcorefw_l,
	"bootcofw_l- boot downloaded coreFW image (no wait for boot) ",
	"<download type (t | x)>\n\n"
	"    - boot downloaded coreFW (wait for boot) by 'download type'\n"
	"    - 'download type' t is TFTP\n"
	"    - 'download type' x is XMODEM\n"
);

U_BOOT_CMD(
	bootcofw, 3, 0,	do_bootcorefw,
	"bootcofw- boot core firmware image(no wait for boot)",
	"[ cfile ] [ sfile ]\n"
	"    - boot coreFW (no wait for boot)"
	" using coreFW binary file 'cfile' and sensorFW binary file 'sfile'\n"
);

