/*
 * (C) Copyright 2007 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <command.h>
#include <common.h>
#include "pinmux.h"
#include "cmd_pinmux.h"
#include "xarina_bool.h"
#include "corefw_comm_param.h"

static int pinmux_cmd_cond_gpioe1_uart0(void);

static struct s_pinmux_info pinmux_tbl[] = {
	{ CMD_PINMUX_TYPE_GPIOE0	,pinmux_gpioe_spi0_select	, PINMUX_GPIOE_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_SPI0		,pinmux_gpioe_spi0_select	, PINMUX_SPI0_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_GPIOE1	,pinmux_gpioe_uart0_select	, PINMUX_GPIOE_SELECT	,pinmux_cmd_cond_gpioe1_uart0	},
	{ CMD_PINMUX_TYPE_UART0		,pinmux_gpioe_uart0_select	, PINMUX_UART0_SELECT	,pinmux_cmd_cond_gpioe1_uart0	},
	{ CMD_PINMUX_TYPE_GPIOG0	,pinmux_gpiog_uart1_spi2_select	, PINMUX_GPIOG_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_UART1		,pinmux_gpiog_uart1_spi2_select	, PINMUX_UART1_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_SPI2		,pinmux_gpiog_uart1_spi2_select	, PINMUX_SPI2_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_GPIOG1	,pinmux_gpiog_uart2_select	, PINMUX_GPIOG_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_UART2		,pinmux_gpiog_uart2_select	, PINMUX_UART2_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_GPIOH		,pinmux_gpioh_sdif_select	, PINMUX_GPIOH_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_SDIF		,pinmux_gpioh_sdif_select	, PINMUX_SDIF_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_GPIOI0	,pinmux_gpioi_spi3_select	, PINMUX_GPIOI_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_SPI3		,pinmux_gpioi_spi3_select	, PINMUX_SPI3_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_GPIOI1	,pinmux_gpioi_pwm0_select	, PINMUX_GPIOI_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_PWM0		,pinmux_gpioi_pwm0_select	, PINMUX_PWM0_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_GPIOI2	,pinmux_gpioi_pwm1_select	, PINMUX_GPIOI_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_PWM1		,pinmux_gpioi_pwm1_select	, PINMUX_PWM1_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_GPIOI3	,pinmux_gpioi_nce1_select	, PINMUX_GPIOI_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_NCE1		,pinmux_gpioi_nce1_select	, PINMUX_NCE1_SELECT	,NULL	},
	{ CMD_PINMUX_TYPE_VOUT_INVALID	,pinmux_video_out_select	, PINMUX_VOUT_DESELECT	,NULL	},
	{ CMD_PINMUX_TYPE_VOUT		,pinmux_video_out_select	, PINMUX_VOUT_SELECT	,NULL	},
#if defined(CONFIG_FOR_DEVELOPMENT)
	{ CMD_PINMUX_TYPE_SVI		,pinmux_video_out_select	, PINMUX_SVI_SELECT	,NULL	},
#endif
};


static int pinmux_cmd_cond_gpioe1_uart0(void)
{
	return !is_corefw_comm_param_uart0_spi1_select();
}


int do_pinmux (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{

	char *type;
	int max;
	int i;
	int b_cond;

	if (argc != 2 ) {
		printf("invalid arg num.\n");
		return E_CMD_PINMUX_ARG_NUM;
	}

	type = argv[1];

	max = sizeof(pinmux_tbl)/sizeof(pinmux_tbl[0]);
	for(i = 0; i < max; i++) {
		/* type check */
		if (strcmp(type, pinmux_tbl[i].p_type) == 0) {
			if (pinmux_tbl[i].p_cond != NULL) {
				b_cond = pinmux_tbl[i].p_cond();
				if (!b_cond) {
					printf("prohibit.\n");
					return E_CMD_PROHIBIT;
				}
			}

			/* pinmux select */
			pinmux_tbl[i].p_func(pinmux_tbl[i].arg);
			printf("set %s.\n", pinmux_tbl[i].p_type);
			break;
		}
	}

	if (i >= max) {
		printf("invalid type.\n");
		return E_CMD_PINMUX_TYPE;
	}

	return 0;
}


U_BOOT_CMD(
	pinmux, 2, 0,	do_pinmux,
	"pinmux    - use set pinmux used arm11 set",
	"[ type ]\n"
	"    - select pin type for pinmux used arm11\n"
);

