/*
 * (C) Copyright 2007 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef COREFW_BOOT_PARAM_H
#define COREFW_BOOT_PARAM_H

#ifdef __cplusplus
extern "C" {
#endif  /* __cplusplus */

/*-----------------------------------------------------------------------------
 * define
 *---------------------------------------------------------------------------*/
/*--------------------------
 * boot response
 *------------------------*/
/* responce code    */
#define ARM9_BOOT_RES_WAIT          (0x00000000)    /* no responce  */

#define ARM9_BOOT_RES_OK_OS_INIT    (0x00000001)    /* os init complete */
#define ARM9_BOOT_RES_OK_RM_INIT    (0x00000002)    /* resource manager init complete */
#define ARM9_BOOT_RES_OK_VOUT_START (0x00000004)    /* monitor out start */

#define ARM9_BOOT_RES_ERR_PARAM     (0x00000100)    /* corefw comm parameter error */
#define ARM9_BOOT_RES_ERR_SPF       (0x00000200)    /* spf init error */
#define ARM9_BOOT_RES_ERR_DRV       (0x00000400)    /* arm9 peripheral driver init error */
#define ARM9_BOOT_RES_ERR_RM        (0x00000800)    /* resource manager init error */
#define ARM9_BOOT_RES_ERR_VIDEO     (0x00001000)    /* video fw init error */
#define ARM9_BOOT_RES_ERR_AUDIO     (0x00002000)    /* audio fw init error */
#define ARM9_BOOT_RES_ERR_CAM       (0x00004000)    /* camera fw init error */
#define ARM9_BOOT_RES_ERR_V_OPEN    (0x00008000)    /* video fw open error */
#define ARM9_BOOT_RES_ERR_A_OPEN    (0x00010000)    /* audio fw open error */
#define ARM9_BOOT_RES_WAR_MASK      (0x00020000)    /* Mask data error (warning) */
#define ARM9_BOOT_RES_WAR_OSD       (0x00040000)    /* OSD data error (warning) */
#define ARM9_BOOT_RES_WAR_COLOR     (0x00080000)    /* OSD color palette error (warning) */

/* nofity type  */
#define ARM9_BOOT_NOTIFY_ON         (0x00000001)    /* enable mailbox notify    */
#define ARM9_BOOT_NOTIFY_OFF        (0x00000002)    /* disable mailbox notify   */

/* response type */
#define ARM9_BOOT_RES_TYPE_OK       (0x00000000)    /* response code ok   */
#define ARM9_BOOT_RES_TYPE_ERR      (0xFFF00000)    /* response code error */


/*-------------------------
 * corefw common parameter
 *-----------------------*/
 /* version str */
#define ARM9_VER_HEAD_STR           "ARM9"

/* memory size parameter    */
#define ARM9_MEM_SIZE_4KB           (0x00000001)    /* memory size = 4KB    */
#define ARM9_MEM_SIZE_8KB           (0x00000002)    /* memory size = 8KB    */
#define ARM9_MEM_SIZE_16KB          (0x00000003)    /* memory size = 16KB   */
#define ARM9_MEM_SIZE_32KB          (0x00000004)    /* memory size = 32KB   */
#define ARM9_MEM_SIZE_64KB          (0x00000005)    /* memory size = 64KB   */
#define ARM9_MEM_SIZE_128KB         (0x00000006)    /* memory size = 128KB  */
#define ARM9_MEM_SIZE_256KB         (0x00000007)    /* memory size = 256KB  */
#define ARM9_MEM_SIZE_512KB         (0x00000008)    /* memory size = 512KB  */
#define ARM9_MEM_SIZE_1MB           (0x00000009)    /* memory size = 1MB    */
#define ARM9_MEM_SIZE_2MB           (0x0000000A)    /* memory size = 2MB    */
#define ARM9_MEM_SIZE_4MB           (0x0000000B)    /* memory size = 4MB    */
#define ARM9_MEM_SIZE_8MB           (0x0000000C)    /* memory size = 8MB    */
#define ARM9_MEM_SIZE_16MB          (0x0000000D)    /* memory size = 16MB   */
#define ARM9_MEM_SIZE_32MB          (0x0000000E)    /* memory size = 32MB   */
#define ARM9_MEM_SIZE_64MB          (0x0000000F)    /* memory size = 64MB   */
#define ARM9_MEM_SIZE_128MB         (0x00000010)    /* memory size = 128MB  */
#define ARM9_MEM_SIZE_256MB         (0x00000011)    /* memory size = 256MB  */
#define ARM9_MEM_SIZE_512MB         (0x00000012)    /* memory size = 512MB  */
#define ARM9_MEM_SIZE_1GB           (0x00000013)    /* memory size = 1GB    */
#define ARM9_MEM_SIZE_2GB           (0x00000014)    /* memory size = 2GB    */
#define ARM9_MEM_SIZE_4GB           (0x00000015)    /* memory size = 4GB    */

/* permission flag  */
#define ARM9_MEM_PERMISSION_ON      (0x00000001)    /* parmission = on (protected linux management area)    */
#define ARM9_MEM_PERMISSION_OFF     (0x00000002)    /* parmission = off(not protected linux management area)    */

/* log output paramter  */
#define ARM9_LOG_LOGOUT_ON          (0x00000001)    /* log output on */
#define ARM9_LOG_TRCOUT_ON          (0x00000002)    /* trace log output on */

/* peripheral io infomation */
#define ARM9_IO_UART0_SELECT        (0x00000001)    /* uart1 enable */
#define ARM9_IO_SPI1_SELECT         (0x00000010)    /* spi mask */
#define ARM9_IO_NOUSE_SELECT        (0x000000FF)    /* spi mask */

#define ARM9_IO_DEBUG_GPIO_OUT_OFF  (0x00000000)    /* debug gpio output off */
#define ARM9_IO_DEBUG_GPIO_OUT_ON   (0x00000001)    /* debug gpio output on  */

/* gpio parameter */
#define ARM9_IO_GPIO_PORT_NONE      (0x00)    /* port none */
#define ARM9_IO_GPIO_PORT_A         (0x10)    /* port a */
#define ARM9_IO_GPIO_PORT_B         (0x20)    /* port b */
#define ARM9_IO_GPIO_PORT_C         (0x30)    /* port c */
#define ARM9_IO_GPIO_PORT_D         (0x40)    /* port d */
#define ARM9_IO_GPIO_PORT_E         (0x50)    /* port e */
#define ARM9_IO_GPIO_PORT_MASK      (0xF0)    /* mask   */

#define ARM9_IO_GPIO_BIT_0          (0x00)    /* bit 0 */
#define ARM9_IO_GPIO_BIT_1          (0x01)    /* bit 1 */
#define ARM9_IO_GPIO_BIT_2          (0x02)    /* bit 2 */
#define ARM9_IO_GPIO_BIT_3          (0x03)    /* bit 3 */
#define ARM9_IO_GPIO_BIT_4          (0x04)    /* bit 4 */
#define ARM9_IO_GPIO_BIT_5          (0x05)    /* bit 5 */
#define ARM9_IO_GPIO_BIT_6          (0x06)    /* bit 6 */
#define ARM9_IO_GPIO_BIT_7          (0x07)    /* bit 7 */
#define ARM9_IO_GPIO_BIT_MASK       (0x0F)    /* mask  */

/* video control infomation */
#define ARM9_VIDEO_TVOUT_525I       (0x00000001)    /* 525i@29.97Hz NTSC */
#define ARM9_VIDEO_TVOUT_625I       (0x00000002)    /* 625@25Hz     PAL  */
#define ARM9_VIDEO_TVOUT_STOP       (0xFFFFFFFF)    /* OUTPUT STOP       */
#define ARM9_VIDEO_REV_NONE         (0x00000000)
#define ARM9_VIDEO_REV_Y            (0x00000001)
#define ARM9_VIDEO_REV_X            (0x00000002)
#define ARM9_VIDEO_REV_X_Y          (0x00000003)

/* OSD & Mask */
#define ARM9_VIDEO_OSD_OUTPUT       (0x00000001)    /* boot OSD output */
#define ARM9_VIDEO_MASK_OUTPUT      (0x00000002)    /* boot Mask output */

/* audio control infomation */
#define ARM9_AUDIO_NO_USE           (0x00000000)
#define ARM9_AUDIO_USE              (0x00000001)
#define ARM9_AUDIO_I2S_MASTER       (0x00000001)
#define ARM9_AUDIO_I2S_SLAVE        (0x00000002)
#define ARM9_AUDIO_FMT_LEFT         (0x00000001)
#define ARM9_AUDIO_FMT_RIGHT        (0x00000002)
#define ARM9_AUDIO_FMT_I2S          (0x00000003)
#define ARM9_AUDIO_BCK_32FS         (0x00000001)
#define ARM9_AUDIO_BCK_64FS         (0x00000002)

/* video control infomation */
#define COREFW_VIDEO_TVOUT_720X486_59I       	(0x00000001) 
#define COREFW_VIDEO_TVOUT_720X576_50I       	(0x00000002) 
#define COREFW_VIDEO_TVOUT_720X480_59I		(0x00000003) 
#define COREFW_VIDEO_TVOUT_720X480_60I  	(0x00000004) 
#define COREFW_VIDEO_TVOUT_720X480_60P   	(0x00000005) 
#define COREFW_VIDEO_TVOUT_720X480_59P   	(0x00000006) 
#define COREFW_VIDEO_TVOUT_720X576_50P       	(0x00000007) 
#define COREFW_VIDEO_TVOUT_1280X720_60P  	(0x00000010) 
#define COREFW_VIDEO_TVOUT_1280X720_59P  	(0x00000011) 
#define COREFW_VIDEO_TVOUT_1280X720_30P  	(0x00000012) 
#define COREFW_VIDEO_TVOUT_1280X720_29P  	(0x00000013) 
#define COREFW_VIDEO_TVOUT_1280X720_50P      	(0x00000014) 
#define COREFW_VIDEO_TVOUT_1280X720_25P      	(0x00000015) 
#define COREFW_VIDEO_TVOUT_1920X1080_60P 	(0x00000020) 
#define COREFW_VIDEO_TVOUT_1920X1080_59P 	(0x00000021) 
#define COREFW_VIDEO_TVOUT_1920X1080_30P 	(0x00000022) 
#define COREFW_VIDEO_TVOUT_1920X1080_29P 	(0x00000023) 
#define COREFW_VIDEO_TVOUT_1920X1080_50P	(0x00000024) 
#define COREFW_VIDEO_TVOUT_1920X1080_25P	(0x00000025) 
#define COREFW_VIDEO_TVOUT_1920X1080_60I 	(0x00000026) 
#define COREFW_VIDEO_TVOUT_1920X1080_59I 	(0x00000027) 
#define COREFW_VIDEO_TVOUT_1920X1080_50I     	(0x00000028) 
#define COREFW_VIDEO_TVOUT_STOP		     	(0x000000FF)    /* OUTPUT STOP       */


/*-----------------------------------------------------------------------------
 * structrure
 *---------------------------------------------------------------------------*/
/*--------------------------
 * boot response
 *------------------------*/
/* boot response information structure  */
typedef struct tagS_Arm9_BootResInfo {
    unsigned long   notifyFlag;         /* notify flag(mailbox) */
    unsigned long   responseCode;       /* responce code    */
    unsigned char   reserve[24];        /* reserve  */
} SArm9_BootResInfo;

/*-------------------------
 * arm9 common parameter
 *-----------------------*/
/* version information structure */
typedef struct tagS_Arm9_VerInfo {
    unsigned char   verStr[32];        /* version string */
} SArm9_VerInfo;

/* memory information structure */
typedef struct tagS_Arm9_MemInfo {
    unsigned char   vramSize;           /* VRAM size    */
    unsigned char   vramBusWidth;       /* VRAM connect bits num */
    unsigned char   reserve0[2];        /* reserve  */
    unsigned long   sysMemSize;         /* system memory size   */
    unsigned long   sysMemParmission;   /* system memory permission flag    */
    unsigned long   linuxMemAddr;       /* linux management area address    */
    unsigned long   linuxMemSize;       /* linux management area size   */
    unsigned long   camInfoNoticeAddr;  /* CamFW infomation notice area address */
    unsigned short  camInfoNoticeSize;  /* CamFW infomation notice area size per a entry */
    unsigned short  camInfoNoticeNum;   /* CamFW infomation notice area entry num */
    unsigned char   reserve[4];        /* reserve  */
} SArm9_MemInfo;

/* log information structure    */
typedef struct tagS_Arm9_LogInfo {
    unsigned long   outputType;         /* log output type  */
    unsigned long   logBufAddr;         /* log buffer address    */
    unsigned long   logBufSize;         /* log buffer size  */
    unsigned long   trcBufAddr;         /* trace log buffer address    */
    unsigned long   trcBufSize;         /* trace log buffer size  */
    unsigned char   reserve[12];        /* reserve  */
} SArm9_LogInfo;

/* peripheral IO information structure  */
typedef struct tagS_Arm9_PeriIoInfo {
    unsigned char   uart0ssp1;          /* select uart0/spi1/none */
    unsigned char   debugGpioOut;       /* debug gpio output */
    unsigned char   camfwExtbusyGpio;   /* camfw extbusy gpio */
    unsigned char   camfwVsubGpio;      /* camfw vsub gpio */
    unsigned long   camfwSsp;           /* camfw ssp setting  */
    unsigned char   reserve[24];        /* reserve  */
} SArm9_PeriIoInfo;

/* video control information structure */
typedef struct tagS_Arm9_VideoInfo {
    unsigned long   tvOutType;          /* TV Output Type */
    unsigned long   muteColor;          /* mute color */
    unsigned long   inversionSet;       /* inversion setting */
    unsigned long   outputSet;          /* init OSD & Mask Output setting */
    unsigned long   osdColorPltAddr;    /* OSD Color Palette address*/
    unsigned long   osdDataAddr;        /* init OSD Data address */
    unsigned long   maskDataAddr;       /* init Mask Data address */
    unsigned char   reserve[4];         /* reserve  */
} SArm9_VideoInfo;

/* audio control information structure */
typedef struct tagS_Arm9_AudioInfo {
    unsigned char   audioUse;           /* TV Output Type */
    unsigned char   i2sMode;            /* I2S mode */
    unsigned char   i2sFormat;          /* I2S serial format */
    unsigned char   i2sBck;             /* I2S bck */
    unsigned char   reserve[28];        /* reserve  */
} SArm9_AudioInfo;


/* boot paramter structure */
typedef struct tagS_Arm9_CommParam {
    unsigned char      bootInfo[32];	/* dummy */
    SArm9_VerInfo      verInfo;         /* version information          */
    SArm9_MemInfo      memInfo;         /* memory information           */
    SArm9_LogInfo      logInfo;         /* log information structure    */
    SArm9_PeriIoInfo   periIoInfo;      /* peripheral IO information    */
    SArm9_VideoInfo    videoInfo;       /* video control information    */
    SArm9_AudioInfo    audioInfo;       /* audio control information    */
} SArm9_CommParam;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* COREFW_BOOT_PARAM_H */

