/*
 * (C) Copyright 2007 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#ifndef CONFIG_XARINA_SPL
#include <version.h>
#include "cam_comm_param.h"
#include "corefw_boot_param.h"
#endif
#include <asm/arch/hw_reg.h>
#include <asm/arch/xarina.h>
#include "hw_reg_ctrl.h"
#include "pinmux.h"

extern void udelay (unsigned long usec);

/************************************/
/* SYSREG1                          */
/************************************/
void xarina_hw_pinmux_set(unsigned long *p_reg)
{
	unsigned long data;

	/* set GPIO B E*/
	data = SYSREG1_RIN(SYSREG1_GPIOABE_SEL);
	data &= ~(SYSREG1_PINSEL_PB_MASK | SYSREG1_PINSEL_PE_MASK);
	data |= (p_reg[PINMUX_GPIO_B] << SYSREG1_PINSEL_PB_SHIFT);
	data |= (p_reg[PINMUX_GPIO_E] << SYSREG1_PINSEL_PE_SHIFT);
	SYSREG1_ROUT(SYSREG1_GPIOABE_SEL, data);
	
	/* set GPIO C*/
	data = SYSREG1_RIN(SYSREG1_VIDEO_GPIO_SVI_SEL);
	data &= ~SYSREG1_PINSEL_PC_MASK;
	data |= (p_reg[PINMUX_GPIO_C] << SYSREG1_PINSEL_PC_SHIFT);
	SYSREG1_ROUT(SYSREG1_VIDEO_GPIO_SVI_SEL, data);

	/* set GPIO G*/
	data = SYSREG1_RIN(SYSREG1_GPIOFG_SEL);
	data &= ~SYSREG1_PINSEL_PG_MASK;
	data |= (p_reg[PINMUX_GPIO_G] << SYSREG1_PINSEL_PG_SHIFT);
	SYSREG1_ROUT(SYSREG1_GPIOFG_SEL, data);

	/* set GPIO H I*/
	data = SYSREG1_RIN(SYSREG1_GPIOHI_SEL);
	data &= ~(SYSREG1_PINSEL_PH_MASK | SYSREG1_PINSEL_PI_MASK);
	data |= (p_reg[PINMUX_GPIO_H] << SYSREG1_PINSEL_PH_SHIFT);
	data |= (p_reg[PINMUX_GPIO_I0] << SYSREG1_PINSEL_PI_SHIFT);
	SYSREG1_ROUT(SYSREG1_GPIOHI_SEL, data);

	/* set GPIO I*/
	data = SYSREG1_RIN(SYSREG1_NAND_GPIOI_SEL);
	data &= ~SYSREG1_NAND_PI_MASK;
	data |= p_reg[PINMUX_GPIO_I1];
	SYSREG1_ROUT(SYSREG1_NAND_GPIOI_SEL, data);

	return;
}

int xarina_hw_jtag_connection_check(void)
{
	int ret = 0;
	
	switch (SYSREG1_RIN(SYSREG1_PORT_MON) & SYSREG1_A9JTAGSEL) {
	case SYSREG1_A9JTAGSEL_CPU1:
		/* Fall through */
	case SYSREG1_A9JTAGSEL_CPU2:
		/* Fall through */
	case SYSREG1_A9JTAGSEL_CPU3:
		/* Fall through */
	case SYSREG1_A9JTAGSEL_DAISYCHAIN:
		ret = 1;
		break;
	default:
		/* do nothing */
		break;
	}
	
	return ret;
}

void xarina_hw_video_clock_select(unsigned long clock)
{
	unsigned long data;
	
	data = SYSREG1_RIN(SYSREG1_VIDEO_GPIO_SVI_SEL);
	data &= ~SYSREG1_VIDEOCLK_SEL_MASK;
	data |= (clock & SYSREG1_VIDEOCLK_SEL_MASK);
	SYSREG1_ROUT(SYSREG1_VIDEO_GPIO_SVI_SEL, data);
	
}


void xarina_hw_bootram_remap(void)
{
	SYSREG1_ROUT(SYSREG1_REMAP_CTR, SYSREG1_BOTR_BOOTREMAP_BIT);
}


void xarina_hw_nfif_clock_set(unsigned long clock)
{
	unsigned long data;
	
	data = SYSREG1_RIN(SYSREG1_CLK_CTL);
	data &= ~SYSREG1_NFIF_CKSEL_MASK;
	data |= clock << SYSREG1_NFIF_CKSEL_SHIFT;
	SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
}

#if !defined(CONFIG_NAND_SPL)
void xarina_hw_gbe_init(void)
{
	unsigned long data;

#if !defined(CONFIG_XARINA_FPGA)
	/* PLL6 Power On */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_567);
	data |= SYSREG1_PLL6_POWER_ON;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_567, data);
	
	/* wait PLL6 ready */
	while(!(SYSREG1_RIN(SYSREG1_PLL_READY) & SYSREG1_PLL6_READY));

	/* PLL6 output enable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_567);
	data |= (SYSREG1_PLL6_POWER_ON | SYSREG1_PLL6_OUT_EN);
	SYSREG1_ROUT(SYSREG1_PLL_POWER_567, data);
	udelay(5);
#endif	

	/* GbE mode & clock setting */
#if defined(CONFIG_XARINA_USE_RGMII)
	SYSREG1_ROUT(SYSREG1_MODE_SEL, SYSREG1_GBEMODE_RGMII);
#else
	SYSREG1_ROUT(SYSREG1_MODE_SEL, SYSREG1_GBEMODE_MII);
#endif
	data = SYSREG1_RIN(SYSREG1_CLK_CTL);
	data &= ~SYSREG1_GBE_CKSEL_MASK;
	
#if defined(CONFIG_XARINA_USE_RGMII)
	data |= (SYSREG1_GBE_CKSEL_RGMII1000M << SYSREG1_GBE_CKSEL_SHIFT);
#else 
	data |= (SYSREG1_GBE_CKSEL_MII10M_100M << SYSREG1_GBE_CKSEL_SHIFT);
#endif

	SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
	
	/* GbE clock enable */
	data = SYSREG1_RIN(SYSREG1_CLK_EN);
	data |= (SYSREG1_GBE_CKON | SYSREG1_GBEEXT_EN); 
	SYSREG1_ROUT(SYSREG1_CLK_EN, data);

	/* GbE unreset */
	data = SYSREG1_RIN(SYSREG1_RST_CTL);	
	data |= SYSREG1_RST_CTL_GBE_RST_CTL_N;
	SYSREG1_ROUT(SYSREG1_RST_CTL, data);
	
	udelay(5);

	return;
}


#if defined(CONFIG_XARINA_SPL)

void xarina_hw_gbe_disable(void)
{
	unsigned long data;

	/* GbE clock disable */
	data = SYSREG1_RIN(SYSREG1_CLK_EN);
	data &= ~SYSREG1_GBE_CKON;
	data &= ~SYSREG1_GBEEXT_EN;
	SYSREG1_ROUT(SYSREG1_CLK_EN, data);
	
	/* GbE reset */
	data = SYSREG1_RIN(SYSREG1_RST_CTL);	
	data &= ~SYSREG1_RST_CTL_GBE_RST_CTL_N;
	SYSREG1_ROUT(SYSREG1_RST_CTL, data);

	return;
}


#else /* defined(CONFIG_XARINA_SPL) */ 


static void xarina_hw_pll2_select(unsigned long pll2)
{
	unsigned long data;
	
	SYSREG1_ROUT(SYSREG1_PLL2, pll2);
	/* PLL2 Power On */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data |= SYSREG1_PLL2_POWER_ON;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);
	/* wait PLL2 ready */
	while(!(SYSREG1_RIN(SYSREG1_PLL_READY) & SYSREG1_PLL2_READY));
	/* PLL2 Clock Enable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data |= SYSREG1_PLL2_OUT_EN;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);
}

static void xarina_hw_pll7_select(unsigned long pll7)
{
	unsigned long data;
	
	SYSREG1_ROUT(SYSREG1_PLL7, pll7);
	/* PLL7 Power On */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_567);
	data |= SYSREG1_PLL7_POWER_ON;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_567, data);
	/* wait PLL7 ready */
	while(!(SYSREG1_RIN(SYSREG1_PLL_READY) & SYSREG1_PLL7_READY));
	/* PLL7 Clock Enable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_567);
	data |= SYSREG1_PLL7_OUT_EN;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_567, data);
}

void xarina_hw_pll2_pll7_unselect(void)
{
	unsigned long data;

	/* PLL2 Power Off and Clock Disable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data &= ~(SYSREG1_PLL2_POWER_ON | SYSREG1_PLL2_OUT_EN);
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);
	
	/* PLL7 Power Off and Clock Disable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_567);
	data &= ~(SYSREG1_PLL7_POWER_ON | SYSREG1_PLL7_OUT_EN);
	SYSREG1_ROUT(SYSREG1_PLL_POWER_567, data);
	

}

static void xarina_hw_pll3_select(unsigned long pll3)
{
	unsigned long data;
	
	SYSREG1_ROUT(SYSREG1_PLL3, pll3);
	/* PLL3 Power On */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data |= SYSREG1_PLL3_POWER_ON;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);
	/* wait PLL3 ready */
	while(!(SYSREG1_RIN(SYSREG1_PLL_READY) & SYSREG1_PLL3_READY));
	/* PLL3 Clock Enable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data |= SYSREG1_PLL3_OUT_EN;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);
}

static void xarina_hw_pll4_select(unsigned long pll4)
{
	unsigned long data;
	
	SYSREG1_ROUT(SYSREG1_PLL4, pll4);
	/* PLL4 Power On */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data |= SYSREG1_PLL4_POWER_ON;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);
	/* wait PLL4 ready */
	while(!(SYSREG1_RIN(SYSREG1_PLL_READY) & SYSREG1_PLL4_READY));
	/* PLL4 Clock Enable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data |= SYSREG1_PLL4_OUT_EN;
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);
}

void xarina_hw_pll3_pll4_unselect(void)
{
	unsigned long data;

	/* PLL3 and PLL4 Power Off and Clock Disable */
	data = SYSREG1_RIN(SYSREG1_PLL_POWER_1234);
	data &= ~(SYSREG1_PLL3_POWER_ON | SYSREG1_PLL3_OUT_EN 
		| SYSREG1_PLL4_POWER_ON | SYSREG1_PLL4_OUT_EN );
	SYSREG1_ROUT(SYSREG1_PLL_POWER_1234, data);

}


void xarina_hw_pll2_pll7_select(int clock_type)
{

#if !defined(CONFIG_XARINA_FPGA_COLO2)
	switch(clock_type) {
	case CAM_PARAM_PLL2_CLK_216MHZ:
		SYSREG1_ROUT(SYSREG1_PLL7, CIS_108_0_PLL7);
		xarina_hw_pll2_select(CIS_108_0_PLL2);
		break;
	case CAM_PARAM_PLL2_CLK_216_216MHZ:
		xarina_hw_pll7_select(CIS_108_1_PLL7);
		xarina_hw_pll2_select(CIS_108_1_PLL2);
		break;
	case CAM_PARAM_PLL2_CLK_148_5MHZ:
		SYSREG1_ROUT(SYSREG1_PLL7, CIS_74_25_0_PLL7);
		xarina_hw_pll2_select(CIS_74_25_0_PLL2);
		break;
	case CAM_PARAM_PLL2_CLK_148_352MHZ:
		xarina_hw_pll7_select(CIS_74_25_1_PLL7);
		xarina_hw_pll2_select(CIS_74_25_1_PLL2);
		break;
	case CAM_PARAM_PLL2_CLK_144MHZ:
		SYSREG1_ROUT(SYSREG1_PLL7, CCD_NTSC_PLL7);
		xarina_hw_pll2_select(CCD_NTSC_PLL2);
		break;
	case CAM_PARAM_PLL2_CLK_116MHZ:
		xarina_hw_pll7_select(CCD_PAL_PLL7);
		xarina_hw_pll2_select(CCD_PAL_PLL2);
		break;
	case CAM_PARAM_PLL2_CLK_75_6MHZ:
		SYSREG1_ROUT(SYSREG1_PLL7, CCD_SVGA_PLL7);
		xarina_hw_pll2_select(CCD_SVGA_PLL2);
		break;
	case CAM_PARAM_PLL2_CLK_90MHZ:
		SYSREG1_ROUT(SYSREG1_PLL7, CCD_YC_5M_PLL7);
		xarina_hw_pll2_select(CCD_YC_5M_PLL2);
		break;
	default:
		/* do nothing */
		break;
	}
#endif
}

void xarina_hw_pll3_pll4_select(int clock_type)
{
	unsigned long data;
	
	data = SYSREG1_RIN(SYSREG1_CLK_CTL);
	data &= ~(SYSREG1_VCLK_CKSEL_MASK | SYSREG1_AUDIO_CKSEL_MASK);
	data |= SYSREG1_AUDIO_CKSEL_SLAVE;

#if !defined(CONFIG_XARINA_FPGA_COLO2)
	switch(clock_type) {
	case COREFW_VIDEO_TVOUT_720X486_59I:	/* fall through */
	case COREFW_VIDEO_TVOUT_720X576_50I:	/* fall through */
	case COREFW_VIDEO_TVOUT_720X480_59I: 	/* fall through */
	case COREFW_VIDEO_TVOUT_720X480_60I:
		data |= SYSREG1_VCLK_CKSEL_13_5MHZ_27MHZ;
		SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
		break;
	case COREFW_VIDEO_TVOUT_720X480_60P: 	/* fall through */
	case COREFW_VIDEO_TVOUT_720X480_59P: 	/* fall through */
	case COREFW_VIDEO_TVOUT_720X576_50P:
		data |= SYSREG1_VCLK_CKSEL_27MHZ_27MHZ;
		SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
		break;
	case COREFW_VIDEO_TVOUT_1280X720_59P:	/* fall through */
	case COREFW_VIDEO_TVOUT_1920X1080_29P:	/* fall through */
	case COREFW_VIDEO_TVOUT_1920X1080_59I:
		data |= SYSREG1_VCLK_CKSEL_74_175MHZ_74_175MHZ;
		SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
		xarina_hw_pll3_select(PLL3_148_352MHZ);
		xarina_hw_pll4_select(PLL4_148_352MHZ);
		break;
	case COREFW_VIDEO_TVOUT_1280X720_60P:	/* fall through */
	case COREFW_VIDEO_TVOUT_1280X720_50P:	/* fall through */
	case COREFW_VIDEO_TVOUT_1920X1080_30P:	/* fall through */
	case COREFW_VIDEO_TVOUT_1920X1080_25P:	/* fall through */
	case COREFW_VIDEO_TVOUT_1920X1080_60I:	/* fall through */
	case COREFW_VIDEO_TVOUT_1920X1080_50I:
		data |= SYSREG1_VCLK_CKSEL_74_25MHZ_74_25MHZ_PLL4;
		SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
		xarina_hw_pll4_select(PLL4_148_5MHZ);
		break;
	case COREFW_VIDEO_TVOUT_1920X1080_60P:	/* fall through */
	case COREFW_VIDEO_TVOUT_1920X1080_50P:
		data |= SYSREG1_VCLK_CKSEL_74_25MHZ_74_25MHZ_PLL4;
		SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
		xarina_hw_pll4_select(PLL4_297MHZ);
		break;
	case COREFW_VIDEO_TVOUT_1280X720_30P:	/* fall through */
	case COREFW_VIDEO_TVOUT_1280X720_25P:
		data |= SYSREG1_VCLK_CKSEL_74_25MHZ_74_25MHZ_PLL4;
		SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
		xarina_hw_pll4_select(PLL4_148_5MHZ);
		/* T.B.D extension H */
		break;
	case COREFW_VIDEO_TVOUT_1280X720_29P:
		data |= SYSREG1_VCLK_CKSEL_74_175MHZ_74_175MHZ;
		SYSREG1_ROUT(SYSREG1_CLK_CTL, data);
		xarina_hw_pll3_select(PLL3_148_352MHZ);
		xarina_hw_pll4_select(PLL4_148_352MHZ);
		/* T.B.D extension H*/
		break;
	default:
		/* do nothing */
		break;
	}
	
	
#endif
}


void xarina_hw_cpu1_reset(void)
{
	unsigned long data;
	
	data = SYSREG1_RIN(SYSREG1_RST_CTL);
	data &= ~SYSREG1_AVCC_ARM9_RST_CTL_N;
	SYSREG1_ROUT(SYSREG1_RST_CTL, data);
	udelay(5);

	return;
}


void xarina_hw_cpu1_unreset(void)
{	

	unsigned long data;

	/* AVARM9 clock enable */	
	data = SYSREG1_RIN(SYSREG1_CLK_EN);
	data |= SYSREG1_AVA9_CKON;
	SYSREG1_ROUT(SYSREG1_CLK_EN, data);

	/* AVARM9 unreset */
	data = SYSREG1_RIN(SYSREG1_RST_CTL);
	data |= SYSREG1_AVCC_ARM9_RST_CTL_N;
	SYSREG1_ROUT(SYSREG1_RST_CTL, data);

	udelay(5);

	return;
}



void xarina_hw_gpio_pullup_init(void)
{
#if 0
	SYSREG1_ROUT(SYSREG1_PAPUL_REG, 0);
	SYSREG1_ROUT(SYSREG1_PBPUL_REG, 0);
	SYSREG1_ROUT(SYSREG1_PCPUL_REG, 0);
	SYSREG1_ROUT(SYSREG1_PEPUL_REG, 0);
	SYSREG1_ROUT(SYSREG1_PFPUL_REG, 0);
	SYSREG1_ROUT(SYSREG1_PGPUL_REG, 0);
	SYSREG1_ROUT(SYSREG1_PHPUL_REG, 0);
	SYSREG1_ROUT(SYSREG1_PIPUL_REG, 0);
#endif
	return;
}

unsigned long xarina_hw_devid_get(void)
{

/*	return SYSREG1_RIN(SYSREG1_DEVID_REG); */
	return 0;
}

int xarina_hw_boot_type_get(void)
{
	int ret = XARINA_BOOT_NAND_BOOT;
	
	switch(SYSREG1_RIN(SYSREG1_PORT_MON) & SYSREG1_BOOTSEL_MASK) {
	case SYSREG1_BOOTSEL_NAND:
		ret = XARINA_BOOT_NAND_BOOT;
		break;
	case SYSREG1_BOOTSEL_UART:
		ret = XARINA_BOOT_UART_BOOT;
		break;
	default:
		/* do nothing */
		break;
	}
	
	return ret;
}

/************************************/
/* SYSREG0                          */
/************************************/
/* SDout clock enable */
void xarina_hw_sdout_clock_enable(void)
{
	unsigned long data;
	
	data = SYSREG0_RIN(SYSREG0_CLK_EN);
	data |= SYSREG0_SLICEVIDEO_CKON;
	SYSREG0_ROUT(SYSREG0_CLK_EN, data);

	return;
}

#endif /* defined(CONFIG_XARINA_SPL) */ 
#endif /* !defined(CONFIG_NAND_SPL) */
