/*
 * (C) Copyright 2007 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <asm/arch/hw_reg.h>
#include <asm/arch/xarina.h>
#include <asm/arch/hw_reg_init.h>


/***********************************/
/* initial unreset                 */
/***********************************/
#if defined(CONFIG_XARINA_NOR_BOOT)
.globl xarina_init_unreset
xarina_init_unreset:

	/* AVCC Block clock enable */
	SYSREG1_RIN	SYSREG1_CLK_EN, r1
	orr	r1, r1, #SYSREG1_AVCAMCTL_CKON
	SYSREG1_ROUT	SYSREG1_CLK_EN, r1
	/* wait */
	nop
	nop
	nop
	nop
	nop
	
	/* AVCC Block unreset */
	SYSREG1_RIN		SYSREG1_RST_CTL, r1
	orr	r1, r1, #SYSREG1_AVCC_RST_CTL_N
	SYSREG1_ROUT	SYSREG1_RST_CTL, r1
	/* wait */
	nop
	nop
	nop
	nop
	nop
	
	/* EMC-SM2 CS1 16bitmode */
	ldr	r1, =EMC_SM2_PHY_AM_DW_16BIT
	EMCSM2_ROUT	EMC_SM2_PHY_AM1_CFG, r1
	/* wait */
	nop
	nop
	nop
	nop
	nop

	mov pc, lr

#endif /* defined(CONFIG_XARINA_NOR_BOOT) */

/***********************************/
/* pll1 select                     */
/***********************************/
.globl xa_pll1_sel_start
xa_pll1_sel_start:
#if !defined(CONFIG_XARINA_FPGA_COLO2)
	/* PLL1 power on */
	SYSREG1_RIN		SYSREG1_PLL_POWER_1234, r1
	orr	r1, r1, #SYSREG1_PLL1_POWER_ON
	SYSREG1_ROUT		SYSREG1_PLL_POWER_1234, r1

pll1_wait_loop:
	SYSREG1_RIN		SYSREG1_PLL_READY, r1
	ands	r1, r1, #SYSREG1_PLL1_READY
	beq	pll1_wait_loop

	/* select system clock */
	SYSREG1_RIN		SYSREG1_CLK_CTL, r1
	orr	r1, r1, #SYSREG1_SYS_CKSEL_PLL1
	SYSREG1_ROUT		SYSREG1_CLK_CTL, r1

	nop
	nop
	nop
	nop
#endif
	mov pc, lr
	
xa_pll1_pll6_sel_end:

/***********************************/
/* ARM9 Peripheral initialze       */
/*                  only use ARM11 */
/***********************************/
.globl ne3_hw_arm9_peri_init
ne3_hw_arm9_peri_init:
#if 0
	/* UART1 */
	SYSREG0_ROUT_VAL	SYSREG0_CES1_REG, SYSREG0_CE1_UR1CLKEN
	
	/* UART0 */
	SYSREG0_ROUT_VAL	SYSREG0_CES1_REG, SYSREG0_CE1_UR0CLKEN
	
	/* SPI0 */
	SYSREG0_ROUT_VAL	SYSREG0_CES1_REG, SYSREG0_CE1_SPI0CLKEN
#endif
	mov pc, lr


/***********************************/
/* I2C for SerialEEP code Relocate */
/***********************************/
#if 0
.globl ne3_hw_se2p_i2c_relocate
ne3_hw_se2p_i2c_relocate:
	ldr	r0, =__scratchpad_src_start
	cmp	r4, #0
	beq	se2p_i2c_relocate_dest_set

	ldr	r1, =TEXT_BASE
	sub	r0, r0, r1
	ldr	r1, =CONFIG_SYS_FLASH_BASE
	add	r0, r0, r1

se2p_i2c_relocate_dest_set:
	ldr	r1, =__scratchpad_dest_start
	ldr	r2, =__scratchpad_dest_end

se2p_i2c_relocate_loop:
	ldr	r3, [r0], #4
	str	r3, [r1], #4
	cmp	r2, r1
	bhi	se2p_i2c_relocate_loop

	mov pc, lr
#endif

#ifndef CONFIG_DDR_INIT_I2C_DATA_GET
/***********************************/
/* VRAM (DDR1 REG) init            */
/***********************************/
.globl ne3_hw_vram_init
ne3_hw_vram_init:
	/* VRAM unreset */
	SYSREG1_RIN		SYSREG1_RST_CTL, r1
	orr	r1, r1, #SYSREG1_RST_CTL_DDR1_RST_CTL_N
	SYSREG1_ROUT	SYSREG1_RST_CTL, r1

	/* VRAM clock enable */
	SYSREG1_RIN		SYSREG1_CLK_EN, r1
	orr	r1, r1, #SYSREG1_DDR1_CKON
	SYSREG1_ROUT	SYSREG1_CLK_EN, r1

ne3_hw_vram_controller_init:
	/* do nothing if the controller has already configured & kicked */
	DDR1_RIN	DDR1_DB_CTL07_REG, r1
	ands	r1, r1, #DDR_START
	bne	vram_init_end

	/* #####DataBahn(vram) Setting##### */
	DDR1_ROUT_VAL	DDR1_DB_CTL00_REG, DDR1_CTL00_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL01_REG, DDR1_CTL01_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL02_REG, DDR1_CTL02_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL03_REG, DDR1_CTL03_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL04_REG, DDR1_CTL04_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL05_REG, DDR1_CTL05_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL06_REG, DDR1_CTL06_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL07_REG, DDR1_CTL07_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL08_REG, DDR1_CTL08_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL09_REG, DDR1_CTL09_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL10_REG, DDR1_CTL10_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL11_REG, DDR1_CTL11_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL12_REG, DDR1_CTL12_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL13_REG, DDR1_CTL13_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL14_REG, DDR1_CTL14_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL15_REG, DDR1_CTL15_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL16_REG, DDR1_CTL16_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL17_REG, DDR1_CTL17_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL18_REG, DDR1_CTL18_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL19_REG, DDR1_CTL19_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL20_REG, DDR1_CTL20_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL21_REG, DDR1_CTL21_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL22_REG, DDR1_CTL22_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL23_REG, DDR1_CTL23_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL24_REG, DDR1_CTL24_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL25_REG, DDR1_CTL25_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL26_REG, DDR1_CTL26_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL27_REG, DDR1_CTL27_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL28_REG, DDR1_CTL28_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL29_REG, DDR1_CTL29_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL30_REG, DDR1_CTL30_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL31_REG, DDR1_CTL31_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL32_REG, DDR1_CTL32_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL33_REG, DDR1_CTL33_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL34_REG, DDR1_CTL34_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL35_REG, DDR1_CTL35_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL36_REG, DDR1_CTL36_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL37_REG, DDR1_CTL37_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL38_REG, DDR1_CTL38_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL39_REG, DDR1_CTL39_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL40_REG, DDR1_CTL40_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL41_REG, DDR1_CTL41_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL42_REG, DDR1_CTL42_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL43_REG, DDR1_CTL43_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL44_REG, DDR1_CTL44_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL45_REG, DDR1_CTL45_DATA
	DDR1_ROUT_VAL	DDR1_DB_CTL46_REG, DDR1_CTL46_DATA

	/* set interrupt mask */
	ldr	r1, =DDR1_CTL22_DATA;
	orr	r1, r1, #DDR_INTERRUPT_MASK;
	DDR1_ROUT	DDR1_DB_CTL22_REG, r1

	/* set START bit */
	ldr	r1, =DDR1_CTL07_DATA;
	orr	r1, r1, #DDR_START;
	DDR1_ROUT	DDR1_DB_CTL07_REG, r1

vram_init_loop:
	DDR1_RIN	DDR1_DB_CTL22_REG, r1
	ands	r1, r1, #DDR_INT_STATUS
	beq	vram_init_loop

	/* clear all interrupts */
	ldr	r1, =DDR1_CTL17_DATA;
	orr	r1, r1, #DDR_INTERRUPT_CLEAR;
	DDR1_ROUT	DDR1_DB_CTL17_REG, r1

vram_init_end:
	mov pc, lr


/***********************************/
/* SysRAM (DDR0 REG) init          */
/***********************************/
.globl ne3_hw_sysram_init
ne3_hw_sysram_init:
	/* do nothing if the controller has already configured & kicked */
	DDR0_RIN	DDR0_DB_CTL07_REG, r1
	ands	r1, r1, #DDR_START
	bne	sysram_init_end

	/* #####DataBahn(sysram) Setting##### */
	DDR0_ROUT_VAL	DDR0_DB_CTL00_REG, DDR0_CTL00_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL01_REG, DDR0_CTL01_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL02_REG, DDR0_CTL02_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL03_REG, DDR0_CTL03_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL04_REG, DDR0_CTL04_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL05_REG, DDR0_CTL05_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL06_REG, DDR0_CTL06_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL07_REG, DDR0_CTL07_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL08_REG, DDR0_CTL08_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL09_REG, DDR0_CTL09_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL10_REG, DDR0_CTL10_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL11_REG, DDR0_CTL11_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL12_REG, DDR0_CTL12_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL13_REG, DDR0_CTL13_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL14_REG, DDR0_CTL14_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL15_REG, DDR0_CTL15_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL16_REG, DDR0_CTL16_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL17_REG, DDR0_CTL17_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL18_REG, DDR0_CTL18_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL19_REG, DDR0_CTL19_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL20_REG, DDR0_CTL20_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL21_REG, DDR0_CTL21_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL22_REG, DDR0_CTL22_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL23_REG, DDR0_CTL23_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL24_REG, DDR0_CTL24_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL25_REG, DDR0_CTL25_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL26_REG, DDR0_CTL26_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL27_REG, DDR0_CTL27_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL28_REG, DDR0_CTL28_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL29_REG, DDR0_CTL29_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL30_REG, DDR0_CTL30_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL31_REG, DDR0_CTL31_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL32_REG, DDR0_CTL32_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL33_REG, DDR0_CTL33_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL34_REG, DDR0_CTL34_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL35_REG, DDR0_CTL35_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL36_REG, DDR0_CTL36_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL37_REG, DDR0_CTL37_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL38_REG, DDR0_CTL38_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL39_REG, DDR0_CTL39_DATA
	DDR0_ROUT_VAL	DDR0_DB_CTL40_REG, DDR0_CTL40_DATA

	/* set interrupt mask */
	ldr	r1, =DDR0_CTL22_DATA;
	orr	r1, r1, #DDR_INTERRUPT_MASK;
	DDR0_ROUT	DDR0_DB_CTL22_REG, r1

	/* set START bit */
	ldr	r1, =DDR0_CTL07_DATA;
	orr	r1, r1, #DDR_START;
	DDR0_ROUT	DDR0_DB_CTL07_REG, r1

sysram_init_loop:
	DDR0_RIN	DDR0_DB_CTL22_REG, r1
	ands		r1, r1, #DDR_INT_STATUS
	beq		sysram_init_loop

	/* clear all interrupts */
	ldr	r1, =DDR0_CTL17_DATA;
	orr	r1, r1, #DDR_INTERRUPT_CLEAR;
	DDR0_ROUT	DDR0_DB_CTL17_REG, r1

sysram_init_end:
	mov pc, lr

#else	/* CONFIG_DDR_INIT_I2C_DATA_GET */

/***********************************/
/* SysRAM (DDR0 REG) init          */
/***********************************/
.globl ne3_hw_sysram_init
ne3_hw_sysram_init:
	mov	r9, lr

	/* do nothing if the controller has already configured & kicked */
	DDR0_RIN	DDR0_DB_CTL07_REG, r1
	ands	r1, r1, #DDR_START
	bne	sysram_init_end

	ldr	r0, = DDR0_DB_BASE
	bl	ne3_hw_ddrcon_init

sysram_init_end:
	mov	lr, r9
	mov pc, lr


/***********************************/
/* VRAM (DDR1 REG) init            */
/***********************************/
.globl ne3_hw_vram_init
ne3_hw_vram_init:
	mov	r9, lr

	/* VRAM unreset */
	SYSREG0_ROUT_VAL	SYSREG0_RSTC_REG, SYSREG0_RST_XRSTVRAM_BIT

	/* VRAM clock enable */
	ldr	r1, =SYSREG0_CE2_VHCLKCLKEN
	orr	r1, r1, #SYSREG0_CE2_VPCLKCLKEN
	SYSREG0_ROUT	SYSREG0_CES2_REG, r1

ne3_hw_vram_controller_init:
	/* do nothing if the controller has already configured & kicked */
	DDR1_RIN	DDR1_DB_CTL07_REG, r1
	ands	r1, r1, #DDR_START
	bne	vram_init_end

	ldr	r0, = DDR1_DB_BASE
	bl	ne3_hw_ddrcon_init

vram_init_end:
	mov	lr, r9
	mov pc, lr


/***********************************/
/* ddrcon init (for databahn)      */
/* r0: set register base addr      */
/***********************************/
ne3_hw_ddrcon_init:
	/* sp set */
	ldr	sp, =XARINA_E2PROM_I2C_SP_TOP
	stmdb	sp!,{r4-r9, ip, lr}

	/* register backup */
	mov	r7, r0					/* r7: target base addr */

	/* init */
	ldr	r3, =se2p_i2c_init			/* call se2p_i2c_init */
	blx	r3

	ldr	r9, =XARINA_E2PROM_DATA_BUF_ADDR	/* r9: read buff addr */
	mov	r8, #0					/* r8: src(i2c subaddr)      */
ddrcon_init_header_read:
	/* base addr & data num read */
	mov	r0, r9					/* r0: dst(read buff addr)   */
	mov	r1, r8					/* r1: src(i2c subaddr)      */
	mov	r2, #XARINA_E2PROM_HEADER_BYTES		/* r2: len(read bytes)       */
	ldr	r3, =se2p_i2c_eeprom_rd			/* call se2p_i2c_eeprom_rd() */
	blx	r3
	add	r8, r8, #XARINA_E2PROM_HEADER_BYTES	/* r8: src += XARINA_E2PROM_HEADER_BYTES bytes */

	/* base addr & data num get */
	mov	r0, r9					/* r0: read buff addr */
	ldr	r4, [r0]				/* r4: base addr  */
	ldr	r6, [r0, #4]				/* r6: data num   */

	cmp	r4, r7
	beq	ddrcon_init_data_get

	/* if base address and data num is 0, data is end */
	cmp	r4, #0
	bne	ddrcon_init_next_header
	cmp	r6, #0
	bne	ddrcon_init_next_header

ddrcon_init_data_none:
	/* The data which you should set are not found. ERROR. */
	b	ddrcon_init_data_none

ddrcon_init_next_header:
	add	r8, r8, r6, lsl #2			/* r8: src += data num * 4bytes */
	b	ddrcon_init_header_read

ddrcon_init_data_get:
	/* data get */
	mov	r0, r9					/* r0: dst(read buff addr)   */
	mov	r1, r8					/* r1: src(i2c subaddr)      */
	mov	r2, r6, lsl #2				/* r2: len(read bytes) = data num * 4bytes */
	ldr	r3, =se2p_i2c_eeprom_rd                 /* call se2p_i2c_eeprom_rd() */
	blx	r3

	/* set register */
	mov	r0, r9					/* r0: read buff addr */
	mov	r1, r4					/* r1: base addr      */
	mov	r2, r6					/* r2: data num       */
ddrcon_set_reg_loop:
	ldr	r3, [r0], #4				/* get data          */
	str	r3, [r1], #4				/* set register      */
	subs	r2, r2, #1				/* counter decriment */
	bne	ddrcon_set_reg_loop


	mov	r0, r9					/* r0: read buff addr        */
	mov	r1, r4					/* r1: register base address */
	/* set interrupt mask */
	ldr	r3, [r0, #DDR0_DB_CTL22_REG]
	orr	r3, r3, #DDR_INTERRUPT_MASK;
	str	r3, [r1, #DDR0_DB_CTL22_REG]

	/* set START bit */
	ldr	r3, [r0, #DDR0_DB_CTL07_REG]
	orr	r3, r3, #DDR_START;
	str	r3, [r1, #DDR0_DB_CTL07_REG]

	/* confirm int_status bit */
ddrcon_set_loop:
	ldr	r3, [r1, #DDR0_DB_CTL22_REG]
	ands	r3, r3, #DDR_INT_STATUS
	beq	ddrcon_set_loop

	/* clear all interrupts */
	ldr	r3, [r0, #DDR0_DB_CTL17_REG]
	orr	r3, r3, #DDR_INTERRUPT_CLEAR;
	str	r3, [r1, #DDR0_DB_CTL17_REG]

	ldmia	sp!,{r4-r9, ip, pc}

#endif	/* CONFIG_DDR_INIT_I2C_DATA_GET */


