/*
 * (C) Copyright 2007 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <common.h>
#include <command.h>
#include "cmd_menu.h"
#include "menu_top.h"
#include "menu_flash.h"
#include "command_exec.h"
#include "xarina_bool.h"

extern char console_buffer[];	/* readline buffer */
extern char cmd_buff[CONFIG_SYS_CBSIZE];

/**************************************/
/* menu item code                     */
/**************************************/
static int menu_fl_loads(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	unsigned long offset;
	int ret;

	offset = 0;
	ret = readline("OFFSET(HEX): ");
	if (ret <= 0) {
		return ret;
	}

	offset = simple_strtoul(console_buffer, NULL, 16);

	loads_command(offset);

	return 0;
}

static int menu_fl_loadx(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	unsigned long addr;
	int ret;
#ifdef CONFIG_AUTOSCRIPT
	int b_autoscript = FALSE;
#endif /* CONFIG_AUTOSCRIPT */

	addr = CONFIG_SYS_LOAD_ADDR;
	printf("OFFSET(HEX)[default %lX]: ", addr);
	ret = readline("");
	if (ret < 0) {
		return ret;
	}
	if (ret > 0) {
		addr = simple_strtoul(console_buffer, NULL, 16);
	}

#ifdef CONFIG_AUTOSCRIPT
	if (((s = getenv("autoscript")) != NULL) && (strcmp(s,"yes") == 0)) {
		run_command("setenv autoscript", 0);	/* delete autoscript */
		b_autoscript = TRUE;
	}
#endif /* CONFIG_AUTOSCRIPT */

	loadx_command(addr);

#ifdef CONFIG_AUTOSCRIPT
	if (b_autoscript) {
		setenv ("autoscript", "yes");	/* set autoscript */
	}
#endif /* CONFIG_AUTOSCRIPT */

	return 0;
}

static int menu_fl_tftp(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	unsigned long addr;
	int ret;
	int b_autostart = FALSE;
#ifdef CONFIG_AUTOSCRIPT
	int b_autoscript = FALSE;
#endif /* CONFIG_AUTOSCRIPT */
	char *s;

	addr = CONFIG_SYS_LOAD_ADDR;
	printf("OFFSET(HEX)[default %lX]: ", addr);
	ret = readline("");
	if (ret < 0) {
		return ret;
	}
	if (ret > 0) {
		addr = simple_strtoul(console_buffer, NULL, 16);
	}

	ret = readline("FILENAME: ");
	if (ret <= 0) {
		return ret;
	}

	/* The bootm command operates after executing the tftpboot command when autostart is set to yes. */
	/* Autostart is deleted because there is an obstacle in the update operation when this is done once. */
	if (((s = getenv("autostart")) != NULL) && (strcmp(s,"yes") == 0)) {
		run_command("setenv autostart", 0);	/* delete autostart */
		b_autostart = TRUE;
	}
#ifdef CONFIG_AUTOSCRIPT
	if (((s = getenv("autoscript")) != NULL) && (strcmp(s,"yes") == 0)) {
		run_command("setenv autoscript", 0);	/* delete autoscript */
		b_autoscript = TRUE;
	}
#endif /* CONFIG_AUTOSCRIPT */

	tftp_command(addr, console_buffer);

	if (b_autostart) {
		setenv ("autostart", "yes");	/* set autostart */
	}
#ifdef CONFIG_AUTOSCRIPT
	if (b_autoscript) {
		setenv ("autoscript", "yes");	/* set autoscript */
	}
#endif /* CONFIG_AUTOSCRIPT */

	if (had_ctrlc()) {
		return 0;
	}

	return 0;
}

static int menu_fl_flinfo(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	run_command("nand device", 0);
	
	return 0;
}

static int menu_fl_erase(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	unsigned long s_addr, size;
	int ret;

	ret = readline("START ADDR(HEX): ");
	if (ret <= 0) {
		return ret;
	}
	s_addr = simple_strtoul(console_buffer, NULL, 16);

	ret = readline("ERASE SIZE(HEX)  : ");
	if (ret <= 0) {
		return ret;
	}
	size = simple_strtoul(console_buffer, NULL, 16);

	sprintf(cmd_buff, "nand erase %08lx %08lx", s_addr, size);
	
	run_command(cmd_buff, 0);

	return 0;
}

static int menu_fl_write(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	unsigned long src, dest, size;
	int ret;

	ret = readline("SRC ADDR(HEX)  : ");
	if (ret <= 0) {
		return ret;
	}
	src = simple_strtoul(console_buffer, NULL, 16);

	ret = readline("DEST ADDR(HEX) : ");
	if (ret <= 0) {
		return ret;
	}
	dest = simple_strtoul(console_buffer, NULL, 16);

	ret = readline("WRITE SIZE BYTES(HEX): ");
	if (ret <= 0) {
		return ret;
	}
	size = simple_strtoul(console_buffer, NULL, 16);

	sprintf(cmd_buff, "nand write %08lx %08lx %08lx", src, dest, size);
	
	run_command(cmd_buff, 0);

	return 0;
}

static int menu_fl_read(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	unsigned long src, dest, size;
	int ret;

	ret = readline("SRC ADDR(HEX)  : ");
	if (ret <= 0) {
		return ret;
	}
	src = simple_strtoul(console_buffer, NULL, 16);

	ret = readline("DEST ADDR(HEX) : ");
	if (ret <= 0) {
		return ret;
	}
	dest = simple_strtoul(console_buffer, NULL, 16);

	ret = readline("READ SIZE BYTES(HEX): ");
	if (ret <= 0) {
		return ret;
	}
	size = simple_strtoul(console_buffer, NULL, 16);

	sprintf(cmd_buff, "nand read %08lx %08lx %08lx", dest, src, size);
	
	run_command(cmd_buff, 0);

	return 0;
}

/**************************************/
/* menu table                         */
/**************************************/
struct menu_item item_fl_loads = {
	"load over serial (s record)", menu_fl_loads, NULL, 0, 0,
};
struct menu_item item_fl_loadx = {
	"load over serial (xmodem)", menu_fl_loadx, NULL, 0, 0,
};
struct menu_item item_fl_tftp = {
	"load over ether (tftp)", menu_fl_tftp, NULL, 0, 0,
};
struct menu_item item_fl_flinfo = {
	"nand info", menu_fl_flinfo, NULL, 0, 0,
};	
struct menu_item item_fl_erase = {
	"erase nand", menu_fl_erase, NULL, 0, 0,
};
struct menu_item item_fl_write = {
	"write nand", menu_fl_write, NULL, 0, 0,
};
struct menu_item item_fl_read = {
	"read nand", menu_fl_read, NULL, 0, 0,
};
struct menu_item item_fl_quit = {
	"quit", NULL, &info_top, 0, 0,
};

struct menu_info info_flash = {
	"load data and nand r/w",
	NULL, NULL,
	NULL,
	7,
	{	&item_fl_loads, &item_fl_loadx, &item_fl_tftp, &item_fl_flinfo,
		&item_fl_erase, &item_fl_write, &item_fl_read
	},
	&item_fl_quit, NULL,
	0,
};

