/*
 * (C) Copyright 2007 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <common.h>
#include <command.h>
#include "cmd_menu.h"
#include "menu_top.h"
#include "menu_input.h"
#include "corefw_boot_param.h"
#include "corefw_comm_param.h"
#include "cam_set_param.h"
#include "cam_comm_param.h"
#include "boot_param.h"
#include "xarina_bool.h"

#define INPUT_CHANGE_MAX_PAGE	(9)
#define INPUT_CHANGE_MAX_ITEM	(9)
extern char console_buffer[];	/* readline buffer */


struct menu_item item_in_cgm[INPUT_CHANGE_MAX_ITEM] ;
static int menu_input_page;
static char menu_input_title[50];

/**************************************/
/* menu item code                     */
/**************************************/
static int menu_in_out(struct menu_info *p_info, struct menu_proc *p_proc)
{

	printf("Current Setting: %s\n", 
		getenv(CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM));
	

	return 0;
}

static int menu_in_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	menu_input_page = 0;
	sprintf(menu_input_title, "input chage page %02d / %02d", 
		menu_input_page, INPUT_CHANGE_MAX_PAGE);
	return 0;
}


/*------------------------------------*/
/* main                               */
/*------------------------------------*/
/*------------------------------------*/
/* factory default                    */
/*------------------------------------*/
static int menu_in_def_restore(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	struct s_param_data_image_header *p_header = 
		(struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;

	/* load saved parameter */
	ret = boot_param_load_file_env(CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM_D,
				 CONFIG_SYS_LOAD_ADDR, 0);

	printf("load Cam FW set param ... ");

	if (!ret) {
		printf("not found!\n");
		return 0;
	}
			
	if (boot_param_data_expansion(p_header, CAM_SET_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done!\n");
	} else {
		printf("invalid file!\n");
		return 0;
	}

	setenv(CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM, 
		getenv(CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM_D));

	if (cam_param_senssystem_ddr_get() & CAM_PARAM_SENSSYS_VIDEOIN_MASK) {
		p_item->next = &info_in_cg_vrs;
	} else {
		p_item->next = &info_in_cg_ets;
	}
	
	return 0;
}


/*------------------------------------*/
/* input_change                       */
/*------------------------------------*/

static int menu_in_cgm_next(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	menu_input_page++;
	
	if (menu_input_page >= INPUT_CHANGE_MAX_PAGE) {
		menu_input_page = 0;
	}
	
	sprintf(menu_input_title, "input chage page %02d / %02d", 
		menu_input_page + 1, INPUT_CHANGE_MAX_PAGE);
	return 0;
}

static int menu_in_cgm_back(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	
	if (menu_input_page == 0) {
		menu_input_page = INPUT_CHANGE_MAX_PAGE -1;
	} else {
		menu_input_page--;
	}
	
	sprintf(menu_input_title, "input chage page %02d / %02d", 
		menu_input_page + 1, INPUT_CHANGE_MAX_PAGE);
	return 0;
}

static int menu_in_cgm_select(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	char env[32];
	int ret;
	struct s_param_data_image_header *p_header = 
		(struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;

	sprintf(env, "menu_input_%02x_%02x_file", menu_input_page + 1, p_proc->select_num);
	
	/* load saved parameter */
	ret = boot_param_load_file_env(env, CONFIG_SYS_LOAD_ADDR, 0);

	printf("load Cam FW set param ... ");

	if (!ret) {
		printf("not found!\n");
		return 0;
	}
			
	if (boot_param_data_expansion(p_header, CAM_SET_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done!\n");
	} else {
		printf("invalid file!\n");
		return 0;
	}

	sprintf(console_buffer, "setenv %s %s", 
			CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM, getenv(env));
	run_command(console_buffer, 0);
	
	if (cam_param_senssystem_ddr_get() & CAM_PARAM_SENSSYS_VIDEOIN_MASK) {
		item_in_cgm[p_proc->select_num - 1].next = &info_in_cg_vrs;
	} else {
		item_in_cgm[p_proc->select_num - 1].next = &info_in_cg_ets;
	}

	return 0;
}

static int menu_in_cgm_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	char env[30];
	char *file_name;
	char *cur_file_name;
	int i;
	
	for (i = 0; i < INPUT_CHANGE_MAX_ITEM; i++) {
		sprintf(env, "menu_input_%02x_%02x_str", menu_input_page + 1, i + 1);
		item_in_cgm[i].p_str = getenv(env);
		if (item_in_cgm[i].p_str != NULL) {
			p_proc->b_disable[i] = FALSE;
		} else {
			p_proc->b_disable[i] = TRUE;
		}
		
		sprintf(env, "menu_input_%02x_%02x_file", menu_input_page + 1, i + 1);
		file_name = getenv(env);
		cur_file_name = getenv(CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM);
		if(!strcmp(file_name, cur_file_name)) {
			p_proc->b_choice[i] = TRUE;
		} else {
			p_proc->b_choice[i] = FALSE;
		}

		item_in_cgm[i].p_func = menu_in_cgm_select;
		item_in_cgm[i].next = &info_in_cgm;
	}

	return 0;
}

/*------------------------------------*/
/* video range select                 */
/*------------------------------------*/
static int menu_in_cg_vrs_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	unsigned char videorange;
	
	videorange = cam_param_videorange_ddr_get();
	
	if (videorange == CAM_PARAM_VIDEORANGE_FULL) {
		p_proc->b_choice[0] = TRUE;
	} else if (videorange == CAM_PARAM_VIDEORANGE_BT601) {
		p_proc->b_choice[1] = TRUE;
	} else {
		p_proc->b_choice[0] = FALSE;
		p_proc->b_choice[1] = FALSE;
	}
	
	return 0;
}
static int menu_in_cg_vrs(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	cam_param_videorange_ddr_set((unsigned char)p_item->info);

	return 0;
}
/*------------------------------------*/
/* EVR type select                  */
/*------------------------------------*/
static int menu_in_cg_ets_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	unsigned char evr_type;
	
	evr_type = cam_param_evr_type_ddr_get();
	
	if (evr_type == CAM_PARAM_EVR_TYPE_SCI) {
		p_proc->b_choice[0] = TRUE;
	} else if (evr_type == CAM_PARAM_EVR_TYPE_SPI) {
		p_proc->b_choice[1] = TRUE;
	} else {
		p_proc->b_choice[0] = FALSE;
		p_proc->b_choice[1] = FALSE;
	}
	
	return 0;
}
static int menu_in_cg_ets(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	cam_param_evr_type_ddr_set((unsigned char)p_item->info);

	return 0;
}
/*------------------------------------*/
/* MIRIS select                       */
/*------------------------------------*/
static int menu_in_cg_ms_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	unsigned char sysmiris;
	
	sysmiris = cam_param_sysmiris_ddr_get();
	
	if (sysmiris == CAM_PARAM_EVR_TYPE_SCI) {
		p_proc->b_choice[0] = TRUE;
	} else if (sysmiris == CAM_PARAM_EVR_TYPE_SPI) {
		p_proc->b_choice[1] = TRUE;
	} else {
		p_proc->b_choice[0] = FALSE;
		p_proc->b_choice[1] = FALSE;
	}
	
	return 0;
}
static int menu_in_cg_ms(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	cam_param_sysmiris_ddr_set((unsigned char)p_item->info);

	return 0;
}
/**************************************/
/* menu table                         */
/**************************************/


/*------------------------------------*/
/* factory default                    */
/*------------------------------------*/
struct menu_item item_in_def_restore = {
	"restore", menu_in_def_restore, &info_input, 0, 0,
};
struct menu_item item_in_def_quit = {
	"quit", NULL, &info_input, 0, 0,
};

struct menu_info info_in_default= {
	"factory default", NULL, NULL,
	NULL,
	1,
	{	&item_in_def_restore,
	},
	&item_in_def_quit, NULL,
	0,
};



/*------------------------------------*/
/* EVR type select                    */
/*------------------------------------*/
struct menu_item item_in_cg_ets_sci = {
	"SCI", menu_in_cg_ets, &info_in_cg_ms, 0, CAM_PARAM_EVR_TYPE_SCI,
};
struct menu_item item_in_cg_ets_spi = {
	"SPI", menu_in_cg_ets, &info_in_cg_ms, 0, CAM_PARAM_EVR_TYPE_SPI,
};
struct menu_info info_in_cg_ets = {
	"EVR type select",
	menu_in_cg_ets_pre, NULL,
	NULL,
	2,
	{&item_in_cg_ets_sci, &item_in_cg_ets_spi},
	NULL, NULL,
	0,
};
/*------------------------------------*/
/* MIRIS select                       */
/*------------------------------------*/
struct menu_item item_in_cg_ms_off = {
	"off", menu_in_cg_ms, &info_in_cg_vrs, 0, CAM_PARAM_MIRIS_SYSMIRIS_NONE,
};
struct menu_item item_in_cg_ms_on = {
	"on", menu_in_cg_ms, &info_in_cg_vrs, 0, CAM_PARAM_MIRIS_SYSMIRIS_YES,
};
struct menu_info info_in_cg_ms = {
	"MIRIS select",
	menu_in_cg_ms_pre, NULL,
	NULL,
	2,
	{&item_in_cg_ms_off, &item_in_cg_ms_on},
	NULL, NULL,
	0,
};

/*------------------------------------*/
/* video range select   	      */
/*------------------------------------*/
struct menu_item item_in_cg_vrs_full = {
	"Full", menu_in_cg_vrs, &info_input, 0, CAM_PARAM_VIDEORANGE_FULL,
};
struct menu_item item_in_cg_vrs_bt601 = {
	"BT.601", menu_in_cg_vrs, &info_input, 0, CAM_PARAM_VIDEORANGE_BT601,
};
struct menu_info info_in_cg_vrs = {
	"video range select",
	menu_in_cg_vrs_pre, NULL,
	NULL,
	2,
	{&item_in_cg_vrs_full, &item_in_cg_vrs_bt601},
	NULL, NULL,
	0,
};
/*------------------------------------*/
/* input change 		    	*/
/*------------------------------------*/

struct menu_item item_in_cgm_quit = {
	"quit", NULL, &info_input, 0, 0,
};
struct menu_item item_in_cgm_next = {
	"next >>", menu_in_cgm_next, &info_in_cgm, 0, 0,
};
struct menu_item item_in_cgm_back = {
	"<< back", menu_in_cgm_back, &info_in_cgm, 0, 0,
};
struct menu_info info_in_cgm = {
	menu_input_title,
	menu_in_cgm_pre, NULL,
	NULL,
	11,
	{	&item_in_cgm[0], &item_in_cgm[1],
		&item_in_cgm[2], &item_in_cgm[3],
		&item_in_cgm[4], &item_in_cgm[5],
		&item_in_cgm[6], &item_in_cgm[7], &item_in_cgm[8],
		&item_in_cgm_next, &item_in_cgm_back
	},
	&item_in_cgm_quit, NULL,
	0,
};

/*------------------------------------*/
/* save                               */
/*------------------------------------*/
static int menu_in_save(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	run_command("saveenv", 0);
	return 0;
}

/*------------------------------------*/
/* input change by file name          */
/*------------------------------------*/
static int menu_in_cgf(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	struct s_param_data_image_header *p_header = 
		(struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;
	
	ret = readline("FILENAME: ");
	if (ret <= 0) {
		return FALSE;
	}
	ret = boot_param_load_file_name(console_buffer, CONFIG_SYS_LOAD_ADDR, 0);
	if (!ret) {
		printf("error\n");
		return FALSE;
	}
	if (boot_param_data_expansion(p_header, CAM_SET_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done!\n");
		setenv(CONFIG_XARINA_ENV_CAMFW_SETTING_PARAM, console_buffer);
	} else {
		printf("invalid file!\n");
		p_item->next = &info_input;
		return FALSE;
	}
	
	if (cam_param_senssystem_ddr_get() & CAM_PARAM_SENSSYS_VIDEOIN_MASK) {
		p_item->next = &info_in_cg_vrs;
	} else {
		p_item->next = &info_in_cg_ets;
	}
		

	return 0;
}


/*------------------------------------*/
/* main                               */
/*------------------------------------*/
struct menu_item item_in_default = {
	"factory default", NULL, &info_in_default, 0, 0,
};
struct menu_item item_in_change_menu = {
	"input change menu", NULL, &info_in_cgm, 0, 0,
};
struct menu_item item_in_change_file = {
	"input change by file name", menu_in_cgf, NULL, 0, 0,
};
struct menu_item item_in_save = {
	"save", menu_in_save, NULL, 0, 0,
};
struct menu_item item_in_quit = {
	"quit", NULL, &info_top, 0, 0,
};

struct menu_info info_input = {
	"input setting",
	menu_in_pre, menu_in_out,
	NULL,
	4,
	{	&item_in_default, &item_in_change_menu, 
		&item_in_change_file, &item_in_save
	},
	&item_in_quit, NULL,
	0,
};


