/*
 * (C) Copyright 2007 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <common.h>
#include <command.h>
#include "cmd_menu.h"
#include "menu_top.h"
#include "menu_output.h"
#include "command_exec.h"
#include "corefw_boot_param.h"
#include "corefw_comm_param.h"
#include "boot_param.h"
#include "xarina_bool.h"
#include "menu_update.h"
#include "venc.h"


extern char console_buffer[];	/* readline buffer */
extern char cmd_buff[CONFIG_SYS_CBSIZE];
extern int data_down_load(unsigned int type, unsigned long addr);



/**************************************/
/* menu item code                     */
/**************************************/
/*------------------------------------*/
/* factory default                    */
/*------------------------------------*/
static int menu_out_def_restore(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	boot_param_corefw_param_restore();
	return 0;
}
static int menu_out_def_setting(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	
	ret = run_command("loadprm copa u", 0);
	
	if (ret) {
		setenv(CONFIG_XARINA_ENV_COREFW_COMM_PARAM, console_buffer);
		run_command("saveenv", 0);
	}
	
	return 0;
}
/*------------------------------------*/
/* resolution select                    */
/*------------------------------------*/
static int menu_out_cg_rs_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	unsigned long type;

	type = corefw_param_tvout_type_get();
	switch(type) {
	case COREFW_VIDEO_TVOUT_720X486_59I:
	case COREFW_VIDEO_TVOUT_720X576_50I:
	case COREFW_VIDEO_TVOUT_720X480_59I:
	case COREFW_VIDEO_TVOUT_720X480_60I:
	case COREFW_VIDEO_TVOUT_720X480_60P:
	case COREFW_VIDEO_TVOUT_720X480_59P:
	case COREFW_VIDEO_TVOUT_720X576_50P:
		p_proc->b_choice[0] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_1280X720_60P:
	case COREFW_VIDEO_TVOUT_1280X720_59P:
	case COREFW_VIDEO_TVOUT_1280X720_30P:
	case COREFW_VIDEO_TVOUT_1280X720_29P:
	case COREFW_VIDEO_TVOUT_1280X720_50P:
	case COREFW_VIDEO_TVOUT_1280X720_25P:
	case COREFW_VIDEO_TVOUT_1920X1080_60P:
	case COREFW_VIDEO_TVOUT_1920X1080_59P:
	case COREFW_VIDEO_TVOUT_1920X1080_30P:
	case COREFW_VIDEO_TVOUT_1920X1080_29P:
	case COREFW_VIDEO_TVOUT_1920X1080_50P:
	case COREFW_VIDEO_TVOUT_1920X1080_25P:
	case COREFW_VIDEO_TVOUT_1920X1080_60I:
	case COREFW_VIDEO_TVOUT_1920X1080_59I:
	case COREFW_VIDEO_TVOUT_1920X1080_50I:
		p_proc->b_choice[1] = TRUE;
		break;
	default:
		p_proc->b_choice[2] = TRUE;
		break;

	}
	return 0;
}
/*------------------------------------*/
/* resolution select SD               */
/*------------------------------------*/
static int menu_out_cg_rs_sd_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	unsigned long type;

	type = corefw_param_tvout_type_get();
	switch (type) {
	case COREFW_VIDEO_TVOUT_720X486_59I:
		p_proc->b_choice[0] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_720X576_50I:
		p_proc->b_choice[1] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_720X480_59I:
		p_proc->b_choice[2] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_720X480_60I:
		p_proc->b_choice[3] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_720X480_60P:
		p_proc->b_choice[4] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_720X480_59P:
		p_proc->b_choice[5] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_720X576_50P:
		p_proc->b_choice[6] = TRUE;
		break;
	default:
		/* do nothing */
		break;

	}
	return 0;
}
/*------------------------------------*/
/* resolution select HD               */
/*------------------------------------*/
static int menu_out_cg_rs_hd_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	unsigned long type;

	type = corefw_param_tvout_type_get();
	switch (type) {
	case COREFW_VIDEO_TVOUT_1280X720_60P:
		p_proc->b_choice[0] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_1280X720_59P:
		p_proc->b_choice[1] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_1280X720_30P:
		p_proc->b_choice[2] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_1280X720_29P:
		p_proc->b_choice[3] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_1280X720_50P:
		p_proc->b_choice[4] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_1280X720_25P:
		p_proc->b_choice[5] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_1920X1080_60P:
		p_proc->b_choice[6] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_1920X1080_59P:
		p_proc->b_choice[7] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_1920X1080_30P:
		p_proc->b_choice[8] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_1920X1080_29P:
		p_proc->b_choice[9] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_1920X1080_50P:
		p_proc->b_choice[10] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_1920X1080_25P:
		p_proc->b_choice[11] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_1920X1080_60I:
		p_proc->b_choice[12] = TRUE;
		break;
	case COREFW_VIDEO_TVOUT_1920X1080_50I:
		p_proc->b_choice[13] = TRUE;
		break;
	default:
		/* do nothing */
		break;

	}
	return 0;
}

/*------------------------------------*/
/* video-out dev pre                 */
/*------------------------------------*/
static int menu_out_vod_pre(struct menu_info *p_info, struct menu_proc *p_proc)
{
	char *env;
	char str;
	
	env = getenv(CONFIG_XARINA_ENV_VIDEOOUT_DEV);
	
	if (!env) {
		sprintf(&str,"%x", VOUT_DEV_DEFAULT);
		setenv(CONFIG_XARINA_ENV_VIDEOOUT_DEV, &str);
		set_vout_dev(VOUT_DEV_DEFAULT);
		p_proc->b_choice[0] = TRUE;
		return 0;
	}

	switch(simple_strtoul(env, NULL, 16)) {
	case VOUT_DEV_VENC:
		p_proc->b_choice[0] = TRUE;
		break;
	case VOUT_DEV_HDMI:
		p_proc->b_choice[1] = TRUE;
		break;
	case VOUT_DEV_DVI:
		p_proc->b_choice[2] = TRUE;
		break;
	case VOUT_DEV_HDCCTV:
		p_proc->b_choice[3] = TRUE;
		break;
	default:
		/* do nothing */
		break;
	}
	
	return 0;
}
/*------------------------------------*/
/* factory default                    */
/*------------------------------------*/

static int menu_out_cg(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	corefw_param_tvout_type_set(p_item->info);
	return 0;
}

/*------------------------------------*/
/* parameter load                     */
/*------------------------------------*/

int corefw_param_load(unsigned int type)
{
	int ret;
	struct s_param_data_image_header *p_header
		= (struct s_param_data_image_header *)CONFIG_SYS_LOAD_ADDR;
		
	/* not create filesize variable when tftp download size 0 file */
	run_command("setenv filesize", 0);	/* delete filesize */

	ret = data_down_load(type, (unsigned long)p_header);
	if (ret <= 0) {
		return 0;
	}

	printf("load CoreFW comm param ... ");
	if (boot_param_data_expansion(p_header, COREFW_PARAM_FILE_IDENTIFIER)
		== E_CMD_EXPANSION_SUCCESS) {
		printf("done!\n");
	} else {
		printf("invalid file!\n");
		return 0;
	}
	
	return 1;

}

static int menu_out_ld_param(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;

	ret = corefw_param_load(p_item->info);

	return ret;
}
/*------------------------------------*/
/* video-out dev                      */
/*------------------------------------*/
static int menu_out_vod(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	char str;

	sprintf(&str,"%lx", p_item->info);
	setenv(CONFIG_XARINA_ENV_VIDEOOUT_DEV, &str);
	set_vout_dev((int)(p_item->info));

	return 0;
}
static int menu_out_vod_save(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	run_command("saveenv", 0);
	return 0;
}
/**************************************/
/* menu table                         */
/**************************************/
/*------------------------------------*/
/* factory default                    */
/*------------------------------------*/
struct menu_item item_out_def_restore = {
	"restore", menu_out_def_restore, &info_output, 0, 0,
};
struct menu_item item_out_def_setting = {
	"setting", menu_out_def_setting, &info_output, 0, 0,
};
struct menu_item item_out_def_quit = {
	"quit", NULL, &info_output, 0, 0,
};

struct menu_info info_out_default = {
	"default setting",
	NULL, NULL,
	NULL,
	2,
	{	&item_out_def_restore, &item_out_def_setting
	},
	&item_out_def_quit, NULL,
	0,
};

/*------------------------------------*/
/* select resolution HD               */
/*------------------------------------*/
struct menu_item item_out_cg_1280x720_60p = {
	"1280x720p@60Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_1280X720_60P,
};
struct menu_item item_out_cg_1280x720_59p = {
	"1280x720p@59.94Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_1280X720_59P,
};
struct menu_item item_out_cg_1280x720_30p = {
	"1280x720p@30Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_1280X720_30P,
};
struct menu_item item_out_cg_1280x720_29p = {
	"1280x720p@29.97Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_1280X720_29P,
};
struct menu_item item_out_cg_1280x720_50p = {
	"1280x720p@50Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_1280X720_50P,
};
struct menu_item item_out_cg_1280x720_25p = {
	"1280x720p@25Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_1280X720_25P,
};
struct menu_item item_out_cg_1920x1080_60p = {
	"1920x1080p@60Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_1920X1080_60P,
};
struct menu_item item_out_cg_1920x1080_59p = {
	"1920x1080p@59.94Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_1920X1080_59P,
};
struct menu_item item_out_cg_1920x1080_30p = {
	"1920x1080p@30Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_1920X1080_30P,
};
struct menu_item item_out_cg_1920x1080_29p = {
	"1920x1080p@29.97Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_1920X1080_29P,
};
struct menu_item item_out_cg_1920x1080_50p = {
	"1920x1080p@50Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_1920X1080_50P,
};
struct menu_item item_out_cg_1920x1080_25p = {
	"1920x1080p@25Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_1920X1080_25P,
};
struct menu_item item_out_cg_1920x1080_60i = {
	"1920x1080i@60Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_1920X1080_60I,
};
struct menu_item item_out_cg_1920x1080_50i = {
	"1920x1080i@50Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_1920X1080_50I,
};
struct menu_item item_out_cg_rs_hd_quit = {
	"quit", NULL, &info_out_cg_rs, 0, 0,
};

struct menu_info info_out_cg_rs_hd = {
	"output change (HD)",
	menu_out_cg_rs_hd_pre, NULL,
	NULL,
	14,
	{	&item_out_cg_1280x720_60p,	&item_out_cg_1280x720_59p,
		&item_out_cg_1280x720_30p, 	&item_out_cg_1280x720_29p,
		&item_out_cg_1280x720_50p,	&item_out_cg_1280x720_25p, 
		&item_out_cg_1920x1080_60p,	&item_out_cg_1920x1080_59p,
		&item_out_cg_1920x1080_30p, 	&item_out_cg_1920x1080_29p,
		&item_out_cg_1920x1080_50p,	&item_out_cg_1920x1080_25p, 	
		&item_out_cg_1920x1080_60i,	&item_out_cg_1920x1080_50i
	},
	&item_out_cg_rs_hd_quit, NULL,
	0,
};
/*------------------------------------*/
/* select resolution SD               */
/*------------------------------------*/
struct menu_item item_out_cg_720x486_59i = {
	"720x486i@59.94Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_720X486_59I,
};
struct menu_item item_out_cg_720x576_50i = {
	"720x576i@50Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_720X576_50I,
};
struct menu_item item_out_cg_720x480_60i = {
	"720x480i@60Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_720X480_60I,
};
struct menu_item item_out_cg_720x480_59i = {
	"720x480i@59.94Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_720X480_59I,
};
struct menu_item item_out_cg_720x480_60p = {
	"720x480p@60Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_720X480_60P,
};
struct menu_item item_out_cg_720x480_59p = {
	"720x480p@59.94Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_720X480_59P,
};
struct menu_item item_out_cg_720x576_50p = {
	"720x576p@50Hz", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_720X576_50P,
};

struct menu_item item_out_cg_rs_sd_quit = {
	"quit", NULL, &info_out_cg_rs, 0, 0,
};

struct menu_info info_out_cg_rs_sd = {
	"output change (SD)",
	menu_out_cg_rs_sd_pre, NULL,
	NULL,
	7,
	{	&item_out_cg_720x486_59i, &item_out_cg_720x576_50i,
		&item_out_cg_720x480_60i, &item_out_cg_720x480_59i,
		&item_out_cg_720x480_60p, &item_out_cg_720x480_59p,
		&item_out_cg_720x576_50p
	},
	&item_out_cg_rs_sd_quit, NULL,
	0,
};


/*------------------------------------*/
/* select resolution                  */
/*------------------------------------*/
struct menu_item item_out_cg_rs_sd = {
	"SD", NULL, &info_out_cg_rs_sd, 0, 0,
};
struct menu_item item_out_cg_rs_hd = {
	"HD", NULL, &info_out_cg_rs_hd, 0, 0,
};
struct menu_item item_out_cg_rs_stop = {
	"output stop", menu_out_cg, &info_output, 0, COREFW_VIDEO_TVOUT_STOP,
};
struct menu_item item_out_cg_rs_quit = {
	"quit", NULL, &info_output, 0, 0,
};

struct menu_info info_out_cg_rs = {
	"resolution select SD/HD",
	menu_out_cg_rs_pre, NULL,
	NULL,
	3,
	{	&item_out_cg_rs_sd, &item_out_cg_rs_hd,
		&item_out_cg_rs_stop
	},
	&item_out_cg_rs_quit, NULL,
	0,
};


/*------------------------------------*/
/* parameter load                     */
/*------------------------------------*/
struct menu_item item_out_ld_xmodem = {
	"load over serial (xmodem)", menu_out_ld_param, &info_output, 0, DOWNLOAD_TYPE_LOADX,
};
struct menu_item item_out_ld_tftp = {
	"load over ether (tftp)", menu_out_ld_param, &info_output, 0, DOWNLOAD_TYPE_TFTP,
};
struct menu_item item_out_ld_quit = {
	"quit", NULL, &info_output, 0, 0,
};

struct menu_info info_out_load = {
	"corefw comm parameter load",
	NULL, NULL,
	NULL,
	2,
	{	&item_out_ld_xmodem, &item_out_ld_tftp
	},
	&item_out_ld_quit, NULL,
	0,
};


/*------------------------------------*/
/* video-out device                   */
/*------------------------------------*/
struct menu_item item_out_vod_venc = {
	"VIDEO ENCODER", menu_out_vod, &info_out_vod, 0, VOUT_DEV_VENC,
};
struct menu_item item_out_vod_hdmi = {
	"HDMI", menu_out_vod, &info_out_vod, 0, VOUT_DEV_HDMI,
};
struct menu_item item_out_vod_dvi = {
	"DVI", menu_out_vod, &info_out_vod, 0, VOUT_DEV_DVI,
};
struct menu_item item_out_vod_save = {
	"save", menu_out_vod_save, &info_output, 0, 0,
};
struct menu_item item_out_vod_quit = {
	"quit", NULL, &info_output, 0, 0,
};

struct menu_info info_out_vod = {
	"video-out device",
	menu_out_vod_pre, NULL,
	NULL,
	4,
	{	&item_out_vod_venc, &item_out_vod_hdmi, 
		&item_out_vod_dvi,  &item_out_vod_save
	},
	&item_out_vod_quit, NULL,
	0,
};


/*------------------------------------*/
/* main                               */
/*------------------------------------*/
struct menu_item item_out_default = {
	"factory default", NULL, &info_out_default, 0, 0,
};
struct menu_item item_out_change = {
	"output change", NULL, &info_out_cg_rs, 0, 0,
};
struct menu_item item_out_vod = {
	"video-out device", NULL, &info_out_vod, 0, 0,
};
struct menu_item item_out_load = {
	"corefw comm param load (XMODEM/TFTP)", NULL, &info_out_load, 0, 0,
};
struct menu_item item_out_quit = {
	"quit", NULL, &info_top, 0, 0,
};

struct menu_info info_output = {
	"output setting",
	NULL, NULL,
	NULL,
	4,
	{	&item_out_default, &item_out_change, &item_out_vod, &item_out_load
	},
	&item_out_quit, NULL,
	0,
};

