/*
 * (C) Copyright 2007 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <common.h>
#include <command.h>
#include "cmd_menu.h"
#include "menu_top.h"
#include "menu_uboot.h"
#include "xarina_bool.h"

extern char console_buffer[];	/* readline buffer */

static char cmd_buff[CONFIG_SYS_CBSIZE];

/**************************************/
/* menu item code                     */
/**************************************/
/*------------------------------------*/
/* static ip setting                  */
/*------------------------------------*/
static int menu_ub_ip_ipaddr(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;

	ret = readline("IPADDR: ");
	if (ret <= 0) {
		return 0;
	}
	sprintf(cmd_buff, "setenv ipaddr %s", console_buffer);
	run_command(cmd_buff, 0);
	return 1;
}


static int menu_ub_ip_netmask(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;

	ret = readline("NETMASK: ");
	if (ret <= 0) {
		return 0;
	}
	sprintf(cmd_buff, "setenv netmask %s", console_buffer);
	run_command(cmd_buff, 0);
	return 1;
}


static int menu_ub_ip_gateway(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;

	ret = readline("GATEWAYIP: ");
	if (ret <= 0) {
		return 0;
	}
	sprintf(cmd_buff, "setenv gatewayip %s", console_buffer);
	run_command(cmd_buff, 0);
	return 1;
}


static int menu_ub_ip_server(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;

	ret = readline("SERVERIP: ");
	if (ret <= 0) {
		return 0;
	}
	sprintf(cmd_buff, "setenv serverip %s", console_buffer);
	run_command(cmd_buff, 0);
	return 1;
}


static int menu_ub_ip_save(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	run_command("saveenv", 0);
	return 0;
}

static int menu_ub_ip_out(struct menu_info *p_info, struct menu_proc *p_proc)
{
	char *s;

	s = getenv("ipaddr");
	if (s != NULL) {
		run_command("printenv ipaddr", 0);
	} else {
		printf("ipaddr is not set.\n");
	}

	s = getenv("netmask");
	if (s != NULL) {
		run_command("printenv netmask", 0);
	} else {
		printf("netmask is not set.\n");
	}

	s = getenv("gatewayip");
	if (s != NULL) {
		run_command("printenv gatewayip", 0);
	} else {
		printf("gatewayip is not set.\n");
	}

	s = getenv("serverip");
	if (s != NULL) {
		run_command("printenv serverip", 0);
	} else {
		printf("serverip is not set.\n");
	}

	return 0;
}


/*------------------------------------*/
/* main                               */
/*------------------------------------*/
static int menu_ub_dhcp(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	run_command("dhcp", 0);
	return 0;
}


static int menu_ub_disp(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	char *s;

	s = getenv("ethaddr");
	if (s != NULL) {
		run_command("printenv ethaddr", 0);
	} else {
		printf("ethaddr is not set.\n");
	}

	s = getenv("ipaddr");
	if (s != NULL) {
		run_command("printenv ipaddr", 0);
	} else {
		printf("ipaddr is not set.\n");
	}

	s = getenv("netmask");
	if (s != NULL) {
		run_command("printenv netmask", 0);
	} else {
		printf("netmask is not set.\n");
	}

	s = getenv("gatewayip");
	if (s != NULL) {
		run_command("printenv gatewayip", 0);
	} else {
		printf("gatewayip is not set.\n");
	}

	s = getenv("serverip");
	if (s != NULL) {
		run_command("printenv serverip", 0);
	} else {
		printf("serverip is not set.\n");
	}

	return 0;
}


/**************************************/
/* menu table                         */
/**************************************/
/*------------------------------------*/
/* static ip setting                  */
/*------------------------------------*/
struct menu_item item_ub_ip_ipaddr = {
	"ip address",  menu_ub_ip_ipaddr, NULL, 0, 0,
};
struct menu_item item_ub_ip_netmask = {
	"netmask",  menu_ub_ip_netmask, NULL, 0, 0,
};
struct menu_item item_ub_ip_gateway = {
	"gateway ip",  menu_ub_ip_gateway, NULL, 0, 0,
};
struct menu_item item_ub_ip_server = {
	"server ip",  menu_ub_ip_server, NULL, 0, 0,
};
struct menu_item item_ub_ip_save = {
	"save", menu_ub_ip_save, NULL, 0, 0,
};
struct menu_item item_ub_ip_quit = {
	"quit", NULL, &info_uboot, 0, 0,
};

struct menu_info info_ub_static_ip = {
	"static ip setting",
	NULL, menu_ub_ip_out,
	NULL,
	5,
	{	&item_ub_ip_ipaddr, &item_ub_ip_netmask, &item_ub_ip_gateway,
		&item_ub_ip_server, &item_ub_ip_save,
	},
	&item_ub_ip_quit, NULL,
	0,
};


/*------------------------------------*/
/* main                               */
/*------------------------------------*/
struct menu_item item_ub_static_ip = {
	"static ip setting", NULL, &info_ub_static_ip, 0,
};
struct menu_item item_ub_dhcp = {
	"dhcp", menu_ub_dhcp, NULL, 0,
};
struct menu_item item_ub_disp = {
	"disp ip variable", menu_ub_disp, NULL, 0,
};
struct menu_item item_ub_quit = {
	"quit", NULL, &info_top, 0, 0,
};

struct menu_info info_uboot = {
	"u-boot setting",
	NULL, NULL,
	NULL,
	3,
	{	&item_ub_static_ip, &item_ub_dhcp, &item_ub_disp,
	},
	&item_ub_quit, NULL,
	0,
};

