/*
 * (C) Copyright 2007 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <common.h>
#include <command.h>
#include "cmd_menu.h"
#include "menu_top.h"
#include "menu_update.h"
#include "command_exec.h"
#include "xarina_bool.h"


extern int pushyesno(int c);
extern char console_buffer[];	/* readline buffer */
extern char cmd_buff[CONFIG_SYS_CBSIZE];


/**************************************/
/* menu item code                     */
/**************************************/
int data_down_load(unsigned int type, unsigned long addr)
{
	int b_autostart = FALSE;
#ifdef CONFIG_AUTOSCRIPT
	int b_autoscript = FALSE;
#endif /* CONFIG_AUTOSCRIPT */
	char *s;
	int ret;

	switch(type) {
	case DOWNLOAD_TYPE_LOADS:
		break;
	case DOWNLOAD_TYPE_LOADX:
#ifdef CONFIG_AUTOSCRIPT
		if (((s = getenv("autoscript")) != NULL) && (strcmp(s,"yes") == 0)) {
			run_command("setenv autoscript", 0);	/* delete autoscript */
			b_autoscript = TRUE;
		}
#endif /* CONFIG_AUTOSCRIPT */

		ret =  loadx_command(addr);

#ifdef CONFIG_AUTOSCRIPT
		if (b_autoscript) {
			setenv ("autoscript", "yes");	/* set autoscript */
		}
#endif /* CONFIG_AUTOSCRIPT */
		break;
	case DOWNLOAD_TYPE_TFTP:
		ret = readline("FILENAME: ");
		if (ret <= 0) {
			return ret;
		}

		/* The bootm command operates after executing the tftpboot command when autostart is set to yes. */
		/* Autostart is deleted because there is an obstacle in the update operation when this is done once. */
		if (((s = getenv("autostart")) != NULL) && (strcmp(s,"yes") == 0)) {
			run_command("setenv autostart", 0);	/* delete autostart */
			b_autostart = TRUE;
		}
#ifdef CONFIG_AUTOSCRIPT
		if (((s = getenv("autoscript")) != NULL) && (strcmp(s,"yes") == 0)) {
			run_command("setenv autoscript", 0);	/* delete autoscript */
			b_autoscript = TRUE;
		}
#endif /* CONFIG_AUTOSCRIPT */

		tftp_command(addr, console_buffer);

		if (b_autostart) {
			setenv ("autostart", "yes");	/* set autostart */
		}
#ifdef CONFIG_AUTOSCRIPT
		if (b_autoscript) {
			setenv ("autoscript", "yes");	/* set autoscript */
		}
#endif /* CONFIG_AUTOSCRIPT */

		if (had_ctrlc()) {
			return 0;
		}
		break;
	default:
		break;
	}

	return 1;
}

static int data_update(unsigned int type, unsigned long start, unsigned long size)
{
	unsigned long end;
	unsigned long data_size;
	char *s;
	int ret;

	/* not create filesize variable when tftp download size 0 file */
	run_command("setenv filesize", 0);	/* delete filesize */

	ret = data_down_load(type, CONFIG_SYS_LOAD_ADDR);
	if (ret <= 0) {
		return 0;
	}

	end = start + size -1;

	s = getenv("filesize");
	if (s == NULL) {	/* for tftp download size 0 file */
		printf("\n");
		printf("error: download data is size 0 or unknown.\n");
		return -1;
	}

	data_size = simple_strtoul(s, NULL, 16);
	if (data_size > size) {
		printf("\n");
		printf("error: download data is bigger than flash area.\n");
		return -1;
	}
	if (data_size <= 0) { /* for xmodem download size 0 file or timeout */
		printf("\n");
		printf("error: download data is size 0.\n");
		return -1;
	}

	protect_off_command(start, end);
	erase_command(start, end);

	cp_b_command(CONFIG_SYS_LOAD_ADDR, start, data_size);

	protect_on_command(start, end);

	return 0;
}

static int menu_ud_uboot(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;

	ret = data_update(p_item->info, CFG_NOR_MONITOR_ADDR, CFG_NOR_MONITOR_AREA_SIZE);

	return ret;
}
static int menu_ud_uboot_nand(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	char *file_size;
	/* not create filesize variable when tftp download size 0 file */
	run_command("setenv filesize", 0);	/* delete filesize */

	ret = data_down_load(p_item->info, CONFIG_SYS_LOAD_ADDR);
	if (ret <= 0) {
		return 0;
	}
	
	file_size = getenv("filesize");
	if (file_size == NULL) {
		return 0;
	}
	
	sprintf(cmd_buff, "update u-boot %x %s", CONFIG_SYS_LOAD_ADDR, file_size);
	
	ret = run_command(cmd_buff, 0);

	return ret;
}

static int menu_ud_spl(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	char *file_size;
	/* not create filesize variable when tftp download size 0 file */
	run_command("setenv filesize", 0);	/* delete filesize */

	ret = data_down_load(p_item->info, CONFIG_SYS_LOAD_ADDR);
	if (ret <= 0) {
		return 0;
	}
	
	file_size = getenv("filesize");
	if (file_size == NULL) {
		return 0;
	}
	
	sprintf(cmd_buff, "update spl %x %s", CONFIG_SYS_LOAD_ADDR, file_size);
	
	ret = run_command(cmd_buff, 0);

	return ret;
}

static int menu_ud_ubifs(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	char *file_size;
	char *part_name;
	char *vol_name;
	/* not create filesize variable when tftp download size 0 file */
	run_command("setenv filesize", 0);	/* delete filesize */

	ret = data_down_load(p_item->info, CONFIG_SYS_LOAD_ADDR);
	if (ret <= 0) {
		return 0;
	}
	
	
	part_name = getenv(CONFIG_XARINA_ENV_UBI_PART);
	if(!part_name) {
		return 0;
	}
	
	sprintf(cmd_buff, "nand erase %s", part_name);
	ret = run_command(cmd_buff, 0);
	if (!ret) {
		printf("nand erase failure\n");
		return 0;
	}

	sprintf(cmd_buff, "ubi part %s", part_name);
	ret = run_command(cmd_buff, 0);
	if (!ret) {
		printf("ubi attach failure\n");
		return 0;
	}

	vol_name = getenv(CONFIG_XARINA_ENV_UBI_VOLUME);
	if(!vol_name) {
		return 0;
	}
	sprintf(cmd_buff, "ubi create %s", vol_name);
	ret = run_command(cmd_buff, 0);
	
	file_size = getenv("filesize");
	if (file_size == NULL) {
		return 0;
	}
	sprintf(cmd_buff, "ubi write %x %s %s",CONFIG_SYS_LOAD_ADDR, vol_name, file_size);
	ret = run_command(cmd_buff, 0);
	if (!ret) {
		printf("ubi write failure\n");
		return 0;
	}
	
	sprintf(cmd_buff, "ubifsmount %s", vol_name);
	ret = run_command(cmd_buff, 0);

	printf("\nubifs update is done!\n\n");


	return ret;
}

static int menu_ud_rootfs(struct menu_info *p_info, struct menu_item *p_item, struct menu_proc *p_proc)
{
	int ret;
	char *file_size;
	char *part_name;
	char *vol_name;
	/* not create filesize variable when tftp download size 0 file */
	run_command("setenv filesize", 0);	/* delete filesize */

	ret = data_down_load(p_item->info, CONFIG_SYS_LOAD_ADDR);
	if (ret <= 0) {
		return 0;
	}
	
	
	part_name = getenv(CONFIG_XARINA_ENV_ROOTFS_UBI_PART);
	if(!part_name) {
		return 0;
	}
	
	sprintf(cmd_buff, "nand erase %s", part_name);
	ret = run_command(cmd_buff, 0);
	if (!ret) {
		printf("nand erase failure\n");
		return 0;
	}

	sprintf(cmd_buff, "ubi part %s", part_name);
	ret = run_command(cmd_buff, 0);
	if (!ret) {
		printf("ubi attach failure\n");
		return 0;
	}

	vol_name = getenv(CONFIG_XARINA_ENV_ROOTFS_UBI_VOLUME);
	if(!vol_name) {
		return 0;
	}
	sprintf(cmd_buff, "ubi create %s", vol_name);
	ret = run_command(cmd_buff, 0);
	
	file_size = getenv("filesize");
	if (file_size == NULL) {
		return 0;
	}
	sprintf(cmd_buff, "ubi write %x %s %s",CONFIG_SYS_LOAD_ADDR, vol_name, file_size);
	ret = run_command(cmd_buff, 0);
	if (!ret) {
		printf("ubi write failure\n");
		return 0;
	}
	
	
	/* mount UBIFS for u-boot */
	part_name = getenv(CONFIG_XARINA_ENV_UBI_PART);
	if(!part_name) {
		return 0;
	}
	sprintf(cmd_buff, "ubi part %s", part_name);
	ret = run_command(cmd_buff, 0);
	if (!ret) {
		printf("ubi attach failure\n");
		return 0;
	}
	vol_name = getenv(CONFIG_XARINA_ENV_UBI_VOLUME);
	if(!vol_name) {
		return 0;
	}	
	
	sprintf(cmd_buff, "ubifsmount %s", vol_name);
	ret = run_command(cmd_buff, 0);

	printf("\nubifs update is done!\n\n");


	return ret;
}

/**************************************/
/* menu table                         */
/**************************************/
/*------------------------------------*/
/* u-boot update                      */
/*------------------------------------*/
struct menu_item item_ud_ub_loadx = {
	"load over serial (xmodem)", menu_ud_uboot, &info_update, 0, DOWNLOAD_TYPE_LOADX,
};
struct menu_item item_ud_ub_tftp = {
	"load over ether (tftp)", menu_ud_uboot, &info_update, 0, DOWNLOAD_TYPE_TFTP,
};
struct menu_item item_ud_ub_quit = {
	"quit", NULL, &info_update, 0, 0,
};

struct menu_info info_ud_uboot = {
	"u-boot update", NULL, NULL,
	NULL,
	2,
	{	&item_ud_ub_loadx, &item_ud_ub_tftp,
	},
	&item_ud_ub_quit, NULL,
	0,
};

/*------------------------------------*/
/* u-boot update nand                 */
/*------------------------------------*/
struct menu_item item_ud_ub_nd_loadx = {
	"load over serial (xmodem)", menu_ud_uboot_nand, &info_update, 0, DOWNLOAD_TYPE_LOADX,
};
struct menu_item item_ud_ub_nd_tftp = {
	"load over ether (tftp)", menu_ud_uboot_nand, &info_update, 0, DOWNLOAD_TYPE_TFTP,
};
struct menu_item item_ud_ub_nd_quit = {
	"quit", NULL, &info_update, 0, 0,
};
struct menu_info info_ud_uboot_nand = {
	"u-boot update nand", NULL, NULL,
	NULL,
	2,
	{	&item_ud_ub_nd_loadx, &item_ud_ub_nd_tftp,
	},
	&item_ud_ub_nd_quit, NULL,
	0,
};
/*------------------------------------*/
/* ubifs update                */
/*------------------------------------*/
struct menu_item item_ud_ubifs_loadx = {
	"load over serial (xmodem)", menu_ud_ubifs, &info_update, 0, DOWNLOAD_TYPE_LOADX,
};
struct menu_item item_ud_ubifs_tftp = {
	"load over ether (tftp)", menu_ud_ubifs, &info_update, 0, DOWNLOAD_TYPE_TFTP,
};
struct menu_item item_ud_ubifs_quit = {
	"quit", NULL, &info_update, 0, 0,
};

struct menu_info info_ud_ubifs = {
	"ubifs update", NULL, NULL,
	NULL,
	2,
	{	&item_ud_ubifs_loadx, &item_ud_ubifs_tftp,
	},
	&item_ud_ubifs_quit, NULL,
	0,
};
/*------------------------------------*/
/* rootfs update 	              */
/*------------------------------------*/
struct menu_item item_ud_rootfs_loadx = {
	"load over serial (xmodem)", menu_ud_rootfs, &info_update, 0, DOWNLOAD_TYPE_LOADX,
};
struct menu_item item_ud_rootfs_tftp = {
	"load over ether (tftp)", menu_ud_rootfs, &info_update, 0, DOWNLOAD_TYPE_TFTP,
};
struct menu_item item_ud_rootfs_quit = {
	"quit", NULL, &info_update, 0, 0,
};

struct menu_info info_ud_rootfs = {
	"rootfs update", NULL, NULL,
	NULL,
	2,
	{	&item_ud_rootfs_loadx, &item_ud_rootfs_tftp,
	},
	&item_ud_ubifs_quit, NULL,
	0,
};

/*------------------------------------*/
/* spl update 		              */
/*------------------------------------*/
struct menu_item item_ud_spl_loadx = {
	"load over serial (xmodem)", menu_ud_spl, &info_update, 0, DOWNLOAD_TYPE_LOADX,
};
struct menu_item item_ud_spl_tftp = {
	"load over ether (tftp)", menu_ud_spl, &info_update, 0, DOWNLOAD_TYPE_TFTP,
};
struct menu_info info_ud_spl = {
	"spl update", NULL, NULL,
	NULL,
	2,
	{	&item_ud_spl_loadx, &item_ud_spl_tftp,
	},
	&item_ud_ub_quit, NULL,
	0,
};


/*------------------------------------*/
/* main                               */
/*------------------------------------*/
struct menu_item item_ud_bootrom = {
	"u-boot", NULL, &info_ud_uboot, 0, 0,
};
struct menu_item item_ud_uboot = {
	"u-boot nand", NULL, &info_ud_uboot_nand, 0, 0,
};
struct menu_item item_ud_ubifs = {
	"ubifs", NULL, &info_ud_ubifs, 0, 0,
};
struct menu_item item_ud_rootfs = {
	"rootfs", NULL, &info_ud_rootfs, 0, 0,
};
struct menu_item item_ud_spl = {
	"spl", NULL, &info_ud_spl, 0, 0,
};
struct menu_item item_ud_quit = {
	"quit", NULL, &info_top, 0, 0,
};

struct menu_info info_update = {
	"update",
	NULL, NULL,
	NULL,
	5,
	{	&item_ud_bootrom, &item_ud_uboot,
		&item_ud_spl, &item_ud_ubifs, &item_ud_rootfs
	},
	&item_ud_quit, NULL,
	0,
};

