/*
 * (C) Copyright 2007 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <config.h>
#include <asm/arch/hw_reg.h>
#include "pinmux.h"
#include "hw_reg_ctrl.h"
#if !defined(CONFIG_XARINA_SPL)
#include "corefw_boot_param.h"
#include "corefw_comm_param.h"
#include "xarina_bool.h"
#endif /* !defined(CONFIG_XARINA_SPL) */

#if !defined(CONFIG_XARINA_SPL)
static int g_pinmux_gpioe_spi0;
static int g_pinmux_gpioe_uart0;
static int g_pinmux_gpiog_uart1_spi2;
static int g_pinmux_gpiog_uart2;
static int g_pinmux_gpioh_sdif;
static int g_pinmux_gpioi_spi3;
static int g_pinmux_gpioi_pwm0;
static int g_pinmux_gpioi_pwm1;
static int g_pinmux_gpioi_nce1;
static int g_pinmux_video_out;




int pinmux_gpioe_uart0_select_val_get(void)
{
	return g_pinmux_gpioe_uart0;
}

int pinmux_gpioe_spi0_select_val_get(void)
{
	return g_pinmux_gpioe_spi0;
}
int pinmux_gpiog_uart1_spi2_select_val_get(void)
{
	return g_pinmux_gpiog_uart1_spi2;
}

int pinmux_gpiog_uart2_val_get(void)
{
	return g_pinmux_gpiog_uart2;
}
int pinmux_gpioh_sdif_val_get(void)
{
	return g_pinmux_gpioh_sdif;
}
int pinmux_gpioi_spi3_val_get(void)
{
	return g_pinmux_gpioi_spi3;
}
int pinmux_gpioi_pwm0_val_get(void)
{
	return g_pinmux_gpioi_pwm0;
}
int pinmux_gpioi_pwm1_val_get(void)
{
	return g_pinmux_gpioi_pwm1;
}
int pinmux_gpioi_nce1_val_get(void)
{
	return g_pinmux_gpioi_nce1;
}
int pinmux_video_out_val_get(void)
{
	return g_pinmux_video_out;
}






static int pinmux_pb_set(unsigned long *p_reg)
{
	unsigned long reg = SYSREG1_PINSEL_PB_GPIO;	/* GPIO B */
	unsigned char audio;

	/* GPIO B or AudioIF */
	audio = corefw_param_audio_use_get();
	if (audio == ARM9_AUDIO_USE) {
		reg |= SYSREG1_PINSEL_PB_AUDIOIF;
	}
	
	*p_reg = reg;
	
	return 0;
}
static int pinmux_pc_set(unsigned long *p_reg)
{
	unsigned long reg = SYSREG1_PINSEL_PC_GPIO;	/* GPIO C */

	switch(g_pinmux_video_out) {
	case PINMUX_VOUT_DESELECT:
		/* Do Nothing */
		break;
#if defined(CONFIG_FOR_DEVELOPMENT)
	case PINMUX_SVI_SELECT:
		reg |= SYSREG1_PINSEL_PC_SVI;
		break;
#endif /* defined(CONFIG_FOR_DEVELOPMENT) */
	default:
		/* default */
		reg |= SYSREG1_PINSEL_PC_VIDEO;
		break;
	}
	
	*p_reg = reg;
	
	return 0;
}
static int pinmux_pe_set(unsigned long *p_reg)
{
	unsigned long reg = SYSREG1_PINSEL_PE_GPIO;	/* GPIO E */
	unsigned char select;

	/* GPIO E or UART0 or SPI1 */
	select = corefw_param_uart0_spi1_select_get();

	switch(select) {
	case ARM9_IO_UART0_SELECT:
		reg |= SYSREG1_PINSEL_PE_UART0;
		break;
	case ARM9_IO_SPI1_SELECT:
		reg |= SYSREG1_PINSEL_PE_SPI1;
		break;
	case ARM9_IO_NOUSE_SELECT:
		/* Do Nothing */
		/* default */
		/* GPIO E */
		break;
	default:
		/* Do Nothing */
		/* default */
		/* GPIO E */
		break;
	}

	if (reg == SYSREG1_PINSEL_PE_GPIO) {
		/* CoreFW param doesn't select UART0 or SPI1. */

		/* GPIO E or UART */
		switch(g_pinmux_gpioe_uart0) {
		case PINMUX_GPIOE_SELECT:
			/* Do Nothing */
			/* default */
			/* GPIO E */
			break;
		case PINMUX_UART0_SELECT:
			reg |= SYSREG1_PINSEL_PE_UART0;
			break;
		default:
			/* Do Nothing */
			/* default */
			/* GPIO E */
			break;
		}
	}
	
	/* GPIO E or SPI0 */
	switch(g_pinmux_gpioe_spi0) {
	case PINMUX_GPIOE_SELECT:
		/* Do Nothing */
		break;
	case PINMUX_SPI0_SELECT:
		reg |= SYSREG1_PINSEL_PE_SPI0;
		break;
	default:
		/* default */
		reg |= SYSREG1_PINSEL_PE_SPI0;
		break;
	}

	*p_reg = reg;
	
	return 0;
}
static int pinmux_pg_set(unsigned long *p_reg)
{
	unsigned long reg = SYSREG1_PINSEL_PG_UART3;	/* UART3 */

	/* GPIO G or UART1 or SPI2 */
	switch(g_pinmux_gpiog_uart1_spi2) {
	case PINMUX_GPIOG_SELECT:
		/* Do Nothing */
		break;
	case PINMUX_UART1_SELECT:
		reg |= SYSREG1_PINSEL_PG_UART1;
		break;
	case PINMUX_SPI2_SELECT:
		reg |= SYSREG1_PINSEL_PG_SPI2;
		break;
	default:
		/* Do Nothing */
		break;
	}

	/* GPIO G or UART2 or VIDEO or SVI*/
	switch (g_pinmux_video_out) { 
	case PINMUX_VOUT_DESELECT:
		switch(g_pinmux_gpiog_uart2) {
		case PINMUX_GPIOG_SELECT:
			/* Do Nothing */
			break;
		case PINMUX_UART2_SELECT:
			reg |= SYSREG1_PINSEL_PG_UART2;
			break;
		default:
			/* Do Nothing */
			break;
		}
		break;
	case PINMUX_VOUT_SELECT:
		reg |= SYSREG1_PINSEL_PG_VIDEO;
		break;
#if defined(CONFIG_FOR_DEVELOPMENT)
	case PINMUX_SVI_SELECT:
		reg |= SYSREG1_PINSEL_PG_SVI;
		break;
#endif
	default:
		/* Do Nothing */
		break;
	}
		
	*p_reg = reg;
	
	return 0;
}
static int pinmux_ph_set(unsigned long *p_reg)
{
	unsigned long reg = SYSREG1_PINSEL_PH_GPIO;	/* GPIO H */

	/* GPIO H or SDIF or VIDEO or SVI*/
	switch (g_pinmux_video_out) { 
	case PINMUX_VOUT_DESELECT:
		switch(g_pinmux_gpioh_sdif) {
		case PINMUX_GPIOH_SELECT:
			/* Do Nothing */
			break;
		case PINMUX_SDIF_SELECT:
			reg |= SYSREG1_PINSEL_PH_SDIF;
			break;
		default:
			/* Do Nothing */
			break;
		}
		break;
	case PINMUX_VOUT_SELECT:
		reg |= SYSREG1_PINSEL_PH_VIDEO;
		break;
#if defined(CONFIG_FOR_DEVELOPMENT)
	case PINMUX_SVI_SELECT:
		reg |= SYSREG1_PINSEL_PH_SVI;
		break;
#endif
	default:
		/* Do Nothing */
		break;
	}
		
	*p_reg = reg;
	
	return 0;
}
static int pinmux_pi0_set(unsigned long *p_reg)
{
	unsigned long reg = SYSREG1_PINSEL_PI_GPIO;	/* GPIO I */

	if (!xarina_hw_jtag_connection_check()) {
		/* GPIO G or SPI3 */
		switch(g_pinmux_gpioi_spi3) {
		case PINMUX_GPIOI_SELECT:
			/* Do Nothing */
			break;
		case PINMUX_SPI3_SELECT:
			reg |= SYSREG1_PINSEL_PI_SPI3;
			break;
		default:
			/* Do Nothing */
			break;
		}

		/* GPIO I or PWM or VIDEO */
		switch (g_pinmux_video_out) { 
		case PINMUX_VOUT_DESELECT:
			switch(g_pinmux_gpioi_pwm0) {
			case PINMUX_GPIOI_SELECT:
				/* Do Nothing */
				break;
			case PINMUX_PWM0_SELECT:
				reg |= SYSREG1_PINSEL_PI_PWM0;
				break;
			default:
				/* Do Nothing */
				break;
			}
			switch(g_pinmux_gpioi_pwm1) {
			case PINMUX_GPIOI_SELECT:
				/* Do Nothing */
				break;
			case PINMUX_PWM1_SELECT:
				reg |= SYSREG1_PINSEL_PI_PWM1;
				break;
			default:
				/* Do Nothing */
				break;
			}
			break;
		case PINMUX_VOUT_SELECT:
			reg |= SYSREG1_PINSEL_PI_VIDEO;
			break;
		default:
			/* Do Nothing */
			break;
		}
	}
	
	*p_reg = reg;
	
	return 0;
}

static int pinmux_pi1_set(unsigned long *p_reg)
{
	unsigned long reg = SYSREG1_NAND_PI_GPIO;	/* GPIO I */

	/* GPIO G or NCE1 */
	switch(g_pinmux_gpioi_nce1) {
	case PINMUX_GPIOI_SELECT:
		/* Do Nothing */
		break;
	case PINMUX_NCE1_SELECT:
		reg |= SYSREG1_PINSEL_PI_NCE1;
		break;
	default:
		/* Do Nothing */
		break;
	}

	*p_reg = reg;
	
	return 0;
}

int pinmux_set(void)
{
	unsigned long reg[PINMUX_REG_NUM];
	
	reg[PINMUX_GPIO_A] = 0;
	pinmux_pb_set(&reg[PINMUX_GPIO_B]);
	pinmux_pc_set(&reg[PINMUX_GPIO_C]);
	reg[PINMUX_GPIO_D] = 0;
	pinmux_pe_set(&reg[PINMUX_GPIO_E]);
	reg[PINMUX_GPIO_F] = 0;
	pinmux_pg_set(&reg[PINMUX_GPIO_G]);
	pinmux_ph_set(&reg[PINMUX_GPIO_H]);
	pinmux_pi0_set(&reg[PINMUX_GPIO_I0]);
	pinmux_pi1_set(&reg[PINMUX_GPIO_I1]);
	
	/* video clock setting */
	switch(g_pinmux_video_out) {
	case PINMUX_VOUT_SELECT:
		xarina_hw_video_clock_select(SYSREG1_VIDEO_HCLK);
		break;
	case PINMUX_SVI_SELECT:
		xarina_hw_video_clock_select(SYSREG1_SVICLK);
		break;
	default:
		/* do nothinig */
		break;
	}

	/* pinmux set */
	xarina_hw_pinmux_set(reg);

	return 0;
}


static int pinmux_env_vari_val_set(char *p_vari_name, int status)
{
	char str[PINMUX_STATUS_STR_NUM];

	sprintf(str, "%x", status);
	setenv(p_vari_name, str);

	return 0;
}


int pinmux_gpioe_spi0_select(int status)
{
	g_pinmux_gpioe_spi0 = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOE_SPI0, status);
	pinmux_set();

	return 0;
}

int pinmux_gpioe_uart0_select(int status)
{
	g_pinmux_gpioe_uart0 = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOE_UART0, status);
	pinmux_set();

	return 0;
}


int pinmux_gpiog_uart1_spi2_select(int status)
{
	g_pinmux_gpiog_uart1_spi2 = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOG_UART1_SPI2, status);
	pinmux_set();

	return 0;
}

int pinmux_gpiog_uart2_select(int status)
{
	g_pinmux_gpiog_uart2 = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOG_UART2, status);
	pinmux_set();

	return 0;
}

int pinmux_gpioh_sdif_select(int status)
{
	g_pinmux_gpioh_sdif = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOH_SDIF, status);
	pinmux_set();

	return 0;
}

int pinmux_gpioi_spi3_select(int status)
{
	g_pinmux_gpioi_spi3 = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOI_SPI3, status);
	pinmux_set();

	return 0;
}

int pinmux_gpioi_pwm0_select(int status)
{
	g_pinmux_gpioi_pwm0 = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOI_PWM0, status);
	pinmux_set();

	return 0;
}

int pinmux_gpioi_pwm1_select(int status)
{
	g_pinmux_gpioi_pwm1 = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOI_PWM1, status);
	pinmux_set();

	return 0;
}

int pinmux_gpioi_nce1_select(int status)
{
	g_pinmux_gpioi_nce1 = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOI_NCE1, status);
	pinmux_set();

	return 0;
}
int pinmux_video_out_select(int status)
{
	g_pinmux_video_out = status;
	pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_VIDEO_OUT, status);
	
	pinmux_set();

	return 0;
}


int pinmux_default_set(void)
{
	pinmux_video_out_select(PINMUX_DEFAULT_VIDEO_OUT);
	pinmux_gpioe_spi0_select(PINMUX_DEFAULT_GPIOE_SPI0);
	pinmux_gpioe_uart0_select(PINMUX_DEFAULT_GPIOE_UART0);
	pinmux_gpiog_uart1_spi2_select(PINMUX_DEFAULT_GPIOG_UART1_SPI2);
	pinmux_gpiog_uart2_select(PINMUX_DEFAULT_GPIOG_UART2);
	pinmux_gpioh_sdif_select(PINMUX_DEFAULT_GPIOH_SDIF);
	pinmux_gpioi_spi3_select(PINMUX_DEFAULT_GPIOI_SPI3);
	pinmux_gpioi_pwm0_select(PINMUX_DEFAULT_GPIOI_PWM0);
	pinmux_gpioi_pwm1_select(PINMUX_DEFAULT_GPIOI_PWM1);
	pinmux_gpioi_nce1_select(PINMUX_DEFAULT_GPIOI_NCE1);

	return 0;
}


int pinmux_env_vari_val_all_get(void)
{
	char *s;

	/* VIDEO OUT */
	s = getenv(PINMUX_ENV_VARI_NAME_VIDEO_OUT);
	if (s) {
		g_pinmux_video_out = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_video_out = PINMUX_DEFAULT_VIDEO_OUT;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_VIDEO_OUT, g_pinmux_video_out);
	}

	/* GPIO E or SPI0 */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOE_SPI0);
	if (s) {
		g_pinmux_gpioe_spi0 = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpioe_spi0 = PINMUX_DEFAULT_GPIOE_SPI0;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOE_SPI0, g_pinmux_gpioe_spi0);
	}

	/* GPIO E or UART0 */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOE_UART0);
	if (s) {
		g_pinmux_gpioe_uart0 = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpioe_uart0 = PINMUX_DEFAULT_GPIOE_UART0;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOE_UART0, g_pinmux_gpioe_uart0);
	}

	/* GPIO G or UART1 or SPI2 */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOG_UART1_SPI2);
	if (s) {
		g_pinmux_gpiog_uart1_spi2 = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpiog_uart1_spi2 = PINMUX_DEFAULT_GPIOG_UART1_SPI2;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOG_UART1_SPI2, g_pinmux_gpiog_uart1_spi2);
	}

	/* GPIO G or UART2 */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOG_UART2);
	if (s) {
		g_pinmux_gpiog_uart2 = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpiog_uart2 = PINMUX_DEFAULT_GPIOG_UART2;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOG_UART2, g_pinmux_gpiog_uart2);
	}

	/* GPIO H or SDIF */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOH_SDIF);
	if (s) {
		g_pinmux_gpioh_sdif = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpioh_sdif = PINMUX_DEFAULT_GPIOH_SDIF;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOH_SDIF, g_pinmux_gpioh_sdif);
	}

	/* GPIO I or SPI3 */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOI_SPI3);
	if (s) {
		g_pinmux_gpioi_spi3 = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpioi_spi3 = PINMUX_DEFAULT_GPIOI_SPI3;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOI_SPI3, g_pinmux_gpioi_spi3);
	}

	/* GPIO I or PWM0 */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOI_PWM0);
	if (s) {
		g_pinmux_gpioi_pwm0 = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpioi_pwm0 = PINMUX_DEFAULT_GPIOI_PWM0;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOI_PWM0, g_pinmux_gpioi_pwm0);
	}

	/* GPIO I or PWM1 */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOI_PWM1);
	if (s) {
		g_pinmux_gpioi_pwm1 = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpioi_pwm1 = PINMUX_DEFAULT_GPIOI_PWM1;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOI_PWM1, g_pinmux_gpioi_pwm1);
	}

	/* GPIO I or NCE1 */
	s = getenv(PINMUX_ENV_VARI_NAME_GPIOI_NCE1);
	if (s) {
		g_pinmux_gpioi_nce1 = simple_strtoul(s, NULL, 16);
	} else {
		g_pinmux_gpioi_nce1 = PINMUX_DEFAULT_GPIOI_NCE1;
		pinmux_env_vari_val_set(PINMUX_ENV_VARI_NAME_GPIOI_NCE1, g_pinmux_gpioi_nce1);
	}

	return 0;
}
#endif /* !defined(CONFIG_XARINA_SPL) */

int pinmux_gpio_uart3_set(void)
{
	unsigned long reg[PINMUX_REG_NUM];

	reg[PINMUX_GPIO_A] = 0;
	reg[PINMUX_GPIO_B] = SYSREG1_PINSEL_PB_GPIO;
	reg[PINMUX_GPIO_C] = SYSREG1_PINSEL_PC_GPIO;
	reg[PINMUX_GPIO_D] = 0;
	reg[PINMUX_GPIO_E] = SYSREG1_PINSEL_PE_GPIO;
	reg[PINMUX_GPIO_F] = 0;
	reg[PINMUX_GPIO_G] = SYSREG1_PINSEL_PG_UART3;
	reg[PINMUX_GPIO_H] = SYSREG1_PINSEL_PH_GPIO;
	reg[PINMUX_GPIO_I0] = SYSREG1_PINSEL_PI_GPIO;
	reg[PINMUX_GPIO_I1] = SYSREG1_NAND_PI_GPIO;
	
	xarina_hw_pinmux_set(reg);

	return 0;
}


