/*
 * (C) Copyright 2007 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <i2c.h>
#include "vdec.h"

/* #define VDEC_DEBUG */

static const struct s_vdec_data_tbl vdec_data_ntsc[] = {
	{ 0x00, 0x00 },
	{ 0x03, 0x0D },
	{ 0x28, 0x0A },
};

static const struct s_vdec_data_tbl vdec_data_pal[] = {
	{ 0x00, 0x00 },
	{ 0x03, 0x0D },
	{ 0x28, 0x04 },
};


int vdec_init(unsigned long type)
{
	int limit = 0;
	int ret;
	int i;

	/* i2c device set */
	ret = i2c_set_bus_num(VDEC_I2C_DEV);
	if (ret < 0) {
		return ret;
	}

#ifdef VDEC_DEBUG
	/* Read data */
	printf("Befor register set.\n");
	for(i = 0; i < 0x100; i++) {
		printf("reg:%02x data:%02x\n", i, i2c_reg_read(VDEC_I2C_SLAVE_ADDR, i));
	}
#endif	/* VDEC_DEBUG */

	switch(type) {
	case VDEC_TYPE_NTSC:
		limit  = sizeof(vdec_data_ntsc)/sizeof(vdec_data_ntsc[0]);

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VDEC_I2C_SLAVE_ADDR, vdec_data_ntsc[i].addr, vdec_data_ntsc[i].data);
		}
		break;
	case VDEC_TYPE_PAL:
		limit  = sizeof(vdec_data_pal)/sizeof(vdec_data_pal[0]);

		/* Write data */
		for(i = 0; i < limit; i++) {
			i2c_reg_write(VDEC_I2C_SLAVE_ADDR, vdec_data_pal[i].addr, vdec_data_pal[i].data);
		}
		break;
	default:
		return -1;
		break;
	}

#ifdef VDEC_DEBUG
	printf("\n" "After register set.\n");
	/* Read data */
	for(i = 0; i < 0x100; i++) {
		printf("reg:%02x data:%02x\n", i, i2c_reg_read(VDEC_I2C_SLAVE_ADDR, i));
	}
#endif	/* VDEC_DEBUG */

	return 0;
}

