/*
 * (C) Copyright 2007 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <i2c.h>
#include "venc.h"

/* #define VENC_DEBUG */

static int vout_dev;

static const unsigned char venc_data_ntsc[] = {
	0x00,	/* 00 *//* read only. not write */
	0x00,	/* 01 */
	0x00,	/* 02 */
	0x00,	/* 03 */
	0x00,	/* 04 */
	0x00,	/* 05 */
	0x00,	/* 06 */
	0x00,	/* 07 */
	0x00,	/* 08 */
	0x00,	/* 09 */
	0x00,	/* 0A */
	0x00,	/* 0B */
	0x00,	/* 0C */
	0x00,	/* 0D */
	0x00,	/* 0E */
	0x00,	/* 0F */
	0x00,	/* 10 */
	0x00,	/* 11 */
	0x00,	/* 12 */
	0x00,	/* 13 */
	0x00,	/* 14 */
	0x00,	/* 15 */
	0x00,	/* 16 */
	0x00,	/* 17 */
	0x00,	/* 18 */
	0x00,	/* 19 */
	0x00,	/* 1A */
	0x00,	/* 1B */
	0x00,	/* 1C */
	0x00,	/* 1D */
	0x00,	/* 1E */
	0x00,	/* 1F */
	0x00,	/* 20 */
	0x00,	/* 21 */
	0x00,	/* 22 */
	0x00,	/* 23 */
	0x00,	/* 24 */
	0x00,	/* 25 */
	0x00,	/* 26 */
	0x00,	/* 27 */
	0x19,	/* 28 */
	0x1d,	/* 29 */
	0x00,	/* 2A */
	0x00,	/* 2B */
	0x00,	/* 2C */
	0x3f,	/* 2D */
	0x00,	/* 2E */
	0x00,	/* 2F */
	0x00,	/* 30 */
	0x00,	/* 31 */
	0x00,	/* 32 */
	0x00,	/* 33 */
	0x00,	/* 34 */
	0x00,	/* 35 */
	0x00,	/* 36 */
	0x00,	/* 37 */
	0x00,	/* 38 */
	0x00,	/* 39 */
	0x13,	/* 3A */
	0x00,	/* 3B */
	0x00,	/* 3C */
	0x00,	/* 3D */
	0x00,	/* 3E */
	0x00,	/* 3F */
	0x00,	/* 40 */
	0x00,	/* 41 */
	0x80,	/* 42 */
	0x80,	/* 43 */
	0x80,	/* 44 */
	0x80,	/* 45 */
	0x80,	/* 46 */
	0x80,	/* 47 */
	0x80,	/* 48 */
	0x80,	/* 49 */
	0x80,	/* 4A */
	0x80,	/* 4B */
	0x80,	/* 4C */
	0x80,	/* 4D */
	0x3f,	/* 4E */
	0x3f,	/* 4F */
	0x3f,	/* 50 */
	0x80,	/* 51 */
	0x80,	/* 52 */
	0x80,	/* 53 */
	0x3c,	/* 54 */
	0x00,	/* 55 */
	0x00,	/* 56 */
	0x00,	/* 57 */
	0x00,	/* 58 */
	0x00,	/* 59 */
	0x35,	/* 5A */
	0x76,	/* 5B */
	0xa5,	/* 5C */
	0x3f,	/* 5D */
	0x2e,	/* 5E */
	0x30,	/* 5F */
	0x00,	/* 60 */
	0x15,	/* 61 */
	0x3f,	/* 62 */
	0x1f,	/* 63 */
	0x7c,	/* 64 */
	0xf0,	/* 65 */
	0x21,	/* 66 */
	0x00,	/* 67 */
	0x00,	/* 68 */
	0x00,	/* 69 */
	0x00,	/* 6A */
	0xd2,	/* 6B */
	0xfa,	/* 6C */
	0x00,	/* 6D */
	0x90,	/* 6E */
	0x00,	/* 6F */
	0x49,	/* 70 */
	0x49,	/* 71 */
	0x00,	/* 72 */
	0x00,	/* 73 */
	0x00,	/* 74 */
	0x03,	/* 75 */
	0x00,	/* 76 */
	0x00,	/* 77 */
	0x00,	/* 78 */
	0x00,	/* 79 */
	0x00,	/* 7A */
	0x01,	/* 7B */
	0x40,	/* 7C */
	0x00,	/* 7D */
	0x00,	/* 7E */
	0x00,	/* 7F */
};


static const unsigned char venc_data_pal[] = {
	0x00,	/* 00 *//* read only. not write */
	0x00,	/* 01 */
	0x00,	/* 02 */
	0x00,	/* 03 */
	0x00,	/* 04 */
	0x00,	/* 05 */
	0x00,	/* 06 */
	0x00,	/* 07 */
	0x00,	/* 08 */
	0x00,	/* 09 */
	0x00,	/* 0A */
	0x00,	/* 0B */
	0x00,	/* 0C */
	0x00,	/* 0D */
	0x00,	/* 0E */
	0x00,	/* 0F */
	0x00,	/* 10 */
	0x00,	/* 11 */
	0x00,	/* 12 */
	0x00,	/* 13 */
	0x00,	/* 14 */
	0x00,	/* 15 */
	0x00,	/* 16 */
	0x00,	/* 17 */
	0x00,	/* 18 */
	0x00,	/* 19 */
	0x00,	/* 1A */
	0x00,	/* 1B */
	0x00,	/* 1C */
	0x00,	/* 1D */
	0x00,	/* 1E */
	0x00,	/* 1F */
	0x00,	/* 20 */
	0x00,	/* 21 */
	0x00,	/* 22 */
	0x00,	/* 23 */
	0x00,	/* 24 */
	0x00,	/* 25 */
	0x00,	/* 26 */
	0x00,	/* 27 */
	0x21,	/* 28 */
	0x1d,	/* 29 */
	0x00,	/* 2A */
	0x00,	/* 2B */
	0x00,	/* 2C */
	0x3f,	/* 2D */
	0x00,	/* 2E */
	0x00,	/* 2F */
	0x00,	/* 30 */
	0x00,	/* 31 */
	0x00,	/* 32 */
	0x00,	/* 33 */
	0x00,	/* 34 */
	0x00,	/* 35 */
	0x00,	/* 36 */
	0x00,	/* 37 */
	0x00,	/* 38 */
	0x00,	/* 39 */
	0x13,	/* 3A */
	0x00,	/* 3B */
	0x00,	/* 3C */
	0x00,	/* 3D */
	0x00,	/* 3E */
	0x00,	/* 3F */
	0x00,	/* 40 */
	0x00,	/* 41 */
	0x80,	/* 42 */
	0x80,	/* 43 */
	0x80,	/* 44 */
	0x80,	/* 45 */
	0x80,	/* 46 */
	0x80,	/* 47 */
	0x80,	/* 48 */
	0x80,	/* 49 */
	0x80,	/* 4A */
	0x80,	/* 4B */
	0x80,	/* 4C */
	0x80,	/* 4D */
	0x3f,	/* 4E */
	0x3f,	/* 4F */
	0x3f,	/* 50 */
	0x80,	/* 51 */
	0x80,	/* 52 */
	0x80,	/* 53 */
	0x3c,	/* 54 */
	0x00,	/* 55 */
	0x00,	/* 56 */
	0x00,	/* 57 */
	0x00,	/* 58 */
	0x00,	/* 59 */
	0x00,	/* 5A */
	0x76,	/* 5B */
	0xa5,	/* 5C */
	0x3f,	/* 5D */
	0x2e,	/* 5E */
	0x30,	/* 5F */
	0x00,	/* 60 */
	0x16,	/* 61 */
	0x2b,	/* 62 */
	0xcb,	/* 63 */
	0x8a,	/* 64 */
	0x09,	/* 65 */
	0x2a,	/* 66 */
	0x00,	/* 67 */
	0x00,	/* 68 */
	0x00,	/* 69 */
	0x00,	/* 6A */
	0xd2,	/* 6B */
	0x02,	/* 6C */
	0x20,	/* 6D */
	0xa0,	/* 6E */
	0x00,	/* 6F */
	0x49,	/* 70 */
	0x49,	/* 71 */
	0x00,	/* 72 */
	0x00,	/* 73 */
	0x00,	/* 74 */
	0x03,	/* 75 */
	0x00,	/* 76 */
	0x00,	/* 77 */
	0x00,	/* 78 */
	0x00,	/* 79 */
	0x1b,	/* 7A */
	0x30,	/* 7B */
	0x40,	/* 7C */
	0x00,	/* 7D */
	0x00,	/* 7E */
	0x00,	/* 7F */
};

#if defined(CONFIG_XARINA_FPGA_COLO2)
static const unsigned char venc_data_adv7344_480i_nstsc[][2] = {

	{0x17, 0x02},
	{0x00, 0x9c},
	{0x01, 0x80},
	{0x02, 0x20},
	{0x0a, 0x00},
	{0x0b, 0x00},
	{0x0d, 0x00},
	{0x10, 0x10},
	{0x80, 0x10},
	{0x82, 0xc9},
	{0x83, 0x04},
	{0x84, 0x00},
	{0x86, 0x02},
	{0x87, 0x00},
	{0x88, 0x00},
	{0x89, 0x00},
	{0x8a, 0x08},
	{0x8b, 0x00},
	{0x8c, 0x1f},
	{0x8d, 0x7c},
	{0x8e, 0xf0},
	{0x8f, 0x21},
	{0x90, 0x00},
	{0x9c, 0x00},

};
#endif


void set_vout_dev(int dev)
{
	vout_dev = dev;
}

int get_vout_dev(void)
{
	return vout_dev;
}

void init_vout_dev(void)
{
	char *env;
	char str;
	
	env = getenv(CONFIG_XARINA_ENV_VIDEOOUT_DEV);
	
	if (!env) {
		sprintf(&str,"%x", VOUT_DEV_DEFAULT);
		setenv(CONFIG_XARINA_ENV_VIDEOOUT_DEV, &str);
		set_vout_dev(VOUT_DEV_DEFAULT);
		return;
	}
	
	set_vout_dev((int)simple_strtoul(env, NULL, 16));
	
}

int venc_init(unsigned long type)
{
	const unsigned char *p_data = NULL;
	int limit = 0;
	int ret;
	int i;

	/* i2c device set */
	ret = i2c_set_bus_num(VENC_I2C_DEV);
	if (ret < 0) {
		return ret;
	}

#ifdef VENC_DEBUG
	/* Read data */
	printf("Befor register set.\n");
	for(i = 0; i < 0x80; i++) {
		printf("reg:%02x data:%02x\n", i, i2c_reg_read(VENC_I2C_SLAVE_ADDR, i));
	}
#endif	/* VENC_DEBUG */

	switch(type) {
	case VENC_TYPE_NTSC:
#if !defined(CONFIG_XARINA_FPGA_COLO2)
		p_data = venc_data_ntsc;
		limit  = sizeof(venc_data_ntsc)/sizeof(venc_data_ntsc[0]);

		/* Write data */
		for(i = 1; i < limit; i++) {	/* subaddress 0 is read only register. not write. */
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, i, *(p_data + i));
		}
#else
		for (i = 0; i < 24; i++) {
			i2c_reg_write(0x6a, venc_data_adv7344_480i_nstsc[i][0],
					venc_data_adv7344_480i_nstsc[i][1]);
		}
#endif
		
		break;
	case VENC_TYPE_PAL:
		p_data = venc_data_pal;
		limit  = sizeof(venc_data_pal)/sizeof(venc_data_pal[0]);

		/* Write data */
		for(i = 1; i < limit; i++) {	/* subaddress 0 is read only register. not write. */
			i2c_reg_write(VENC_I2C_SLAVE_ADDR, i, *(p_data + i));
		}

		break;
	case VENC_TYPE_OUT_STOP:
#if !defined(CONFIG_XARINA_FPGA_COLO2)
		/* output stop */
		i2c_reg_write(VENC_I2C_SLAVE_ADDR,
		              VENC_I2C_OUT_PORT_CTRL_SADDR,
		              VENC_I2C_OUT_PORT_CTRL_STOP);
#endif
		break;
	default:
		return -1;
		break;
	}

#ifdef VENC_DEBUG
	printf("\n" "After register set.\n");
#if 0
	/* Read data */
	for(i = 0; i < 0x80; i++) {
		printf("reg:%02x data:%02x\n", i, i2c_reg_read(VENC_I2C_SLAVE_ADDR, i));
	}
#else
	uchar buffer[0x80];
	i2c_read(VENC_I2C_SLAVE_ADDR, 0, 1, buffer, 0x80);
	for(i = 0; i < 0x80; i++) {
		printf("reg:%02x data:%02x\n", i, buffer[i]);
	}
#endif
#endif	/* VENC_DEBUG */

	return 0;
}



/*
	I do not do a loop of i2c_reg_read, when I want to perform read at once:
		i2c_read(VENC_I2C_SLAVE_ADDR, 0, 1, buffer, 0x80);

	I do not do a loop of i2c_reg_read, when I want to perform write at once:
		i2c00 performs repeat start every 64 bytes transmission of a message.
		Because it includes slave address and subaddress to transmission of a message size, it is every 62 bytes for pure data.
		I have to appoint a subaddress again when I perform repeat start in a write.

		I put a subaddress to do a write next in every 62 bytes of write data.

		Or I cannot do it, I divide by 62 bytes
			i2c_write(VENC_I2C_SLAVE_ADDR, 0x01, 1, &p_data[0x01], 0x3E);
			i2c_write(VENC_I2C_SLAVE_ADDR, 0x3F, 1, &p_data[0x3F], 0x3E);
			i2c_write(VENC_I2C_SLAVE_ADDR, 0x7D, 1, &p_data[0x7D], 0x03);
*/

