/*
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * (C) Copyright 2002
 * David Mueller, ELSOFT AG, <d.mueller@elsoft.ch>
 *
 * (C) Copyright 2003
 * Texas Instruments, <www.ti.com>
 * Kshitij Gupta <Kshitij@ti.com>
 *
 * (C) Copyright 2004
 * ARM Ltd.
 * Philippe Robin, <philippe.robin@arm.com>
 *
 * (C) Copyright 2007 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <i2c.h>
#include <flash.h>
#include <asm/arch/xarina.h>
#include "boot_param.h"
#include "corefw_comm_param.h"
#include "cam_set_param.h"
#include "pinmux.h"
#include "hw_reg_ctrl.h"

DECLARE_GLOBAL_DATA_PTR;
#if !defined(CONFIG_SYS_NO_FLASH)
extern flash_info_t flash_info[];		/* FLASH chips info */
#endif /* !defind(ONFIG_SYS_NO_FLASH) */

void flash__init (void);
void ether__init (void);
int l2cache_init(void);
int biu_set(void);

#if defined(CONFIG_SHOW_BOOT_PROGRESS)
void show_boot_progress(int progress)
{
    printf("Boot reached stage %d\n", progress);
}
#endif

static inline void delay (unsigned long loops)
{
	__asm__ volatile ("1:\n"
		"subs %0, %1, #1\n"
		"bne 1b":"=r" (loops):"0" (loops));
}

/*
 * Miscellaneous platform dependent initialisations
 */

int xarina_init_f(void)
{
	/* GbE init */
	xarina_hw_gbe_init();

	/* PG is selected as UART3. */
	/* Other as GPIO */
	pinmux_gpio_uart3_set();
	
	/* NFIF clock set*/
	xarina_hw_nfif_clock_set(SYSREG1_NFIF_CKSEL_80MHZ);

#if !defined(CONFIG_XARINA_SPL)
	/* GPIO Pullup init */
	xarina_hw_gpio_pullup_init();
#endif /* !defined(CONFIG_XARINA_SPL) */
	return 0;
}

/* from start.S */
extern unsigned long _ddr_vector_start;
extern unsigned long _ddr_vector_end;

int xarina_init_r(void)
{
	unsigned long vtbl_start_addr = (unsigned long)&_ddr_vector_start;
	unsigned long vtbl_end_addr   = (unsigned long)&_ddr_vector_end;
	size_t len = vtbl_end_addr - vtbl_start_addr;

	/* vector table for ddr copy to BootRAM */
	memcpy((void *)XARINA_BOOTRAM_BASE, (void *)vtbl_start_addr, len);

	/* invalidate both caches and flush btb */
	asm ("mcr p15, 0, %0, c7, c7, 0": :"r" (0));
	/* mem barrier to sync things */
	asm ("mcr p15, 0, %0, c7, c10, 4": :"r" (0));

	/* BootRAM remap */
	xarina_hw_bootram_remap();

	return 0;
}


void abort(void)
{
	hang();
}


int board_init (void)
{
#if !defined(CONFIG_XARINA_SPL)
	/* To acquire the state of DipSW of the NOR boot or the UART boot early, it calls it here. */
	/* xarina_hw_boot_type_get(); */

	/* arch number of Versatile Board */
	gd->bd->bi_arch_number = MACH_TYPE_XARINA;

	/* adress of boot parameters */
	gd->bd->bi_boot_params = PHYS_SDRAM_1 + 0x100;
#endif /* !defined(CONFIG_XARINA_SPL) */
	gd->flags = 0;

/*	icache_enable (); */

	flash__init ();
	ether__init ();

	return 0;
}


int misc_init_r (void)
{
	setenv("verify", "n");

	return (0);
}

/******************************
 Routine:
 Description:
******************************/
void flash__init (void)
{
}
/*************************************************************
 Routine:ether__init
 Description: take the Ethernet controller out of reset and wait
	  		   for the EEPROM load to complete.
*************************************************************/
void ether__init (void)
{
}

#ifdef CONFIG_CMD_NET
#ifdef CONFIG_SMAC
extern int smac_initialize(bd_t *bis);
#endif /* CONFIG_SMAC */
#ifdef CONFIG_DRIVER_SMSC9118
extern int smsc9118_initialize(bd_t *bis);
#endif	/* CONFIG_DRIVE_SMSC9118 */

int board_eth_init(bd_t *bis)
{
	int rc = 0;

#ifdef CONFIG_SMAC
	rc = smac_initialize(bis);
#endif /* CONFIG_SMAC */

#ifdef CONFIG_DRIVER_SMSC9118
	rc = smsc9118_initialize(bis);
#endif	/* CONFIG_DRIVE_SMSC9118 */

	return rc;
}
#endif /* CONFIG_CMD_NET */

/******************************
 Routine:
 Description:
******************************/
int dram_init (void)
{              
	gd->bd->bi_dram[0].start	= PHYS_SDRAM_1;
	gd->bd->bi_dram[0].size		= PHYS_SDRAM_1_SIZE;

	return 0;
}
