/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <common.h>
#include <command.h>
#include <asm/arch/xarina.h>
#include <nand.h>

#define DEBUG

#ifdef DEBUG
#define DBG(format, arg...)	printk(KERN_ALERT "[TB]: " format, ## arg)
#else
#define DBG(format, arg...)
#endif
#define XARINA_TB_DELIMITER	(const char *) 0
#define XARINA_TB_BITERROR	"biterr"
#define XARINA_TB_BITERROR_OOB	"biterr_oob"
#define XARINA_TB_MARKBAD	"markbad"
#define XARINA_TB_MARKGOOD	"markgood"
#define XARINA_TB_PARAM		"param"
#define XARINA_TB_COREFW	"corefw"
#define XARINA_TB_VENC		"venc"
#define XARINA_TB_HDMI		"hdmi"
#define XARINA_TB_TIME		"time"
#define XARINA_TB_UPDATE	"update"
#define XARINA_TB_CHKOOB	"chkoob"
#define XARINA_TB_RMW		"rmw"
#define XARINA_TB_SCRIPT	"script"
#define XARINA_TB_CHKBLK	"chkblk"
#define XARINA_TB_MD_NOCACHE	"mdnc"
#define XARINA_TB_WEB_UPDATE	"webupdate"
#define TEST_BENCH_WORK_AREA	(0x60c00000)


/**************************/
/* prototype              */
/**************************/
static int tb_time(int argc, char * const argv[], int id);
static int tb_update(int argc, char * const argv[], int id);
static int tb_rmw(int argc, char * const argv[], int id);
static int tb_nand_biterr(int argc, char * const argv[], int id);
static int tb_nand_biterr_oob(int argc, char * const argv[], int id);
static int tb_param(int argc, char * const argv[], int id);
static int tb_corefw(int argc, char * const argv[], int id);
static int tb_venc(int argc, char * const argv[], int id);
static int tb_hdmi(int argc, char * const argv[], int id);
static int tb_markbad(int argc, char * const argv[], int id);
static int tb_markgood(int argc, char * const argv[], int id);
static int tb_chkoob(int argc, char * const argv[], int id);
static int tb_script(int argc, char * const argv[], int id);
static int tb_chkblk(int argc, char * const argv[], int id);
static int tb_mdnc(int argc, char * const argv[], int id);
static int tb_web_update(int argc, char * const argv[], int id);
static void tb_nand_biterr_usage(void);
static void tb_nand_biterr_oob_usage(void);
static void tb_time_usage(void);
static void tb_update_usage(void);
static void tb_rmw_usage(void);
static void tb_param_usage(void);
static void tb_corefw_usage(void);
static void tb_venc_usage(void);
static void tb_hdmi_usage(void);
static void tb_markbad_usage(void);
static void tb_markgood_usage(void);
static void tb_chkoob_usage(void);
static void tb_script_usage(void);
static void tb_chkblk_usage(void);
static void tb_mdnc_usage(void);
static void tb_web_update_usage(void);

/**************************/
/* test bench structure   */
/**************************/
struct s_test_bench_info {
	const char *target;
	int (*p_func)(int argc, char * const argv[], int id);
	void (*p_usage)(void);
};

/**************************/
/* global variable        */
/**************************/
static struct s_test_bench_info test_bench_table[] = {
	{XARINA_TB_BITERROR,		tb_nand_biterr,		tb_nand_biterr_usage},
	{XARINA_TB_BITERROR_OOB,	tb_nand_biterr_oob,	tb_nand_biterr_oob_usage},
	{XARINA_TB_TIME,		tb_time,		tb_time_usage},
	{XARINA_TB_UPDATE,		tb_update,		tb_update_usage},
	{XARINA_TB_RMW,			tb_rmw,			tb_rmw_usage},
	{XARINA_TB_PARAM,		tb_param,		tb_param_usage},
	{XARINA_TB_COREFW,		tb_corefw,		tb_corefw_usage},
	{XARINA_TB_VENC,		tb_venc,		tb_venc_usage},
	{XARINA_TB_HDMI,		tb_hdmi,		tb_hdmi_usage},
	{XARINA_TB_MARKBAD,		tb_markbad,		tb_markbad_usage},
	{XARINA_TB_MARKGOOD,		tb_markgood,		tb_markgood_usage},
	{XARINA_TB_CHKOOB,		tb_chkoob,		tb_chkoob_usage},
	{XARINA_TB_SCRIPT,		tb_script,		tb_script_usage},
	{XARINA_TB_CHKBLK,		tb_chkblk,		tb_chkblk_usage},
	{XARINA_TB_MD_NOCACHE,		tb_mdnc,		tb_mdnc_usage},
	{XARINA_TB_WEB_UPDATE,		tb_web_update,		tb_web_update_usage},
	{XARINA_TB_DELIMITER,		NULL,			NULL}
};
extern char cmd_buff[];

/**************************/
/* common func            */
/**************************/
static int target_check(char * const argv[], int *p_id)
{
	const char *p;
	int i;

	for (i = 0; ; i++) {
		p = test_bench_table[i].target;
		if (p == XARINA_TB_DELIMITER) {
			break;
		} else if (strcmp(p, argv[1]) == 0) {
			*p_id = i;
			return 1;
		}
	}

	return 0;

}

static void target_print(void)
{
	int i;

	printf("\nSupported test bench targets are below.\n");
	for (i = 0; test_bench_table[i].target != XARINA_TB_DELIMITER; i++) {
		printf("\t\"%s\"\n",test_bench_table[i].target);
	}
}

static void usage_print(int id)
{
	int i;

	/* check id */
	for (i = 0; i <= id; i++) {
		if (test_bench_table[i].target == XARINA_TB_DELIMITER) {
			printf("invalid id!\n");
			return;
		}
	}
	/* usage print */
	if (test_bench_table[id].p_usage != NULL) {
		test_bench_table[id].p_usage();
	}

}
static void tb_run_command(char *cmd)
{
	DBG("%s\n", cmd);
	run_command(cmd, 0);
}

#if 0
/**************************/
/*  skeleton              */
/**************************/
static int tb_XXXXXX(int argc, char * const argv[], int id)
{
	printf("exec %s\n", __func__);
	return 0;
}

static void tb_XXXXX_usage(void)
{
	printf("\nusage:\n");
	printf("# testbench %s \n",
		XARINA_TB_XXXXX);
}

#endif

/**************************/
/*  web update            */
/**************************/
static int tb_web_update(int argc, char * const argv[], int id)
{
	int ret;
	unsigned long value;
	char cmd[64];

	DBG("exec %s\n", __func__);
	if (argc != 3) {
		usage_print(id);
		return 1;
	}

	value = simple_strtoul(argv[2], NULL, 16);

	sprintf(cmd, "setenv web_update_indicator %x", value);
	tb_run_command(cmd);

	tb_run_command("saveenv");
	tb_run_command("reset");

	return 0;
}

static void tb_web_update_usage(void)
{
	printf("\nusage:\n");
	printf("# testbench %s <value> \n",
		"  - <value> is web update indicator environment value\n",
		XARINA_TB_WEB_UPDATE);
}

/**************************/
/*  script                */
/**************************/
static int tb_script(int argc, char * const argv[], int id)
{
	if (argc != 4) {
		usage_print(id);
		return 1;
	}
	int ret;
	int i, size, idx, ign;
	unsigned char *addr;



	/* remove filesize */
	run_command("setenv filesize", 0);

	if (*(argv[2]) == 't') {
		sprintf(cmd_buff, "tftpboot %x %s", 
				CONFIG_XARINA_SD_BOOTSCRIPT_LOAD_ADDR, argv[3]);
	} else {
		sprintf(cmd_buff, "ubifsload %x %s", 
				CONFIG_XARINA_SD_BOOTSCRIPT_LOAD_ADDR, argv[3]);

	}
	ret = run_command(cmd_buff, 0);


	if (getenv("filesize") == NULL) {
		printf("boot script load error.\n");
		return 1;
	}
	/* size check */
	size = (int)simple_strtoul(getenv("filesize"), NULL, 16);
	if (size >= CONFIG_XARINA_SD_BOOTSCRIPT_MAX_SIZE) {
		printf("script file is too big\n");
		return 1;
	}

	addr = (unsigned char *)CONFIG_XARINA_SD_BOOTSCRIPT_LOAD_ADDR;


	for (i = 0, idx = 0, ign = 0; i < size; i++) {
		if (*addr == '\n' || *addr == '\r') {
			if (idx != 0) {
				cmd_buff[idx] = '\0';
				tb_run_command(cmd_buff);
			}
			idx = 0;
			ign = 0;
		} else if (*addr == '#') {
			ign = 1;
		} else {
			if (ign == 0) {
				cmd_buff[idx] = *addr;
				idx++;
				if (idx >= CONFIG_SYS_CBSIZE - 1) {
					ign = 1;
				}
			}
		}

		addr++;
	}
	if (idx != 0) {
		cmd_buff[idx] = '\0';
		tb_run_command(cmd_buff);
	}

	return 0;
}

static void tb_script_usage(void)
{
	printf("\nusage:\n");
	printf("# testbench %s \n",
		XARINA_TB_SCRIPT);
}


/**************************/
/*  venc                  */
/**************************/
static void tb_venc_common(unsigned char fmt)
{
	char cmd[64];

	
	sprintf(cmd, "mw.b 600000a0 %x", fmt);
	tb_run_command("setenv camfw_setting_param test/camfw_param_normal0.dat");
	tb_run_command("setenv corefw_comm_param test/corefw_param_normal0.dat");
	tb_run_command("setenv corefw_bin test/corefw_normal.bin");
	tb_run_command("mw.b 60001001 3");
	tb_run_command("setenv cam_bin_imx122 test/sens_imx122_normal.bin");
	tb_run_command("setenv vout_dev 0");
	tb_run_command(cmd);
	tb_run_command("setenv debug yes");
	tb_run_command("bootcofw");

}
static void tb_venc_scenario_00(void)
{
	tb_venc_common(0x00);
}
static void tb_venc_scenario_01(void)
{
	tb_venc_common(0x01);

}
static void tb_venc_scenario_02(void)
{
	tb_venc_common(0x02);

}
static void tb_venc_scenario_03(void)
{
	tb_venc_common(0x03);

}
static void tb_venc_scenario_04(void)
{
	tb_venc_common(0x04);

}
static void tb_venc_scenario_05(void)
{
	tb_venc_common(0x05);

}
static void tb_venc_scenario_06(void)
{
	tb_venc_common(0x06);

}
static void tb_venc_scenario_07(void)
{
	tb_venc_common(0x07);

}
static void tb_venc_scenario_08(void)
{
	tb_venc_common(0x08);

}
static void tb_venc_scenario_09(void)
{
	tb_venc_common(0x09);

}
static void tb_venc_scenario_0a(void)
{
	tb_venc_common(0x0a);

}
static void tb_venc_scenario_0b(void)
{
	tb_venc_common(0x0b);

}
static void tb_venc_scenario_0c(void)
{
	tb_venc_common(0x0c);

}
static void tb_venc_scenario_0d(void)
{
	tb_venc_common(0x0d);

}
static void tb_venc_scenario_0e(void)
{
	tb_venc_common(0x0e);

}
static void tb_venc_scenario_0f(void)
{
	tb_venc_common(0x21);

}
static void tb_venc_scenario_10(void)
{
	tb_venc_common(0x22);

}
static void tb_venc_scenario_11(void)
{
	tb_venc_common(0x23);

}
static void tb_venc_scenario_12(void)
{
	tb_venc_common(0x24);

}
static void tb_venc_scenario_13(void)
{
	tb_venc_common(0x25);

}
static void tb_venc_scenario_14(void)
{
	tb_venc_common(0x26);

}
static void tb_venc_scenario_15(void)
{
	tb_venc_common(0x27);

}
static void tb_venc_scenario_16(void)
{
	tb_venc_common(0x28);

}
static void tb_venc_scenario_17(void)
{
	tb_venc_common(0x29);

}
static void tb_venc_scenario_18(void)
{
	tb_venc_common(0x2a);

}
static void tb_venc_scenario_19(void)
{
	tb_venc_common(0x2b);

}
static void tb_venc_scenario_1a(void)
{
	tb_venc_common(0x2c);

}
static void tb_venc_scenario_1b(void)
{
	tb_venc_common(0x41);

}
static void tb_venc_scenario_1c(void)
{
	tb_venc_common(0x42);

}
static void tb_venc_scenario_1d(void)
{
	tb_venc_common(0x43);

}
static void tb_venc_scenario_1e(void)
{
	tb_venc_common(0x44);

}
static void tb_venc_scenario_1f(void)
{
	tb_venc_common(0x45);

}
static void tb_venc_scenario_20(void)
{
	tb_venc_common(0x46);

}
static void tb_venc_scenario_21(void)
{
	tb_venc_common(0x47);

}
static void tb_venc_scenario_22(void)
{
	tb_venc_common(0x48);

}
static void tb_venc_scenario_23(void)
{
	tb_venc_common(0x49);

}
static void tb_venc_scenario_24(void)
{
	tb_venc_common(0x4a);

}
static void tb_venc_scenario_25(void)
{
	tb_venc_common(0x4b);

}
static void tb_venc_scenario_26(void)
{
	tb_venc_common(0x4c);

}
static void tb_venc_scenario_27(void)
{
	tb_venc_common(0x4d);

}
static void tb_venc_scenario_28(void)
{
	tb_venc_common(0x4e);

}
static void tb_venc_scenario_29(void)
{
	tb_venc_common(0x4f);

}
static void tb_venc_scenario_2a(void)
{
	tb_venc_common(0x50);

}
static void tb_venc_scenario_2b(void)
{
	tb_venc_common(0x51);

}
static void tb_venc_scenario_2c(void)
{
	tb_venc_common(0x52);

}
static void tb_venc_scenario_2d(void)
{
	tb_venc_common(0xff);

}

static void tb_venc_scenario_2e(void)
{
	tb_venc_common(0x0f);

}
static void tb_venc_scenario_2f(void)
{
	tb_venc_common(0x10);

}
static void tb_venc_scenario_30(void)
{
	tb_venc_common(0x11);

}
static void tb_venc_scenario_31(void)
{
	tb_venc_common(0x12);

}
static void tb_venc_scenario_32(void)
{
	tb_venc_common(0x13);

}
static void tb_venc_scenario_33(void)
{
	tb_venc_common(0x14);

}
static void tb_venc_scenario_34(void)
{
	tb_venc_common(0x15);

}
static void tb_venc_scenario_35(void)
{
	tb_venc_common(0x16);

}
static void tb_venc_scenario_36(void)
{
	tb_venc_common(0x17);

}
static void tb_venc_scenario_37(void)
{
	tb_venc_common(0x18);
}

static void (*tb_venc_table[])(void) = {
	tb_venc_scenario_00,
	tb_venc_scenario_01,
	tb_venc_scenario_02,
	tb_venc_scenario_03,
	tb_venc_scenario_04,
	tb_venc_scenario_05,
	tb_venc_scenario_06,
	tb_venc_scenario_07,
	tb_venc_scenario_08,
	tb_venc_scenario_09,
	tb_venc_scenario_0a,
	tb_venc_scenario_0b,
	tb_venc_scenario_0c,
	tb_venc_scenario_0d,
	tb_venc_scenario_0e,
	tb_venc_scenario_0f,
	tb_venc_scenario_10,
	tb_venc_scenario_11,
	tb_venc_scenario_12,
	tb_venc_scenario_13,
	tb_venc_scenario_14,
	tb_venc_scenario_15,
	tb_venc_scenario_16,
	tb_venc_scenario_17,
	tb_venc_scenario_18,
	tb_venc_scenario_19,
	tb_venc_scenario_1a,
	tb_venc_scenario_1b,
	tb_venc_scenario_1c,
	tb_venc_scenario_1d,
	tb_venc_scenario_1e,
	tb_venc_scenario_1f,
	tb_venc_scenario_20,
	tb_venc_scenario_21,
	tb_venc_scenario_22,
	tb_venc_scenario_23,
	tb_venc_scenario_24,
	tb_venc_scenario_25,
	tb_venc_scenario_26,
	tb_venc_scenario_27,
	tb_venc_scenario_28,
	tb_venc_scenario_29,
	tb_venc_scenario_2a,
	tb_venc_scenario_2b,
	tb_venc_scenario_2c,
	tb_venc_scenario_2d,
	tb_venc_scenario_2e,
	tb_venc_scenario_2f,
	tb_venc_scenario_30,
	tb_venc_scenario_31,
	tb_venc_scenario_32,
	tb_venc_scenario_33,
	tb_venc_scenario_34,
	tb_venc_scenario_35,
	tb_venc_scenario_36,
	tb_venc_scenario_37,
	NULL
};

static int tb_venc(int argc, char * const argv[], int id)
{
	int scenario;
	int i;

	if (argc != 3) {
		usage_print(id);
		return 1;
	}

	if (strcmp(argv[2], "all") == 0) {
		for (i = 0; i < 256; i++) {
			if (tb_venc_table[i] != NULL) {
				printf("\n\n\n** scenario 0x%02x start ***********************\n", i);
				tb_venc_table[i]();
			} else {
				return 0;
			}
		}
	}

	scenario = (int)simple_strtoul(argv[2], NULL, 16);
	
	for (i = 0; i < 256; i++) {
		if (tb_venc_table[i] == NULL) {
			printf("scenario number is invalid\n");
			return 1;
		} else if (i == scenario) {
			tb_venc_table[i]();
			return 0;
		}
	}

	return 0;

}

static void tb_venc_usage(void)
{
	printf("\nusage:\n");
	printf("# testbench %s <scenario num>\n",
		XARINA_TB_VENC);
}
/**************************/
/*  hdmi                  */
/**************************/
static void tb_hdmi_common(unsigned char fmt)
{
	char cmd[64];

	
	sprintf(cmd, "mw.b 600000a0 %x", fmt);
	tb_run_command("setenv camfw_setting_param test/camfw_param_normal0.dat");
	tb_run_command("setenv corefw_comm_param test/corefw_param_normal0.dat");
	tb_run_command("setenv corefw_bin test/corefw_normal.bin");
	tb_run_command("mw.b 60001001 3");
	tb_run_command("setenv cam_bin_imx122 test/sens_imx122_normal.bin");
	tb_run_command("setenv vout_dev 1");
	tb_run_command(cmd);
	tb_run_command("setenv debug yes");
	tb_run_command("bootcofw");

}
static void tb_hdmi_scenario_00(void)
{
	tb_hdmi_common(0x00);
}
static void tb_hdmi_scenario_01(void)
{
	tb_hdmi_common(0x01);

}
static void tb_hdmi_scenario_02(void)
{
	tb_hdmi_common(0x02);

}
static void tb_hdmi_scenario_03(void)
{
	tb_hdmi_common(0x03);

}
static void tb_hdmi_scenario_04(void)
{
	tb_hdmi_common(0x04);

}
static void tb_hdmi_scenario_05(void)
{
	tb_hdmi_common(0x05);

}
static void tb_hdmi_scenario_06(void)
{
	tb_hdmi_common(0x06);

}
static void tb_hdmi_scenario_07(void)
{
	tb_hdmi_common(0x07);

}
static void tb_hdmi_scenario_08(void)
{
	tb_hdmi_common(0x08);

}
static void tb_hdmi_scenario_09(void)
{
	tb_hdmi_common(0x09);

}
static void tb_hdmi_scenario_0a(void)
{
	tb_hdmi_common(0x0a);

}
static void tb_hdmi_scenario_0b(void)
{
	tb_hdmi_common(0x0b);

}
static void tb_hdmi_scenario_0c(void)
{
	tb_hdmi_common(0x0c);

}
static void tb_hdmi_scenario_0d(void)
{
	tb_hdmi_common(0x0d);

}
static void tb_hdmi_scenario_0e(void)
{
	tb_hdmi_common(0x0e);

}
static void tb_hdmi_scenario_0f(void)
{
	tb_hdmi_common(0x21);

}
static void tb_hdmi_scenario_10(void)
{
	tb_hdmi_common(0x22);

}
static void tb_hdmi_scenario_11(void)
{
	tb_hdmi_common(0x23);

}
static void tb_hdmi_scenario_12(void)
{
	tb_hdmi_common(0x24);

}
static void tb_hdmi_scenario_13(void)
{
	tb_hdmi_common(0x25);

}
static void tb_hdmi_scenario_14(void)
{
	tb_hdmi_common(0x26);

}
static void tb_hdmi_scenario_15(void)
{
	tb_hdmi_common(0x27);

}
static void tb_hdmi_scenario_16(void)
{
	tb_hdmi_common(0x28);

}
static void tb_hdmi_scenario_17(void)
{
	tb_hdmi_common(0x29);

}
static void tb_hdmi_scenario_18(void)
{
	tb_hdmi_common(0x2a);

}
static void tb_hdmi_scenario_19(void)
{
	tb_hdmi_common(0x2b);

}
static void tb_hdmi_scenario_1a(void)
{
	tb_hdmi_common(0x2c);

}
static void tb_hdmi_scenario_1b(void)
{
	tb_hdmi_common(0x41);

}
static void tb_hdmi_scenario_1c(void)
{
	tb_hdmi_common(0x42);

}
static void tb_hdmi_scenario_1d(void)
{
	tb_hdmi_common(0x43);

}
static void tb_hdmi_scenario_1e(void)
{
	tb_hdmi_common(0x44);

}
static void tb_hdmi_scenario_1f(void)
{
	tb_hdmi_common(0x45);

}
static void tb_hdmi_scenario_20(void)
{
	tb_hdmi_common(0x46);

}
static void tb_hdmi_scenario_21(void)
{
	tb_hdmi_common(0x47);

}
static void tb_hdmi_scenario_22(void)
{
	tb_hdmi_common(0x48);

}
static void tb_hdmi_scenario_23(void)
{
	tb_hdmi_common(0x49);

}
static void tb_hdmi_scenario_24(void)
{
	tb_hdmi_common(0x4a);

}
static void tb_hdmi_scenario_25(void)
{
	tb_hdmi_common(0x4b);

}
static void tb_hdmi_scenario_26(void)
{
	tb_hdmi_common(0x4c);

}
static void tb_hdmi_scenario_27(void)
{
	tb_hdmi_common(0x4d);

}
static void tb_hdmi_scenario_28(void)
{
	tb_hdmi_common(0x4e);

}
static void tb_hdmi_scenario_29(void)
{
	tb_hdmi_common(0x4f);

}
static void tb_hdmi_scenario_2a(void)
{
	tb_hdmi_common(0x50);

}
static void tb_hdmi_scenario_2b(void)
{
	tb_hdmi_common(0x51);

}
static void tb_hdmi_scenario_2c(void)
{
	tb_hdmi_common(0x52);

}
static void tb_hdmi_scenario_2d(void)
{
	tb_hdmi_common(0xff);

}
static void tb_hdmi_scenario_2e(void)
{
	tb_hdmi_common(0x0f);

}
static void tb_hdmi_scenario_2f(void)
{
	tb_hdmi_common(0x10);

}
static void tb_hdmi_scenario_30(void)
{
	tb_hdmi_common(0x11);

}
static void tb_hdmi_scenario_31(void)
{
	tb_hdmi_common(0x12);

}
static void tb_hdmi_scenario_32(void)
{
	tb_hdmi_common(0x13);

}
static void tb_hdmi_scenario_33(void)
{
	tb_hdmi_common(0x14);

}
static void tb_hdmi_scenario_34(void)
{
	tb_hdmi_common(0x15);

}
static void tb_hdmi_scenario_35(void)
{
	tb_hdmi_common(0x16);

}
static void tb_hdmi_scenario_36(void)
{
	tb_hdmi_common(0x17);

}
static void tb_hdmi_scenario_37(void)
{
	tb_hdmi_common(0x18);

}


static void (*tb_hdmi_table[])(void) = {
	tb_hdmi_scenario_00,
	tb_hdmi_scenario_01,
	tb_hdmi_scenario_02,
	tb_hdmi_scenario_03,
	tb_hdmi_scenario_04,
	tb_hdmi_scenario_05,
	tb_hdmi_scenario_06,
	tb_hdmi_scenario_07,
	tb_hdmi_scenario_08,
	tb_hdmi_scenario_09,
	tb_hdmi_scenario_0a,
	tb_hdmi_scenario_0b,
	tb_hdmi_scenario_0c,
	tb_hdmi_scenario_0d,
	tb_hdmi_scenario_0e,
	tb_hdmi_scenario_0f,
	tb_hdmi_scenario_10,
	tb_hdmi_scenario_11,
	tb_hdmi_scenario_12,
	tb_hdmi_scenario_13,
	tb_hdmi_scenario_14,
	tb_hdmi_scenario_15,
	tb_hdmi_scenario_16,
	tb_hdmi_scenario_17,
	tb_hdmi_scenario_18,
	tb_hdmi_scenario_19,
	tb_hdmi_scenario_1a,
	tb_hdmi_scenario_1b,
	tb_hdmi_scenario_1c,
	tb_hdmi_scenario_1d,
	tb_hdmi_scenario_1e,
	tb_hdmi_scenario_1f,
	tb_hdmi_scenario_20,
	tb_hdmi_scenario_21,
	tb_hdmi_scenario_22,
	tb_hdmi_scenario_23,
	tb_hdmi_scenario_24,
	tb_hdmi_scenario_25,
	tb_hdmi_scenario_26,
	tb_hdmi_scenario_27,
	tb_hdmi_scenario_28,
	tb_hdmi_scenario_29,
	tb_hdmi_scenario_2a,
	tb_hdmi_scenario_2b,
	tb_hdmi_scenario_2c,
	tb_hdmi_scenario_2d,
	tb_hdmi_scenario_2e,
	tb_hdmi_scenario_2f,
	tb_hdmi_scenario_30,
	tb_hdmi_scenario_31,
	tb_hdmi_scenario_32,
	tb_hdmi_scenario_33,
	tb_hdmi_scenario_34,
	tb_hdmi_scenario_35,
	tb_hdmi_scenario_36,
	tb_hdmi_scenario_37,
	NULL
};

static int tb_hdmi(int argc, char * const argv[], int id)
{
	int scenario;
	int i;

	if (argc != 3) {
		usage_print(id);
		return 1;
	}

	if (strcmp(argv[2], "all") == 0) {
		for (i = 0; i < 256; i++) {
			if (tb_hdmi_table[i] != NULL) {
				printf("\n\n\n** scenario 0x%02x start ***********************\n", i);
				tb_hdmi_table[i]();
			} else {
				return 0;
			}
		}
	}

	scenario = (int)simple_strtoul(argv[2], NULL, 16);
	
	for (i = 0; i < 256; i++) {
		if (tb_hdmi_table[i] == NULL) {
			printf("scenario number is invalid\n");
			return 1;
		} else if (i == scenario) {
			tb_hdmi_table[i]();
			return 0;
		}
	}

	return 0;

}

static void tb_hdmi_usage(void)
{
	printf("\nusage:\n");
	printf("# testbench %s <scenario num>\n",
		XARINA_TB_HDMI);
}
/**************************/
/*  corefw                */
/**************************/
static void tb_corefw_scenario_pre(void)
{
	tb_run_command("setenv camfw_setting_param test/camfw_param_normal0.dat");
	tb_run_command("setenv corefw_comm_param test/corefw_param_normal0.dat");
	tb_run_command("setenv corefw_bin test/corefw_normal.bin");
	tb_run_command("setenv cam_bin_imx122 test/sens_imx122_normal.bin");
	tb_run_command("mw.b 60001001 3");
}

static void tb_corefw_scenario_00(void)
{
	tb_run_command("mw.b 600000ac 1");
	tb_run_command("setenv osd_color_palette test/osd_color_param_normal0.dat");
	tb_run_command("bootcofw");
	tb_run_command("ubifsload 65000000 test/osd_color_param_normal0.dat");
	tb_run_command("cmp.b 65000080 607c0400 3fc");
}
static void tb_corefw_scenario_01(void)
{
	tb_run_command("mw.b 600000ac 1");
	tb_run_command("setenv osd_color_palette test/osd_color_param_normal1.dat");
	tb_run_command("bootcofw");
	tb_run_command("ubifsload 65000000 test/osd_color_param_normal1.dat");
	tb_run_command("cmp.b 65000080 697c0400 3fc");
}
static void tb_corefw_scenario_02(void)
{
	tb_run_command("mw.b 600000ac 0");
	tb_run_command("setenv osd_color_palette test/osd_color_param_normal1.dat");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_03(void)
{
	tb_run_command("mw.b 600000ac 1");
	tb_run_command("setenv osd_color_palette");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_04(void)
{
	tb_run_command("mw.b 600000ac 1");
	tb_run_command("setenv osd_color_palette hoge.dat");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_05(void)
{
	tb_run_command("mw.b 600000ac 1");
	tb_run_command("setenv osd_color_palette test/osd_color_param_fourcc_invalid.dat");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_06(void)
{
	tb_run_command("mw.b 600000ac 1");
	tb_run_command("setenv osd_color_palette test/osd_color_param_addr_invalid.dat");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_07(void)
{
	tb_run_command("mw.b 600000ac 1");
	tb_run_command("setenv osd_color_palette test/osd_color_param_size_invalid.dat");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_08(void)
{
	tb_run_command("mw.b 600000ac 1");
	tb_run_command("setenv osd_data test/osd_data_param_normal0.dat");
	tb_run_command("bootcofw");
	tb_run_command("ubifsload 65000000 test/osd_data_param_normal0.dat");
	tb_run_command("cmp.b 65000080 60740400 840");
}
static void tb_corefw_scenario_09(void)
{
	tb_run_command("mw.b 600000ac 1");
	tb_run_command("setenv osd_data test/osd_data_param_normal1.dat");
	tb_run_command("bootcofw");
	tb_run_command("ubifsload 65000000 test/osd_data_param_normal1.dat");
	tb_run_command("cmp.b 65000080 69740400 840");

}
static void tb_corefw_scenario_0a(void)
{
	tb_run_command("mw.b 600000ac 0");
	tb_run_command("setenv osd_data test/osd_data_param_normal0.dat");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_0b(void)
{
	tb_run_command("mw.b 600000ac 1");
	tb_run_command("setenv osd_data");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_0c(void)
{
	tb_run_command("mw.b 600000ac 1");
	tb_run_command("setenv osd_data hoge.dat");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_0d(void)
{
	tb_run_command("mw.b 600000ac 1");
	tb_run_command("setenv osd_data test/osd_data_param_fourcc_invalid.dat");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_0e(void)
{
	tb_run_command("mw.b 600000ac 1");
	tb_run_command("setenv osd_data test/osd_data_param_addr_invalid.dat");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_0f(void)
{
	tb_run_command("mw.b 600000ac 1");
	tb_run_command("setenv osd_data test/osd_data_param_size_invalid.dat");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_10(void)
{
	tb_run_command("mw.b 600000ac 2");
	tb_run_command("setenv mask_data test/mask_param_normal0.dat");
	tb_run_command("bootcofw");
	tb_run_command("ubifsload 65000000 test/mask_param_normal0.dat");
	tb_run_command("cmp.b 65000080 60740000 2a0");

}
static void tb_corefw_scenario_11(void)
{
	tb_run_command("mw.b 600000ac 2");
	tb_run_command("setenv mask_data test/mask_param_normal1.dat");
	tb_run_command("bootcofw");
	tb_run_command("ubifsload 65000000 test/mask_param_normal1.dat");
	tb_run_command("cmp.b 65000080 69740000 2a0");

}
static void tb_corefw_scenario_12(void)
{
	tb_run_command("mw.b 600000ac 0");
	tb_run_command("setenv mask_data test/mask_param_normal0.dat");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_13(void)
{
	tb_run_command("mw.b 600000ac 2");
	tb_run_command("setenv mask_data");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_14(void)
{
	tb_run_command("mw.b 600000ac 2");
	tb_run_command("setenv mask_data hoge.dat");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_15(void)
{
	tb_run_command("mw.b 600000ac 2");
	tb_run_command("setenv mask_data test/mask_param_fourcc_invalid.dat");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_16(void)
{
	tb_run_command("mw.b 600000ac 2");
	tb_run_command("setenv mask_data test/mask_param_addr_invalid.dat");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_17(void)
{
	tb_run_command("mw.b 600000ac 2");
	tb_run_command("setenv mask_data test/mask_param_size_invalid.dat");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_18(void)
{
	tb_run_command("setenv corefw_bin test/corefw_normal.bin");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_19(void)
{
	tb_run_command("setenv corefw_bin test/corefw_fourcc_invalid.bin");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_1a(void)
{
	tb_run_command("setenv corefw_bin");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_1b(void)
{
	tb_run_command("setenv corefw_bin hoge.bin");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_1c(void)
{
	tb_run_command("mw.b 60001001 0");
	tb_run_command("setenv cam_bin_imx035 test/sens_imx035.bin");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_1d(void)
{
	tb_run_command("mw.b 60001001 1");
	tb_run_command("setenv cam_bin_imx137 test/sens_imx137.bin");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_1e(void)
{
	tb_run_command("mw.b 60001001 2");
	tb_run_command("setenv cam_bin_imx139 test/sens_imx139.bin");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_1f(void)
{
	tb_run_command("mw.b 60001001 3");
	tb_run_command("setenv cam_bin_imx122 test/sens_imx122.bin");
	tb_run_command("bootcofw");
}

static void tb_corefw_scenario_20(void)
{
	tb_run_command("mw.b 60001001 4");
	tb_run_command("setenv cam_bin_imx140 test/sens_imx140.bin");
	tb_run_command("bootcofw");
}
static void tb_corefw_scenario_21(void)
{
	tb_run_command("mw.b 60001001 5");
	tb_run_command("setenv cam_bin_imx103 test/sens_imx103.bin");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_22(void)
{
	tb_run_command("mw.b 60001001 6");
	tb_run_command("setenv cam_bin_imx123mipi test/sens_imx123mipi.bin");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_23(void)
{
	tb_run_command("mw.b 60001001 7");
	tb_run_command("setenv cam_bin_imx123mipiw test/sens_imx123mipiw.bin");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_24(void)
{
	tb_run_command("mw.b 60001001 10");
	tb_run_command("setenv cam_bin_icx445 test/sens_icx445.bin");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_25(void)
{
	tb_run_command("mw.b 60001001 11");
	tb_run_command("setenv cam_bin_icx692 test/sens_icx692.bin");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_26(void)
{
	tb_run_command("mw.b 60001001 12");
	tb_run_command("setenv cam_bin_icx693 test/sens_icx693.bin");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_27(void)
{
	tb_run_command("mw.b 60001001 20");
	tb_run_command("setenv cam_bin_vin test/sens_vin.bin");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_28(void)
{
	tb_run_command("mw.b 60001001 21");
	tb_run_command("setenv cam_bin_vin test/sens_vin.bin");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_29(void)
{
	tb_run_command("mw.b 60001001 22");
	tb_run_command("setenv cam_bin_vin test/sens_vin.bin");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_2a(void)
{
	tb_run_command("mw.b 60001001 23");
	tb_run_command("setenv cam_bin_vin test/sens_vin.bin");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_2b(void)
{
	tb_run_command("mw.b 60001001 0");
	tb_run_command("setenv cam_bin_imx035 test/sens_fourcc_invalid.bin");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_2c(void)
{
	tb_run_command("mw.b 60001001 0");
	tb_run_command("setenv cam_bin_imx035");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_2d(void)
{
	tb_run_command("mw.b 60001001 0");
	tb_run_command("setenv cam_bin_imx035 hoge.bin");
	tb_run_command("bootcofw");

}
static void tb_corefw_scenario_2e(void)
{
	tb_run_command("mw.b 600000c1 1");
	tb_run_command("bootcofw");
	tb_run_command("i2c md 18 8");
}
static void tb_corefw_scenario_2f(void)
{
	tb_run_command("mw.b 600000c1 2");
	tb_run_command("bootcofw");
	tb_run_command("i2c md 18 8");
}

static void tb_corefw_scenario_30(void)
{
	tb_run_command("mw.b 600000c1 0");
	tb_run_command("bootcofw");
	tb_run_command("i2c md 18 8");
}
static void tb_corefw_scenario_31(void)
{
	tb_run_command("mw.b 600000c2 1");
	tb_run_command("bootcofw");
	tb_run_command("i2c md 18 9");

}
static void tb_corefw_scenario_32(void)
{
	tb_run_command("mw.b 600000c2 2");
	tb_run_command("bootcofw");
	tb_run_command("i2c md 18 9");

}
static void tb_corefw_scenario_33(void)
{
	tb_run_command("mw.b 600000c2 3");
	tb_run_command("bootcofw");
	tb_run_command("i2c md 18 9");

}
static void tb_corefw_scenario_34(void)
{
	tb_run_command("mw.b 600000c2 0");
	tb_run_command("bootcofw");
	tb_run_command("i2c md 18 9");

}
static void tb_corefw_scenario_35(void)
{

}
static void tb_corefw_scenario_36(void)
{

}


static void (*tb_corefw_table[])(void) = {
	tb_corefw_scenario_00,
	tb_corefw_scenario_01,
	tb_corefw_scenario_02,
	tb_corefw_scenario_03,
	tb_corefw_scenario_04,
	tb_corefw_scenario_05,
	tb_corefw_scenario_06,
	tb_corefw_scenario_07,
	tb_corefw_scenario_08,
	tb_corefw_scenario_09,
	tb_corefw_scenario_0a,
	tb_corefw_scenario_0b,
	tb_corefw_scenario_0c,
	tb_corefw_scenario_0d,
	tb_corefw_scenario_0e,
	tb_corefw_scenario_0f,
	tb_corefw_scenario_10,
	tb_corefw_scenario_11,
	tb_corefw_scenario_12,
	tb_corefw_scenario_13,
	tb_corefw_scenario_14,
	tb_corefw_scenario_15,
	tb_corefw_scenario_16,
	tb_corefw_scenario_17,
	tb_corefw_scenario_18,
	tb_corefw_scenario_19,
	tb_corefw_scenario_1a,
	tb_corefw_scenario_1b,
	tb_corefw_scenario_1c,
	tb_corefw_scenario_1d,
	tb_corefw_scenario_1e,
	tb_corefw_scenario_1f,
	tb_corefw_scenario_20,
	tb_corefw_scenario_21,
	tb_corefw_scenario_22,
	tb_corefw_scenario_23,
	tb_corefw_scenario_24,
	tb_corefw_scenario_25,
	tb_corefw_scenario_26,
	tb_corefw_scenario_27,
	tb_corefw_scenario_28,
	tb_corefw_scenario_29,
	tb_corefw_scenario_2a,
	tb_corefw_scenario_2b,
	tb_corefw_scenario_2c,
	tb_corefw_scenario_2d,
	tb_corefw_scenario_2e,
	tb_corefw_scenario_2f,
	tb_corefw_scenario_30,
	tb_corefw_scenario_31,
	tb_corefw_scenario_32,
	tb_corefw_scenario_33,
	tb_corefw_scenario_34,
	tb_corefw_scenario_35,
	tb_corefw_scenario_36,
	NULL
};


static int tb_corefw(int argc, char * const argv[], int id)
{
	int scenario;
	int i;

	if (argc != 3) {
		usage_print(id);
		return 1;
	}

	if (strcmp(argv[2], "all") == 0) {
		for (i = 0; i < 256; i++) {
			if (tb_corefw_table[i] != NULL) {
				printf("\n\n\n** scenario 0x%02x start ***********************\n", i);
				tb_corefw_scenario_pre();
				tb_corefw_table[i]();
			} else {
				return 0;
			}
		}
	}

	scenario = (int)simple_strtoul(argv[2], NULL, 16);
	
	for (i = 0; i < 256; i++) {
		if (tb_corefw_table[i] == NULL) {
			printf("scenario number is invalid\n");
			return 1;
		} else if (i == scenario) {
			tb_corefw_table[i]();
			return 0;
		}
	}


	return 0;
}

static void tb_corefw_usage(void)
{
	printf("\nusage:\n");
	printf("# testbench %s <scenario num>\n",
		XARINA_TB_COREFW);
}

/**************************/
/*  parameter             */
/**************************/


static void tb_param_scenario_00(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_01(void)
{
	tb_run_command("setenv u-boot_ubi_part hoge");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_02(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume foo");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_03(void)
{
	tb_run_command("setenv u-boot_ubi_part hoge");
	tb_run_command("setenv u-boot_ubi_volume foo");
	tb_run_command("saveenv");
	tb_run_command("reset");
}

static void tb_param_scenario_04(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param test/corefw_param_normal0.dat");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");
}

static void tb_param_scenario_05(void)
{
	memset((void*)0x65000000, 0, 0x10000);
	tb_run_command("ubifsload 65000000 test/corefw_param_normal0.dat");
	tb_run_command("cmp.b 65000080 60000000 220");

}

static void tb_param_scenario_06(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param test/corefw_param_normal1.dat");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");

}
static void tb_param_scenario_07(void)
{
	memset((void*)0x65000000, 0, 0x10000);
	tb_run_command("ubifsload 65000000 test/corefw_param_normal1.dat");
	tb_run_command("cmp.b 65000080 69000000 220");

}
static void tb_param_scenario_08(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_09(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param hoge.dat");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_0a(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param test/corefw_param_fourcc_invalid.dat");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_0b(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param test/corefw_param_addr_invalid.dat");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_0c(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param test/corefw_param_size_invalid.dat");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");

}
static void tb_param_scenario_0d(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param test/camfw_param_normal0.dat");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_0e(void)
{
	memset((void*)0x65000000, 0, 0x10000);
	tb_run_command("ubifsload 65000000 test/camfw_param_normal0.dat");
	tb_run_command("cmp.b 65000080 60001000 8000");
}
static void tb_param_scenario_0f(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param test/camfw_param_normal1.dat");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_10(void)
{
	memset((void*)0x65000000, 0, 0x10000);
	tb_run_command("ubifsload 65000000 test/camfw_param_normal1.dat");
	tb_run_command("cmp.b 65000080 60001000 8000");
}
static void tb_param_scenario_11(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param hoge.dat");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_12(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param test/camfw_param_fourcc_invalid.dat");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_13(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param test/camfw_param_addr_invalid.dat");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");

}
static void tb_param_scenario_14(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param test/camfw_param_size_invalid.dat");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");

}
static void tb_param_scenario_15(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data test/defect_param_normal0.dat");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_16(void)
{
	memset((void*)0x65000000, 0, 0x10000);
	tb_run_command("ubifsload 65000000 test/defect_param_normal0.dat");
	tb_run_command("cmp.b 65000080 60011000 1400");
}
static void tb_param_scenario_17(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data test/defect_param_normal1.dat");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_18(void)
{
	memset((void*)0x65000000, 0, 0x10000);
	tb_run_command("ubifsload 65000000 test/defect_param_normal1.dat");
	tb_run_command("cmp.b 65000080 69011000 1400");
}
static void tb_param_scenario_19(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data hoge.dat");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_1a(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data test/defect_param_fourcc_invalid.dat");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");
}

static void tb_param_scenario_1b(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data test/defect_param_addr_invalid.dat");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_1c(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data test/defect_param_size_invalid.dat");
	tb_run_command("setenv camfw_shading_data");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_1d(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data test/shading_param_normal0.dat");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_1e(void)
{
	memset((void*)0x65000000, 0, 0x10000);
	tb_run_command("ubifsload 65000000 test/shading_param_normal0.dat");
	tb_run_command("cmp.b 65000080 60012400 1000");
}
static void tb_param_scenario_1f(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data test/shading_param_normal1.dat");
	tb_run_command("saveenv");
	tb_run_command("reset");

}

static void tb_param_scenario_20(void)
{
	memset((void*)0x65000000, 0, 0x10000);
	tb_run_command("ubifsload 65000000 test/shading_param_normal1.dat");
	tb_run_command("cmp.b 65000080 60012400 1000");

}
static void tb_param_scenario_21(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data hoge.dat");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_22(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data test/shading_param_fourcc_invalid.dat");
	tb_run_command("saveenv");
	tb_run_command("reset");
}
static void tb_param_scenario_23(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data test/shading_param_addr_invalid.dat");
	tb_run_command("saveenv");
	tb_run_command("reset");

}
static void tb_param_scenario_24(void)
{
	tb_run_command("setenv u-boot_ubi_part ubifs");
	tb_run_command("setenv u-boot_ubi_volume ubootfs");
	tb_run_command("setenv corefw_comm_param");
	tb_run_command("setenv camfw_setting_param");
	tb_run_command("setenv camfw_defect_data");
	tb_run_command("setenv camfw_shading_data test/shading_param_size_invalid.dat");
	tb_run_command("saveenv");
	tb_run_command("reset");

}




static void (*tb_param_table[])(void) = {
	tb_param_scenario_00,
	tb_param_scenario_01,
	tb_param_scenario_02,
	tb_param_scenario_03,
	tb_param_scenario_04,
	tb_param_scenario_05,
	tb_param_scenario_06,
	tb_param_scenario_07,
	tb_param_scenario_08,
	tb_param_scenario_09,
	tb_param_scenario_0a,
	tb_param_scenario_0b,
	tb_param_scenario_0c,
	tb_param_scenario_0d,
	tb_param_scenario_0e,
	tb_param_scenario_0f,
	tb_param_scenario_10,
	tb_param_scenario_11,
	tb_param_scenario_12,
	tb_param_scenario_13,
	tb_param_scenario_14,
	tb_param_scenario_15,
	tb_param_scenario_16,
	tb_param_scenario_17,
	tb_param_scenario_18,
	tb_param_scenario_19,
	tb_param_scenario_1a,
	tb_param_scenario_1b,
	tb_param_scenario_1c,
	tb_param_scenario_1d,
	tb_param_scenario_1e,
	tb_param_scenario_1f,
	tb_param_scenario_20,
	tb_param_scenario_21,
	tb_param_scenario_22,
	tb_param_scenario_23,
	tb_param_scenario_24,
	NULL
};


static int tb_param(int argc, char * const argv[], int id)
{
	int scenario;
	int i;

	if (argc != 3) {
		usage_print(id);
		return 1;
	}

	scenario = (int)simple_strtoul(argv[2], NULL, 16);
	
	for (i = 0; i < 256; i++) {
		if (tb_param_table[i] == NULL) {
			printf("scenario number is invalid\n");
			return 1;
		} else if (i == scenario) {
			tb_param_table[i]();
			return 0;
		}
	}


	return 0;
}

static void tb_param_usage(void)
{
	printf("\nusage:\n");
	printf("# testbench %s <scenario num>\n",
		XARINA_TB_PARAM);
}


/**************************/
/* read modify write      */
/**************************/
static int tb_rmw(int argc, char * const argv[], int id)
{
	unsigned long *addr;
	unsigned long val;
	unsigned long mask;

	if (argc != 5) {
		usage_print(id);
		return 1;
	}
	addr = (unsigned long *)simple_strtoul(argv[2], NULL, 16);
	val =  simple_strtoul(argv[3], NULL, 16);
	mask = simple_strtoul(argv[4], NULL, 16);

	*addr &= ~mask;
	*addr |= val & mask;

	return 0;

}

static void tb_rmw_usage(void)
{
	printf("\nRead modify write.\n");

	printf("\nusage:\n");
	printf("# testbench %s <addr> <val> <mask>\n"
		"   - <addr> is address where data to be written is. (must be word alignment)\n"
		"   - <val> is value to write\n"
		"   - <mask> is bit mask.\n\n",
		XARINA_TB_RMW);
}
/**************************/
/* update test            */
/**************************/

extern int do_test_update(int argc, char * const argv[]);


static int tb_update(int argc, char * const argv[], int id)
{

	/* arg check */
	if ((argc < 8) || (9 < argc)) {
		usage_print(id);
		return 1;
	}

	do_test_update(argc - 1, &(argv[1]));

	return 0;
}


static void tb_update_usage(void)
{
	printf("\nWrite NAND with u-boot format.\n");
	printf("\nusage:\n");
	printf("# testbench %s <addr> <offset> <size> <version> <interval> <mark> [r]\n", XARINA_TB_UPDATE);
	printf("   - <addr> is address where data to be written is.\n");
	printf("   - <size> is data size to be written (Byte unit).\n");
	printf("   - <version> is version number witten to OOB area.\n");
	printf("   - <interval> is intarval blocks.\n");
	printf("   - <mark> is mark in OOB area.\n");
	printf("   - [r] reverse order of data write.\n\n");

}
/**************************/
/*  markbad               */
/**************************/
static int tb_markbad(int argc, char * const argv[], int id)
{

	nand_info_t *nand;
	struct nand_chip *chip;
	int block;
	unsigned long ofs;

	if (argc != 3) {
		usage_print(id);
		return 1;
	}

	ofs = simple_strtoul(argv[2], NULL, 16);


	/* the following commands operate on the current device */
	if (nand_curr_device < 0 || nand_curr_device >= CONFIG_SYS_MAX_NAND_DEVICE ||
	    !nand_info[nand_curr_device].name) {
		puts("\nno devices available\n");
		return 1;
	}
	nand = &nand_info[nand_curr_device];
	chip = nand->priv;


	/* Get block number */
	block = (int)(ofs >> chip->bbt_erase_shift);
	if (chip->bbt)
		chip->bbt[block >> 2] |= 0x01 << ((block & 0x03) << 1);


	DBG("0x%08lx:mark bad\n", (unsigned long)block <<  chip->bbt_erase_shift);


	return 0;
}

static void tb_markbad_usage(void)
{
	printf("\nusage:\n");
	printf("# testbench %s <offset>\n"
		"  - <offset> is offset address of NAND to be marked bad\n",
		XARINA_TB_MARKBAD);
}
/**************************/
/*  markgood               */
/**************************/
static int tb_markgood(int argc, char * const argv[], int id)
{

	nand_info_t *nand;
	struct nand_chip *chip;
	int block;
	unsigned long ofs;

	if (argc != 3) {
		usage_print(id);
		return 1;
	}

	ofs = simple_strtoul(argv[2], NULL, 16);


	/* the following commands operate on the current device */
	if (nand_curr_device < 0 || nand_curr_device >= CONFIG_SYS_MAX_NAND_DEVICE ||
	    !nand_info[nand_curr_device].name) {
		puts("\nno devices available\n");
		return 1;
	}
	nand = &nand_info[nand_curr_device];
	chip = nand->priv;

	if (strcmp(argv[2], "all") == 0) {
		for (block = 0; block < 0x130; block++) {
			if (chip->bbt)
				chip->bbt[block >> 2] &= ~(0x03 << ((block & 0x03) << 1));
		}
		DBG("mark good 0x00000000 - 0x%08lx\n", (unsigned long)block << chip->phys_erase_shift);
	} else {

		/* Get block number */
		block = (int)(ofs >> chip->bbt_erase_shift);
		if (chip->bbt)
			chip->bbt[block >> 2] &= ~(0x03 << ((block & 0x03) << 1));
		DBG("0x%08lx:mark good\n", (unsigned long)block << chip->bbt_erase_shift);
	}

	return 0;
}

static void tb_markgood_usage(void)
{
	printf("\nusage:\n");
	printf("# testbench %s <offset>\n"
		"  - <offset> is offset address of NAND to be marked good\n",
		XARINA_TB_MARKGOOD);
}
/**************************/
/*  check oob             */
/**************************/
static int tb_chkoob(int argc, char * const argv[], int id)
{

	nand_info_t *nand;
	struct nand_chip *chip;
	struct mtd_oob_ops ops;
	unsigned long ofs;
	unsigned char oob_buf[128];
	int ooblen, pagesize, oobcnt;
	int i, j;
	int block;
	int cmp;
	unsigned char str[5];

	if (argc != 3) {
		usage_print(id);
		return 1;
	}

	ofs = simple_strtoul(argv[2], NULL, 16);


	/* the following commands operate on the current device */
	if (nand_curr_device < 0 || nand_curr_device >= CONFIG_SYS_MAX_NAND_DEVICE ||
	    !nand_info[nand_curr_device].name) {
		puts("\nno devices available\n");
		return 1;
	}
	nand = &nand_info[nand_curr_device];
	chip = nand->priv;


	/* Get block number */
	block = (int)(ofs >> chip->bbt_erase_shift);

	if (chip->page_shift == 12) {
		ooblen = 16;
		oobcnt = 2;
		pagesize = 0x1000;
	} else {
		ooblen = 32;
		oobcnt = 4;
		pagesize = 0x800;
	}

	ops.ooblen = ooblen;
	ops.oobbuf = oob_buf;
	ops.ooboffs = 0;
	ops.datbuf = NULL;
	ops.mode = MTD_OOB_PLACE;
	ops.len = pagesize;

	for (j = 0; j < 64; j++) {
		nand->read_oob(nand, block * (1 << chip->phys_erase_shift) + j * pagesize, &ops);
		printf("page->0x%08lx:", (unsigned long)(block * (1 << chip->phys_erase_shift) + j * pagesize));
		for (i = 1; i < oobcnt; i++) {
			if ((cmp = memcmp(oob_buf, oob_buf + i * 8, 8)) != 0) {
				break;
			}
		}
		if (cmp) {
			printf("cmp->NG:");
		} else {
			printf("cmp->OK:");
		}
		memcpy(str, oob_buf, 4);
		str[4] = '\0';
		printf("mark->%s:",str);
		printf("LBN->0x%02x:", oob_buf[4]);
		printf("end->0x%02x:", oob_buf[5]);
		printf("ver->0x%02x\n", oob_buf[6]);
	}

	return 0;
}

static void tb_chkoob_usage(void)
{
	printf("\nusage:\n");
	printf("# testbench %s <offset>\n"
		"  - <offset> is offset address of NAND to dump oob info.\n",
		XARINA_TB_CHKOOB);
}

/**************************/
/*  check block           */
/**************************/
static int tb_chkblk(int argc, char * const argv[], int id)
{

	nand_info_t *nand;
	struct nand_chip *chip;
	struct mtd_oob_ops ops;
	unsigned long ofs, len;
	unsigned char oob_buf[128];
	int ooblen, oobcnt, pagesize;
	int i, j;
	int block;
	int cmp;
	unsigned char str[5];

	if (argc != 4) {
		usage_print(id);
		return 1;
	}

	ofs = simple_strtoul(argv[2], NULL, 16);
	len = simple_strtoul(argv[3], NULL, 16);


	/* the following commands operate on the current device */
	if (nand_curr_device < 0 || nand_curr_device >= CONFIG_SYS_MAX_NAND_DEVICE ||
	    !nand_info[nand_curr_device].name) {
		puts("\nno devices available\n");
		return 1;
	}
	nand = &nand_info[nand_curr_device];
	chip = nand->priv;



	if (chip->page_shift == 12) {
		ooblen = 16;
		oobcnt = 2;
		pagesize = 0x1000;
	} else {
		ooblen = 32;
		oobcnt = 4;
		pagesize = 0x800;
	}

	ops.ooblen = ooblen;
	ops.oobbuf = oob_buf;
	ops.ooboffs = 0;
	ops.datbuf = NULL;
	ops.mode = MTD_OOB_PLACE;
	ops.len = pagesize;

	for (j = 0; j < len; j++) {
		/* Get block number */
		block = (int)(ofs >> chip->bbt_erase_shift) + j;
		nand->read_oob(nand, block * (1 << chip->phys_erase_shift), &ops);
		printf("page->0x%08lx:", (unsigned long)(block * (1 << chip->phys_erase_shift)));
		for (i = 1; i < oobcnt; i++) {
			if ((cmp = memcmp(oob_buf, oob_buf + i * 8, 8)) != 0) {
				break;
			}
		}
		if (cmp) {
			printf("cmp->NG:");
		} else {
			printf("cmp->OK:");
		}
		memcpy(str, oob_buf, 4);
		str[4] = '\0';
		printf("mark->%s:",str);
		printf("LBN->0x%02x:", oob_buf[4]);
		printf("end->0x%02x:", oob_buf[5]);
		printf("ver->0x%02x\n", oob_buf[6]);
	}

	return 0;
}

static void tb_chkblk_usage(void)
{
	printf("\nusage:\n");
	printf("# testbench %s <offset> <blocks>\n"
		"  - <offset> is offset address of NAND to dump oob info.\n"
		"  - <blocks> is the number of blocks to dump.\n",
		XARINA_TB_CHKBLK);
}

/**************************/
/* time test              */
/**************************/
static int tb_time(int argc, char * const argv[], int id)
{

	int i;
	unsigned long time0, time1;
	char *tmp_buf = cmd_buff;

	if (argc < 3) {
		usage_print(id);
		return 1;
	}

	for (i = 2; i < argc; i++) {
		sprintf(tmp_buf, "%s ", argv[i]);
		tmp_buf += (strlen(argv[i]) + 1);
	}

	printf("exec \"%s\"\n", cmd_buff);

	time0 = (*(volatile ulong *)(CONFIG_SYS_TIMERBASE+4));
	run_command(cmd_buff, 0);
	time1 = (*(volatile ulong *)(CONFIG_SYS_TIMERBASE+4));

	printf("exec time: %ld.%02ld [sec]\n", 
		(time0 - time1) / CONFIG_SYS_HZ, ((time0 - time1) / (CONFIG_SYS_HZ / 100)) % 100);


	return 0;
}

static void tb_time_usage(void)
{
	printf("\nMesurement of execution time of u-boot command.\n");
	printf("\nusage:\n");
	printf("# testbench %s <command> [arg1] [arg2] ...\n"
		"   - <command> is a u-boot command.\n"
		"   - [arg1] [arg2] ... are arguments of the command.\n\n",
		XARINA_TB_TIME);
}


/**************************/
/* nand biterror          */
/**************************/
static int bit_err_make(unsigned char *byte, unsigned char bit_inv)
{
	unsigned char tmp;

	tmp = *byte;

	*byte ^= bit_inv;

	printf("make error 0x%02x -> 0x%02x\n", tmp, *byte);

	return 1;

}


static int tb_nand_biterr(int argc, char * const argv[], int id)
{
	nand_info_t *nand;
	struct nand_chip *chip;
	struct mtd_oob_ops ops;
	unsigned long err_ofs;
	unsigned long blk_ofs;
	unsigned long target_ofs;
	unsigned long target_page;
	unsigned char bit_inv;
	unsigned char *blk_buf;
	nand_erase_options_t opts;
	int i;

	if (argc != 4) {
		usage_print(id);
		return 1;
	}


	/* the following commands operate on the current device */
	if (nand_curr_device < 0 || nand_curr_device >= CONFIG_SYS_MAX_NAND_DEVICE ||
	    !nand_info[nand_curr_device].name) {
		puts("\nno devices available\n");
		return 1;
	}


	nand = &nand_info[nand_curr_device];
	chip = nand->priv;

	err_ofs = simple_strtoul(argv[2], NULL, 16);
	bit_inv = (unsigned char)simple_strtoul(argv[3], NULL, 16);

	target_page = ((err_ofs >> chip->page_shift) & chip->pagemask) & 0x3f;

	/* block align offset */
	blk_ofs = err_ofs & ~(nand->erasesize -1);

	target_ofs = target_page * ((1 << chip->page_shift) + nand->oobsize) 
			+ (err_ofs & ((1 << chip->page_shift) - 1));

	DBG("page align mask = 0x%08lx\n", (unsigned long)~((1 << chip->page_shift) - 1));
	DBG("err_ofs = 0x%08lx\n", (unsigned long)err_ofs);
	DBG("target_ofs = 0x%08lx\n", (unsigned long)target_ofs);
	DBG("target_page = 0x%08lx\n", (unsigned long)target_page);
	DBG("pagemask = 0x%08lx\n", (unsigned long)chip->pagemask);
	DBG("page_shift = 0x%08lx\n", (unsigned long)chip->page_shift);
	DBG("blk_ofs = 0x%08lx\n", (unsigned long)blk_ofs);
	DBG("erasesize = 0x%08lx\n", (unsigned long)nand->erasesize);

	blk_buf = (unsigned char *)TEST_BENCH_WORK_AREA;

	/* nand read option setting */
	ops.ooblen = nand->oobsize;
	ops.oobbuf = blk_buf;
	ops.ooboffs = 0;
	ops.datbuf = blk_buf;
	ops.mode = MTD_OOB_RAW;	/* without ecc */
	ops.len = nand->erasesize;

	/* nand read */
	nand->read_oob(nand, blk_ofs, &ops);

	bit_err_make(&(blk_buf[target_ofs]), bit_inv);

	/* nand erase option setting */
	memset(&opts, 0, sizeof(opts));
	opts.offset = blk_ofs;
	opts.length = nand->erasesize;
	opts.quiet  = 1;

	/* nand erase */
	nand_erase_opts(nand, &opts);


	for (i = 0; i < 64; i++ ) {
		/* nand write option setting */
		ops.ooblen = nand->oobsize;
		ops.oobbuf = blk_buf + (1 << chip->page_shift) + 
				((1 << chip->page_shift) + nand->oobsize) * i;
		ops.ooboffs = 0;
		ops.datbuf = blk_buf + ((1 << chip->page_shift) + nand->oobsize) * i;
		ops.mode = MTD_OOB_RAW;	/* without ecc */
		ops.len = 1 << chip->page_shift;

		/* nand write */
		nand->write_oob(nand, blk_ofs + (1 << chip->page_shift) * i, &ops);
	}

	return 0;
}


static void tb_nand_biterr_usage(void)
{
	printf("\nusage:\n");
	printf("# testbench %s <offset> <bitinv>\n", XARINA_TB_BITERROR);
}

/**************************/
/* nand biterror oob      */
/**************************/
static int tb_nand_biterr_oob(int argc, char * const argv[], int id)
{
	nand_info_t *nand;
	struct nand_chip *chip;
	struct mtd_oob_ops ops;
	unsigned long err_ofs;
	unsigned long blk_ofs;
	unsigned long page_ofs;
	unsigned long target_ofs;
	unsigned long target_page;
	unsigned char bit_inv;
	unsigned char *blk_buf;
	nand_erase_options_t opts;
	int i;

	printf("exec %s\n", __func__);

	if (argc != 5) {
		usage_print(id);
		return 1;
	}


	/* the following commands operate on the current device */
	if (nand_curr_device < 0 || nand_curr_device >= CONFIG_SYS_MAX_NAND_DEVICE ||
	    !nand_info[nand_curr_device].name) {
		puts("\nno devices available\n");
		return 1;
	}


	nand = &nand_info[nand_curr_device];
	chip = nand->priv;

	page_ofs = simple_strtoul(argv[2], NULL, 16) &  ~((1 << chip->page_shift) - 1);
	target_page = (page_ofs >> chip->page_shift) & 0x3f;
	err_ofs =  simple_strtoul(argv[3], NULL, 16);
	bit_inv = (unsigned char)simple_strtoul(argv[4], NULL, 16);

	/* offset check */
	if (err_ofs >= nand->oobsize) {
		printf("oob offset must be smaller than 0x%08x.\n", nand->oobsize);
		return 1;
	}

	/* block align offset */
	blk_ofs = page_ofs & ~(nand->erasesize -1);

	target_ofs = target_page * ((1 << chip->page_shift) + nand->oobsize)
			+ (1 << chip->page_shift) + err_ofs;

	DBG("page align mask = 0x%08lx\n", (unsigned long)~((1 << chip->page_shift) - 1));
	DBG("err_ofs = 0x%08lx\n", (unsigned long)err_ofs);
	DBG("target_ofs = 0x%08lx\n", (unsigned long)target_ofs);
	DBG("target_page = 0x%08lx\n", (unsigned long)target_page);
	DBG("pagemask = 0x%08lx\n", (unsigned long)chip->pagemask);
	DBG("page_shift = 0x%08lx\n", (unsigned long)chip->page_shift);
	DBG("blk_ofs = 0x%08lx\n", (unsigned long)blk_ofs);
	DBG("erasesize = 0x%08lx\n", (unsigned long)nand->erasesize);

	blk_buf = (unsigned char *)TEST_BENCH_WORK_AREA;

	/* nand read option setting */
	ops.ooblen = nand->oobsize;
	ops.oobbuf = blk_buf;
	ops.ooboffs = 0;
	ops.datbuf = blk_buf;
	ops.mode = MTD_OOB_RAW;	/* without ecc */
	ops.len = nand->erasesize;

	/* nand read */
	nand->read_oob(nand, blk_ofs, &ops);

	bit_err_make(&(blk_buf[target_ofs]), bit_inv);

	/* nand erase option setting */
	memset(&opts, 0, sizeof(opts));
	opts.offset = blk_ofs;
	opts.length = nand->erasesize;
	opts.quiet  = 1;

	/* nand erase */
	nand_erase_opts(nand, &opts);


	for (i = 0; i < 64; i++ ) {
		/* nand write option setting */
		ops.ooblen = nand->oobsize;
		ops.oobbuf = blk_buf + (1 << chip->page_shift) + 
				((1 << chip->page_shift) + nand->oobsize) * i;
		ops.ooboffs = 0;
		ops.datbuf = blk_buf + ((1 << chip->page_shift) + nand->oobsize) * i;
		ops.mode = MTD_OOB_RAW;	/* without ecc */
		ops.len = 1 << chip->page_shift;

		/* nand write */
		nand->write_oob(nand, blk_ofs + (1 << chip->page_shift) * i, &ops);
	}

	return 0;
}


static void tb_nand_biterr_oob_usage(void)
{
	printf("\nusage:\n");
	printf("# testbench %s <page offset> <oob offset> <bit inv>\n"
		"   - <page offset> is must be page align.\n"
		"   - <oob offset> is must be smaller than available oob size.\n"
		"   - <bit inv> generates bit err where bit is set to 1.\n"
		, XARINA_TB_BITERROR_OOB);
}


/**************************/
/*  mdnc                  */
/**************************/
static int tb_mdnc(int argc, char * const argv[], int id)
{
	unsigned long addr;

	if (argc != 3) {
		usage_print(id);
		return 1;
	}

	addr = simple_strtoul(argv[2], NULL, 16);

	if (addr & 0x1F) {
		printf("address is not 32-byte aligned\n");
		return 1;
	}
	
	sprintf(cmd_buff, "md %x 8", (unsigned int)addr);
	invalidate_dcache_range(addr, addr + 32);
	run_command(cmd_buff, 0);
	

	return 0;
}

static void tb_mdnc_usage(void)
{
	printf("\nusage:\n");
	printf("# testbench %s <addr>\n"
		"   - <addr> is dump address which must be 32-byte aligned.\n"
		"Memory dump without D-cache\n",

		XARINA_TB_MD_NOCACHE);
}



/**************************/
/* testbench command      */
/**************************/
int do_test_bench(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[]) 
{
	int target_id;
	int ret = 0;

	/* arg num check */
	if (argc < 2) {
		cmd_usage(cmdtp);
		target_print();
		return -1;
	}

	/* target check */
	if (target_check(argv, &target_id) == 0) {
		printf("target is illegal.\n");
		target_print();
		return -1;
	}

	/* run test bench */
	if (test_bench_table[target_id].p_func != NULL) {
		ret = test_bench_table[target_id].p_func(argc, argv, target_id);
	}
	
	return ret;
}


U_BOOT_CMD(
	testbench, 20, 0,	do_test_bench,
	"run test bench",
	"<target>\n"
	"    - run test bench 'target'\n"
);





