/*
 * (C) Copyright 2007, 2011 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef _SMAC_MII_H
#define _SMAC_MII_H

#include <net.h>

#ifdef CONFIG_XARINA
#include <asm/io.h>
#include <asm/arch/hw_reg.h>

/********************************/
/* I/O                          */
/********************************/
#define SYSREG1_RIN(n)		readl(SYSREG1_REG_BASE+(n))
#define SYSREG1_ROUT(n, v)	writel((v), SYSREG1_REG_BASE+(n))
#endif /* CONFIG_XARINA */


#define SMAC_BASE_00 			(0x40002000)	/* smac base address */
#define SMAC_MAX_DEVICE			(1)		/* device number */

/* timeout */
#define SMAC_LINK_TIMEOUT		(5)		/* link timeout seconds */
#define SMAC_TX_TIMEOUT			(1)		/* send timeout seconds */


#define SMAC_PBL			(16)		/* AHB bus burst length*/

#define SMAC_MULTICAST_ADDR_NUM		(16)

/* speed */
#define SPEED_10    10
#define SPEED_100   100
#define SPEED_1000  1000

/* duplex */
#define HALF_DUPLEX 1
#define FULL_DUPLEX 2

/* bool */
#define TRUE	1
#define FALSE	0

/********************************/
/* DEBUG PRINT                  */
/********************************/
#if 0
#define DEBUG_PRINTK(frmt,args...)  printf("[drv][smac][DEBUG]" frmt ,##args)
#else
#define DEBUG_PRINTK(frmt,args...)
#endif
#if 0
#define DEBUG_PRINTK2(frmt,args...)  printf("[drv][smac][DEBUG]" frmt ,##args)
#else
#define DEBUG_PRINTK2(frmt,args...)
#endif

/********************************/
/* ENDIAN                       */
/********************************/
#ifdef CONFIG_CPU_BIG_ENDIAN
#define SMAC_CPU_DESC_BE
#else
#define SMAC_CPU_DESC_LE
#endif

/********************************/
/* SMAC Register Map            */
/********************************/
#define SMAC_INTR_STAT			(0x0000)
#define SMAC_INTR_ENABLE		(0x0004)
#define SMAC_INTR_DISABLE		(0x0008)
#define SMAC_INTR_CLEAR			(0x000c)
#define SMAC_INTR_TEST			(0x0010)

#define SMAC_INTR_SWRD			(1 << 18)
#define SMAC_INTR_PSUD			(1 << 17)
#define SMAC_INTR_RU			(1 << 7)
#define SMAC_INTR_RI			(1 << 6)
#define SMAC_INTR_TU			(1 << 2)
#define SMAC_INTR_TI			(1 << 0)
#define SMAC_INTR_ALL			0x7c01abff

#define SMAC_OPERATION			(0x0100)
#define SMAC_OP_RSF			(1 << 25)
#define SMAC_OP_TSF			(1 << 21)
#define SMAC_OP_ST			(1 << 13)    /* start transmit */
#define SMAC_OP_RTC_64BYTE		(0 << 3)
#define SMAC_OP_RTC_32BYTE		(1 << 3)
#define SMAC_OP_SR			(1 << 1)    /* start receive */


#define SMAC_TX_DB_BASE			(0x010c)
#define SMAC_TX_DB_LENGTH_MINUS1	(0x0110)
#define SMAC_TX_DB_HEAD			(0x0114)
#define SMAC_TX_DB_TAIL			(0x0118)
#define SMAC_RX_DB_BASE			(0x011c)
#define SMAC_RX_DB_LENGTH_MINUS1	(0x0120)
#define SMAC_RX_DB_HEAD			(0x0124)
#define SMAC_RX_DB_TAIL			(0x0128)

#define SMAC_BUS_MODE			(0x012c)
#define SMAC_BM_S_DCT			25
#define SMAC_BM_DBO_BIG			(1 << 20)
#define SMAC_BM_DBO_LITTLE		(0 << 20)
#define SMAC_BM_RMW_ENA			(1 << 16)
#define SMAC_BM_BLE_BIG			(1 << 7)
#define SMAC_BM_BLE_LITTLE		(0 << 7)
#define SMAC_BM_S_PBL			8
#define SMAC_BM_SWR			(1 << 0)    // s/w reset

#define SMAC_BUS_ARB			(0x0130)
#define SMAC_RX_FRAME_BYTE_LIMIT	(0x013c)
#define SMAC_RX_FRAME_COUNT_LIMIT	(0x0140)
#define SMAC_RX_MINSIZE_LIMIT		(0x0144)
#define SMAC_RX_DELAY_TIMER		(0x0148)
#define SMAC_RX_ABSOLUTE_TIMER		(0x014c)
#define SMAC_RX_DESC_LIMIT		(0x0150)

#define SMAC_MAC_CONFIG			(0x1000)
#define SMAC_MC_TC			(1 << 24)
#define SMAC_MC_WD			(1 << 23)
#define SMAC_MC_JD			(1 << 22)
#define SMAC_MC_JE			(1 << 20)	 /* jumbo frame enable */
#define SMAC_MC_M_IFG			(7 << 17)
#define SMAC_MC_IFG_64BIT		(4 << 17)
#define SMAC_MC_PS_MII			(1 << 15)	 /* MII */
#define SMAC_MC_FES_100			(1 << 14)	 /* 100Mbps */
#define SMAC_MC_DM_FULL			(1 << 11)	 /* full duplex */
#define SMAC_MC_RCS			(1 << 10)	 /* to enable Rx checksum offload */
#define SMAC_MC_TE			(1 << 3)
#define SMAC_MC_RE			(1 << 2)
#define SMAC_MC_DFS_ENA			(0 << 7)	/* enable FCS stripping */
#define SMAC_MC_DFS_DIS			(1 << 7)

#define SMAC_MAC_FRAME_FILTER		(0x1004)
#define SMAC_MFF_PM			(1 << 4)	/* enable pass all multicast */
#define SMAC_MFF_PR			(1 << 0)	/* enable promiscuous mode */

#define SMAC_HASH_TABLE_HIGH		(0x1008)
#define SMAC_HASH_TABLE_LOW		(0x100c)

#define SMAC_PHY_ADDR			(0x1010)
#define SMAC_PA_M_PA			0xf800
#define SMAC_PA_S_PA			11
#define SMAC_PA_M_GR			0x07c0
#define SMAC_PA_S_GR			6
#define SMAC_PA_M_MCR			0x001c
#define SMAC_PA_S_MCR			2
#define SMAC_PA_M_GW			0x0002
#define SMAC_PA_READ			0x0000
#define SMAC_PA_WRITE			0x0002
#define SMAC_PA_M_GB			0x0001
#define SMAC_PA_KICK			0x0001
#define SMAC_PA_BUSY			0x0001
#define SMAC_PA_FREE			0x0000

#define SMAC_PHY_DATA			(0x1014)
#define SMAC_CONTROL_FRM		(0x1018)
#define SMAC_VLAN_TAG			(0x101c)
#define SMAC_MAC_VERSION		(0x1020)
#define SMAC_WAKE_UP_FILTER		(0x1028)
#define SMAC_POWER_MANAGEMENT		(0x102c)
#define SMAC_POWER_MANAGEMENT		(0x102c)

#define SMAC_MAC_ADDR_HIGH(n)		(0x1040 + n * 8)
#define SMAC_MAC_ADDR_LOW(n)		(0x1044 + n * 8)
#define SMAC_MAC_ADDR_0_HIGH		SMAC_MAC_ADDR_HIGH(0)
#define SMAC_MAC_ADDR_0_LOW		SMAC_MAC_ADDR_LOW(0)
#define SMAC_MAC_AE			0x80000000

#define SMAC_PHY_POLLING		(0x0440)
#define SMAC_PP_M_PA			0xf800
#define SMAC_PP_S_PA			11
#define SMAC_PP_M_GR			0x07c0
#define SMAC_PP_S_GR			6
#define SMAC_PP_PE			(1 << 5)

#define SMAC_PHY_POLLED_DATA		(0x0444)

#define SMAC_FRM_COUNTER0		(0x0500)
#define SMAC_FRM_COUNTER1		(0x0580)

#define SMAC_MISS_COUNTER0		(0x0504)
#define SMAC_MISS_COUNTER1		(0x0584)

#define SMAC_TX_DESC_ADDR0		(0x0530)
#define SMAC_TX_DESC_ADDR1		(0x05b0)

#define SMAC_RX_DESC_ADDR0		(0x0534)
#define SMAC_RX_DESC_ADDR1		(0x05b4)

#define SMAC_TX_DESC0			(0x0538)
#define SMAC_TX_DESC1			(0x053c)
#define SMAC_TX_DESC2			(0x0540)
#define SMAC_TX_DESC3			(0x0544)

#define SMAC_RX_DESC0			(0x0548)
#define SMAC_RX_DESC1			(0x054c)
#define SMAC_RX_DESC2			(0x0550)
#define SMAC_RX_DESC3			(0x0554)

#define SMAC_FW_POLICY			(0x0700)

#define SMAC_FW_CONST_SWITCH		(0x0710)
#define SMAC_FW_CONST_SA		(0x0714)
#define SMAC_FW_CONST_SA_MASK		(0x0718)
#define SMAC_FW_CONST_DA		(0x071c)
#define SMAC_FW_CONST_DA_MASK		(0x0720)
#define SMAC_FW_CONST_SPN		(0x0724)
#define SMAC_FW_CONST_DPN		(0x0728)
#define SMAC_FW_CONST_FLAG		(0x072c)

#define SMAC_STATE_SWR			(0x0D40)
#define SMAC_STATE_RA			(0x0D60)
#define SMAC_STATE_RA_PL		(0x0D64)

#define SMAC_HW_VERSION			(0x0f00)
#define SMAC_HV_S_MAV			24
#define SMAC_HV_M_MAV			0xff000000
#define SMAC_HV_S_MIV0			16
#define SMAC_HV_M_MIV0			0x00ff0000
#define SMAC_HV_S_MIV1			8
#define SMAC_HV_M_MIV1			0x0000ff00
#define SMAC_HV_S_MIV			0
#define SMAC_HV_M_MIV2			0x000000ff

#define SMAC_MMC_CONTROL		(0x1100)
#define SMAC_MMC_CR			(1 << 0)
#define SMAC_MMC_CSR			(1 << 1)
#define SMAC_MMC_RR			(1 << 2)
#define SMAC_MMC_CF			(1 << 3)

#define SMAC_TXOCTETCOUNT_GB		(0x1114)
#define SMAC_TXFRAMECOUNT_GB		(0x1118)
#define SMAC_TXBROADCASTFRAMES_G	(0x111C)
#define SMAC_TXMULTICASTFRAMES_G	(0x1120)
#define SMAC_TX64OCTETS_GB		(0x1124)
#define SMAC_TX65TO127OCTETS_GB		(0x1128)
#define SMAC_TX128TO255OCTETS_GB	(0x112C)
#define SMAC_TX256TO511OCTETS_GB	(0x1130)
#define SMAC_TX512TO1023OCTETS_GB	(0x1134)
#define SMAC_TX1024TOMAXOCTETS_GB	(0x1138)
#define SMAC_TXUNICASTFRAMES_GB		(0x113C)
#define SMAC_TXMULTICASTFRAMES_GB	(0x1140)
#define SMAC_TXBROADCASTFRAMES_GB	(0x1144)
#define SMAC_TXUNDERFLOWERROR		(0x1148)
#define SMAC_TXSINGLECOL_G		(0x114C)
#define SMAC_TXMULTICOL_G		(0x1150)
#define SMAC_TXDEFERRED			(0x1154)
#define SMAC_TXLATECOL			(0x1158)
#define SMAC_TXEXCESSCOL		(0x115C)
#define SMAC_TXCARRIERERROR		(0x1160)
#define SMAC_TXOCTETCOUNT_G		(0x1164)
#define SMAC_TXFRAMECOUNT_G		(0x1168)
#define SMAC_TXEXCESSDEF		(0x116C)
#define SMAC_TXPAUSEFRAMES_G		(0x1170)
#define SMAC_TXVLANFRAMES_G		(0x1174)

#define SMAC_RXFRAMECOUNT_GB		(0x1180)
#define SMAC_RXOCTETCOUNT_GB		(0x1184)
#define SMAC_RXOCTETCOUNT_G		(0x1188)
#define SMAC_RXBROADCASTFRAMES_G	(0x118C)
#define SMAC_RXMULTICASTFRAMES_G	(0x1190)
#define SMAC_RXCRCERROR			(0x1194)
#define SMAC_RXALIGNMENTERROR		(0x1198)
#define SMAC_RXRUNTERROR		(0x119C)
#define SMAC_RXJABBERERROR		(0x11A0)
#define SMAC_RXUNDERSIZE_G		(0x11A4)
#define SMAC_RXOVERSIZE_G		(0x11A8)
#define SMAC_RX64OCTETS_GB		(0x11AC)
#define SMAC_RX65TO127OCTETS_GB		(0x11B0)
#define SMAC_RX128TO255OCTETS_GB	(0x11B4)
#define SMAC_RX256TO511OCTETS_GB	(0x11B8)
#define SMAC_RX512TO1023OCTETS_GB	(0x11BC)
#define SMAC_RX1024TOMAXOCTETS_GB	(0x11C0)
#define SMAC_RXUNICASTFRAMES_G		(0x11C4)
#define SMAC_RXLENGTHERROR		(0x11C8)
#define SMAC_OUTOFRANGETYPE		(0x11CC)
#define SMAC_RXPAUSEFRAMES_G		(0x11D0)
#define SMAC_RXFIFOOVERFLOW		(0x11D4)
#define SMAC_RXVLANFRAMES_GB		(0x11D8)
#define SMAC_RXWATCHDOGERROR		(0x11DC)

struct smac_tx_desc {
	volatile uint32_t std_flags;
	uint32_t std_length;
	uint32_t std_addr1;
	uint32_t std_addr2;
};

#define SMAC_STDD_M_WB			(1 << 31)
#define SMAC_STDD_M_DESC_DATA		(0 << 29)    /* data descriptor */
#define SMAC_STDD_M_DESC_CTX		(1 << 29)    /* context descriptor */
#define SMAC_STDD_M_IC			(1 << 28)
#define SMAC_STDD_M_ICS			(1 << 25)
#define SMAC_STDD_M_TCS			(1 << 24)
#define SMAC_STDD_M_PHCS		(1 << 23)
#define SMAC_STDD_M_LS			(1 << 27)
#define SMAC_STDD_M_FS			(1 << 26)
#define SMAC_STDD_M_DFCS_ENA		(1 << 22)
#define SMAC_STDD_M_DFCS_DIS		(0 << 22)
#define SMAC_STDD_M_DPAD_ENA		(1 << 21)
#define SMAC_STDD_M_DPAD_DIS		(0 << 21)
#define SMAC_STDD_M_LSE			(1 << 19)
#define SMAC_STDD_M_ES			(1 << 15)

#define SMAC_STDD_M_BUF_LEN2		0x3fff0000
#define SMAC_STDD_S_BUF_LEN2		16
#define SMAC_STDD_M_BUF_LEN1		0x000000ff
#define SMAC_STDD_S_BUF_LEN1		0

struct smac_rx_desc {
	volatile uint32_t srd_flags;
	uint32_t srd_length;
	uint32_t srd_addr1;
	uint32_t srd_addr2;
};

#define SMAC_SRD_M_WB			(1 << 31)
#define SMAC_SRD_M_SHF			(1 << 30)
#define SMAC_SRD_M_FRAME_LEN		0x3fff0000
#define SMAC_SRD_S_FRAME_LEN		16
#define SMAC_SRD_M_ES			(1 << 15)
#define SMAC_SRD_M_TCS			(1 << 14)
#define SMAC_SRD_M_SAF			(1 << 13)
#define SMAC_SRD_M_LE			(1 << 12)
#define SMAC_SRD_M_OE			(1 << 11)
#define SMAC_SRD_M_ICSE			(1 << 7)
#define SMAC_SRD_M_LC			(1 << 6)
#define SMAC_SRD_M_ICS			(1 << 5)
#define SMAC_SRD_M_RWT			(1 << 4)
#define SMAC_SRD_M_RE			(1 << 3)
#define SMAC_SRD_M_DE			(1 << 2)
#define SMAC_SRD_M_CE			(1 << 1)
#define SMAC_SRD_M_TCSE			(1 << 0)

#define SMAC_SRD_S_BUF_LEN2		16
#define SMAC_SRD_M_BUF_LEN2		0x3fff0000
#define SMAC_SRD_S_BUF_LEN1		0
#define SMAC_SRD_M_BUF_LEN1		0x000000ff
#define SMAC_SRD_SHP_ON			(1 << 11)


/********************************/
/* PHY                          */
/********************************/
/* PHY */
// CONFIG_PHY_ADDRESS itogah
#define SMAC_PHY_LAYER_ADDR		1    /* H/W fixed address */
#if defined(CONFIG_XARINA_EVBRD_PRE) | defined(CONFIG_XARINA_FPGA)
#define SMAC_PHY_CLK			(2 << SMAC_PA_S_MCR)
#else 
#define SMAC_PHY_CLK			(0 << SMAC_PA_S_MCR)
#endif

#define SMAC_PHY_CONTROL		(0x00)
#define SMAC_PHY_STATUS			(0x01)
#define SMAC_PHY_AUTONEG_ADVERTISE	(0x04)
#define SMAC_PHY_PHY_CONTROL1		(0x1E)


#define SMAC_PHY_S_RESET		(1 << 15)
#define SMAC_PHY_S_AUTO_NEG_ENA		(1 << 12)
#define SMAC_PHY_S_AUTO_NEG_RESTART	(1 << 9)
#define SMAC_PHY_S_10BASE_FULL		(0 << 13) | (1 << 8) | (0 << 6)
#define SMAC_PHY_S_10BASE_HALF		(0 << 13) | (0 << 8) | (0 << 6)
#define SMAC_PHY_S_100BASE_FULL		(1 << 13) | (1 << 8) | (0 << 6)
#define SMAC_PHY_S_100BASE_HALF		(1 << 13) | (0 << 8) | (0 << 6)
#define SMAC_PHY_S_1000BASE_FULL	(0 << 13) | (1 << 8) | (1 << 6)
#define SMAC_PHY_S_1000BASE_HALF	(0 << 13) | (0 << 8) | (1 << 6)

#define SMAC_PHY_M_SPEED		(3)
#define SMAC_PHY_M_DUPLEX		(1 << 2)
#define SMAC_PHY_M_LINK			(1 << 8)


#define SMAC_PHY_S_ASSERT_CRS		(1 << 11)


/********************************/
/* descriptor & buffer          */
/********************************/
#define SMAC_MAX_TXD		(1)	/* TX Descriptor is fixed to 1. */
#define SMAC_MAX_RXD		(2)	/* RX Descriptor */

#define SMAC_FRAME_SIZE		(PKTSIZE - 4)	/* one frame size - FCB */
#define SMAC_BUFF_SIZE		(PKTSIZE_ALIGN)	/* one frame size (32byte align size)*/

/* index to tail set value */
#define INDEX2TAIL_RX(x)	((x) < (SMAC_MAX_RXD - 1)) ? ((unsigned long)&rx->desc[(x)+1]) : ((unsigned long)&rx->desc[0])
/* tail to index set value */
#define TAIL2INDEX_RX(x)	((unsigned long)(x) == (unsigned long)&rx->desc[0]) ? (SMAC_MAX_RXD - 1) : \
				((((unsigned long)(x) - (unsigned long)&rx->desc[0]) / sizeof(struct smac_rx_desc)) - 1)
#define HEAD2INDEX_RX(x)	TAIL2INDEX_RX(x)
/* index increment */
#define INDEX_INC_RX(x)		((x) < (SMAC_MAX_RXD - 1)) ? ((x) +1) : 0

/* TX resource */
struct smac_tx_res {
	struct smac_tx_desc	desc[SMAC_MAX_TXD];			/* descriptor */
	unsigned char		buff[SMAC_MAX_TXD][SMAC_BUFF_SIZE];	/* buffer */
	unsigned long		idx;					/* index */
};

/* TX resource */
struct smac_rx_res {
	struct smac_rx_desc	desc[SMAC_MAX_RXD];			/* descriptor */
	unsigned char		buff[SMAC_MAX_RXD][SMAC_BUFF_SIZE];	/* buffer */
	unsigned long		idx;					/* index */
};

struct smac_resource {
	struct smac_tx_res	tx;	/* tx resource */
	struct smac_rx_res	rx;	/* rx resource */
};

#endif /* _SMAC_MII_H */
