#include <common.h>
#include <exports.h>
#include "xarina_nandif.h"
#include "xarina_nand_ctrl.h"

#define NANDIF_RIN(n)		readl(NANDIF_BASE+(n))
#define NANDIF_ROUT(n, v)	writel((v), NANDIF_BASE+(n))
#define PAGE_SIZE		(2048)
#define ECC_SIZE		(512)
#define ECC_LEN			(4)
#define OOB_SIZE		(8)
#define BLOCK_SIZE		(64)



//static void test_nand_write(unsigned char *pbuf, unsigned long offset, size_t len)
static void test_nand_write(unsigned long page, unsigned long ofs, size_t byte, unsigned char *addr)
{
	unsigned long stat;
	unsigned long page_addr;
	int	cnt;

	if (byte & 0x1) {
		printf("test_nand_write: len must be even\n");
		return;
	}
	
	
	/* chip enable 0 , write protect off */
	NANDIF_ROUT(NANDIF_NF_CTLSTS, 0x0A);
	
	NANDIF_ROUT(NANDIF_NF_CMD1, NANDIF_CMD_PAGEPRG1);
	/* set address */
	NANDIF_ROUT(NANDIF_NF_CADR01, ofs);
	NANDIF_ROUT(NANDIF_NF_PADR01, page & 0xffff);
	NANDIF_ROUT(NANDIF_NF_PADR2, (page >> 16) & 0xff);

	/* ECC clear */
	NANDIF_ROUT(NANDIF_NF_ECCLR, 0x00);
	/* set send data size */
	NANDIF_ROUT(NANDIF_NF_DSIZE, byte);


	NANDIF_ROUT(NANDIF_NF_TASK, 
		NANDIF_TASK_SENDDATA | 
		NANDIF_TASK_COLADR0 | 
		NANDIF_TASK_COLADR1 | 
		NANDIF_TASK_PAGADR0 | 
		NANDIF_TASK_PAGADR1 |
		NANDIF_TASK_PAGADR2 |
		NANDIF_TASK_COMMAND1
		);
		
	for(cnt = 0; cnt < byte; cnt += 2) {
		while (NANDIF_RIN(NANDIF_NF_CTLSTS) & 0x800);
		NANDIF_ROUT(NANDIF_NF_DATA01, *(unsigned short *)(addr + cnt));		
	}

	NANDIF_ROUT(NANDIF_NF_CMD2, NANDIF_CMD_PAGEPRG2);
	NANDIF_ROUT(NANDIF_NF_CMD3, NANDIF_CMD_READSTATUS);

	printf("check status\n");
	NANDIF_ROUT(NANDIF_NF_TASK, 
		NANDIF_TASK_COMMAND2 | 
		NANDIF_TASK_COMMAND3 | 
		NANDIF_TASK_WAITRDY1 | 
		NANDIF_TASK_RECV2REG);
	while(NANDIF_RIN(NANDIF_NF_TASK));
	
	stat = NANDIF_RIN(NANDIF_NF_ID_ST_RDT_12) & 0xFF;
	
	printf("stat = %lx\n", stat);

	/* chip disable 0, write protect on */
	NANDIF_ROUT(NANDIF_NF_CTLSTS, 0x0C);
	
	return;
}
int do_test_nand_write_noecc(int argc, char * const argv[])
{
	unsigned char *addr;
	unsigned long len;
	unsigned long offset;
	
	if (argc == 4) {
		addr = (unsigned char *)simple_strtoul(argv[2], NULL, 16);
		offset = (unsigned long)simple_strtoul(argv[3], NULL, 16);
		//len = (unsigned long *)simple_strtoul(argv[3], NULL, 16);
		len = 512;
	
		printf("start test nand write raw\n");
		printf("addr %08x, offset = %08x, len = %lu\n", 
			(unsigned int)addr, (unsigned int)offset, (unsigned long)len);
		test_nand_write(offset / PAGE_SIZE, 0, len, addr);
	} else {
		printf("usage: %s w addr offset\n", argv[0]);
	}

	return 0;
}


static void nand_read_byte(unsigned long page, unsigned long ofs, unsigned long byte, unsigned char *addr)
{
	unsigned char *buf = addr;
	unsigned short tmp;
	int cnt;
	
	/* chip enable 0 & write protect off */
	NANDIF_ROUT(NANDIF_NF_CTLSTS, 0x0A);

	NANDIF_ROUT(NANDIF_NF_CMD1, NANDIF_CMD_READ1);
	/* set address */
	NANDIF_ROUT(NANDIF_NF_CADR01, ofs);
	NANDIF_ROUT(NANDIF_NF_PADR01, page & 0xFFFF);
	NANDIF_ROUT(NANDIF_NF_PADR2, (page >> 16) & 0xFF);

	NANDIF_ROUT(NANDIF_NF_CMD2, NANDIF_CMD_READ2);

	/* ECC clear */
	NANDIF_ROUT(NANDIF_NF_ECCLR, 0x00);
	/* set read data size */
	NANDIF_ROUT(NANDIF_NF_DSIZE, byte);

	NANDIF_ROUT(NANDIF_NF_TASK, 
		NANDIF_TASK_CHKECC | 
		NANDIF_TASK_RECVDATA | 
		NANDIF_TASK_WAITRDY1 | 
		NANDIF_TASK_COMMAND2 | 
		NANDIF_TASK_COLADR0 | 
		NANDIF_TASK_COLADR1 |
		NANDIF_TASK_PAGADR0 | 
		NANDIF_TASK_PAGADR1 |
		NANDIF_TASK_PAGADR2 |
		NANDIF_TASK_COMMAND1
		);
	
	for(cnt = 0; cnt < byte; cnt +=2 ) {
		while (NANDIF_RIN(NANDIF_NF_CTLSTS) & 0x400);
		tmp = NANDIF_RIN(NANDIF_NF_DATA01);
		buf[cnt] = (unsigned char)(tmp & 0xFF);
		buf[cnt + 1] = (unsigned char)((tmp >> 8) & 0xFF);
		//if(!(NANDIF_RIN(NANDIF_NF_TASK) & NANDIF_TASK_RECVDATA)) break;
	}
		
	while(NANDIF_RIN(NANDIF_NF_TASK) & NANDIF_TASK_CHKECC);
	
	if(NANDIF_RIN(NANDIF_NF_ECCHK)) {
		printf("err occured\n");
		printf("page = %d ofs = %d\n", (unsigned int)page, (unsigned int)ofs);
	}
	
	/* chip disable 0 , write protect on */
	NANDIF_ROUT(NANDIF_NF_CTLSTS, 0x0C);


}

static void test_nand_read_page(unsigned long page, unsigned char *addr)
{
	int i;
	unsigned short tmp;
	unsigned char *buf = addr;
	unsigned char oobbuf[OOB_SIZE];
	int ofs = 0;
	
	while (!(NANDIF_RIN(NANDIF_NF_CTLSTS) & 0x400)) {
		tmp = NANDIF_RIN(NANDIF_NF_DATA01);
	}
	
	for (i = 0; i < 4; i++) {

		nand_read_byte(page, ofs, ECC_SIZE, buf);
		ofs += ECC_SIZE + ECC_LEN;
		buf += ECC_SIZE;
		nand_read_byte(page, ofs, OOB_SIZE, oobbuf);
		ofs += OOB_SIZE + ECC_LEN;
	}
	
	printf("oob = ");
	for (i = 0; i < OOB_SIZE; i++) {
		printf("0x%02x, ", oobbuf[i]);
	}
	printf("\n");
	
}

static void test_nand_read(unsigned long page, size_t len, unsigned char *addr)
{
	unsigned char *buf = addr;
	unsigned long page_num =  (len - 1) / PAGE_SIZE + 1;
	int i;

	for (i = 0; i < page_num; i++) {
		test_nand_read_page(page + i, buf + i * PAGE_SIZE);
	}
	
	return;
}
int do_test_nand_read(int argc, char * const argv[])
{
	unsigned long page;
	unsigned long len;
	unsigned char *addr;
	
	if (argc != 5) {
		printf("error argc = %d\n", argc);
		return 1;
	}
	
	page = simple_strtoul(argv[2], NULL, 16);	
	len = simple_strtoul(argv[3], NULL, 16);	
	addr = (unsigned char *)simple_strtoul(argv[4], NULL, 16);	

	printf("nand read page = 0x%lx, len = 0x%lx, addr = 0x%lx\n", 
		(unsigned long)page, (unsigned long)len, (unsigned long)addr);

	test_nand_read(page, len, addr);
	
	return 0;
}

int do_test_nand_biterr(int argc, char * const argv[])
{
	unsigned char tmp[ECC_SIZE];
	unsigned long offset;
	unsigned long page;
	unsigned long col;
	unsigned long oob_num;
	int i, j, cnt, size;
	unsigned char bit_mask;
	unsigned char tmp_mask;
	int err_flag = 0;
	
	if (argc > 5 || argc < 3) {
		printf("usage: %s offset [c|u] [o num]\n", argv[0]);
		return 1;
	}
	
	offset = simple_strtoul(argv[1], NULL, 16);
	
	if (*argv[2] == 'c') {
		bit_mask = 0x01;
		cnt = 8;
		printf("add 1 bit error %08x\n", (unsigned int)offset);
	} else {
		bit_mask = 0x03;
		cnt = 7;
		printf("add 2 bit error %08x\n", (unsigned int)offset);
	}
	
	if (*argv[3] == 'o') {
		size = OOB_SIZE;
		oob_num = simple_strtoul(argv[4], NULL, 16);
		oob_num &= 0x3;
		col = ECC_SIZE + 4 + oob_num * (ECC_SIZE + 16);
	} else {
		size = ECC_SIZE;
		col = 0;
	}
	
	page = offset / PAGE_SIZE;
	
	nand_read_byte(page, col, ECC_SIZE, tmp);

	for (i = 0; i < size; i++) {
		tmp_mask = bit_mask;
		for (j = 0; j < cnt; j++) {
			if((tmp[i] & tmp_mask) == tmp_mask) {
				printf("src data = %02x\n", (unsigned int)tmp[i]);
				tmp[i] ^= tmp_mask;
				err_flag = 1;
				printf("address = %08x\n", (unsigned int)i);
				printf("dst data = %02x\n", (unsigned int)tmp[i]);
				break;
			}			
			tmp_mask <<= 1;
		}
		if (err_flag) {
			break;
		}
	}
	
	if (err_flag == 0) {
		printf("failure\n");
		return 1;
	}
	
	test_nand_write(page, col, size, tmp);
	printf("success\n");
	
	return 0;
	
}

static void test_nand_erase(unsigned long page)
{
	NANDIF_ROUT(NANDIF_NF_CMD1, NANDIF_CMD_BLOCKERASE1);
	/* set address */
	NANDIF_ROUT(NANDIF_NF_CADR01, 0x0000);
	NANDIF_ROUT(NANDIF_NF_PADR01, page & 0xFFFF);
	NANDIF_ROUT(NANDIF_NF_PADR2, (page >> 16) & 0xFFFF);

	NANDIF_ROUT(NANDIF_NF_CMD2, NANDIF_CMD_BLOCKERASE2);
	/* set status */
	// NANDIF_ROUT(NANDIF_NF_CTLSTS, NANDIF_FWPN | 0x08);


	NANDIF_ROUT(NANDIF_NF_TASK, 
		NANDIF_TASK_WAITRDY1 | 
		NANDIF_TASK_COMMAND2 | 
		NANDIF_TASK_PAGADR0 | 
		NANDIF_TASK_PAGADR1 |
		NANDIF_TASK_PAGADR2 |
		NANDIF_TASK_COMMAND1
		);
	
	while(NANDIF_RIN(NANDIF_NF_TASK));
	
	return;
}
int do_test_nand_erase(int argc, char * const argv[])
{
	unsigned long page;
	
	if (argc != 3) {
		printf("error argc = %d\n", argc);
		return 1;
	}
	
	page = simple_strtoul(argv[2], NULL, 16);	
	if ((page % BLOCK_SIZE) != 0) {
		printf("error page number\n");
		return 1;
	}

	printf("nand erase [0x%lx]\n", page);
	test_nand_erase(page);
	printf("erase done\n");
	return 0;
}	


static void test_nand_reset(void)
{
	
	NANDIF_ROUT(NANDIF_NF_CMD1, NANDIF_CMD_RESET);
	/* set status */
	NANDIF_ROUT(NANDIF_NF_MODE, (NANDIF_RIN(NANDIF_NF_MODE) & ~0x03) | 0x02 );

	NANDIF_ROUT(NANDIF_NF_TASK, NANDIF_TASK_WAITRDY1);
	while(NANDIF_RIN(NANDIF_NF_TASK));
	
	NANDIF_ROUT(NANDIF_NF_TASK, NANDIF_TASK_COMMAND1 | NANDIF_TASK_WAITRDY1);
	while(NANDIF_RIN(NANDIF_NF_TASK));
	
	return;
}


void test_nand_init(void)
{
	printf("nand init\n");

	/* set status */
	NANDIF_ROUT(NANDIF_NF_CTLSTS, 0x08);
	/* set mode */
	NANDIF_ROUT(NANDIF_NF_MODE, (NANDIF_RIN(NANDIF_NF_MODE) & ~0x03) | 0x02 );

	/* nand reset */
	NANDIF_ROUT(NANDIF_NF_CMD1, NANDIF_CMD_RESET);
	NANDIF_ROUT(NANDIF_NF_MODE, (NANDIF_RIN(NANDIF_NF_MODE) & ~0x03) | 0x02 );
	NANDIF_ROUT(NANDIF_NF_TASK, NANDIF_TASK_WAITRDY1);
	while(NANDIF_RIN(NANDIF_NF_TASK));
	NANDIF_ROUT(NANDIF_NF_TASK, NANDIF_TASK_COMMAND1 | NANDIF_TASK_WAITRDY1);
	while(NANDIF_RIN(NANDIF_NF_TASK));


	return;
}
