/*
 * (C) Copyright 2006-2008
 * Stefan Roese, DENX Software Engineering, sr@denx.de.
 *
 * (C) Copyright 2012 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <nand.h>
#include <asm/io.h>
#include <asm/arch/xarina.h>

/* define NAND_BOOT_DEBUG */

#define CONFIG_SYS_NAND_READ_DELAY \
	{ volatile int dummy; int i; for (i=0; i<10000; i++) dummy = i; }


#if !defined(CONFIG_XARINA)
static int nand_ecc_pos[] = CONFIG_SYS_NAND_ECCPOS;

#else
#include <asm/arch/xarina_nand1.h>
#include <asm/arch/xarina_nand2.h>
#include <asm/arch/hw_reg.h>
/* 16bit access macro for xarina */
#define WRITE16	writew
#define READ16	readw
#define MAIN_TABLE		(0)
#define MIRR_TABLE		(1)
#define INVALID			(0xff)
#define MAX_LOGICAL_BLOCK	(10)
#define ECC_NO_ERR		(0)
#define ECC_CO_ERR		(1)	/* correctable error */
#define ECC_UC_ERR		(2)	/* uncorrectable error */
#define MARK_LEN		(4)
#define TABLE_NUM		(2)	/* main table and mirror table */
#define MAX_TBL_SET		(2)
#define TBL_SET_X		(0)
#define TBL_SET_Y		(1)
#define NAND_PAGE_COUNT		(64)
#define NAND1_OOB_COUNT		(4)
#define NAND2_OOB_COUNT		(2)
#define OOB_FREE_MAX 		(32)


typedef struct s_ecc_info {
	unsigned short chk;
	unsigned short err_bit;
	unsigned short err_addr;
} ecc_info;

typedef struct s_xa_oob {
	unsigned char mark[MARK_LEN];
	unsigned char loblock;
	unsigned char contflag;
	unsigned char version;	
	unsigned char reserved;	
} xa_oob;

typedef struct s_lblk_tbl_set {
	unsigned char lblk[TABLE_NUM][MAX_LOGICAL_BLOCK];
	unsigned char version;
} lblk_tbl;



/* global variable */
static const unsigned char mark_uboot[] = {'u', 'b', 'o', 't' };
static const char char_list[] = {'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f' };
static lblk_tbl xarina_lblk[MAX_TBL_SET];

static int (*xarina_nand_read_oob)(struct mtd_info *mtd, int block, int page, uchar *dst);
static int (*xarina_nand_read_page)(struct mtd_info *mtd, int block, 
			int page, uchar *dst_data, uchar *dst_oob);
static int (*xarina_nand_command)(struct mtd_info *mtd, int block, int page, int offs, u8 cmd);
static int xarina_nand_block_size;
static int xarina_nand_page_size;
static int xarina_nand_page_count;
static int xarina_nand_oob_count;


/* prototype declaration */
static int nand1_read_page(struct mtd_info *mtd, int block,
				int page, uchar *dst_data, uchar *dst_oob);
static void xarina_put_num(char num);
static int nand1_command(struct mtd_info *mtd, int block, int page, int offs, u8 cmd);
static int nand2_command(struct mtd_info *mtd, int block, int page, int offs, u8 cmd);
static int nand1_correct_data(struct mtd_info *mtd, unsigned char *data, unsigned short len, ecc_info *ecc);
static int nand2_correct_data(struct mtd_info *mtd, unsigned char *data, unsigned short len, ecc_info *ecc);
static int nand1_read_page(struct mtd_info *mtd, int block, 
				int page, uchar *dst_data, uchar *dst_oob);
static int nand2_read_page(struct mtd_info *mtd, int block, 
				int page, uchar *dst_data, uchar *dst_oob);
static int nand1_read_oob(struct mtd_info *mtd, int block, int page, uchar *dst);
static int nand2_read_oob(struct mtd_info *mtd, int block, int page, uchar *dst);
static int nand_load(struct mtd_info *mtd, uchar *dst, lblk_tbl *table);
static int cmp_mark(unsigned char *src0, const unsigned char *src1);
static void init_lblk_tbl(lblk_tbl *table);
static int cmp_version(unsigned char ver0, unsigned char ver1);
static int create_lblk_table(struct mtd_info *mtd, int offset, 
				int block_len, unsigned char *ver_cnt);
static void set_blk_num(lblk_tbl *table, unsigned char lblk_num, unsigned char pblk_num);


/* external reference */
extern int pinmux_gpio_uart3_set(void);
extern void xarina_hw_nfif_clock_set(unsigned long clock);
extern int xarina_hw_nand_cont_type_get(void);
extern int serial_init (void);
extern void serial_putc (const char c);
extern void serial_puts (const char *s);
extern void xarina_nand1_write_protect(struct mtd_info *mtd, int protect);
extern void xarina_nand2_write_protect(struct mtd_info *mtd, int protect);
extern void board_nand1_info_set(struct nand_chip *nand);
extern void board_nand2_info_set(struct nand_chip *nand);




#endif

#if !defined(CONFIG_XARINA)
#if (CONFIG_SYS_NAND_PAGE_SIZE <= 512)
/*
 * NAND command for small page NAND devices (512)
 */
static int nand_command(struct mtd_info *mtd, int block, int page, int offs, u8 cmd)
{
	struct nand_chip *this = mtd->priv;
	int page_addr = page + block * CONFIG_SYS_NAND_PAGE_COUNT;

	if (this->dev_ready)
		while (!this->dev_ready(mtd))
			;
	else
		CONFIG_SYS_NAND_READ_DELAY;

	/* Begin command latch cycle */
	this->cmd_ctrl(mtd, cmd, NAND_CTRL_CLE | NAND_CTRL_CHANGE);
	/* Set ALE and clear CLE to start address cycle */
	/* Column address */
	this->cmd_ctrl(mtd, offs, NAND_CTRL_ALE | NAND_CTRL_CHANGE);
	this->cmd_ctrl(mtd, page_addr & 0xff, NAND_CTRL_ALE); /* A[16:9] */
	this->cmd_ctrl(mtd, (page_addr >> 8) & 0xff,
		       NAND_CTRL_ALE); /* A[24:17] */
#ifdef CONFIG_SYS_NAND_4_ADDR_CYCLE
	/* One more address cycle for devices > 32MiB */
	this->cmd_ctrl(mtd, (page_addr >> 16) & 0x0f,
		       NAND_CTRL_ALE); /* A[28:25] */
#endif
	/* Latch in address */
	this->cmd_ctrl(mtd, NAND_CMD_NONE, NAND_NCE | NAND_CTRL_CHANGE);

	/*
	 * Wait a while for the data to be ready
	 */
	if (this->dev_ready)
		while (!this->dev_ready(mtd))
			;
	else
		CONFIG_SYS_NAND_READ_DELAY;

	return 0;
}
#else
/*
 * NAND command for large page NAND devices (2k)
 */
static int nand_command(struct mtd_info *mtd, int block, int page, int offs, u8 cmd)
{
	struct nand_chip *this = mtd->priv;
	int page_addr = page + block * CONFIG_SYS_NAND_PAGE_COUNT;

	if (this->dev_ready)
		while (!this->dev_ready(mtd))
			;
	else
		CONFIG_SYS_NAND_READ_DELAY;

	/* Emulate NAND_CMD_READOOB */
	if (cmd == NAND_CMD_READOOB) {
		offs += CONFIG_SYS_NAND_PAGE_SIZE;
		cmd = NAND_CMD_READ0;
	}

	/* Begin command latch cycle */
	this->cmd_ctrl(mtd, cmd, NAND_CTRL_CLE | NAND_CTRL_CHANGE);
	/* Set ALE and clear CLE to start address cycle */
	/* Column address */
	this->cmd_ctrl(mtd, offs & 0xff,
		       NAND_CTRL_ALE | NAND_CTRL_CHANGE); /* A[7:0] */
	this->cmd_ctrl(mtd, (offs >> 8) & 0xff, NAND_CTRL_ALE); /* A[11:9] */
	/* Row address */
	this->cmd_ctrl(mtd, (page_addr & 0xff), NAND_CTRL_ALE); /* A[19:12] */
	this->cmd_ctrl(mtd, ((page_addr >> 8) & 0xff),
		       NAND_CTRL_ALE); /* A[27:20] */
#ifdef CONFIG_SYS_NAND_5_ADDR_CYCLE
	/* One more address cycle for devices > 128MiB */
	this->cmd_ctrl(mtd, (page_addr >> 16) & 0x0f,
		       NAND_CTRL_ALE); /* A[31:28] */
#endif
	/* Latch in address */
	this->cmd_ctrl(mtd, NAND_CMD_READSTART,
		       NAND_CTRL_CLE | NAND_CTRL_CHANGE);
	this->cmd_ctrl(mtd, NAND_CMD_NONE, NAND_NCE | NAND_CTRL_CHANGE);

	/*
	 * Wait a while for the data to be ready
	 */
	if (this->dev_ready)
		while (!this->dev_ready(mtd))
			;
	else
		CONFIG_SYS_NAND_READ_DELAY;

	return 0;
}
#endif

static int nand_is_bad_block(struct mtd_info *mtd, int block)
{
	struct nand_chip *this = mtd->priv;

	nand_command(mtd, block, 0, CONFIG_SYS_NAND_BAD_BLOCK_POS, NAND_CMD_READOOB);

	/*
	 * Read one byte
	 */
	if (readb(this->IO_ADDR_R) != 0xff)
		return 1;

	return 0;
}

static int nand_read_page(struct mtd_info *mtd, int block, int page, uchar *dst)
{
	struct nand_chip *this = mtd->priv;
	u_char *ecc_calc;
	u_char *ecc_code;
	u_char *oob_data;
	int i;
	int eccsize = CONFIG_SYS_NAND_ECCSIZE;
	int eccbytes = CONFIG_SYS_NAND_ECCBYTES;
	int eccsteps = CONFIG_SYS_NAND_ECCSTEPS;
	uint8_t *p = dst;
	int stat;

	xarina_nand_command(mtd, block, page, 0, NAND_CMD_READ0);

	/* No malloc available for now, just use some temporary locations
	 * in SDRAM
	 */
	ecc_calc = (u_char *)(CONFIG_SYS_SDRAM_BASE + 0x10000);
	ecc_code = ecc_calc + 0x100;
	oob_data = ecc_calc + 0x200;

	for (i = 0; eccsteps; eccsteps--, i += eccbytes, p += eccsize) {
		this->ecc.hwctl(mtd, NAND_ECC_READ);
		this->read_buf(mtd, p, eccsize);
		this->ecc.calculate(mtd, p, &ecc_calc[i]);
	}
	this->read_buf(mtd, oob_data, CONFIG_SYS_NAND_OOBSIZE);

	/* Pick the ECC bytes out of the oob data */
	for (i = 0; i < CONFIG_SYS_NAND_ECCTOTAL; i++)
		ecc_code[i] = oob_data[nand_ecc_pos[i]];

	eccsteps = CONFIG_SYS_NAND_ECCSTEPS;
	p = dst;

	for (i = 0 ; eccsteps; eccsteps--, i += eccbytes, p += eccsize) {
		/* No chance to do something with the possible error message
		 * from correct_data(). We just hope that all possible errors
		 * are corrected by this routine.
		 */
		stat = this->ecc.correct(mtd, p, &ecc_code[i], &ecc_calc[i]);
	}

	return 0;
}

static int nand_load(struct mtd_info *mtd, unsigned int offs,
		     unsigned int uboot_size, uchar *dst)
{
	unsigned int block, lastblock;
	unsigned int page;

	/*
	 * offs has to be aligned to a page address!
	 */
	block = offs / CONFIG_SYS_NAND_BLOCK_SIZE;
	lastblock = (offs + uboot_size - 1) / CONFIG_SYS_NAND_BLOCK_SIZE;
	page = (offs % CONFIG_SYS_NAND_BLOCK_SIZE) / CONFIG_SYS_NAND_PAGE_SIZE;

	while (block <= lastblock) {

		if (!nand_is_bad_block(mtd, block)) {
			/*
			 * Skip bad blocks
			 */
			while (page < CONFIG_SYS_NAND_PAGE_COUNT) {
				nand_read_page(mtd, block, page, dst);
				dst += CONFIG_SYS_NAND_PAGE_SIZE;
				page++;
			}

			page = 0;
		} else {
			lastblock++;
		}

		block++;
	}

	return 0;
}

#else /* !defined(CONFIG_XARINA) */


/***************************************
*
*	for XARINA NAND SPL
*
****************************************/
/**
 * xarina_put_num
 *
 * print number by hex
 * @num:	number
 * @return:	none
*/

static void xarina_put_num(char num)
{
	char tmp[3];

	tmp[0] = char_list[(num / 16) % 16];
	tmp[1] = char_list[(num % 16)];
	tmp[2] = 0;
	
	serial_puts(tmp);

	return;
}


/**
 * nand_command for NAND1
 *
 * send nand command (READ0 and READOOB are supported)
 * @mtd: 	MTD device structure
 * @block:	physical block number
 * @page:	page number
 * @offs:	page offset (not used)
 * @cmd:	command to send
 * @return:	none
*/
static int nand1_command(struct mtd_info *mtd, int block, int page, int offs, u8 cmd)
{

	register struct nand_chip *chip = mtd->priv;
	unsigned long page_addr = (unsigned int)(page + block * NAND_PAGE_COUNT);
	unsigned long tmp_addr;
	unsigned short task, tmp_task;

	if (chip->dev_ready)
		while (!chip->dev_ready(mtd))
			;
	else
		CONFIG_SYS_NAND_READ_DELAY;


	xarina_nand1_write_protect(mtd, XARINA_NAND1_WP_OFF);

	switch (cmd) {
	/* Emulate NAND_CMD_READOOB */
	case NAND_CMD_READOOB:
	case NAND_CMD_READ0:

		/* command setting */
		WRITE16(cmd, chip->IO_ADDR_W + XARINA_NAND1_COMMAND1_REG);
		WRITE16(NAND_CMD_READSTART, chip->IO_ADDR_W + XARINA_NAND1_COMMAND2_REG);
		/* address setting */
		tmp_addr = (unsigned short)(page_addr & XARINA_NAND1_MASK_16BIT);
		/* set page0, page1 */
		WRITE16(tmp_addr, chip->IO_ADDR_W + XARINA_NAND1_PAGE_ADDR0_REG);
		tmp_addr = (unsigned short)((page_addr >> XARINA_NAND1_SHIFT_16) & XARINA_NAND1_MASK_8BIT);
		/* set page2 */
		WRITE16(tmp_addr, chip->IO_ADDR_W + XARINA_NAND1_PAGE_ADDR2_REG);
		/* set col0, col1 */
		WRITE16(0, chip->IO_ADDR_W + XARINA_NAND1_COL_ADDR0_REG);
		/* set task */
		task = XARINA_NAND1_BIT_TASK_COMMAND1
				| XARINA_NAND1_BIT_TASK_COMMAND2
				| XARINA_NAND1_BIT_TASK_COL_ADR0
				| XARINA_NAND1_BIT_TASK_COL_ADR1
				| XARINA_NAND1_BIT_TASK_PAGE_ADR0
				| XARINA_NAND1_BIT_TASK_PAGE_ADR1
				| XARINA_NAND1_BIT_TASK_PAGE_ADR2
				| XARINA_NAND1_BIT_TASK_WAIT_READY1;

		tmp_task = READ16(chip->IO_ADDR_W + XARINA_NAND1_TASK_REG);
		tmp_task |= task;
		WRITE16(tmp_task, chip->IO_ADDR_W + XARINA_NAND1_TASK_REG);
		/* ready waiting */
		while (1) {
			if (chip->dev_ready(mtd)) {
				break;
			}
		}
		break;
	case NAND_CMD_RESET:
		xarina_nand1_write_protect(mtd, XARINA_NAND1_WP_OFF);
		/* command setting */
		WRITE16(cmd, chip->IO_ADDR_W + XARINA_NAND1_COMMAND1_REG);

		task = XARINA_NAND1_BIT_TASK_COMMAND1
				| XARINA_NAND1_BIT_TASK_WAIT_READY1;

		tmp_task = READ16(chip->IO_ADDR_W + XARINA_NAND1_TASK_REG);
		tmp_task |= task;
		WRITE16(tmp_task, chip->IO_ADDR_W + XARINA_NAND1_TASK_REG);

		/* ready waiting */
		while (1) {
			if (chip->dev_ready(mtd)) {
				break;
			}
		}
		xarina_nand1_write_protect(mtd, XARINA_NAND1_WP_ON);
	default:
		break;
	}

	if (chip->dev_ready)
		while (!chip->dev_ready(mtd))
			;
	else
		CONFIG_SYS_NAND_READ_DELAY;

	return 0;
}

/**
 * nand_command for NAND2
 *
 * send nand command (READ0 and READOOB are supported)
 * @mtd: 	MTD device structure
 * @block:	physical block number
 * @page:	page number
 * @offs:	page offset (not used)
 * @cmd:	command to send
 * @return:	none
*/
static int nand2_command(struct mtd_info *mtd, int block, int page, int offs, u8 cmd)
{

	register struct nand_chip *chip = mtd->priv;
	unsigned long page_addr = (unsigned int)(page + block * NAND_PAGE_COUNT);
	unsigned long tmp_addr;
	unsigned short task, tmp_task;

	if (chip->dev_ready)
		while (!chip->dev_ready(mtd))
			;
	else
		CONFIG_SYS_NAND_READ_DELAY;


	xarina_nand2_write_protect(mtd, XARINA_NAND2_WP_OFF);

	switch (cmd) {
	/* Emulate NAND_CMD_READOOB */
	case NAND_CMD_READOOB:
	case NAND_CMD_READ0:

		/* command setting */
		WRITE16(cmd, chip->IO_ADDR_W + XARINA_NAND2_COMMAND1_REG);
		WRITE16(NAND_CMD_READSTART, chip->IO_ADDR_W + XARINA_NAND2_COMMAND2_REG);
		/* address setting */
		tmp_addr = (unsigned short)(page_addr & XARINA_NAND2_MASK_16BIT);
		/* set page0, page1 */
		WRITE16(tmp_addr, chip->IO_ADDR_W + XARINA_NAND2_PAGE_ADDR01_REG);
		tmp_addr = (unsigned short)((page_addr >> XARINA_NAND2_SHIFT_16) & XARINA_NAND2_MASK_8BIT);
		/* set page2 */
		WRITE16(tmp_addr, chip->IO_ADDR_W + XARINA_NAND2_PAGE_ADDR2_REG);
		/* set col0, col1 */
		WRITE16(0, chip->IO_ADDR_W + XARINA_NAND2_COL_ADDR_REG);
		/* set task */
		task = XARINA_NAND2_BIT_TASK_COMMAND1
				| XARINA_NAND2_BIT_TASK_COMMAND2
				| XARINA_NAND2_BIT_TASK_COL_ADR0
				| XARINA_NAND2_BIT_TASK_COL_ADR1
				| XARINA_NAND2_BIT_TASK_PAGE_ADR0
				| XARINA_NAND2_BIT_TASK_PAGE_ADR1
				| XARINA_NAND2_BIT_TASK_PAGE_ADR2
				| XARINA_NAND2_BIT_TASK_WAIT_READY1;

		tmp_task = READ16(chip->IO_ADDR_W + XARINA_NAND2_TASK_REG);
		tmp_task |= task;
		WRITE16(tmp_task, chip->IO_ADDR_W + XARINA_NAND2_TASK_REG);
		/* ready waiting */
		while (1) {
			if (chip->dev_ready(mtd)) {
				break;
			}
		}
		break;
	case NAND_CMD_RESET:
		xarina_nand2_write_protect(mtd, XARINA_NAND2_WP_OFF);
		/* command setting */
		WRITE16(cmd, chip->IO_ADDR_W + XARINA_NAND2_COMMAND1_REG);

		task = XARINA_NAND2_BIT_TASK_COMMAND1
				| XARINA_NAND2_BIT_TASK_WAIT_READY1;

		tmp_task = READ16(chip->IO_ADDR_W + XARINA_NAND2_TASK_REG);
		tmp_task |= task;
		WRITE16(tmp_task, chip->IO_ADDR_W + XARINA_NAND2_TASK_REG);

		/* ready waiting */
		while (1) {
			if (chip->dev_ready(mtd)) {
				break;
			}
		}
		xarina_nand2_write_protect(mtd, XARINA_NAND2_WP_ON);
	default:
		break;
	}

	if (chip->dev_ready)
		while (!chip->dev_ready(mtd))
			;
	else
		CONFIG_SYS_NAND_READ_DELAY;

	return 0;
}


/**
 * nand_correct_data for NAND1
 *
 * correct data by ecc information
 * @data: 	pointer to data
 * @len:	length of data
 * @ecc:	ecc information structure	
 * @retval:	ECC_NO_ERR	no error
 * @retval:	ECC_CO_ERR	correctable error and correct data
 * @retval:	ECC_UC_ERR	uncorrectable error
*/
static int nand1_correct_data(struct mtd_info *mtd, unsigned char *data, unsigned short len, ecc_info *ecc)
{
	int ret = ECC_NO_ERR;
	
	if(ecc->chk == 0) {
		/* no error */
		return ret;
	}
	
	if(ecc->chk & XARINA_NAND1_CERR) {
		ret = ECC_CO_ERR;
		if((ecc->err_addr * sizeof(unsigned short)) < len) {
			/* data correct */
			*(unsigned short *)(data + ecc->err_addr * sizeof(unsigned short)) ^= ecc->err_bit;
		}
		serial_puts("correctable error\n");
	}
	
	if(ecc->chk & XARINA_NAND1_UCERR) {
		ret = ECC_UC_ERR;
		serial_puts("uncorrectable error\n");
	}
	
	return ret;
}

/**
 * nand_correct_data for NAND2
 *
 * correct data by ecc information
 * @data: 	pointer to data
 * @len:	length of data
 * @ecc:	ecc information structure	
 * @retval:	ECC_NO_ERR	no error
 * @retval:	ECC_CO_ERR	correctable error and correct data
 * @retval:	ECC_UC_ERR	uncorrectable error
*/
static int nand2_correct_data(struct mtd_info *mtd, unsigned char *data, unsigned short len, ecc_info *ecc)
{
	int ret = ECC_NO_ERR;
	int error_corrected = 0;
	int ecc_num, i;
	struct nand_chip *chip = mtd->priv;
	u_short *e_addr;
	u_short e_bits;
	u_short ecc_result;
	
	if(ecc->chk == 0) {
		/* no error */
		return ret;
	}


	/* calculation start */
	WRITE16(XARINA_NAND2_BIT_ECC_CALC_START, chip->IO_ADDR_W + XARINA_NAND2_ECC_CALC_START_REG);

	while(1){
		if(READ16(chip->IO_ADDR_R + XARINA_NAND2_ECC_CALC_START_REG) == 0)
			break;
	}

	/* Error Check */
	ecc_result = READ16(chip->IO_ADDR_R + XARINA_NAND2_ECC_RESULT_REG) >> XARINA_NAND2_ECC_RESULT_SHIFT;

	switch(ecc_result){
	case XARINA_NAND2_BIT_ECC_NO_ERR:
		/* No Error */
		return ECC_NO_ERR;
	case XARINA_NAND2_BIT_ECC_C_ERR:
		serial_puts("correctable error\n");
		break;
	case XARINA_NAND2_BIT_ECC_UC_ERR:
	case XARINA_NAND2_BIT_ECC_5_ERR:
	default:
		serial_puts("uncorrectable error\n");
		return ECC_UC_ERR;
	}


	ecc_num = READ16(chip->IO_ADDR_R + XARINA_NAND2_ECC_RESULT_REG) & XARINA_NAND2_BIT_ECC_CORRECT_MASK;

	for(i = 0; i < ecc_num; i++){
		e_addr = (u_short *)(data + READ16(chip->IO_ADDR_R + XARINA_NAND2_ECC_ERR_ADDR_BASE_REG + XARINA_NAND2_ECC_ERROR_REG_OFFSET * i));
		e_bits = READ16(chip->IO_ADDR_R + XARINA_NAND2_ECC_ERR_BIT_BASE_REG + XARINA_NAND2_ECC_ERROR_REG_OFFSET * i);
		*(e_addr) ^= e_bits;
		error_corrected++;
	}
	
	return ret;
}


/**
 * nand_read_page
 *
 * read data and oob of NAND page. for NAND1
 * @mtd: 	MTD device structure 
 * @block:	block number
 * @page:	page number	
 * @dst_data:	address to save data
 * @dst_oob:	address to save oob
 * @retval:	0	read done successfully
 * @retval:	1	read done unsuccessfully 
 */
static int nand1_read_page(struct mtd_info *mtd, int block, 
			int page, uchar *dst_data, uchar *dst_oob)
{
	struct nand_chip *this = mtd->priv;
	int i, j;
	int eccsize = XARINA_NAND1_ECC_SIZE;
	int eccsteps = XARINA_NAND1_ECC_STEPS;
	ecc_info ei;
	int ret = 0;
	int oob_read_flag = 0;

	nand1_command(mtd, block, page, 0, NAND_CMD_READ0);

	for(i = 0; i < eccsteps; i++){
		/* read data */
		/* enable hw ecc */
		this->ecc.hwctl(mtd, NAND_ECC_READ);
		this->read_buf(mtd, dst_data + (i * eccsize), eccsize);
		/* store ecc(data) calculation result */
		ei.chk 	    = READ16(this->IO_ADDR_W + XARINA_NAND1_ECC_CHECK_REG);
		ei.err_bit  = READ16(this->IO_ADDR_W + XARINA_NAND1_ECC_ERR_BIT_REG);
		ei.err_addr = READ16(this->IO_ADDR_W + XARINA_NAND1_ECC_ERR_ADDR_REG);
		
		if (nand1_correct_data(mtd, dst_data + (i * eccsize), eccsize, &ei) == ECC_UC_ERR) {
			return 1;
		}

		/* read oob */
		/* enable hw ecc */
		this->ecc.hwctl(mtd, NAND_ECC_READ);
		this->read_buf(mtd, dst_oob + i * 8, 8);
		/* store ecc(oob) calculation result */
		ei.chk 	    = READ16(this->IO_ADDR_W + XARINA_NAND1_ECC_CHECK_REG);
		ei.err_bit  = READ16(this->IO_ADDR_W + XARINA_NAND1_ECC_ERR_BIT_REG);
		ei.err_addr = READ16(this->IO_ADDR_W + XARINA_NAND1_ECC_ERR_ADDR_REG);
		
		if (nand1_correct_data(mtd, dst_oob + (i * eccsize), eccsize, &ei) == ECC_UC_ERR) {
			for (j = 0; j < sizeof(xa_oob); j++) {
				/* delete error oob data */
				*(dst_oob + i * sizeof(xa_oob) + j) = INVALID;
			}
			ret = 1;
		} else {
			oob_read_flag = 1;
		}
	}
	
	if (oob_read_flag) {
		ret = 0;
	}
	
	return ret;
}
/**
 * nand_read_page
 *
 * read data and oob of NAND page. for NAND2
 * @mtd: 	MTD device structure 
 * @block:	block number
 * @page:	page number	
 * @dst_data:	address to save data
 * @dst_oob:	address to save oob
 * @retval:	0	read done successfully
 * @retval:	1	read done unsuccessfully 
 */
static int nand2_read_page(struct mtd_info *mtd, int block, 
			int page, uchar *dst_data, uchar *dst_oob)
{
	struct nand_chip *this = mtd->priv;
	int i, j, k;
	int eccsize = XARINA_NAND2_ECC_SIZE;
	int eccsteps = XARINA_NAND2_ECC_STEPS;
	ecc_info ei;
	int ret = 0;
	int oob_read_flag = 0;

	nand2_command(mtd, block, page, 0, NAND_CMD_READ0);

	for (i = 0; i < 2; i++) {
		for (j = 0; j < (eccsteps / 2); j++) {
			/* read data */
			/* enable hw ecc */
			this->ecc.hwctl(mtd, NAND_ECC_READ);
			this->read_buf(mtd, dst_data + (j * eccsize) + (i * XARINA_NAND2_PAGESIZE / 2), 
					eccsize);
			ei.chk  = READ16(this->IO_ADDR_W + XARINA_NAND2_ECC_CHECK_REG) & XARINA_NAND2_BIT_ECCCHK;

			if (nand2_correct_data(mtd, 
					dst_data + (j * eccsize) + (i * XARINA_NAND2_PAGESIZE / 2), 
					eccsize, &ei) == ECC_UC_ERR) {
				return 1;
			}

		}
		/* read oob */
		/* enable hw ecc */
		this->ecc.hwctl(mtd, NAND_ECC_READ);
		this->read_buf(mtd, dst_oob + i * sizeof(xa_oob), sizeof(xa_oob));
		ei.chk  = READ16(this->IO_ADDR_W + XARINA_NAND2_ECC_CHECK_REG) & XARINA_NAND2_BIT_ECCCHK;

		if (nand2_correct_data(mtd, dst_oob + i * sizeof(xa_oob), 
						sizeof(xa_oob), &ei) == ECC_UC_ERR) {
			for (k = 0; k < sizeof(xa_oob); k++) {
				/* delete error oob data */
				*(dst_oob + i * sizeof(xa_oob) + k) = INVALID;
			}
			ret = 1;
		} else {
			oob_read_flag = 1;
		}
	}
	
	if (oob_read_flag) {
		ret = 0;
	}
	
	return ret;
}
/**
 * nand_read_oob
 *
 * read oob of NAND page. (data area is read but not stored) for NAND1
 * @mtd: 	MTD device structure 
 * @block:	block number
 * @page:	page number	
 * @dst:	address to save oob
 * @retval:	0	read done successfully
 * @retval:	1	read done unsuccessfully 
 */
static int nand1_read_oob(struct mtd_info *mtd, int block, int page, uchar *dst)
{
	struct nand_chip *this = mtd->priv;
	int i, j;
	int eccsize = XARINA_NAND1_ECC_SIZE;
	int eccsteps = XARINA_NAND1_ECC_STEPS;
	unsigned char tmp_data[XARINA_NAND1_ECC_SIZE];
	ecc_info ei;
	int ret = 0;

	nand1_command(mtd, block, page, 0, NAND_CMD_READ0);

	for(i = 0; i < eccsteps; i++){
		/* read data */
		/* enable hw ecc */
		this->ecc.hwctl(mtd, NAND_ECC_READ);
		this->read_buf(mtd, tmp_data, eccsize);
		/* read oob */
		/* enable hw ecc */
		this->ecc.hwctl(mtd, NAND_ECC_READ);
		this->read_buf(mtd, dst + i * sizeof(xa_oob), sizeof(xa_oob));
		/* store ecc(oob) calculation result */
		ei.chk 	    = READ16(this->IO_ADDR_W + XARINA_NAND1_ECC_CHECK_REG);
		ei.err_bit  = READ16(this->IO_ADDR_W + XARINA_NAND1_ECC_ERR_BIT_REG);
		ei.err_addr = READ16(this->IO_ADDR_W + XARINA_NAND1_ECC_ERR_ADDR_REG);

		if (nand1_correct_data(mtd, dst + i * sizeof(xa_oob), sizeof(xa_oob), &ei) == ECC_UC_ERR) {
			for (j = 0; j < sizeof(xa_oob); j++) {
				/* delete error oob data */
				*(dst + i * sizeof(xa_oob) + j) = INVALID;
			}
			ret = 1;
		};
	}

	return ret;
}

/**
 * nand_read_oob
 *
 * read oob of NAND page. (data area is read but not stored) for NAND2
 * @mtd: 	MTD device structure 
 * @block:	block number
 * @page:	page number	
 * @dst:	address to save oob
 * @retval:	0	read done successfully
 * @retval:	1	read done unsuccessfully 
 */
static int nand2_read_oob(struct mtd_info *mtd, int block, int page, uchar *dst)
{
	struct nand_chip *this = mtd->priv;
	int i, j, k;
	int eccsize = XARINA_NAND2_ECC_SIZE;
	int eccsteps = XARINA_NAND2_ECC_STEPS;
	unsigned char tmp_data[XARINA_NAND2_ECC_SIZE];
	ecc_info ei;
	int ret = 0;

	nand2_command(mtd, block, page, 0, NAND_CMD_READ0);

	for (i = 0; i < 2; i++) {
		for (j = 0; j < (eccsteps / 2); j++) {
			/* read data */
			/* enable hw ecc */
			this->ecc.hwctl(mtd, NAND_ECC_READ);
			this->read_buf(mtd, tmp_data, eccsize);
		}
		/* read oob */
		/* enable hw ecc */
		this->ecc.hwctl(mtd, NAND_ECC_READ);
		this->read_buf(mtd, dst + i * sizeof(xa_oob), sizeof(xa_oob));
		ei.chk  = READ16(this->IO_ADDR_W + XARINA_NAND2_ECC_CHECK_REG) & XARINA_NAND2_BIT_ECCCHK;

		if (nand2_correct_data(mtd, dst + i * sizeof(xa_oob), 
						sizeof(xa_oob), &ei) == ECC_UC_ERR) {
			for (k = 0; k < sizeof(xa_oob); k++) {
				/* delete error oob data */
				*(dst + i * sizeof(xa_oob) + k) = INVALID;
			}
			ret = 1;
		}
	}


	return ret;
}


/**
 * nand_load
 *
 * load U-Boot binary from NAND device
 * @mtd: 	MTD device structure 
 * @dst:	address to save U-Boot binary
 * @retval:	0	load done successfully
 * @retval:	1	load done unsuccessfully 
 */
static int nand_load(struct mtd_info *mtd, uchar *dst, lblk_tbl *table)
{
	int i, j, k;
	unsigned char tmp_oob[OOB_FREE_MAX];
	xa_oob *oob_data;
	int result;
	
	serial_puts("load version: ");
	xarina_put_num(table->version);
	serial_puts("\n");

	
	for (i = 0; i < MAX_LOGICAL_BLOCK; i++) {
		if (table->lblk[MAIN_TABLE][i] == INVALID) {
			break;
		}
#if defined(NAND_BOOT_DEBUG)
		serial_puts("load block: ");
		xarina_put_num(table->lblk[MAIN_TABLE][i]);
		serial_puts("\n");
#endif	/* defined(NAND_BOOT_DEBUG) */
		for (j = 0; j < xarina_nand_page_count; j++) {
		
			result = xarina_nand_read_page(mtd, table->lblk[MAIN_TABLE][i], j, 
				dst + i * xarina_nand_block_size 
				    + j * xarina_nand_page_size,
				tmp_oob);
			if(result) {
#if defined(NAND_BOOT_DEBUG)
				serial_puts("read mirror block table\n");
				serial_puts("load block: ");
				xarina_put_num(table->lblk[MIRR_TABLE][i]);
				serial_puts("\n");
#endif	/* defined(NAND_BOOT_DEBUG) */
				/* read mirror table */
				if (table->lblk[MIRR_TABLE][i] == INVALID) {
					serial_puts("can not find mirror block version ");
					xarina_put_num(table->version);
					serial_puts("\n");
					return 1;
				}
				serial_puts("read mirror block\n");
				result = xarina_nand_read_page(mtd, table->lblk[MIRR_TABLE][i], j, 
					dst + i * xarina_nand_block_size 
					    + j * xarina_nand_page_size,
					tmp_oob);				
				if (result) {
					serial_puts("can not load U-Boot image version ");
					xarina_put_num(table->version);
					serial_puts("\n");
					return 1;
				}
			}
			
			for (k = 0; k < xarina_nand_oob_count; k++) {
				oob_data = (xa_oob *)(tmp_oob + sizeof(xa_oob) * k);
				if (cmp_mark(oob_data->mark, mark_uboot) && 
				    (oob_data->contflag != 0xff)) {
				    return 0;
				}		
			}
		}
	}
	serial_puts("can not find last page.\n");
	
	return 1;
}

#if 0
static int nand_load_legacy(struct mtd_info *mtd, unsigned int offs,
		     unsigned int uboot_size, uchar *dst)
{
	unsigned int block, lastblock;
	unsigned int page;
	unsigned char oob[CONFIG_XARINA_NAND_OOB_SIZE];
	int ret;

	/*
	 * offs has to be aligned to a page address!
	 */
	block = offs / CONFIG_SYS_NAND_BLOCK_SIZE;
	lastblock = (offs + uboot_size - 1) / CONFIG_SYS_NAND_BLOCK_SIZE;
	page = (offs % CONFIG_SYS_NAND_BLOCK_SIZE) / CONFIG_SYS_NAND_PAGE_SIZE;

	while (block <= lastblock) {
		while (page < CONFIG_SYS_NAND_PAGE_COUNT) {
			ret = xarina_nand_read_page(mtd, block, page, dst, oob);
			if (ret) {
				dst -= page * CONFIG_SYS_NAND_PAGE_SIZE;
				lastblock++;
				break;
			}
			dst += CONFIG_SYS_NAND_PAGE_SIZE;
			page++;
		}

		page = 0;

		block++;
	}

	return 0;
}
#endif
/**
 * cmp_mark
 *
 * commpare two marks in oob
 * @src0: 	address to mark0
 * @src1:	address to mark1
 * @retval:	0	two marks are different
 * @retval:	1	two marks are same 
 */
static int cmp_mark(unsigned char *src0, const unsigned char *src1)
{
	int i;
	int ret = 1;
	
	for(i = 0; i < MARK_LEN; i++) {
		if(src0[i] != src1[i]) {
			ret = 0;
		}
	}
	return ret;
}
/**
 * init_lblk_tbl
 *
 * fill logical block table with 0xff.
 * @table: 	address to logical block table
 * @return:	none
 */
static void init_lblk_tbl(lblk_tbl *table)
{
	int i ,j;
	
	for(i = 0; i < TABLE_NUM; i++) {
		for (j = 0; j < MAX_LOGICAL_BLOCK; j++) {
			table->lblk[i][j] = INVALID;
		}
	}
	table->version = INVALID;
}

/**
 * set_blk_num
 *
 * set physical block number to logical block table.
 * @table: 	address to logical block table
 * @lblk_num: 	logical block number
 * @pblk_num: 	physical block number
 * @return:	none
 */
static void set_blk_num(lblk_tbl *table, unsigned char lblk_num, unsigned char pblk_num)
{
	if (table->lblk[MAIN_TABLE][lblk_num] == INVALID) {
		table->lblk[MAIN_TABLE][lblk_num] = pblk_num;
	} else if (table->lblk[MIRR_TABLE][lblk_num] == INVALID) {
		table->lblk[MIRR_TABLE][lblk_num] = pblk_num;
	} else {
		/* do nothing */
	}
}


/**
 * cmp_version
 *
 * compare two numbers.
 * notice: 0x00 is bigger than 0xff.
 * @ver0: 	version number to compare	
 * @ver1: 	version number to compare	
 * @retval:	0	ver1 is smaller than ver0
 * @retval:	1	ver1 is bigger than ver0
 */
static int cmp_version(unsigned char ver0, unsigned char ver1)
{
	if (ver0 < ver1) {
		return (0 < ((ver1 - ver0 + 0x01) & 0xff));
	} else {
		return (1 > ((ver0 - ver1 + 0x01) & 0xff)); 
	}
}
/**
 * create_lblk_table
 *
 * scan oob and create logical block table.
 * @mtd: 	MTD device structure 
 * @offset: 	physical block number to start scanning
 * @block_len: physical block length to scan
 * @ver_cnt: 	pointer to valuable to save version count in
 * @retval:	0	done successfully
 * @retval:	1	done unsuccessfully
 */
static int create_lblk_table(struct mtd_info *mtd, int offset, 
			    int block_len, unsigned char *ver_cnt)
{

	int i, j;
	unsigned char oob_buf[OOB_FREE_MAX];
	xa_oob *oob_data;
	int ret = 1;

	/* initialize tables */
	init_lblk_tbl(&(xarina_lblk[TBL_SET_X]));
	init_lblk_tbl(&(xarina_lblk[TBL_SET_Y]));
	*ver_cnt = 0;
	
	
#if defined(NAND_BOOT_DEBUG)
	serial_puts("PBN:LBN:VER (valid block list)\n");
#endif	/* defined(NAND_BOOT_DEBUG) */				
	
	for (i = 0; i < block_len; i++) {
		if(xarina_nand_read_oob(mtd, i + offset, 0, oob_buf)) {
			serial_puts("scan oob table: uncorrectable error\n");
		}

		for (j = 0; j < xarina_nand_oob_count; j++) {
			oob_data = (xa_oob *)(oob_buf + j * sizeof(xa_oob));
			if (cmp_mark(oob_data->mark, mark_uboot) && 
			    (oob_data->loblock < MAX_LOGICAL_BLOCK)) {
				switch (*ver_cnt) {
				case 0:
					xarina_lblk[TBL_SET_X].version = oob_data->version;
					set_blk_num(&(xarina_lblk[TBL_SET_X]), 
						oob_data->loblock, i + offset);
					(*ver_cnt)++;
					break;
				case 1:
					if (xarina_lblk[TBL_SET_X].version == oob_data->version) {
						set_blk_num(&(xarina_lblk[TBL_SET_X]), 
							oob_data->loblock, i + offset);
					} else {
						xarina_lblk[TBL_SET_Y].version = oob_data->version;
						set_blk_num(&(xarina_lblk[TBL_SET_Y]), 
							oob_data->loblock, i + offset);
						(*ver_cnt)++;
					}
					break;
				case 2:
					if (xarina_lblk[TBL_SET_X].version == oob_data->version) {
						set_blk_num(&(xarina_lblk[TBL_SET_X]), 
							oob_data->loblock, i + offset);
					} else if (xarina_lblk[TBL_SET_Y].version == oob_data->version){
						set_blk_num(&(xarina_lblk[TBL_SET_Y]), 
							oob_data->loblock, i + offset);
					} else {
						/* do nothing */
					}
					break;
				default:
					/* no pass */
					break;
				}
				
#if defined(NAND_BOOT_DEBUG)
				serial_puts(" ");
				xarina_put_num((char)(i + offset));
				serial_puts(": ");
				xarina_put_num(oob_data->loblock);
				serial_puts(": ");
				xarina_put_num(oob_data->version);
				serial_puts("\n");
#endif	/* defined(NAND_BOOT_DEBUG) */				

				break;
			}
		}
	}
#if defined(NAND_BOOT_DEBUG)
	serial_puts("version count: ");
	xarina_put_num(*ver_cnt);
	serial_puts("\n");
	for(j = 0; j < *ver_cnt; j++) { 
		serial_puts("version ");
		xarina_put_num(xarina_lblk[j].version);
		serial_puts(" main table -----------------\n");
		for (i = 0; i < MAX_LOGICAL_BLOCK; i++) {
			xarina_put_num(i);
			serial_puts(":"); 
			xarina_put_num(xarina_lblk[j].lblk[MAIN_TABLE][i]);
			serial_puts("\n");	
		}
		serial_puts("version ");
		xarina_put_num(xarina_lblk[j].version);
		serial_puts(" mirror table -----------------\n");
		for (i = 0; i < MAX_LOGICAL_BLOCK; i++) {
			xarina_put_num(i);
			serial_puts(":"); 
			xarina_put_num(xarina_lblk[j].lblk[MIRR_TABLE][i]);
			serial_puts("\n");	
		}
	}
#endif	/* defined(NAND_BOOT_DEBUG) */
	
	if (1 <= *ver_cnt) {
		ret = 0;
	}
	
	return ret;
}

/**
 * xarina_board_init
 *
 * initialize specific board setting.
 * @return	none
 */
void xarina_board_init(void)
{
	/* enable uart3 */
	pinmux_gpio_uart3_set();
	/* NFIF clock set*/
	xarina_hw_nfif_clock_set(SYSREG1_NFIF_CKSEL_80MHZ);
	/* wait */
	CONFIG_SYS_NAND_READ_DELAY;
}

static void boot_type_set(void)
{
	unsigned char *addr = (unsigned char*)XARINA_BOOT_TYPE_ADDR;

	*addr = XARINA_BOOT_NAND_BOOT;

}


#endif /*  !defined(CONFIG_XARINA) */

/*
 * The main entry for NAND booting. It's necessary that SDRAM is already
 * configured and available since this code loads the main U-Boot image
 * from NAND into SDRAM and starts it from there.
 */
 
 
/**
 * nand_boot
 *
 * load U-Boot binary from NAND deviec and boot it.
 * @return	none
 */
void nand_boot(void)
{
	struct nand_chip nand_chip;
	nand_info_t nand_info;
	int ret;
	__attribute__((noreturn)) void (*uboot)(void);

#ifdef CONFIG_XARINA
	unsigned char ver_cnt = 0;
	unsigned char read_ver;
	int block_len;

	xarina_board_init();
	serial_init();
	serial_puts("wake up XARINA NAND SPL\n");
#endif	
	/*
	 * Init board specific nand support
	 */
	nand_info.priv = &nand_chip;
	nand_chip.IO_ADDR_R = nand_chip.IO_ADDR_W = (void  __iomem *)CONFIG_SYS_NAND_BASE;
	nand_chip.dev_ready = NULL;	/* preset to NULL */


#ifdef CONFIG_XARINA

	if (xarina_hw_nand_cont_type_get() == XARINA_NAND_CONTROLLER1) {
		board_nand1_info_set(&nand_chip);
		xarina_nand_read_oob = nand1_read_oob;
		xarina_nand_read_page = nand1_read_page;
		xarina_nand_command = nand1_command;
		xarina_nand_block_size = XARINA_NAND1_PAGESIZE * NAND_PAGE_COUNT;
		xarina_nand_page_size = XARINA_NAND1_PAGESIZE;
		xarina_nand_page_count = NAND_PAGE_COUNT;
		xarina_nand_oob_count = NAND1_OOB_COUNT;
		block_len = CONFIG_XARINA_NAND1_U_BOOT_BIN_LEN;
	} else {
		board_nand2_info_set(&nand_chip);
		xarina_nand_read_oob = nand2_read_oob;
		xarina_nand_read_page = nand2_read_page;
		xarina_nand_command = nand2_command;
		xarina_nand_block_size = XARINA_NAND2_PAGESIZE * NAND_PAGE_COUNT;
		xarina_nand_page_size = XARINA_NAND2_PAGESIZE;
		xarina_nand_page_count = NAND_PAGE_COUNT;
		xarina_nand_oob_count = NAND2_OOB_COUNT;
		block_len = CONFIG_XARINA_NAND2_U_BOOT_BIN_LEN;
	}


	if (nand_chip.select_chip)
		nand_chip.select_chip(&nand_info, 0);

	if (create_lblk_table(&nand_info, 
				CONFIG_XARINA_U_BOOT_BIN_START,
				block_len, &ver_cnt)) {
		serial_puts("oob scan error\n");
		while(1);
	}

	
	/*
	 * Load U-Boot image from NAND into RAM
	 */

	/* NAND Reset */
	xarina_nand_command(&nand_info, 0, 0, 0, NAND_CMD_RESET);
	
	if (ver_cnt == 1) {
		ret = nand_load(&nand_info, 
				(uchar *)CONFIG_SYS_NAND_U_BOOT_DST, 
				&(xarina_lblk[TBL_SET_X]));
	} else {
		read_ver = cmp_version(xarina_lblk[TBL_SET_X].version, 
				       xarina_lblk[TBL_SET_Y].version) ? TBL_SET_Y : TBL_SET_X;
		ret = nand_load(&nand_info, 
				(uchar *)CONFIG_SYS_NAND_U_BOOT_DST, 
				&(xarina_lblk[read_ver]));
		if (ret != 0) {
			serial_puts("Loading version is changed\n");
			read_ver ^= 0x1;
			ret = nand_load(&nand_info, 
					(uchar *)CONFIG_SYS_NAND_U_BOOT_DST, 
					&(xarina_lblk[read_ver]));
		}
	}
	
	if(ret != 0) {
		serial_puts("Load U-Boot Failure\n");
		while(1);
	} else {
		serial_puts("Jump U-Boot\n");
		boot_type_set();
	}
#endif	
	

#ifdef CONFIG_NAND_ENV_DST
	nand_load_legacy(&nand_info, CONFIG_ENV_OFFSET, CONFIG_ENV_SIZE,
		  (uchar *)CONFIG_NAND_ENV_DST);

#ifdef CONFIG_ENV_OFFSET_REDUND
	nand_load_legacy(&nand_info, CONFIG_ENV_OFFSET_REDUND, CONFIG_ENV_SIZE,
		  (uchar *)CONFIG_NAND_ENV_DST + CONFIG_ENV_SIZE);
#endif
#endif

	if (nand_chip.select_chip)
		nand_chip.select_chip(&nand_info, -1);

	/*
	 * Jump to U-Boot image
	 */
	uboot = (void *)CONFIG_SYS_NAND_U_BOOT_START;
	(*uboot)();
}

